package ar;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import edu.unh.sdb.common.FloatValues;
import edu.unh.sdb.datasource.ARSparseDataSource;
import edu.unh.sdb.datasource.DataBlock;
import edu.unh.sdb.datasource.DataSource;
import edu.unh.sdb.datasource.ISBounds;
import edu.unh.sdb.datasource.IntegerIndexSpaceID;
import edu.unh.sdb.datasource.SparseBlockInfo;

public class ARSparseDataGeneration {

	private ARSparseDataSource arSparseDS ; 
	
	private List<List<ARNodeBlock>> arInfoList;
	
	private List<List<DataBlock>> arDataBlocksList ;

	private Integer noOfResolutions;
	
	private boolean justAddedNode ; 
	
	private List<List<SparseBlockInfo>> arSparseBlockInfoList ; 

	public ARSparseDataGeneration(int noOfResolutions, ARSparseDataSource arSparseDS) {
		this.noOfResolutions = noOfResolutions;
		this.justAddedNode = false ;
		this.arSparseDS = arSparseDS ; 
		arInfoList = new ArrayList<List<ARNodeBlock>>(noOfResolutions);

		for (int i = 0; i < noOfResolutions; i++) {
			List<ARNodeBlock> list = new ArrayList<ARNodeBlock>();
			arInfoList.add(i, list);
		}
	}

	public List<List<ARNodeBlock>> getArInfoList() {
		return arInfoList;
	}

	public void setArInfoList(List<List<ARNodeBlock>> arInfoList) {
		this.arInfoList = arInfoList;
	}

	public Integer getNoOfResolutions() {
		return noOfResolutions;
	}

	public void setNoOfResolutions(Integer noOfResolutions) {
		this.noOfResolutions = noOfResolutions;
	}
	
	

	public boolean isJustAddedNode() {
		return justAddedNode;
	}

	public void setJustAddedNode(boolean justAddedNode) {
		this.justAddedNode = justAddedNode;
	}

	
	public List<List<DataBlock>> getArDataBlocksList() {
		return arDataBlocksList;
	}

	public void setArDataBlocksList(List<List<DataBlock>> arDataBlocksList) {
		this.arDataBlocksList = arDataBlocksList;
	}
	

	public ARSparseDataSource getArSparseDS() {
		return arSparseDS;
	}

	public void setArSparseDS(ARSparseDataSource arSparseDS) {
		this.arSparseDS = arSparseDS;
	}

	public void updateList(ARNode tree) {
		
	// numChildren not getting set
		
//		if(tree.getNumChildren() == 0){
//			arInfoList.get(tree.getBlock().getResolution()).add(tree.getBlock()) ; 
//			return ; 
//		}
		
		boolean calledChild = false ; 
	
		
		for (int j = 0; j <= 7; j++) {
			ARNode child = (ARNode) tree.getChild(j);
			if(child != null){
				calledChild = true ; 
				traverseAndUpdateList(child);
			}
		}
		
		if(calledChild == false ){
			arInfoList.get(tree.getBlock().getResolution()).add(tree.getBlock()) ; 
		}
	}

	private void traverseAndUpdateList(ARNode node) {
		if (node == null) {
			return;
		}

		int nodeRes = node.getBlock().getResolution();

		if (node.isLeaf() && !isJustAddedNode()) {
			checkBounds(node.getBlock().getBounds() , nodeRes) ;
			arInfoList.get(nodeRes).add(node.getBlock());
			
		} else {
			setJustAddedNode(false) ;
			for (int j = 0; j <= 7; j++) {
				ARNode nodeChild = (ARNode) node.getChild(j);
				if (nodeChild != null) {
					int nodeChildRes = nodeChild.getBlock().getResolution();
					if (nodeRes == nodeChildRes) {
						traverseAndUpdateList(nodeChild);
					} else {
						
						if(!arInfoList.get(nodeRes).contains(node.getBlock())){
							checkBounds(node.getBlock().getBounds() , nodeRes) ; 
							arInfoList.get(nodeRes).add(node.getBlock());
						}
						checkBounds(nodeChild.getBlock().getBounds(), nodeChildRes) ; 
						arInfoList.get(nodeChildRes).add(nodeChild.getBlock());
						setJustAddedNode(true) ;
						traverseAndUpdateList(nodeChild);
					}
				}
			}
		}
	}

	private boolean checkBounds(ISBounds bounds , int res ){
		
		for(int i=0; i <=2 ; i++){
			int val = bounds.getUpper(i) - bounds.getLower(i) + 1 ; 
			if((val % 2)!= 0){
				System.out.println("Odd bnds :  for res " + res + "=> " + bounds) ; 
				return false ; 
			}
		}
		
		return true ;
		
	}
	
	public void printingARSparseDataBlocksCounts(){
		  // create a Data Block list for each resolution. 
		  // pad them with zeros.
		
		for(int m = 0 ; m<arInfoList.size() ; m++){
			ISBounds actBounds  = arInfoList.get(m).get(0).getBounds() ; 
			String actBoundsStr = actBounds.toString() ; 
//			
//			int diffCount = 0 ;
//			List<ARNodeBlock> arList = arInfoList.get(m) ; 
//			
//			for(int n = 0 ; n < arList.size() ; n++){
//				String currBoundsStr = (arList.get(n).getBounds()).toString() ; 
//				if(!actBoundsStr.equals(currBoundsStr)){
//					diffCount+=1 ; 
//				}
//			}
			
			System.out.println("Difference sizes at resolution : " + m + "  => " + arInfoList.get(m).size()) ; 
			
			
		}
	}
	
	public void paddingARSparseDataBlocks(){
		
	 arDataBlocksList = new ArrayList<List<DataBlock>>(noOfResolutions) ; 
	 for(int k=0; k < noOfResolutions ; k++){
		 
		 arDataBlocksList.add(k, new ArrayList<DataBlock>()) ; 
		 List<DataBlock> currDataBlocksList = arDataBlocksList.get(k) ; 
		 DataSource arDs = arSparseDS.getArSparseStorage().getDsArr(k) ; 
		 ISBounds actualBlkSize = new ISBounds(new IntegerIndexSpaceID(3, 3, 3 )) ; 
		 for(ARNodeBlock nodeBlock : arInfoList.get(k)){
//			 DataBlock arBlock = arDs.subblock(nodeBlock.getBounds()) ; 
//			 arDs.subblock(arBlock, nodeBlock.getBounds());
//			 currDataBlocksList.add(arBlock) ; 
			 
			 // creation of to be adjusted ARDataBlock
			 DataBlock adjustedARBlock = arDs.subblock(actualBlkSize) ; 
			 FloatValues initValues = new FloatValues(actualBlkSize.volume()) ; 
			 adjustedARBlock.setValues(0, initValues) ; 
			 DataBlock arBlock = arDs.subblock(nodeBlock.getBounds()) ; 
			 arDs.subblock(arBlock, nodeBlock.getBounds());
			 
			 adjustedARBlock.setValues(0, arBlock.getValues()) ; 
			 currDataBlocksList.add(adjustedARBlock) ; 
		 }
	}	
		
}
	
	public void writeARSparseData(){
		
		File[] myfiles  = new File[3];
		
	}
	
}
