#include "ControlModule.h"


extern "C" {
   extern void registerCallbacks( struct function_ptrs fun_ptrs);
   extern void registerGUICommandCallbacks( struct function_ptrs2 fun_ptrs2);
   extern void runSimCode();
   extern void Update_UI_Commands();
   extern void setMetaData(VisIt_SimulationMetaData *smd);
}


GraniteRemoteIF* ControlModule::remoteConn=NULL;
vector<RemoteDataSource*> ControlModule::dsVec;

ControlModule::ControlModule(GraniteRemoteIF* remote) {
 
   remoteConn = remote;

   struct function_ptrs fptrs;
   fptrs.GetMesh       = this->GetMesh;
   fptrs.GetMetaData   = this->GetMetaData; 
   fptrs.GetScalar     = this->GetScalar;                  
   fptrs.GetDomainList = this->GetDomainList; 
   registerCallbacks(fptrs);


   struct function_ptrs2 fptrs2;
   fptrs2.createDataSource = this->CreateDataSource;
   registerGUICommandCallbacks( fptrs2);
   runSimCode();     
}

ControlModule::~ControlModule(){

}


int ControlModule::CreateDataSource(char* name, char* path) {
   int val = -99;
   if (name != NULL && path != NULL) {

      RemoteDataSource* rds = new RemoteDataSource(remoteConn, name, path);
      if (rds != NULL) {
         fprintf(stderr, "  -- Created Datasource with id = %d\n", rds->getID());
         val = rds->getID();
         dsVec.push_back(rds);
      }
      else {
         fprintf(stderr, "  -- Failed to create Datasource\n");
      }
   }
   else { 
     fprintf(stderr, " Error: Cannot create data source with NULL name "
                     "and path!!\n");
   }
   return val;
}



VisIt_MeshData* ControlModule::GetMesh(int dsID, int domain, const char* name){
   for (vector<RemoteDataSource*>::iterator it = dsVec.begin(); it!=dsVec.end(); ++it) {
     if ((*it)->getID() == dsID) 
       return ((*it)->GetMesh(domain, name));
   }
   return NULL; 
}
    

VisIt_SimulationMetaData* ControlModule::GetMetaData(int dsID){
   for (vector<RemoteDataSource*>::iterator it = dsVec.begin(); it!=dsVec.end(); ++it) {
     if ((*it)->getID() == dsID) {
        fprintf(stderr, "  found remoteDatasource and calling getmetadata!\n");
        return ((*it)->GetMetaData());
     }
   }
   fprintf(stderr, " Failed to find DataSource with id = %d\n", dsID);
   return NULL; 
}
    

VisIt_ScalarData* ControlModule::GetScalar(int dsID, int domain, 
                                           const char *name)
{
   for (vector<RemoteDataSource*>::iterator it = dsVec.begin(); it!=dsVec.end(); ++it) {
     if ((*it)->getID() == dsID) 
       return ((*it)->GetScalar(domain, name));
   }
   return NULL; 
}
    
   
VisIt_DomainList* ControlModule::GetDomainList(int dsID){
   for (vector<RemoteDataSource*>::iterator it = dsVec.begin(); it!=dsVec.end(); ++it) {
     if ((*it)->getID() == dsID) 
       return ((*it)->GetDomainList());
   }
   return NULL;
}
