///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//          U N I V E R S I T Y    O F    N E W    H A M P S H I R E         //
//                                                                           //
//                       COMPUTER  SCIENCE  DEPARTMENT                       //
//                                                                           //
//                    Graphics And Visualization Research                    //
//                                                                           //
//          --------------------------------------------------------         //
//  File: RemoteDataSource.h                                                 //
//                                                                           //
//  The RemoteDatasource encapsulatesd everything required to render a       //
//  Granite DataSource using visIt                                           // 
//                                                                           //
//                                                                           //
//  Programmer:  Jon Frain                                                   //
//  Creation Date: 7/21/09                                                   //
//                                                                           //
//  History:                                                                 //
//       7-27-09 - Converted ControlModule class to RemoteDataSource         //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#ifndef _REMOTE_DATASOURCE_H_
#define _REMOTE_DATASOURCE_H_

#include "GraniteRemoteIF.h"
extern "C" {
   #include <SimCodeGlobals.h>
}
#include <VisItDataInterface_V1.h>
#include <VisItControlInterface_V1.h>
#include <SimHelperFunc.h>


class RemoteDataSource
{
public:  // constructors
    RemoteDataSource(GraniteRemoteIF *remote, char* name, char* path);
    virtual ~RemoteDataSource();

public:  // api
    VisIt_MeshData*           GetMesh(int domain, const char* name);
    VisIt_SimulationMetaData* GetMetaData(void);
    VisIt_ScalarData*         GetScalar(int domain, const char *name);  
    VisIt_DomainList*         GetDomainList();
    int                       getID() { return dsID;};

private:  // methods
    void InitializeScalarData();
    void InitializeMD(int MaxNumCustCMD);
    void InitializeMesh();
    bool parseGridFile(const char* filename);

private:  // members
    enum endianness {BIG=0, LITTLE};

    GraniteRemoteIF* remoteConn;

    int dsID;
    VisIt_SimulationMetaData *md; 

    /* size of each dimension */
    int* dim_sizes;
    int numDims;

    /* array of each coordinate in that axis*/
    float *p_xcoords;
    float *p_ycoords;
    float *p_zcoords;

    float **scalarData;
  
    int numAttributes;

    /* domains represent how many (parallel processing) sections this has been
       broken into */
    int numdomains;
    
    int par_rank;
    int par_size;

    int dsEndian;
    int osEndian;


private: // disable default and copy constructor and operator= 
         // - disabling to force passing in of a remote connection 
    RemoteDataSource();
    RemoteDataSource( RemoteDataSource& );
    RemoteDataSource& operator=( RemoteDataSource& ); 
};


#endif // _REMOTE_DATASOURCE_H_
