#include <jni.h>

#include <string.h>
#include <fstream>
#include <cstdlib>
#include <iostream>
#include <udt.h>
#include <netdb.h>

#include "edu_unh_sdb_remote_UDTSender.h"

using namespace std;

extern "C" {
void* monitor(void*);
bool done = false;

JNIEXPORT jint JNICALL Java_edu_unh_sdb_remote_UDTSender_nativeSendData
  (JNIEnv *env, jclass cls, jstring targetIP, jint port, jbyteArray buf, jint size)
{
	int recvStatus;
   char clienthost[NI_MAXHOST];
   char clientservice[NI_MAXSERV];
	int time_out = 10*1000;
    const char *clientIP;
    clientIP = (env)->GetStringUTFChars(targetIP, NULL);
    
    if (clientIP == NULL) {
        return 1; /* OutOfMemoryError already thrown */
    }
   
	jboolean iscopy = JNI_FALSE;
    //Try to get a direct pointer to the java direct byte buffer
    //jbyte *cppPtr = env->GetByteArrayElements(buf, 0);
    jbyte *cppPtr = env->GetByteArrayElements(buf, &iscopy); //not (env, buf)
    if(cppPtr == NULL){
      cout << "GetByteArrayElements returned NULL" << endl;
    	return 1; /* Exception occurred */
    }
		//cout << "Size:  " << size << "\n";
		//return 1;
    //If successfully got the pointer, send the data after notifying the peer 
    //of the data size
    //char data[size];
    //memcpy(data, cppPtr, size);
    char* data = (char*)(&(*cppPtr));
    
    UDTSOCKET serv; // = UDT::socket(AF_INET, SOCK_STREAM, 0);
    if(UDT::INVALID_SOCK == (serv = UDT::socket(AF_INET, SOCK_STREAM, 0))){
       	cout << "INVALID SERVER SOCKET -- socket error: " <<
			UDT::getlasterror().getErrorMessage() << endl;
		   env->ReleaseStringUTFChars(targetIP, clientIP);
    		env->ReleaseByteArrayElements(buf, cppPtr, JNI_ABORT);
			UDT::close(serv);
			sleep(50);
       	return 1;
    }
    //printf("--------------------------------------------\n"); 
    //printf("In native method, server socket created ... \n"); 
    //cout << "serv: " << serv <<endl;

//
//#ifdef WIN32
//   int mss = 1052;
//   UDT::setsockopt(serv, 0, UDT_MSS, &mss, sizeof(int));
//#endif
//
   sockaddr_in my_addr;
   my_addr.sin_family = AF_INET;
   my_addr.sin_port = htons(port);
   my_addr.sin_addr.s_addr = INADDR_ANY;
   memset(&(my_addr.sin_zero), '\0', 8);

   //printf("In native method, trying to bind server socket to port %d ... \n", port); 
   if (UDT::ERROR == UDT::bind(serv, (sockaddr*)&my_addr, sizeof(my_addr)))
   {
      cout << "bind: " << UDT::getlasterror().getErrorMessage() << endl;
		env->ReleaseStringUTFChars(targetIP, clientIP);
		env->ReleaseByteArrayElements(buf, cppPtr, JNI_ABORT);
		UDT::close(serv);
		sleep(50);
      return 1;
   }
   //printf("In native method, server socket is bound to port %d and ready to transfer data ... \n", port); 
	
   //cout << "server is ready at port: " << port << endl;
   UDT::listen(serv, 1);
   //printf("In native method, server socket is listening ... \n"); 
   //cout << "server is listening ... " << endl;

   sockaddr_in their_addr;
   //cout << "sockaddr_in their_addr is created, &their_addr: " << &their_addr << endl;
   
   int namelen = sizeof(their_addr);
	 //cout << "namelen: " << namelen << endl;
	 //cout << "&namelen: " << &namelen << endl;
	 
   UDTSOCKET fhandle;
	 //cout << "fhandle: " << fhandle << endl;
	 //cout << "&fhandle: " << &fhandle << endl;
	 
	 while(true){
	 	 
		 //cout << "About to accept a new connection ..." << endl;
		 //cout << &their_addr << endl;
		 //cout << ((sockaddr*)&their_addr) << endl;
		 //cout << &namelen << endl;
		  
      //printf("\nIn native method, server is about to accept a new connection ... "); 
	   if (UDT::INVALID_SOCK == (fhandle = UDT::accept(serv, (sockaddr*)&their_addr, &namelen)))
	   {
	      cout << "accept: " << UDT::getlasterror().getErrorMessage() << endl;
			env->ReleaseStringUTFChars(targetIP, clientIP);
			env->ReleaseByteArrayElements(buf, cppPtr, JNI_ABORT);
			UDT::close(serv);
			UDT::close(fhandle);
			sleep(50);
	      return 1;
	   }
		 //cout << "About to call getnameinfo() ..." << endl;
		 //cout << &their_addr << endl;
		 //cout << ((sockaddr*)&their_addr) << endl;
		 //cout << "namelen: " << namelen << endl;
		 //cout << "&namelen: " << &namelen << endl;
		 //cout << "clienthost:" << clienthost << endl;		 
		 //cout << "&clienthost:" << &clienthost << endl;
		 //cout << "clientservice:" << clientservice << endl;		 
		 //cout << "&clientservice:" << &clientservice << endl;

	   getnameinfo((sockaddr *)&their_addr, namelen, clienthost, sizeof(clienthost), clientservice, sizeof(clientservice), NI_NUMERICHOST|NI_NUMERICSERV);
		 
   		 //printf("In native method, new connection: %s:%d ... \n", clienthost, clientservice); 
		 //cout << "new connection: " << clienthost << ":" << clientservice << endl;
		 
		 //printf("Clienthost = %s, clientIP=%s\n", clienthost, clientIP);
		 
		 
		 if(strcmp(clienthost,clientIP)==0)
		 		break;
	 }
	 //printf("In native method, server successfully accepted a new connection.\n"); 
	 
   UDT::close(serv);

   //done = false;
//    pthread_t monitorThread = new pthread_t;
//    pthread_create(monitorThread, NULL, monitor, &fhandle);

//    if (UDT::ERROR == UDT::send(fhandle, (char*)&size, sizeof(jint), 0))
//    {
//       cout << "send data size: " << UDT::getlasterror().getErrorMessage() << endl;
//       return 0;
//    }


   //UDT::TRACEINFO trace;
   //UDT::perfmon(fhandle, &trace);
	 int sentDataSize = 0;
	 int handle;
   // send the file
   //printf("In native method, server sending data of size %d...", size); 
   fflush(stdout);
   UDT::setsockopt(fhandle, 0, UDT_SNDTIMEO, &time_out, sizeof(int));
   
//    cout << "Setting UDT_LINGER for sending socket." << endl;
//    UDT::setsockopt(fhandle, 0, UDT_LINGER, &time_out, sizeof(int)); // PJR
   
   if (UDT::ERROR == (sentDataSize=UDT::send(fhandle, data, size, 0, &handle)))
   {
	   jclass newExcCls;
      cout << "ERROR: In native method, send data: " << UDT::getlasterror().getErrorMessage() << endl;
      fflush(stdout);
		/////////////////////////////////////////////////// 
		//////// Newly added to throw exception ///////////

	   newExcCls = (env)->FindClass("java/io/IOException");
	   if (newExcCls == NULL) {
			/* Unable to find the exception class, give up. */
			UDT::close(fhandle);
			env->ReleaseStringUTFChars(targetIP, clientIP);
			 //env->ReleaseByteArrayElements(buf, cppPtr, 0);
			env->ReleaseByteArrayElements(buf, cppPtr, JNI_ABORT);
			sleep(50);
			return 1;
	   }
		UDT::close(fhandle);
		 
		env->ReleaseStringUTFChars(targetIP, clientIP);
		 //env->ReleaseByteArrayElements(buf, cppPtr, 0);
		env->ReleaseByteArrayElements(buf, cppPtr, JNI_ABORT);
	   env->ThrowNew(newExcCls, "Send Timeout Exception from Native Code!");
		sleep(50);

		///////////////////////////////////////////////////

      return 1;
   }
   
//   pthread_cancel(monitorThread);
   
   //printf("In native method, %d bytes sent to client ... \n", sentDataSize); 
   //cout << sentDataSize <<" bytes sent to client" << endl;
   //ReleaseByteArrayElements(buf, data, JNI_ABORT);
   //UDT::perfmon(fhandle, &trace);
   //cout << "speed = " << trace.mbpsSendRate << " mbps" << endl;

   //cout << "Waiting for Client ACK" << endl;

	 //Wait until the client send 0 to denote success of receiving data
	 int recvSize;
	 time_out=30 * 1000;  // PJR
	 UDT::setsockopt(fhandle, 0, UDT_RCVTIMEO, &time_out, sizeof(int));
    UDT::setsockopt(fhandle, 0, UDT_LINGER, &time_out, sizeof(int)); // PJR
    if (UDT::ERROR == (recvSize=UDT::recv(fhandle, (char*)&recvStatus, sizeof(int), 0, &handle)))
    {
       cout << "recv: " << UDT::getlasterror().getErrorMessage() << endl;
		 UDT::close(fhandle);
		 
		 env->ReleaseStringUTFChars(targetIP, clientIP);
		 //env->ReleaseByteArrayElements(buf, cppPtr, 0);
		 env->ReleaseByteArrayElements(buf, cppPtr, JNI_ABORT);
	 	 fflush(stdout);
       return 1;
    }
    
    //cout << "recvStatus == " << (int) recvStatus << endl;

    if (recvStatus != 0) {
   
      cout << "Non-zero recvStatus received from client." << endl;
    }
   
    UDT::close(fhandle);
    
    env->ReleaseStringUTFChars(targetIP, clientIP);
    //env->ReleaseByteArrayElements(buf, cppPtr, 0);
    env->ReleaseByteArrayElements(buf, cppPtr, JNI_ABORT);
    //done=true;
    //printf("In native method, done and returning to Java!!!\n"); 
    //printf("--------------------------------------------\n"); 
    
	 //printf("done.\n");
	 fflush(stdout);
	 
    return recvStatus;
}

void* monitor(void* s)
{

//    int oldcanceltype;
//    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,&oldcanceltype);
   UDTSOCKET u = *(UDTSOCKET*)s;

   UDT::TRACEINFO perf;

   cout << "SendRate(Mb/s) RTT(ms) FlowWindow PktSndPeriod(us) RecvACK RecvNAK" << endl;

   //while (true)
   while(!done)
   {
      sleep(1);
      if (UDT::ERROR == UDT::perfmon(u, &perf))
      {
         cout << "perfmon: " << UDT::getlasterror().getErrorMessage() << endl;
         break;
      }

      cout << perf.mbpsSendRate << "\t" 
           << perf.msRTT << "\t" 
           << perf.pktFlowWindow << "\t" 
           << perf.usPktSndPeriod << "\t" 
           << perf.pktRecvACK << "\t" 
           << perf.pktRecvNAK << endl;
   }

   return NULL;
}

};
