/* 
      BoundsError.java
      Copyright 2000 University of New Hampshire
      Durham, NH 03824
*/

//---------------------- Edit history as of 11/24/05 ------------------------------
// 11/24/05 rdb: revised (and simplified) to use new SDBError.logError 
//---------------------------------------------------------------------------------

package edu.unh.sdb.common;
//import java.io.*;

/** This error is used whenever there is some kind of bounds violation.
 *     For example, if a DataSource is given an IndexSpaceID that is outside of its
 *     outer bounds, this error could be generated. Similarly, a call to the 
 *     setCoord() method of ISIterator might generate this exception. 
 *     
 *     In cases where it would be appropriate to create either an 
 *     IllegalArgumentError or BoundsError, we should
 *     use BoundsError, since it is more specific.
 *     
 *     Creating an error will cause the message argument to be logged, and
 *     an exception to be raised, if appropriate. This behavior is governed
 *     by the error level given to the constructor, and by the error reporting
 *     level maintained in the SDBError class. 
 *     
 *     Invoking the constructor is sufficient to log the error. There is no
 *     need to save a reference to the error object.
    
 */

public class BoundsError extends SDBError{
    
    /** Create an error with the specified error level and message.
        The error level can be either SDBError.Fatal or 
        SDBError.Recoverable.
    */
    
    public BoundsError( int errorLevel, String message )
    {
        super( errorLevel, new BoundsException( message ));
    }  
} 