package edu.unh.sdb.common;

import edu.unh.sdb.datasource.SliceUtil;
import javax.vecmath.Matrix4d;
import javax.vecmath.Vector3d;

/**
 * {@link EulerAngles} class for which heading rotates around Y axis, pitch
 * rotates around X axis and bank rotates around Z axes.
 * @author dlipsa
 */
public class EulerAnglesYXZ extends EulerAngles
{      
   public EulerAnglesYXZ (double heading, double pitch, double bank)
   {
      super (heading, pitch, bank);
   }
   
   public EulerAnglesYXZ (Vector3d v)
   {
      super (v);
   }
   
   public void rotateHeading (Matrix4d m)
   {
      m.rotY (((double)getHeading() * Math.PI) / 180);
   }
   public void rotatePitch (Matrix4d m)
   {
      m.rotX (((double)getPitch()) * Math.PI / 180);
   }
   
   public void rotateBank (Matrix4d m)
   {
      m.rotZ (((double)getBank()) * Math.PI / 180);
   }
   
   public Vector3d getInitialOrientation ()
   {
      return new Vector3d (0, 0, 1);
   }
   
   /**
    * Sets this EulerAngles object from the matrix m.
    * From "3d Math Primer fro Graphics and Game Development" by Fletcher Dunn
    * and Ian Parberry adjusted for right hand coordinates
    * @param m
    */
   public void setFromMatrix (Matrix4d m)
   {      
      double sp = -m.m21;
      if (sp <= -1 + SliceUtil.ZERO)
      {
         setPitch (-90);
         sp = -sp;
      }
      else if (sp >= 1 - SliceUtil.ZERO)
         setPitch (90);
      else
         setPitch (Math.toDegrees (Math.asin (sp)));
      
      if (sp > 1 - SliceUtil.ZERO)
      {
         setBank (0);
         setHeading (Math.toDegrees (Math.atan2 (m.m02, m.m00)));
      }
      else
      {
         setBank (Math.toDegrees (Math.atan2 (-m.m01, m.m11)));
         setHeading (Math.toDegrees (Math.atan2 (-m.m20, m.m22)));
      }
   }
}
