/*
 * MixedDatum.java
 * 
 * Implements a Datum of Mixed types stored as a byte array
 * 
 * ------------------------- Edit history  -----------
 * 10/21/05 rdb: Removed unused variables
 * 01/14/05 rdb: Added RecordDescriptor
 *               Added getStorageType() returns StorageType.Record
 *               Added getValues()
 *               <still needs a way to return the actual byte array>
 *
 * 01/11/05 rdb: -Redefined semantics of MixedDatum( byte[], int start, int numattr );
 *                -renamed assign methods to setMixed, but changed semantics of
 *                    setMixed( byte[], int start, int numattr ) 
 *
 * 01/06/05  rdb Serious attempt at correct implementation.
 *               Realized that must really have and save a RecordDescriptor.
 * 12/12/04  rdb created from ShortDatum.java, but it's not at all correct.
 */

/**
 * @author rdb
 *
 */

package edu.unh.sdb.common;

//import java.util.*;
import java.nio.*;


/**
 * Represent multiple data values for a single point. This implementation stores
 * values as an array of bytes.
 */

public class MixedDatum extends Datum
{
   private RecordDescriptor recDesc = null;
   private byte[]           values  = null;
   private ByteBuffer       buffer  = null;
   
   /** Creates a Datum based on a RecordDescriptor. This is the standard way of
    *  creating a MixedDatum.
    * 
    * @param rd   A RecordDescriptor describing all the fields of the Datum
    */
   public MixedDatum( RecordDescriptor rd )
   {
      this.recDesc = rd;
      this.values  = new byte[ rd.byteSize() ];
      this.buffer  = ByteBuffer.wrap( this.values );
   }
   
   /** Creates a null datum with the specified number of values. 
    *  The fields will all be set to float (for now).
    * @deprecated
    */
   public MixedDatum( int numvalues )
   {
      StringBuffer types = new StringBuffer();
      for ( int i = 0; i < numvalues; i++ )
         types.append( "f" );
      this.recDesc = new RecordDescriptor( new String( types ));
      this.values = new byte[ recDesc.byteSize() ];
      this.buffer = ByteBuffer.wrap( this.values );
   }

   /** Creates a datum with attributes containing the argument values. 
    * @deprecated
    */
   public MixedDatum( float vals[] )
   {
      StringBuffer types = new StringBuffer();
      for ( int i = 0; i < vals.length; i++ )
         types.append( "f" );
      this.recDesc = new RecordDescriptor( new String( types ));
      this.values = new byte[ recDesc.byteSize() ];
      this.buffer = ByteBuffer.wrap( this.values );

      // could do this more efficiently by converting to byte array and
      // assigning to the this.values, but who cares???
      for ( int i = 0; i < vals.length; i++ )
         setFloat( i, vals[i] );
   }

   /** Creates a datum by copying a subset of values from an array. 
    * The 3rd argument specifies where to start copying;
    * we copy enough bytes to match the data size from the RecordDescriptor.
    *
    */
   public MixedDatum( RecordDescriptor rd, byte values[], int startIndex )
   {
      this.recDesc = rd; 
      this.values = new byte[ rd.byteSize() ];
      this.buffer = ByteBuffer.wrap( this.values );
      
      //rdb int startIndex = startIndex * rd.byteSize();

      for ( int i = 0; i < rd.byteSize(); i++ )
      {
         this.values[i] = values[startIndex + i];
      }
   }

   /** Creates a datum with attributes containing the argument values. 
    *  @deprecated
    */
   public MixedDatum( int vals[] )
   {
      this.recDesc = new RecordDescriptor( 'i', vals.length );
      this.values = new byte[ this.recDesc.byteSize() ];
      this.buffer = ByteBuffer.wrap( this.values );

      // could do this more efficiently by converting to byte array and
      // assigning to the this.values, but who cares???
      for ( int i = 0; i < vals.length; i++ )
         setInt( i, vals[i] );
   }

   /** Copy Constructor creates a copy of the argument datum. */
   public MixedDatum( MixedDatum d )
   {
      this.recDesc = d.recDesc.copy();
      this.values  = d.values.clone();
      this.buffer = ByteBuffer.wrap( this.values );
   }

   /** Copy Constructor creates a copy of the argument datum. */
   public MixedDatum( Datum d )
   {
      // Pretty ugly code, but who would do this??? and why?
      // It doesn't seem worth the effort to try to be efficient.
      if ( d instanceof MixedDatum )  // shouldn't even get here
      {
         this.recDesc = ((MixedDatum)d).recDesc.copy();
         this.values  = ((MixedDatum)d).values.clone();
         this.buffer = ByteBuffer.wrap( this.values );
      }
      else if ( d instanceof DoubleDatum )
      {
         this.recDesc = new RecordDescriptor( 'd', d.size() );
         for ( int i = 0; i < d.size(); i++ )
            setDouble( i, d.getDouble( i ));
      }
      else if ( d instanceof FloatDatum )
      {
         this.recDesc = new RecordDescriptor( 'f', d.size() );
         for ( int i = 0; i < d.size(); i++ )
            setFloat( i, d.getFloat( i ));
      }
      else if ( d instanceof IntDatum )
      {
         this.recDesc = new RecordDescriptor( 'i', d.size() );
         for ( int i = 0; i < d.size(); i++ )
            setInt( i, d.getInt( i ));
      }
      else if ( d instanceof ShortDatum )
      {
         this.recDesc = new RecordDescriptor( 's', d.size() );
         for ( int i = 0; i < d.size(); i++ )
            setShort( i, d.getShort( i ));
      }
      else if ( d instanceof ByteDatum )
      {
         this.recDesc = new RecordDescriptor( 'b', d.size() );
         for ( int i = 0; i < d.size(); i++ )
            setByte( i, d.getByte( i ));
      }
   }

   /**
    * This constructor creates a single MixedDatum from an array of datums. The
    * Datums in the array are copied in the order they are given.
    */
   public MixedDatum( MixedDatum[] datums )
   {
   }

   /**
    * This constructor creates a single MixedDatum from an array of Datums. The
    * Datums in the array are copied in the order they are given.
    */
   public MixedDatum( Datum[] datums )
   {
      int totalSize = 0;   // total number bytes needed to store
      int numAttr   = 0;   // total number attributes
      MixedDatum    mDatums[] = new MixedDatum[ datums.length ];

      for ( int i = 0; i < datums.length; i++ )
      {
         if ( datums[i] instanceof MixedDatum )
            mDatums[i] = (MixedDatum) datums[i];
         else
            mDatums[i] = new MixedDatum( datums[i] );
         numAttr   += mDatums[i].size();
         totalSize += mDatums[i].getRecordDescriptor().byteSize();
      }
      
      AttributeDescriptor ad[] = new AttributeDescriptor[ numAttr ];
      int attr = 0;
      for ( int i = 0; i < mDatums.length; i++ )
      {
         RecordDescriptor rd = mDatums[i].getRecordDescriptor();
         for ( int j = 0; j < mDatums[i].size() ; j++ )
            ad[ attr++ ] = rd.getAttribute( j );
      }
      this.recDesc = new RecordDescriptor( ad );

      // create the values array
      this.values = new byte[ totalSize ];

      // and copy the data to it from the input Datums
      int valIndex = 0;    // index into value[] byte array
      for ( int i = 0; i < mDatums.length; i++ )
      {
          MixedDatum md = mDatums[i];
          for ( int j = 0; j < md.byteSize(); j++ )
              this.values[ valIndex++ ] = md.values[j];
      }
      this.buffer = ByteBuffer.wrap( this.values );
   }

   /** Return a copy of this datum. */
   public Object clone()
   {
      return new MixedDatum( this );
   }

   /** Copy the values of this MixedDatum object into the argument datum. */
   public void copyTo( MixedDatum d )
   {
      try
      {
         System.arraycopy( this.values, 0, d.values, 0, this.values.length );
      } catch ( ArrayIndexOutOfBoundsException e )
      {
         new SDBError( SDBError.Recoverable, "Datum size mismatch." );
      }

      catch ( NullPointerException e )
      {
         if ( d == null )
         {
            new IllegalArgumentError( SDBError.Recoverable,
                  "Argument Datum was null or had no data." );
         }
         else
         {
            new InternalFaultError( SDBError.Recoverable, "This Datum has no data." );
         }
      }
   }

   /** Copy the values of this MixedDatum object into the argument datum. */
   public void copyTo( Datum d )
   {
      try
      {
         for ( int i = 0; i < d.size(); i++ )
         {
            switch( this.recDesc.getAttribute( i ).getStorageType() )
            {
               case StorageType.Double:
                  d.setDouble( i, this.getDouble( i ));
                  break;
               case StorageType.Float:
                  d.setFloat( i, this.getFloat( i ));
                  break;
               case StorageType.Int:
                  d.setInt( i, this.getInt( i ));
                  break;
               case StorageType.Short:
                  d.setShort( i, this.getShort( i ));
                  break;
               case StorageType.Byte:
                  d.setByte( i, this.getByte( i ));
                  break;
               default:      //???? for now set to float
                  new SDBError( SDBError.Warning, "Unsupported type in MixedDatum copy" );
                  d.setFloat( i, this.getFloat( i ));
                  break;
            }
         }
      } catch ( ArrayIndexOutOfBoundsException e )
      {
         new SDBError( SDBError.Recoverable, "Datum size mismatch." );
      }
      catch ( NullPointerException e )
      {
         if ( d == null )
         {
            new IllegalArgumentError( SDBError.Recoverable, "Argument Datum was null." );
         }
         else
         {
            new InternalFaultError( SDBError.Recoverable, "This Datum has no data." );
         }
      }
   }

   /** Copy the values in the argument to this datum. */
   public void assign( byte[] data )
   {
      assign( data, 0 );
   }

   /**
    * Assigns values to an existing datum by copying values from a portion of an array.
    * The second argument indicates where to start the copy. We try to copy enough 
    * data to fill the Datum.
    * If the input array or this object contain too few attributes, an error
    * is generated.
    */
   public void assign( byte data[], int startIndex )
   {
      try
      {
         for ( int i = 0; i < this.values.length; i++ )
         {
            this.values[i] = data[startIndex + i];
         }
      } catch ( ArrayIndexOutOfBoundsException e )
        {
           new IllegalArgumentError( SDBError.Warning,
               "Tried to assign to a MixedDatum with too little input." );
        }
   }

   /**
    * Assigns values to an existing datum by copying values from a portion of an array.
    * The second argument indicates where to start the copy, the 3rd indicates
    * the number of attributes to copy (as opposed to the number of bytes!)
    * If the input array or this object contain too few attributes, an error
    * is generated.
    */
   public void assign( byte values[], int startIndex, int numAttributes )
   {
      if ( numAttributes > this.size() )
      {
         new IllegalArgumentError( SDBError.Warning, 
               "Too many attributes in assign method parameter.");
         numAttributes = this.size();
      }
      int copySize = 0;
      for ( int i=0; i < numAttributes; i++ )
         copySize += this.recDesc.getAttribute( i ).byteSize();
               
      try
      {
         for ( int i = 0; i < copySize; i++ )
         {
            this.values[i] = values[startIndex + i];
         }
      } catch ( ArrayIndexOutOfBoundsException e )
      {
         new IllegalArgumentError( SDBError.Recoverable,
                   "Tried to assign to a MixedDatum with too little input." );
      }
   }

   /** Determine if the argument is a MixedDatum with the same set of data values. */
   public boolean equals( Object o )
   {
       try 
       {
           MixedDatum d = (MixedDatum) o;
           if ( d.values.length != this.values.length )
              return false;
           for ( int i=0; i < this.values.length; i++ )
           {
   			if ( this.values[i] != d.values[i] )
                   return false;
   		}
       } catch ( Exception e ) // any exception means not equal
         {
             return false;
         }
       return true;
   }

   /**
    * Set the attribute indicated by the first argument to the value of the
    * second argument.
    */
   public void setDouble( int i, double v )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               this.buffer.putDouble( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Float:
               this.buffer.putFloat( this.recDesc.byteOffset( i ), (float) v );
               break;
            case StorageType.Int:
               this.buffer.putInt( this.recDesc.byteOffset( i ), (int) v );
               break;
            case StorageType.Short:
               this.buffer.putShort( this.recDesc.byteOffset( i ), (short) v );
               break;
            case StorageType.Byte:
               this.buffer.put( this.recDesc.byteOffset( i ), (byte) v );
               break;
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
        {
            new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
        } 
        catch ( ArithmeticException aex )
        {
           new SDBError( SDBError.Recoverable, "Error casting double to byte" );
        }
   }

   /**
    * Set the attribute indicated by the first argument to the value of the
    * second argument.
    */
   public void setFloat( int i, float v ) 
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               this.buffer.putDouble( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Float:
               this.buffer.putFloat( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Int:
               this.buffer.putInt( this.recDesc.byteOffset( i ), (int) v );
               break;
            case StorageType.Short:
               this.buffer.putShort( this.recDesc.byteOffset( i ), (short) v );
               break;
            case StorageType.Byte:
               this.buffer.put( this.recDesc.byteOffset( i ), (byte) v );
               break;
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      } catch ( ArithmeticException aex )
      {
         new SDBError( SDBError.Recoverable, "Error casting float to byte" );
      }
   }

   /**
    * Set the attribute indicated by the first argument to the value of the
    * second argument.
    */
   public void setInt( int i, int v )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               this.buffer.putDouble( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Float:
               this.buffer.putFloat( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Int:
               this.buffer.putInt( this.recDesc.byteOffset( i ),  v );
               break;
            case StorageType.Short:
               this.buffer.putShort( this.recDesc.byteOffset( i ), (short) v );
               break;
            case StorageType.Byte:
               this.buffer.put( this.recDesc.byteOffset( i ), (byte) v );
               break;
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      } catch ( ArithmeticException aex )
      {
         new SDBError( SDBError.Recoverable, "Error casting int to byte" );
      }
   }

   /**
    * Set the attribute indicated by the first argument to the value of the
    * second argument.
    */
   public void setShort( int i, short v )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               this.buffer.putDouble( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Float:
               this.buffer.putFloat( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Int:
               this.buffer.putInt( this.recDesc.byteOffset( i ),  v );
               break;
            case StorageType.Short:
               this.buffer.putShort( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Byte:
               this.buffer.put( this.recDesc.byteOffset( i ), (byte) v );
               break;
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      }
   }

   /**
    * Set the attribute indicated by the first argument to the value of the
    * second argument.
    */
   public void setByte( int i, byte v )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               this.buffer.putDouble( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Float:
               this.buffer.putFloat( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Int:
               this.buffer.putInt( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Short:
               this.buffer.putShort( this.recDesc.byteOffset( i ), v );
               break;
            case StorageType.Byte:
               this.buffer.put( this.recDesc.byteOffset( i ), v );
               break;
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
         if ( this.recDesc.getStorageType(i) == StorageType.Byte )
            this.buffer.put( this.recDesc.byteOffset( i ), v );
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      }
   }

   /** Return the field specified by the argument as a double. */
   public double getDouble( int i )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               return this.buffer.getDouble( this.recDesc.byteOffset( i ) );
            case StorageType.Float:
               return this.buffer.getFloat( this.recDesc.byteOffset( i ) );
            case StorageType.Int:
               return this.buffer.getInt( this.recDesc.byteOffset( i ) );
            case StorageType.Short:
               return this.buffer.getShort( this.recDesc.byteOffset( i ) );
            case StorageType.Byte:
               return this.buffer.get( this.recDesc.byteOffset( i ) );
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      }
      return Double.NaN;
   }

   /** Return the field specified by the argument as a float. */
   public float getFloat( int i )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               return (float) this.buffer.getDouble( this.recDesc.byteOffset( i ) );
            case StorageType.Float:
               return this.buffer.getFloat( this.recDesc.byteOffset( i ) );
            case StorageType.Int:
               return this.buffer.getInt( this.recDesc.byteOffset( i ) );
            case StorageType.Short:
               return this.buffer.getShort( this.recDesc.byteOffset( i ) );
            case StorageType.Byte:
               return this.buffer.get( this.recDesc.byteOffset( i ) );
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      }
      return Float.NaN;
   }

   /** Return the field specified by the argument as a int. */
   public int getInt( int i )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               return (int) this.buffer.getDouble( this.recDesc.byteOffset( i ) );
            case StorageType.Float:
               return (int) this.buffer.getFloat( this.recDesc.byteOffset( i ) );
            case StorageType.Int:
               return this.buffer.getInt( this.recDesc.byteOffset( i ) );
            case StorageType.Short:
               return this.buffer.getShort( this.recDesc.byteOffset( i ) );
            case StorageType.Byte:
               return this.buffer.get( this.recDesc.byteOffset( i ) );
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      }
      return Integer.MAX_VALUE;
   }

   /** Return the field specified by the argument as a short. */
   public short getShort( int i )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               return (short) this.buffer.getDouble( this.recDesc.byteOffset( i ) );
            case StorageType.Float:
               return (short) this.buffer.getFloat( this.recDesc.byteOffset( i ) );
            case StorageType.Int:
               return (short) this.buffer.getInt( this.recDesc.byteOffset( i ) );
            case StorageType.Short:
               return this.buffer.getShort( this.recDesc.byteOffset( i ) );
            case StorageType.Byte:
               return this.buffer.get( this.recDesc.byteOffset( i ) );
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      }
      return Byte.MAX_VALUE;
   }

   /** Return the field specified by the argument as a byte. */
   public byte getByte( int i )
   {
      try
      {
         switch ( this.recDesc.getStorageType(i) )
         {
            case StorageType.Double:
               return (byte) this.buffer.getDouble( this.recDesc.byteOffset( i ) );
            case StorageType.Float:
               return (byte) this.buffer.getFloat( this.recDesc.byteOffset( i ) );
            case StorageType.Int:
               return (byte) this.buffer.getInt( this.recDesc.byteOffset( i ) );
            case StorageType.Short:
               return (byte) this.buffer.getShort( this.recDesc.byteOffset( i ) );
            case StorageType.Byte:
               return this.buffer.get( this.recDesc.byteOffset( i ) );
            default:
               new SDBError( SDBError.Warning, "Unimplemented StorageType: " + 
                     this.recDesc.getStorageType(i) );
         }
      } catch ( ArrayIndexOutOfBoundsException ex )
      {
         new SDBError( SDBError.Recoverable, "Invalid attr index: " + i );
      }
      return Byte.MAX_VALUE;
   }

   public void setBytes( byte[] newVals )
   {
      int count = this.size();
      if ( newVals.length < count )
      {
         count = newVals.length;
         new SDBError( SDBError.Recoverable,
               "Datum.setBytes: extra array elements ignored." );
      }
      for ( int i = 0; i < count; i++ )
         this.values[i] = newVals[i];
   }

   /** Return all attributes of the Datum as a Values object  */
   public Values getValues()
   {  
      return new MixedValues( this.recDesc, values );
   }

   /** Return a new datum with the attributes indicated by the RecordSpec. The values of these
    *  attributes are taken from this MixedDatum object. 
    */
   public Datum subset( RecordSpec spec )
   {
      int specsize = spec.size();
      AttributeDescriptor ad[] = new AttributeDescriptor[ specsize ];
      
      for ( int i = 0; i < specsize; i++ )
         ad[ i ] = this.recDesc.getAttribute( spec.getFieldNumber( i ));

      RecordDescriptor newRD = new RecordDescriptor( ad );
      MixedDatum newdatum = new MixedDatum( newRD );
      int thisOff[] = this.recDesc.byteOffsets();
      int newOff[]  = newRD.byteOffsets();

      for ( int i = 0; i < specsize; i++ )
      {
         int srcStart = thisOff[ spec.getFieldNumber( i ) ];
         int dstStart = newOff[ i ];
         for ( int j = 0; j < newRD.attributeByteSizes()[ j ]; j++ )
            newdatum.values[ dstStart + j] = this.values[ srcStart + j];
      }
      return newdatum;
   }

   /** Copy the attributes specified by the RecordSpec into the argument datum. */
   public void subset( MixedDatum d, RecordSpec spec )
   {
      int i;
      int specsize = spec.size();

      try
      {
         for ( i = 0; i < specsize; i++ )
         {
            int srcIndex = spec.getFieldNumber( i );
            int type = d.recDesc.getStorageType( i ); 
            switch ( type )
            {
               case StorageType.Double:
                  d.setDouble( i, this.getDouble( srcIndex ));
                  break;
               case StorageType.Float:
                  d.setFloat( i, this.getFloat( srcIndex ));
                  break;
               case StorageType.Int:
                  d.setInt( i, this.getInt( srcIndex ));
                  break;
               case StorageType.Short:
                  d.setShort( i, this.getShort( srcIndex ));
                  break;
               case StorageType.Byte:
                  d.setByte( i, this.getByte( srcIndex ));
                  break;               
               default:
                  new SDBError( SDBError.Warning, "Unsupported type: " + type );
            }
         }
      } catch ( ArrayIndexOutOfBoundsException e )
      {
         new IllegalArgumentError( SDBError.Recoverable,
               "Argument datum was not large enough for RecordSpec." );
      }
   }

   /** Copy the attributes specified by the RecordSpec into the argument datum. */
   public void subset( Datum d, RecordSpec spec )
   {
      int i;
      int specsize = spec.size();

      try
      {
         for ( i = 0; i < specsize; i++ )
         {
            int srcIndex = spec.getFieldNumber( i );
            int type = this.recDesc.getStorageType( srcIndex ); 
            switch ( type )
            {
               case StorageType.Double:
                  d.setDouble( i, this.getDouble( srcIndex ));
                  break;
               case StorageType.Float:
                  d.setFloat( i, this.getFloat( srcIndex ));
                  break;
               case StorageType.Int:
                  d.setInt( i, this.getInt( srcIndex ));
                  break;
               case StorageType.Short:
                  d.setShort( i, this.getShort( srcIndex ));
                  break;
               case StorageType.Byte:
                  d.setByte( i, this.getByte( srcIndex ));
                  break;  
               default:
                  new SDBError( SDBError.Warning, "Unsupported type: " + type );
            }
         }
      } catch ( ArrayIndexOutOfBoundsException e )
      {
         new IllegalArgumentError( SDBError.Recoverable,
               "RecordSpec was too large." );
      }
   }

   /** Return a string representation of the datum. */
   public String toString()
   {

      int i, size = this.values.length;
      String result = new String( "{" );

      for ( i = 0; i < size - 1; i++ )
      {
         result += values[i] + ", ";
      }

      result += values[i] + "}";

      return result;
   }

   /** Return the number of attributes in the datum. */
   public int size()
   {
      return this.recDesc.size();
   }

   /** Return the size in bytes needed to store the data for the attributes. */
   public int byteSize()
   {
      return this.values.length;
   }

   /**
    * Return the StorageType for this Datum
    */
   public int getStorageType()
   {
      return StorageType.Record;
   }

   /** Return the RecordDescriptor associated with MixedDatum. */
   public RecordDescriptor getRecordDescriptor()
   {
      return this.recDesc;
   }
}