package edu.unh.sdb.common;

//--------------------------RangedTypeBuilder class ----------------------
//  Created by Denise Mitchell Summer 2002.

/** Manufactures RangedType objects. Hold relevant data and validates before
 *instantiating the appropriate objects.
 */
public class RangedTypeBuilder extends UserTypeBuilder
{
  double low; 
  double high;
  double exception;
  
  /** Default constructor takes the base UserTypeBuilder and instantiates a new RangedTypebuilder 
    adds a low/high range and exceptional value.  Note that UserTypeBuilder must be defined  **/
  public RangedTypeBuilder(UserTypeBuilder t, String l, String h, String e)
  {
    super(t.getName(), t.getStorageTypeValue());
    setLow(l);
    setHigh(h);
    setException(e);
  }
    
  /** Overidden constructor takes the base UserTypeBuilder and instantiates a new RangedTypebuilder 
    adds a low/high range and default exceptional value.  Note that UserTypeBuilder must be defined **/
  public RangedTypeBuilder(UserTypeBuilder t, String l, String h)
  {
    super(t.getName(), t.getStorageTypeValue());
    setLow(l);
    setHigh(h);
    setDefaultException();
  }
    
  /** Overidden constructor takes a string name, string storage type, a low/high range and
    an exceptional value. **/
  public RangedTypeBuilder(String n, String t, String l, String h, String e)
  {
    super(n, t);
    setLow(l);
    setHigh(h);
    setException(e);
  }
    
  /** Overidden constructor takes a string name, string storage type, a low/high range and
    sets a default exceptional value. **/
  public RangedTypeBuilder(String n, String t, String l, String h)
  {
    super(n, t);
    setLow(l);
    setHigh(h);
    setDefaultException();
  }
    
  /** Set the low value for the range to the argument value **/
  public void setLow(double v)
  {
    low = v;
  }
    
  /** Set the low value for the range value represented by the argument string**/
  public void setLow(String s)
  {
    if (s != null && s.length() > 0)
      low = Double.parseDouble(s);
  }
    
  /** Set the high value for the range **/
  public void setHigh(double v)
  {
    high = v;
  }
    
  /** Set the high value for the range value represented by the argument string**/
  public void setHigh(String s)
  {
    if (s != null && s.length() > 0)
      high = Double.parseDouble(s);
  }
    
  /** Set the high value for the range **/
  public void setException(double v)
  {
    exception = v;
  }
    
  /** Set the exception value for the range value represented by the argument string**/
  public void setException(String s)
  {
    if (s != null && s.length() > 0)
      exception = Double.parseDouble(s);
  }
  
  /** Set the default exception value for the range value to the low value**/
  // Note: this behavior is probably best to be put into the RangedUserType class itself.
  private void setDefaultException()
  {
     setException(getLow());
  }
  
  /** Returns the low value as a double **/
  public double getLow()
  {
    return low;
  }
    
  /** Returns the high value as a double **/
  public double getHigh()
  {
    return high;
  }
    
  /** Returns the exception value as a double **/
  public double getException()
  {
    return exception;
  }
    
    
  /** Returns a Type object that has been instantiated as a RangedUserType object.  If unable
    to create the object a null Type object will be returned. */
  public UserType makeUserType()
  {
       // this needs to be modified once the RangedUserType handles exception value //
    return new RangedType(getName(), getStorageTypeValue(), getLow(), getHigh());
  }
    
   /** diagnostic utility method for dumping this class to a string **/
    public String toString()
    {
      String s = super.toString();
      s = s+ " [" + getLow() + "," + getHigh() + "," + getException() +"]";
      return s;
    }
 }
