package edu.unh.sdb.common;

//------------------------- Edit history (since 10/21/05) -----------------------
// 06/05/08 rdb: Made ArrayLists generic
// 10/21/05 rdb: Cleaned up unused variables and some reformatting.
//               Added getAttributes (mainly to get rid of compile time warnings.
//-------------------------------------------------------------------------------

import java.util.*;

//------------------------- RecordDescriptorBuilder class ----------------------
//  Created by Denise Mitchell Summer 2002.

/** Manufactures Record Descriptor objects. Hold relevant data and validates before
 * instantiating the appropriate objects.
 */


public class RecordDescriptorBuilder extends UserTypeBuilder
{
  private AttributeDescriptor[] defs;
  private TypeTable types;
    
  /** Declares a vector to hold the field name strings **/
  protected Vector<String> fieldNames;
        
  /** Declares a vector to hold the field type strings**/
  protected Vector<String> fieldTypes;
  
  /** Default constructor takes a string name and array of attributes (fields)**/
  public RecordDescriptorBuilder(String n)
  {
    super(n, StorageType.Record);
    setTypes(null);
    setDefaultFields();
}
    
  /** Overridden constructor takes a string name and array of attributes (fields)**/
  public RecordDescriptorBuilder(String n,  AttributeDescriptor[] attrs)
  {
    super(n, StorageType.Record);
    setTypes(null);
    setAttributes(attrs);
    setDefaultFields();
  }
  /** Overridden constructor takes a string name, string storage type, a low/high range and
    an exceptional value. **/
  public RecordDescriptorBuilder(String n, TypeTable types, AttributeDescriptor[] attrs)
  {
    super(n, StorageType.Record);
    setTypes(types);
    setAttributes(attrs);
    setDefaultFields();
  }
    
  /** Sets the type table. **/
  public void setTypes(TypeTable t)
  {
    types = t;
  }
  
  /** Returns the type table. **/
  public TypeTable getTypes()
  {
    return types;
  }
    
  /** Sets the attributes to cloned objects passed in the argument **/
  public void setAttributes(AttributeDescriptor[] a)
  {
    defs = a.clone();
  }

  /** Gets the attributes for this RD **/
  public AttributeDescriptor[] getAttributes()
  {
    return this.defs.clone();
  }

  private void setDefaultFields()
  {
      fieldNames = new Vector<String>();
      fieldTypes = new Vector<String>();
  }

  /** adds a field name and type to the list for the pending record descriptor**/
  public void addField(String n, String t)
  {
    fieldNames.addElement(n);
    fieldTypes.addElement(t);
  }
    
    //generate a record descriptor using the fieldNames and fieldTypes vectors
    //if there are no field names/types, then an empty RecordDescriptor will be returned

 /* public RecordDescriptor getRecordDescriptor()
    {
      RecordDescriptor    recordDesc;
      int elements = fieldNames.size();   //assume that the parser only gave us pairs of names/types
      int i = 0;
      AttributeDescriptor[] attrDescs= new AttributeDescriptor[elements];
      for(i=0;i<elements;i++){
        attrDescs[i]=new AttributeDescriptor( (String)fieldTypes.elementAt(i), (String)fieldNames.elementAt(i));
      }

      recordDesc = new RecordDescriptor(attrDescs);  
      return recordDesc;

    }
   */
    
  /** Returns a UserType object that has been instantiated as a RangedUserType object.  If unable
    to create the object a null Type object will be returned. */
  public UserType makeUserType()
  {
      int elements = fieldNames.size(); //assume that the parser only gave us pairs of names/types
      int i = 0;
      AttributeDescriptor[] attrDescs= new AttributeDescriptor[elements];
      for(i=0;i<elements;i++){
        attrDescs[i]=new AttributeDescriptor( fieldTypes.elementAt(i),
                                              fieldNames.elementAt(i));
      }

      return new RecordDescriptor(getName(), attrDescs);  
  }
    
   /** diagnostic utility method for dumping this class to a string **/
    public String toString()
    {
      String s = super.toString();
      s = s+ " [";
       
      for(int i=0 ; i<fieldNames.size() ; i++)
      {
        s = s + fieldTypes.elementAt(i);
        s = s + " ";
        s = s + fieldNames.elementAt(i) + "\n";
      }
        
      s = s + "]";
      return s;
    }
 }
