//
//  SDBIterator.java
//  CacheTest
//
//  Created by Philip Rhodes on Sat Apr 24 2004.
//  Copyright (c) 2004 University of New Hampshire. All rights reserved.
//

package edu.unh.sdb.common;

/** The SDBIterator interface is a simple interface designed for 
 *  efficient implementation. There is no hasNext() method, as
 *  this method can be inefficient for the kinds of iterations used
 *  in the SDB system. The valid() method provides similar functionality
 *  slightly different semantics.
 *
 *  Usage example:
 * 
 *     for(iter.init(); iter.valid(); iter.next()){
 *
 *           System.out.println(iter);
 *     }
 *
 */
public interface SDBIterator {

    /** Perform any necessary initialization, and set the iterator value
     *  to the first value in the iteration.
     */
    void init();
    
    /** Advance the iterator by one step. If the iterator cannot be advanced
     *  to produce a valid value, the value of the iterator is undefined, but
     *  the next() method will return false, as will a subsequent call to
     *  valid().
     */
    boolean next();
    
    /** Return true if the iterator currently has a valid value, false otherwise.*/
    boolean valid();
    
    /** Advance the iterator to the last valid value in the iteration. (optional operation)*/
    void end();

}
