package edu.unh.sdb.common;

//--------------------------UserTypeBuilder class ----------------------
//  Created by Denise Mitchell Summer 2002.

/** Manufactures UserType objects. Hold relevant data and validates before
 * instantiating the appropriate objects.
 */


public class UserTypeBuilder
{
  // the name of the pending datasource
  private String name;
    
  protected int storageType;
  
  public UserTypeBuilder(String n, int st)
  {
    setName(n);
    setStorageType(st);
  }
    
  public UserTypeBuilder(String n, String st)
  {
    setName(n);
    setStorageType(st);
  }
   
  /** Sets the name to the String provided*/
  public void setName(String s)
  {
    name = s;
  }
  
  /** Returns the name*/
  public String getName()
  {
    return name;
  }
    
  /** Sets the base storage type given the string name of the type within the TypeTable. **/
  public void setStorageType(String st)
  {
    storageType = StorageType.typeStringtoInt(st);
  }

  /** Sets the base storage type given the integer value of the type within the TypeTable. **/
  public void setStorageType(int st)
  {
    storageType = st;
  }
  
  /** Returns the integer value from the TypeTable of the storage type.**/
  public int getStorageTypeValue()
  {
    return storageType;
  }
    
  /** Returns the name of the storage type.**/
  public String getStorageTypeName()
  {
    return StorageType.intToTypeString(storageType);
  }

  /** Returns a Type object that has been instantiated as a UserType object.  If unable
    to create the object a null Type object will be returned */
  public UserType makeUserType()
  {
    return new UserType(getName(), getStorageTypeValue());
  }
    
   /** diagnostic utility method for dumping this class to a string **/
    public String toString()
    {
      String s = "Type: ";
      s = getName(); 
      if (s == null)
        s = "NO TYPE NAME SPECIFIED";
      s = s + " Storage Type Name: <" + getStorageTypeName() + ">";       
      s = s + " Storage Type Code: <" + getStorageTypeValue()+ ">";       
      return s;
    }
 }
