//
//  WorkSpace.java
//  DS2Test
//
//  Created by Phil Rhodes on Tue Jan 15 2002.
//  Copyright (c) 2001 University of New Hampshire. All rights reserved.
//---------------- edit history ------------------------------------
// 06/06/08 rdb: made HashMap generic

package edu.unh.sdb.common;

import java.util.*;

import edu.unh.sdb.datasource.*;
import edu.unh.sdb.lattice.*;
import edu.unh.sdb.mdb.*;


/** The WorkSpace class is a container holding the various objects that an SDB
 *  user will require during an SDB work session. This includes a list of datasources,
 *  lattices, and types.
 */
public class WorkSpace {

    private DBIF dbConnection; // Is this what we want?
    private Hashtable<String,WorkSpace> lookUpTable = new Hashtable<String,WorkSpace>();
    private String name;  // I think this should also be the name of the DB
    private DataSourceCollection dsCollection;
    private LatticeCollection latticeCollection;
    private TypeTable types;
    private URL url;

    public WorkSpace(URL url){
    
        this.name=new String(url.dbName);
        this.types = new TypeTable();
        this.dsCollection = new DataSourceCollection();
        this.latticeCollection = new LatticeCollection();
            
        try{
            dbConnection= DBIF.open(url);
        } catch(Exception e) {
            new MetaDataBaseError(SDBError.Fatal,"Error opening metadata database in Workspace(String, URL).");
        }
    }

   
    public WorkSpace(String name){
        
        this.name=new String(name);
        this.types=new TypeTable();
        this.dsCollection=new DataSourceCollection();
        this.latticeCollection=new LatticeCollection();
    }

    
    public WorkSpace(String name, TypeTable t){
        
        this.name=new String(name);
        this.types=t;
        this.dsCollection=new DataSourceCollection();
        this.latticeCollection=new LatticeCollection();
    }

    
    public WorkSpace(String name, DataSourceCollection collection){

        this.name=new String(name);
        this.types=new TypeTable();
        this.dsCollection=collection;
        this.latticeCollection=new LatticeCollection();
    }
    
    public WorkSpace(String name, DataSourceCollection collection, TypeTable t){

        this.name=new String(name);
        this.types=t;
        this.dsCollection=collection;
        this.latticeCollection=new LatticeCollection();
    }

   // public void load(DataSource d){
    
    
  // }
    
    /** Returns the DataSourceCollection. */ 
    public DataSourceCollection getDSCollection(){
        
        return this.dsCollection;
    }
 
    /** Returns the LatticeCollection. */ 
    public LatticeCollection getLatticeCollection(){
        
        return this.latticeCollection;
    }
   
    /** Returns the TypeTable. */ 
    public TypeTable getTypeTable(){
    
        return this.types;
    }
    
    /** Returns the WorkSpace name as a String. */ 
    public String getName(){
    
        return this.name;
    }

    /** Returns the DataBase Connection. */
    public DBIF getDBConnection()
    {
        return this.dbConnection;
    }
    

   /** Returns the URL of the primary database which the DataSource 
       specified by the name is saved*/
   public URL getURL(String name)
   {
      
    WorkSpace ws = lookUpTable.get(name);

    //not an AliasDataSource or ForeignReferenceDataSource
    if(ws == null)
    {
        // already saved in database
        if(dbConnection != null && dbConnection.isDSNameUsed(name))
        	return url;
        else
            return null;  //dynamically created by the workspace
    }
    else
        return ws.getURL(name);
  }
 

  /** add the name of the AliasDataSource or ForeignReferenceDataSource to the look up table */
  public void addWorkSpace(String name, WorkSpace ws)
 {
    if (ws != null)
      lookUpTable.put( name, ws );
      
 }

  /** add the DataSource specified by the name from database to workspace */ 
  public DataSource loadDS(String name)throws Exception
  {
    DataSource ws = null;

    if (dbConnection != null)
       ws = this.dbConnection.loadDS(name, this);

    return ws;
  }  

  /** add all the DataSources' in database to workspace */
  public void loadFully()throws Exception
  {
    if(dbConnection != null)
      this.dbConnection.loadWorkSpace(this);
  }  

}
