/*                                                                           
  ARRSCellID.java
  Copyright 2004 University of New Hampshire
  Durham, NH 03824
  Programmer: LI YE
*/

/**
 * The ARRSCellID class indicates the identifier of a cell in a ARRS tree  
 */

package edu.unh.sdb.datasource;

//import java.io.*;
import edu.unh.sdb.common.*;
                                                                                
public class ARRSCellID extends RectilinearCellID {

   private int[] path; //store the path of a cell in a ARRS tree

   /** Consturctor without arguments */
   public ARRSCellID()
   { 
      super(); 
      this.path = null;
   }

   /** Consturctor takes the specified IndexSpaceID in a cell space */
   public ARRSCellID(IndexSpaceID p)
   { 
      super(p); 
      this.path = null;
   }

   /** Consturction takes the specified IndexSpaceID in a cell space 
     * and the specified path in a ARRS tree.
     */
   public ARRSCellID(IndexSpaceID p, int[] path)
   { 
      super(p); 
      this.path =  path.clone();
   }

   /** Get the path of a cell in a ARRS tree */
   public int[] getPath()
   {
      return this.path;
   }

   /** Set the path of a cell in a ARRS tree */
   public void setPath(int[] path)
   {
      this.path = path.clone();
   } 

   /** Return a String representing the object indented by the argument string.
     */
   public String toString(String indent){
      int i; 
      String indent2 = indent+Globals.toStringIndent;
      StringBuffer result = new StringBuffer(indent+"{ ARRSCellID:\n");
                   
      result.append(indent2+"Position is: ");
      result.append(this.position.toString()+"\n");          

      if (this.path != null) {                   
         result.append(indent2+"Path is:     <PATH (");
         for (i = 0; i < this.path.length-1; i++) {
            result.append(this.path[i]+", ");
         }
      
         result.append(this.path[this.path.length-1]+")>\n");
      } 
      result.append(indent+"}");
      return new String(result);
   }
}
