/*
    ARRSCellIterator.java
    Copyright 2004 University of New Hampshire
    Durham, NH 03824
    Programmer: LI YE
*/
                                                                                
/**
 * The ARRSCellIterator class implements an interface to iterate through
 * an ARRS cell space to visit each cell in each leaf cell block.
 */

package edu.unh.sdb.datasource;
                                                                                
import java.util.*;
//import java.io.*;
import edu.unh.sdb.common.*;
                                                                                
public class ARRSCellIterator extends CellIterator {
    private CellIndexable cellblock;
       //store an ARRS cell block
    private ARRSCellBlockIterator arscellBlockIterator;
       //store an ARRSCellBlockIterator object
    private URCellIterator urcellIterator;
       //store a URCellIterator object

    /** Constructor takes an ARRS cell block */
    //public ARRSCellIterator(ARRSCellBlock cblock)
    public ARRSCellIterator(CellIndexable cblock)
    {
       super();
       this.cellblock = cblock;
    }

    /** Initialize this ARRSCellIterator */
    public void init()
    {
       CellIndexable cblock;
       int[] path = null;
       this.arscellBlockIterator = new ARRSCellBlockIterator(this.cellblock);
       this.arscellBlockIterator.init();
       ArrayList leafblock = this.arscellBlockIterator.getLeafBlockList();
       System.out.println("lenght of leaf blocks: "+leafblock.size());
       if (this.arscellBlockIterator.valid()) {
          //get the leaf cell block (a uniform cell block)
          cblock = this.arscellBlockIterator.getLeafCellBlock();

          //create a URCellIterator object with the leaf cell block
          this.urcellIterator = new URCellIterator(cblock);
          this.urcellIterator.init();
          if (this.urcellIterator.valid()) {
             //get a ARRS cell 
             this.cell = (ARRSCell)this.urcellIterator.getCell();
             //set the path of the ARRS cell
             if (cblock instanceof RegCellDataSource){
                path = ((RegCellDataSource)cblock).getPath().clone();
             }
             if (cblock instanceof URCellBlock){
                path = ((URCellBlock)cblock).getPath().clone();
             }
             ((ARRSCellID)this.cell.getCellID()).setPath(path);
             this.valid = true;
          }
      } 
    }

 
    /** Moving the ARRSCellIterator to the next position. */
    public boolean next()
    {
       //URCellBlock cblock;
       CellIndexable cblock;
       int[] path = null;

       if (this.valid == false)
          return false;
      
       this.urcellIterator.next(); 
       if (this.arscellBlockIterator.valid() && this.urcellIterator.valid()) {
          this.valid = true;
          this.cell = (ARRSCell)this.urcellIterator.getCell();
          //set the path of the ARRS cell
          cblock = this.urcellIterator.getDataBlock();
          if (cblock instanceof RegCellDataSource){
             path = ((RegCellDataSource)cblock).getPath().clone();
          }
          if (cblock instanceof URCellBlock){
             path = ((URCellBlock)cblock).getPath().clone();
          }
          ((ARRSCellID)this.cell.getCellID()).setPath(path);
          return true;
       }

       else if (this.arscellBlockIterator.valid() 
                && !this.urcellIterator.valid()) {
          this.arscellBlockIterator.next();
          if (this.arscellBlockIterator.valid()) {
             //get a leaf cell block (a uniform cell block)
             cblock = this.arscellBlockIterator.getLeafCellBlock();
             //create a URCellIterator
             this.urcellIterator = new URCellIterator(cblock);
             this.urcellIterator.init();
             if (this.urcellIterator.valid()) {
                //get a ARRS cell
                this.cell = (ARRSCell)this.urcellIterator.getCell();
                //set the path of the ARRS cell
                if (cblock instanceof RegCellDataSource){
                   path = ((RegCellDataSource)cblock).getPath().clone();
                }
                if (cblock instanceof URCellBlock){
                   path = ((URCellBlock)cblock).getPath().clone();
                }
                ((ARRSCellID)this.cell.getCellID()).setPath(path);
                this.valid = true;
                return true;
             }
          }
          else {
             this.valid = false;
             return false;
          }
       }
 
       else {
          this.valid = false;
          return false;
       }

       return false;
    }
 
    /** Get a cell */
    public Cell getCell()
    {
       return this.cell;
    }

    /** Return description of this ARRSCell */
    public String toString(String indent){
       String indent2=indent+Globals.toStringIndent;
       StringBuffer result=new StringBuffer(indent+"{ ARRSCellIterator:\n");
       result.append(indent2+"valid = "+this.valid+"\n");
       result.append(indent2+"Cell info: \n");                           
       result.append(this.cell.toString(indent2)+"\n");
       result.append(indent+"}");
       return new String(result);
   }
}

