/*
    ARRSNode.java
    Copyright 2004 University of New Hampshire
    Durham, NH 03824
    Programmer: LI YE
*/

/* ------------------- Edit history since 12/10/05--------------------------------
 * 12/12/05 rdb: Renamed setNodeID and getNodeID to setID and getID
 * 12/10/05 rdb: Added copy() abstract method
 *                     copy constructor
 */

/**
 * The ARRSNode class develop an ARRS node structure based on the concepts of 
 * the tree to represent an ARR structure data source. 
 */
                                                                                

package edu.unh.sdb.datasource;

import java.util.*;
//import java.io.*;

//import edu.unh.sdb.common.*;

abstract public class ARRSNode
{

   protected int nodeId;
           //store the node's id
   protected int siblingId;
           //store the sibling's id of this node
   protected ISBounds baseBounds;
           //store the area base bounds covered by this node
   protected int[] path ;
           //store the path of an ARRS node in ARRS tree
   protected CellIndexable cellSource;
           //store ARRSCellCopyDS or ARRSCellBlock

   /** Constructor without argument */
   public ARRSNode() 
   {
      this( (ISBounds) null );
   }
   
   /** Constructor takes a single ISBounds */
   public ARRSNode( ISBounds bounds ) 
   {
      this( bounds, null );
   }
   
   /** Constructor takes a ISbounds and a ARR cell source */
   public ARRSNode( ISBounds bounds, CellIndexable cellSource )
   {
      this.nodeId = -1;
      this.siblingId = -1;
      this.baseBounds = (ISBounds)bounds.clone();
      this.path = null;
      this.cellSource = cellSource;
   }

   /** Copy constructor  */
   public ARRSNode( ARRSNode src )
   {
      this( src.baseBounds, src.cellSource );
   }   

   /**
    * Abstract clone method
    */
   abstract public ARRSNode copy();
   
   /** Set the node ID value for an ARRS node */
   public void setID(int nodeId)
   {
      this.nodeId = nodeId;
   }
  

   /** Set the path value for an ARRS node */
   public void setNodePath(int[] path)
   {
      this.path = path.clone();
   }
  

   /** Set the base bound value */
   public void setBaseBounds(ISBounds bounds)
   {
      this.baseBounds = (ISBounds)bounds.clone();
   }

   
   /** Set the sibling value */
   public void setSibling(int sib)
   {
      this.siblingId = sib;
   }

   

   /** get the sibling of the current node */
   public ARRSNode getSibling(ArrayList tree)
   {
      if (this.siblingId == -1) 
         return null;

      return (ARRSNode)tree.get(this.siblingId);
   }
   
   /** get the id of the current node */
   public int getID()
   {
      return this.nodeId;
   }
 
   /** get the the id of the sibling */
   public int getSiblingID()
   {
      return this.siblingId;
   }


   /** get the bounds of this node */
   public ISBounds getBaseBounds()
   { 
      return this.baseBounds;
   }

   /** get the path of the current node */
   public int[] getPath()
   {
      return this.path;
   }

   /** Set an ARR cell source */
   public void setCellIndexable(CellIndexable cellSource)
   {
      this.cellSource = cellSource;
   }

   /** Get the ARR cell source */
   public CellIndexable getCellIndexable()
   {
      return this.cellSource;
   }

}
  
