/*
    ARRSTree.java
    Copyright 2004 University of New Hampshire
    Durham, NH 03824
    Programmer: LI YE
*/
//------------------- Edit history --------------------------------------
// 06/05/08 rdb: Made ArrayLists generic
// 01/02/06 rdb: buildLeafList: added code to clear old list. Fixed doing it twice.
//                I believe I must have added code that caused it to be done twice;
//                Not worth finding out where or why; this code is going away anyway.
// 12/10/05 rdb: Revised addNode
// 11/24/05 rdb: Took some steps towards making code independent of type of Leaf 
//               implementation by Referencing only parent classes, not children.
// 10/21/05 rdb: removed unused variables; commented out rootId uses
//-----------------------------------------------------------------------

                                                                                
/**
 * The ARRSTree class provide a view of data in ARR structure data source 
 * which contains internal nodes and leaf nodes. A leaf node refers to
 * a uniform rectilinear region or a set of uniform contiguous cells
 */
                                                                           
package edu.unh.sdb.datasource;

import java.util.*;
import edu.unh.sdb.common.*;
//import java.io.*;

public class ARRSTree 
{
   private ArrayList<ARRSNode> nodeList;
           //store all the nodes of an ARS tree
   private ArrayList<ARRSLeafNode> leafNodeList;
           //store all the leaf cell block
   private int totalNodes;
           //store the total number of the nodes
   //private int rootId;
           //store the id for root node

   /** Constructor without arguments */
   public ARRSTree() 
   {
      this.nodeList = new ArrayList<ARRSNode>();
      this.leafNodeList = new ArrayList<ARRSLeafNode>();
      //this.rootId = -1; 
      this.totalNodes = -1;
   }
 
   /** Add a node into an ARRS tree */
   public boolean addNode( ARRSNode node, int parentId )
   {  
      ARRSInternalNode parent = null;
      //int numchildren;

      //add a root node
      if ( this.nodeList.isEmpty() ) 
      {        //it is not root node
         if ( parentId != -1 )
         {
            new SDBError( SDBError.Recoverable, "Empty tree, parentID not -1");
            return false;
         }

         //add the root node
         this.totalNodes++;
         node.setID( 0 );      
         this.nodeList.add( node );
         
         if ( node instanceof ARRSLeafNode )
            this.leafNodeList.add( (ARRSLeafNode)node );
        
         return true;
      }

      //add the other nodes except a root node 
      try 
      {
         parent = (ARRSInternalNode) this.nodeList.get( parentId );
      }
      catch ( IndexOutOfBoundsException e ) 
      {
         new SDBError( SDBError.Recoverable, "Parent id invalid: " + parentId );
         return false;
      }
      catch ( ClassCastException cce )
      {
         new SDBError( SDBError.Recoverable, "Parent id not an Internal Node: " + parentId );;
      }
     
      this.totalNodes++;
      node.setID( this.totalNodes );
      
      // try to add the node to its parent
      boolean success = parent.addChild( this.nodeList, node );
      if ( success )
      {
         this.nodeList.add( node );
         if ( node instanceof ARRSLeafNode )
            this.leafNodeList.add( (ARRSLeafNode) node );        
      }
      else
         this.totalNodes--;
      return success;
         
      /********** rdb: old code, revised above
      if ( parent instanceof NTreeNode ) 
      {
         numchildren = 1 << ((NTreeNode)parent).getBaseBounds().dim();
         return ( (NTreeNode)parent).addNode(
                        this.nodeList, node, parentId,  totalNodes, numchildren );
      } 
      else if ( parent instanceof KDNode ) 
      {
         return ((KDNode)parent).addNode( this.nodeList, node, parentId, totalNodes, 2 );
      }
      else 
      {
         this.totalNodes--; 
         return false;
      }
      /***************************/
   }
   
   /** Set the path of this tree */
   public void buildTreePath()
   {
      if ( nodeList.size() == 0 )
         return;
  
      ARRSNode node = this.getNode( 0 );
      int[] p = {-1};
      node.setNodePath( p );
      if (( node instanceof NTreeNode ) || ( node instanceof KDNode ))
         this.buildSubTreePath( node, null );
   } 

   /* help function for the buildTreePath function */
   private void buildSubTreePath( ARRSNode node, int[] p )
   {
      ARRSNode[] nodes;
      int childnum = 0;
      int[] childpath;

      nodes = this.getChildren( node );
      if ( nodes == null || nodes.length == 0 )
         return;

      for ( int i = 0; i < nodes.length; i++ ) 
      {
         if ( p == null ) 
         {
            childpath = new int[ 1 ];
            childpath[ 0 ] = childnum;
         }
         else 
         {
            childpath = new int[ p.length + 1 ];
            for ( int j = 0; j < p.length; j++ )
               childpath[j] = p[j];
            childpath[ p.length ] = childnum;
         }
         
         nodes[ i ].setNodePath(childpath);
         if (( nodes[ i ] instanceof NTreeNode ) || ( nodes[ i ] instanceof KDNode )) 
            this.buildSubTreePath( nodes[i], childpath );

         childnum++;
      }
   }
 
   /** Find the node with id number */
   public ARRSNode getNode( int id ) 
   {
      Object obj = null;
      try
      {
         obj = this.nodeList.get( id ); 
      } 
      catch ( IndexOutOfBoundsException e ) 
      {
         System.out.println( "ARRSTree.getNode: Index out of range: " + id );
      };

      return (ARRSNode)obj;
   } 

   
   /** Get one child of the specified node */
   public ARRSNode getChild(ARRSNode node, int i)
   {
      if ( node instanceof ARRSInternalNode )
         return (( ARRSInternalNode)node ).getChild( this.nodeList, i );
      return null;
   }


   /** Get the sibling of the specified node */
   public ARRSNode getSibling( ARRSNode node )
   {
      return node.getSibling( this.nodeList );
   }


   /** Get all the children of the specified node */
   public ARRSNode[] getChildren( ARRSNode node )
   {
      if ( node instanceof ARRSInternalNode )
         return ( (ARRSInternalNode)node ).getChildren( this.nodeList );
      return null;
   } 


   /** Get all the nodes in the tree */
   public ArrayList getNodeList()
   {
      return this.nodeList;
   }

   /** Get the number of all the nodes in the tree */
   public int getNumOfNodes()
   {
      return this.totalNodes;
   }

   /** Get the leaf cell block list */
   //public ArrayList getLeafCellBlockList()
   public ArrayList<ARRSLeafNode> getLeafList()
   {
      return this.leafNodeList;
   }


   /** Clear the list of leaf cell blocks */
   //public void detraverseLeafCellBlock()
   public void clearLeafList()
   {
      this.leafNodeList.clear();
   }


   /** Clear the ARRS tree */
   public void treeClear()
   {
      this.nodeList.clear();
      this.leafNodeList.clear();
   }

 
   /** Traverse all the leaf cell block in a ARR tree */
   //public void traverseLeafCellBlock()
   public void buildLeafList()
   {
      ARRSNode node;
      
      this.leafNodeList.clear();   // delete previous list if there is one
      if ( this.nodeList.isEmpty() )
         return;

      node = this.getNode( 0 );
      if ( node instanceof ARRSLeafNode ) 
         this.leafNodeList.add( (ARRSLeafNode) node );
      else if ( node instanceof ARRSInternalNode ) 
         this._buildLeafList( node );
      else 
         new SDBError( SDBError.Warning, "unexpected leaf: " 
                                       + node.getClass().getName() );
   }
     
   /* traverse an internal node */
   private void _buildLeafList( ARRSNode node )
   { 
      ARRSNode[] nodes;
      nodes = ( (ARRSInternalNode)node ).getChildren( this.nodeList );
      if ( nodes != null || nodes.length > 0)
      {
         for ( int i = 0; i < nodes.length; i++ ) 
         {
            if ( nodes[i] instanceof ARRSLeafNode )
               this.leafNodeList.add( (ARRSLeafNode) nodes[i] );
            
            else if (( nodes[i] instanceof NTreeNode) || (nodes[i] instanceof KDNode)) 
               this._buildLeafList(nodes[i]);
         }
      }
   }
      
   /** Print the info of a tree */
   public String toString()
   {
      ARRSNode node = this.nodeList.get(0);
      return this.printNode(node);
   }

   /** Print the info of a node */
   public String printNode(ARRSNode node)
   {
      return this.printNode(0,node);
   }

   /** Print the info of a node with some indent */
   public String printNode(int indent, ARRSNode node)
   {
      int i, j, len;
      ARRSNode[] children;
      int[] path;

      int space1 = 3 + indent;
      int space2 = 6 + indent;
      StringBuffer result = new StringBuffer("");
      for (i = 0; i < indent; i++) {
         result.append(" ");
      }
 
      if (node.getID() == 0) 
         result.append("Root: \n");

      else if ((node.getID() != 0) && (node instanceof ARRSClippedCopyLeafNode))
         result.append("Leaf clipped node: \n");

      else if ((node.getID() != 0) && (node instanceof ARRSLeafNode))
         result.append("Leaf node: \n");

      else
         result.append("Internal node: \n");

      for (i = 0; i < space1; i++) {
         result.append(" ");
      }
      result.append("Node ID = " + node.getID() + "\n");

      for (i = 0; i < space1; i++) {
         result.append(" ");
      }
      result.append("Base bounds: " + node.getBaseBounds() + "\n");

      for (i = 0; i < space1; i++) {
         result.append(" ");
      }
      if (node instanceof ARRSClippedCopyLeafNode){
         result.append("Sub bounds: " + ((ARRSClippedCopyLeafNode)node).getSubBounds() + "\n");

         for (i = 0; i < space1; i++) {
            result.append(" ");
         }
      }

      path = node.getPath();
      if (path != null){
         result.append("Path: <PATH (");
         for (i = 0; i < path.length-1; i++) {
            result.append(path[i]+", ");
         }
         result.append(path[path.length-1]+")>\n");
      }
      else
         result.append("Path: null\n"); 

      for (i = 0; i < space1; i++) {
         result.append(" ");
      }
      result.append("Sibling = " + node.getSiblingID() + "\n");

      if (node instanceof ARRSInternalNode) {
         for (i = 0; i < space1; i++) {
            result.append(" ");
         }
         result.append("Eldest = " +((ARRSInternalNode)node).getFirstChildID() + "\n");
      }

      for (i = 0; i < space1; i++) {
         result.append(" ");
      }
      result.append("Children: " + "\n");
      
      if ((node instanceof ARRSCopyLeafNode) || 
          ((node instanceof ARRSInternalNode)&&
          (((ARRSInternalNode)node).getNumChildren() == 0 )))
      {
         for (j = 0; j < space2; j++) {
               result.append(" ");
         }
         result.append( "No children\n\n" );
         return new String(result);
      }

      children = ((ARRSInternalNode)node).getChildren(this.nodeList);
      len = children.length;
      for (i = 0; i < len; i++) {
         result.append(this.printNode(space2, children[i]));
      }
      return new String(result);
   }
}