package edu.unh.sdb.datasource;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.application.particles.flow.Streamlet;



import ar.ARNode;
import ar.ARNodeBlock;
import ar.ARSparseStorage;
import ar.ARStorage;
import edu.unh.sdb.common.Datum; 
import edu.unh.sdb.common.FieldIDMapper;

/**
 *
 * ARSparseDataSource class, this data source class supports the ARSparse func
 * 
 * Under implementation 
 * 
 * @author AbinayaPitchai
 *
 */
public class ARSparseDataSource extends DataSource {

	/*
	 *  We are defining this field with the sparse file xfdl. For each resolution file we need to create a sparseStorage class
	 *  	based on the respective resolution sparse xfdl file.
	 *  
	 *  SPARSESTORAGE  ( Important fields for getting sparse data ) 
	 *  	sparseTable < Integer , SparseBlockInfo > : Hash table < sparseIndex, sparse block info > 
	 *  	SPARSEBLOCKINFO -> sparse Index, start position of the block, seek address in sparse file, block size, original block index
	 */
	//SparseStorage sparseStorage ; 
	
	/*
	 * ARSTORAGE ( Important fields for getting the AR data ) 
	 * 	 aRDataSource -> holds the ARTree info in the ARNode variable 'root'
	 * 	 blockList -> list of leaves info of ARNodeBlock type. ( need to explicitly call ARStorage.createValidARNodeBlockList method for generating this list)
	 * 	 arTable ->  hash table of index, ARStorageaBlock ( need to explicitly call ARStorage.createARDataBlocksAndAddToARTable )
	 */
	//ARStorage arStorage ; 
	
	ARSparseStorage arSparseStorage ; 
	
	ARNode treeRoot ;
	
	private int startIndex ; 
	// subblock ( ARBlockList, same as others ) ARNodeList -> that returns the full data in the list.
	// 
	
	public ARSparseDataSource(String[] arfilePrefixSuffixArr, String sparseFile){
		super(sparseFile); // name of the ds name of tehe xfdl file that defines it
		//sparseStorage = new SparseStorage("ARSparseFile", sparseFile) ; 
		//arStorage = new ARStorage(arfilePrefixSuffixArr);
		arSparseStorage = new ARSparseStorage(arfilePrefixSuffixArr, sparseFile);
		treeRoot = arSparseStorage.getArDataSource().getRoot();
		startIndex = 0 ;
		
	}
	
	public Map<Integer,ISBounds> getSparsePartsMap(ISBounds bounds, int resolution){
		// based on the AR node resolution  we will be getting the valid parts 
		//from the respective resolution sparse file.
		return arSparseStorage.getSparseArr()[resolution].getParts(bounds) ; 
	}
	
	/*
	 *  Method that returns a map of sparse Data Blocks with sparse Bounds of each Block as key. 
	 *  Returned map contains only the parts that was found in the sparse data.
	 *  If given bounds are not present in sparse then empty map is returned.
	 *  
	 */
	public Map<ISBounds, DataBlock> subblock(ISBounds bounds, int resolution){
		// first we need to check whether the given bounds are valid and find the 
		// valid and invalid sparse block parts for it.
		
		// based on the AR node resolution  we will be getting the valid parts from 
		//the respective resolution sparse file.
		
		SparseStorage sparseStorage = arSparseStorage.getSparseArr()[resolution] ;
 		Map<Integer, ISBounds> partsMap = sparseStorage.getParts(bounds) ; 
		Map<ISBounds, DataBlock> blocksMap  = new HashMap<ISBounds, DataBlock>() ; 
		
		if(!partsMap.isEmpty()){ 
			
			
			for(Map.Entry<Integer, ISBounds> mapEntry : partsMap.entrySet()){
				DataBlock dBlock = sparseStorage.readPartition(mapEntry.getKey()) ; 
				SparseBlockInfo sparseBlock = sparseStorage.getSparseDataTable().get(mapEntry.getKey()); 
				Integer[] sparseLower =  sparseBlock.getStartPosition() ; 
				ISBounds entryBounds = mapEntry.getValue() ; 
				int[] entryBoundsLower = entryBounds.getLowerArray(); 
				//int[] dbLower = dBlock.getLowerBoundsArray() ; 
				int[] lower = { entryBoundsLower[0] - sparseLower[0], 
								entryBoundsLower[1] - sparseLower[1], 
								entryBoundsLower[2] - sparseLower[2]} ; 
				
				int[] entryBoundsSize = entryBounds.getDimensionsArray(); 
				int[] upper = { (lower[0] + entryBoundsSize[0]) - 1 , 
								(lower[1] + entryBoundsSize[1]) - 1 ,
								(lower[2] + entryBoundsSize[2]) - 1  } ;
						
						    
			
				ISBounds currPartBounds = new ISBounds(lower, upper) ; 
				dBlock = dBlock.subblock(currPartBounds) ; 
				
				blocksMap.put(entryBounds, dBlock) ; 
			}
		}
		return blocksMap ; 
	}
	
	
	public void  Datum (Datum dtm, IndexSpaceID did, int resolution ){
		// based on the resolution we will be calling that resolution sparseStorage class object
		SparseStorage sparseStorage = arSparseStorage.getSparseArr()[resolution] ;
		sparseStorage.datum(dtm, did) ;
		if(dtm == null){
			System.out.println(" Given index space id doesn't exist in the Sparse data of resolution" + resolution) ; 
		}
		
	}
	
	public void Datum (Datum dtm, IndexSpaceID did, FieldIDMapper mp, int resolution){
	// based on the resolution we will be calling that resolution sparseStorage class object
		SparseStorage sparseStorage = arSparseStorage.getSparseArr()[resolution] ;
		sparseStorage.datum(dtm, did, mp) ; 
		
	}

	@Override
	public void datum(edu.unh.sdb.common.Datum dtm, IndexSpaceID did,
			FieldIDMapper mp) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void subblock(DataBlock theBlock, ISBounds dstBounds,
			ISBounds srcBounds) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void subblock(DataBlock theBlock, ISBounds dstBounds,
			ISBounds srcBounds, FieldIDMapper mp) {
		// TODO Auto-generated method stub
		
	}
	
	// currently assuming that octents bounds are given as input and the method is called in or
	//		Map , or loop traversal	
	// Should this be in the ARSparseStorage ?? instead of ARSparseDS
	public void getLeaves(int regionIndex, ARNode treeNode, List<ARNode> myList) {
			getLeaves((ARNode)treeNode.getChild(regionIndex), myList) ; 
	}
	
	private void getLeaves(ARNode currNode, List<ARNode> myList){
		if(currNode == null){
			return ; 
		}
		for (int i = 0; i <= 7; i++) {
			ARNode childNode = (ARNode) currNode.getChild(i);
			if (childNode != null && childNode.isLeaf()) {
				myList.add(childNode);
			}

			getLeaves(childNode, myList);

		}
	}
	
	private boolean withinRegion(ISBounds nbnds, ISBounds leafBounds) {
		boolean withinBlock_x = false;
		boolean withinBlock_y = false;
		boolean withinBlock_z = false;

		if (leafBounds.getLower(0) >= nbnds.getLower(0) && leafBounds.getUpper(0) < nbnds.getUpper(0)) {
			withinBlock_x = true;
		}
		if (leafBounds.getLower(1) >= nbnds.getLower(1) && leafBounds.getUpper(1) < nbnds.getUpper(1)) {
			withinBlock_y = true;
		}
		if (leafBounds.getLower(2) > nbnds.getLower(2) && leafBounds.getUpper(2) >= nbnds.getUpper(2)) {
			withinBlock_z = true;
		}

		return (withinBlock_x && withinBlock_y && withinBlock_z);
	}

	public ARSparseStorage getArSparseStorage() {
		return arSparseStorage;
	}

	public void setArSparseStorage(ARSparseStorage arSparseStorage) {
		this.arSparseStorage = arSparseStorage;
	}

	public ARNode getTreeRoot() {
		return treeRoot;
	}

	public void setTreeRoot(ARNode treeRoot) {
		this.treeRoot = treeRoot;
	}
	
	
	
	
}
