package edu.unh.sdb.datasource;

import java.util.HashMap;
import java.util.Map;



import ar.ARNode;
import ar.ARStorage;
import edu.unh.sdb.common.Datum; 
import edu.unh.sdb.common.FieldIDMapper;

/**
 * 
 * ARStorage and SparseStorage class, this data source class supports the ARSparse func 
 * Developed on 06/19/14
 * 
 * @author AbinayaPitchai
 *
 */
public class ARSparseDataSourceOld {

	/*
	 *  We are defining this field with the sparse file xfdl. For each resolution file we need to create a sparseStorage class
	 *  	based on the respective resolution sparse xfdl file.
	 *  
	 *  SPARSESTORAGE  ( Important fields for getting sparse data ) 
	 *  	sparseTable < Integer , SparseBlockInfo > : Hash table < sparseIndex, sparse block info > 
	 *  	SPARSEBLOCKINFO -> sparse Index, start position of the block, seek address in sparse file, block size, original block index
	 */
	SparseStorage sparseStorage ; 
	
	/*
	 * ARSTORAGE ( Important fields for getting the AR data ) 
	 * 	 aRDataSource -> holds the ARTree info in the ARNode variable 'root'
	 * 	 blockList -> list of leaves info of ARNodeBlock type. ( need to explicitly call ARStorage.createValidARNodeBlockList method for generating this list)
	 * 	 arTable ->  hash table of index, ARStorageaBlock ( need to explicitly call ARStorage.createARDataBlocksAndAddToARTable )
	 */
	ARStorage arStorage ; 
	
	ARNode treeRoot ;
	// subblock ( ARBlockList, same as others ) ARNodeList -> that returns the full data in the list.
	// 
	
	public ARSparseDataSourceOld(String sparseFile, String[] arfilePrefixSuffixArr){
		sparseStorage = new SparseStorage("ARSparseFile", sparseFile) ; 
		arStorage = new ARStorage(arfilePrefixSuffixArr);
		treeRoot = arStorage.getArDataSource().getRoot();
		
	}
	
	public Map<Integer,ISBounds> getSparsePartsMap(ISBounds bounds, int resolution){
		// based on the AR node resolution  we will be getting the valid parts 
		//from the respective resolution sparse file.
		return sparseStorage.getParts(bounds) ; 
	}
	
	/*
	 *  Method that returns a map of sparse Data Blocks with sparse Bounds of each Block as key. 
	 *  Returned map contains only the parts that was found in the sparse data.
	 *  If given bounds are not present in sparse then empty map is returned.
	 *  
	 */
	public Map<ISBounds, DataBlock> subblock(ISBounds bounds, int resolution){
		// first we need to check whether the given bounds are valid and find the 
		// valid and invalid sparse block parts for it.
		
		// based on the AR node resolution  we will be getting the valid parts from 
		//the respective resolution sparse file.
		Map<Integer, ISBounds> partsMap = sparseStorage.getParts(bounds) ; 
		Map<ISBounds, DataBlock> blocksMap  = new HashMap<ISBounds, DataBlock>() ; 
		
		if(!partsMap.isEmpty()){ 
			
			
			for(Map.Entry<Integer, ISBounds> mapEntry : partsMap.entrySet()){
				DataBlock dBlock = sparseStorage.readPartition(mapEntry.getKey()) ; 
				SparseBlockInfo sparseBlock = sparseStorage.getSparseDataTable().get(mapEntry.getKey()); 
				Integer[] sparseLower =  sparseBlock.getStartPosition() ; 
				ISBounds entryBounds = mapEntry.getValue() ; 
				int[] entryBoundsLower = entryBounds.getLowerArray(); 
				//int[] dbLower = dBlock.getLowerBoundsArray() ; 
				int[] lower = { entryBoundsLower[0] - sparseLower[0], 
								entryBoundsLower[1] - sparseLower[1], 
								entryBoundsLower[2] - sparseLower[2]} ; 
				
				int[] entryBoundsSize = entryBounds.getDimensionsArray(); 
				int[] upper = { (lower[0] + entryBoundsSize[0]) - 1 , 
								(lower[1] + entryBoundsSize[1]) - 1 ,
								(lower[2] + entryBoundsSize[2]) - 1  } ;
						
						    
			
				ISBounds currPartBounds = new ISBounds(lower, upper) ; 
				dBlock = dBlock.subblock(currPartBounds) ; 
				
				blocksMap.put(entryBounds, dBlock) ; 
			}
		}
		return blocksMap ; 
	}
	
	
	public void  Datum (Datum dtm, IndexSpaceID did, int resolution ){
		// based on the resolution we will be calling that resolution sparseStorage class object
		sparseStorage.datum(dtm, did) ;
		if(dtm == null){
			System.out.println(" Given index space id doesn't exist in the Sparse data of resolution" + resolution) ; 
		}
		
	}
	
	public void Datum (Datum dtm, IndexSpaceID did, FieldIDMapper mp, int resolution){
	// based on the resolution we will be calling that resolution sparseStorage class object
		sparseStorage.datum(dtm, did, mp) ; 
		
	}
	
	
	
}
