//
//  BlockListCache.java
//
//  Created by Xuan Tang on Jan 21 2004.
//  Copyright (c) 2003 University of New Hampshire. All rights reserved.
//
//------------------ Edit history as of 10/21/05 -------------------------------
// 10/21/05 rdb: Added getEnumWindow to avoid warning over enumWindow variable.
//               commented or deleted unused variables.
//------------------------------------------------------------------------------

/* 
    BlockListCache.java
    Copyright 2000 University of New Hampshire
    Durham, NH 03824
*/

package edu.unh.sdb.datasource;

//import edu.unh.sdb.datasource.*;
import edu.unh.sdb.common.*;
//import java.io.*;

/**
  */


public class BlockListCache extends Cache{

   
   private PhysicalDataSource datasource;
   private int numCacheBlocks;
   private DataBlock[] cacheBlocks ;

  
   private IndexSpaceID[] blockIDs;
   private AxisOrdering ordering;
   private ISBounds sourceBounds;
   private int rodAxis;
   private int sourceRodSize ;
   private int lruID;
   
   // use RandomBlockIterator first, should make iterator a interface
   private RandomBlockIterator iterator;


   // this is the query bounds that cache uses to query datasource
   private ISBounds cacheQueryBounds;
   private BasicBlock cacheQueryBlock;
   private ISBounds listBlockBounds;



   // variables used for improving performance
   private  IntegerIndexSpaceID targetUpper;
   private  IntegerIndexSpaceID targetLower; 
   private  int numDim;
   //private  int[] upper;
   //private  int[] lower;

   private ISIDMapper mapper;
   private ISBounds[] enumWindow;

   public BlockListCache(PhysicalDataSource dataSource, int numCacheBlocks,
                                                    ISBounds listBlockSize )
  {
     super(dataSource);
     this.iterator = null;
     this.numCacheBlocks = numCacheBlocks;
     this.listBlockBounds = listBlockSize.copy();
     this.datasource = dataSource;
     this.ordering = dataSource.getAxisOrdering();
     this.rodAxis = this.ordering.getLeastSignificantAxis();  
     this.sourceBounds = dataSource.copyBounds();
     this.sourceRodSize = this.sourceBounds.copyUpper().getCoord(this.rodAxis);
     this.cacheQueryBounds = this.createCacheBuffer(this.listBlockBounds);

     this.cacheQueryBlock = new BasicBlock( this.cacheQueryBounds, 
                                     datasource.getRecordDescriptor(), this.ordering);

     //System.out.println("************ cacheQueryBounds "+ this.cacheQueryBounds ); 
     this.numDim = dataSource.dim();

     this.cacheBlocks = new DataBlock[ this.numCacheBlocks ];
     //this.outBlockIndex =0;
     this.blockIDs = new IndexSpaceID[ this.numCacheBlocks ];
     this.lruID =0;
     for ( int i =0; i< this.numCacheBlocks; i++)
     {
         this.blockIDs[i] = null;
         this.cacheBlocks[i] = new BasicBlock( this.listBlockBounds,
                               datasource.getRecordDescriptor(), this.ordering);
     }
  }
 

  /** given a query bounds, this method extend this bounds along the 
      LestSignificant axis and return the expaned bounds.
      This method is used for prefetching
  */
   private ISBounds createCacheBuffer(ISBounds userQueryBounds) 
   {
        targetUpper = userQueryBounds.copyUpper();
        targetLower = userQueryBounds.copyLower();
        
        targetUpper.setCoord(this.rodAxis, this.sourceRodSize);
        targetLower.setCoord(this.rodAxis, 0);
        return new ISBounds(targetLower,targetUpper);
   } 
   
   private void moveQueryBuffer( ISBounds userQueryBounds )
   {
        targetUpper = userQueryBounds.copyUpper();
        targetLower = userQueryBounds.copyLower();
        //System.out.println(" rodAxis is :" + this.rodAxis);
        for(int i=0; i< this.numDim; i++)
        {
           if( i != this.rodAxis ){
              //rdb: reordered arguments 02/03/05
              //rdb cacheQueryBounds.setUpper( targetUpper.getCoord(i), i );
              //rdb cacheQueryBounds.setLower( targetLower.getCoord(i), i );
              cacheQueryBounds.setUpper( i, targetUpper.getCoord(i) );
              cacheQueryBounds.setLower( i, targetLower.getCoord(i) );
           }
        }
   }
  
 
   public long checkSize(ISBounds bounds)
   {
      RecordDescriptor rec = this.datasource.getRecordDescriptor();
      int bytesize = rec.byteSize();
      return bytesize * bounds.volume();
   }
 
  /** lookup the cached blocks to see if the the query block is already in 
      cache. if not it will get it from the cache block, and get neighbor 
      blocks
   */ 
   public void lookup( BasicBlock queryBlock, ISBounds dstBounds, ISBounds userQueryBounds)
   {
        //System.out.println(" user query bounds  lower : " + userQueryBounds.getLower() ); 
        //System.out.println(" queried block lower is :" + userQueryBounds.getLower() );

        int startWindow;

        for( int i=0; i< this.numCacheBlocks; i++)
         {
             //System.out.println("move the cache check "+ i);
             
             if( (this.blockIDs[i] != null) ){
                //System.out.println(" cached block lower is :" + i +"  "+ this.blockIDs[i] );
                //System.out.println( );
               if( this.blockIDs[i].equals( userQueryBounds.copyLower() ) )  
               {
                 System.out.println(" find in cache ");
                 queryBlock.copyArray((BasicBlock) this.cacheBlocks[i] );
                 return ;
               }
             }
          }
         if ((iterator != null)  && iterator.isWindowOpen() ) // the enumeration window is open 
         {
            this.moveQueryBuffer( userQueryBounds);
            //System.out.println(" after move cache queryBounds " + this.cacheQueryBounds );
            this.datasource.subblock(cacheQueryBlock, cacheQueryBlock.getBounds(),cacheQueryBounds);
            mapper = new ISIDMapper( cacheQueryBlock.getBounds(), cacheQueryBounds);
            //System.out.println(" mapp component " + cacheQueryBlock.getBounds() ); 
            //System.out.println(" mapp composite " + cacheQueryBounds ); 
            int windowSize = iterator.getEnumWindow().length; 
            ISBounds windowElement;
            startWindow = iterator.getEnumWindowIndex();
            System.out.println( " enumeration start window is :"+  startWindow );
            for ( int i = iterator.getEnumWindowIndex(); i< windowSize; i++ )
            {
               System.out.println( "window index   :" + startWindow);
               windowElement = iterator.getWindowElement(i);
               System.out.println( "window index  " +i +"   "+ windowElement );
               if ( windowElement == null )
                  System.out.println( "ERROR in test: windowElement is null ");
               else if ( cacheQueryBounds.contains( windowElement.copyLower() ) )
                { 
                     //System.out.println(" find block in enumeration window index :" + i);
                     //System.out.println(" enum " + mapper.toComponent( windowElement) );
                     cacheQueryBlock.subblock( this.cacheBlocks[lruID],this.listBlockBounds, 
                                                           mapper.toComponent( windowElement ) ); 
                     this.setLRU( windowElement.copyLower() );        
                }
            }
         }
   }

   public void lookup(Datum theDatum, IndexSpaceID id){}

   /** set the enum window so that the cache can perform efficient prefetch
    */ 
   protected void setEnumWindow( ISBounds[] window)
   {
      this.enumWindow = window;
   }
   
   /** return the enum window 
    */ 
   protected ISBounds[] getEnumWindow()
   {
      return this.enumWindow;
   }
   
   protected void setIterator( RandomBlockIterator iter)
   {
      this.iterator = iter;
   } 

   /** specify the cache block, which will be replace if next cache miss happens
    */
   private void setLRU( IndexSpaceID newBlockID ){
       //System.out.println( "replaceCacheBlock :" + lruID );
       //System.out.println(" cached block :" + newBlockID);
       this.blockIDs[lruID] = newBlockID;
       if( lruID == this.numCacheBlocks -1 )
         lruID = 0;
       else 
         lruID ++;
   }


    public static void main(String[] args)
    {
      //ISBounds[] enumBounds;
      PhysicalDataSource ds =(PhysicalDataSource)DataSource.create(
                                                 "foo", "simple.8x8x8.1.xfdl");
      ds.activate();

      RecordDescriptor rec = ds.getRecordDescriptor();
      System.out.println("rec " + rec);
      System.out.println("ds " + ds);

      int[] lower = {0,0,0};
      int[] upper = {3,3,3};
      ISBounds partBounds = new ISBounds( new IntegerIndexSpaceID(lower),
                                             new IntegerIndexSpaceID(upper));
      
      
      RandomBlockIterator iter = new RandomBlockIterator( partBounds,ds.getBounds(), ds );
      iter.setWindowSize(10);
      ISBounds iterBounds = iter.getIteratorBounds();
      BasicBlock bblock = new BasicBlock(iterBounds, ds.getRecordDescriptor(), 
                                         ds.getAxisOrdering() );
      
      BlockListCache cache  = new BlockListCache(ds, 2, iterBounds);
      //enumBounds = iter.getEnumWindow();
      //cache.setEnumWindow( enumBounds );
      cache.setIterator( iter );
      
//      for( int i =0 ; i< enumBounds.length; i++){
//         System.out.println("enum bounds " + enumBounds[i] );
         //System.out.println("iter bounds " + iterBounds );
//      }
      //System.out.println("queryBlock" + bblock );
      

      int[] l = {0,0,0};
      int[] u = {2,2,2};
      ISBounds queryBounds = new ISBounds( new IntegerIndexSpaceID(l),
                                             new IntegerIndexSpaceID(u));
    


      //ds.subblock(bblock, iterBounds, enumBounds[0] ); 
      //cache.lookup(bblock, iterBounds, enumBounds[0] ); 
      cache.lookup(bblock, iterBounds, queryBounds ); 
      cache.lookup(bblock, iterBounds, queryBounds ); 
     

/*
       BlockListCache bcache = new BlockListCache();
       int[] order = {0,1,2};
       AxisOrdering axisOrdering = new AxisOrdering(order);
       int[] sourceDim = { 500,500,500};
       int[] upper = {300,300,300};
       int[] lower = {100,100,100};
       ISBounds targetBounds = new ISBounds( new IntegerIndexSpaceID(upper),
                                             new IntegerIndexSpaceID(lower));
       ISBounds sourceBounds= new ISBounds(sourceDim);
       ISBounds brushing = bcache.createCacheBuffer(targetBounds );
       System.out.println(brushing);
*/      
    } 
}



