package edu.unh.sdb.datasource;

import java.nio.ByteBuffer;
import edu.unh.sdb.common.*;
import edu.unh.sdb.datasource.*;

public class ChomboRemoteClient extends GraniteRemoteClient
{
    private int          dimensionality;
    private int          numLevels;
    private int[]        refRatio;
    private ISBounds[]   qBoxes;
    
    public ChomboRemoteClient(String name, RecordDescriptor recordDesc, ISBounds theBounds,
                              String URL, String serverURL)
    {
        this(name, recordDesc, theBounds, new AxisOrdering(theBounds.dim()), URL, serverURL);
    }
    
    public ChomboRemoteClient(String name, RecordDescriptor recordDesc, ISBounds theBounds,
                              AxisOrdering ordering, String URL, String serverURL)
    {
        super(name, recordDesc, theBounds, ordering, URL, serverURL);
    }
    
    public void getStencilBlocks(DSBlockStencil stencil)
    {
        int datatype = stencil.getStorageType();
        int size = stencil.size();
        int offset;
        
        switch(datatype)
        {
            case StorageType.Double:
                                
                ReceiveData();
                
                // The rest of this stuff can be executed as normal. ;-)
                stencil.setDoubleAMRData(dstByteArray);
                break;
                
            default:
                new InternalFaultError(SDBError.Fatal,"Unimplemented storage type in ChomboRemoteClient.getStencilBlocks()");
                break;
        }
    }
    
    public void feedData(byte[] data)
    {
        this.dstByteArray = data;
    }
    
    
    // The code that follows below is testing code used by a test program that
    // includes a method that accepts a byte stream created by a server, and a
    // method that prints that information.  These lines may not be necessary
    // in later versions of the client, and may eventually go away.
    
    public void testBytes(byte[] data)
    {
        feedData(data);
        
        ByteBuffer buffer = ByteBuffer.wrap(this.dstByteArray);
        
        // Read header information
        dimensionality = buffer.getInt();
        numLevels      = buffer.getInt();
        refRatio = new int[numLevels];
        for(int lv = 0; lv < numLevels; lv++)
            refRatio[lv] = buffer.getInt();
        
        int numQBoxes = buffer.getInt();
        
        qBoxes = new ISBounds[numQBoxes];
        
        int[] lower = new int[this.dimensionality];
        int[] upper = new int[this.dimensionality];
        
        for(int q = 0; q < numQBoxes; q++)
        {
            // Load the lower and upper coords of the current query box,
            // then create the query box's ISBounds
            buffer.asIntBuffer().get(lower, 0, lower.length);
            buffer.position(buffer.position() + (4 * this.dimensionality));
            buffer.asIntBuffer().get(upper, 0, upper.length);
            buffer.position(buffer.position() + (4 * this.dimensionality));
            qBoxes[q] = new ISBounds(lower, upper);
            
            System.out.printf("Original query box %d: %s\n", q, qBoxes[q]);
            System.out.println("------------------------------------------------");
            
            for(int lv = 0; lv < numLevels; lv++)
            {
                System.out.printf("Level %d information:\n", lv);
                
                int numBoxes = buffer.getInt();
                int dataElems = buffer.getInt();
                ISBounds boxes[] = new ISBounds[numBoxes];
                
                System.out.printf("  Number of boxes, this level: %d\n", numBoxes);
                System.out.printf("  Number of data elemens:      %d\n", dataElems);
                
                for(int b = 0; b < numBoxes; b++)
                {
                    // Read lower and upper coords for the current level box
                    buffer.asIntBuffer().get(lower, 0, lower.length);
                    buffer.position(buffer.position() + (4 * this.dimensionality));
                    buffer.asIntBuffer().get(upper, 0, upper.length);
                    buffer.position(buffer.position() + (4 * this.dimensionality));
                    boxes[b] = new ISBounds(lower, upper);
                    System.out.printf("  Box %d: %s\n", b, boxes[b]);
                }
                
                System.out.println("  Raw data skipped; moving on to next metadata section.");
                buffer.position(buffer.position() + (8 * dataElems));
            }
        }
    }
    
    public void printInfo()
    {
        System.out.printf("Number of diemsions:  %d\n" +
                          "Number of levels:     %d\n" +
                          "Refinement Ratios (below max level):\n",
                          this.dimensionality, this.numLevels);
        
        for(int i = 0; i < numLevels - 1; i++)
            System.out.printf("   Level %d -> %d:  %d\n", i, i+1, refRatio[i]);
        
        System.out.printf("\nNumber of query box selections: %d\n", qBoxes.length);
    }
}