/*    
    ComponentInfo.java
    last modified 2/20/2001
    Copyright 2000 University of New Hampshire
    Durham, NH 03824
    ------------------------ Edit history since 02/14/05 -----------------------
    02/14/05 rdb: allowed constructor FIDMap and ISIDMap arguments to be null;
                    -create ISID identity map
                    -leave FIDMap as null, had to fix "equals" method.
*/

package edu.unh.sdb.datasource;
import edu.unh.sdb.common.*;
//import java.io.*;

/** This class records information about a component DataSource of a
 *  AttributeJoinDataSource or BlockedDataSource.
 */
public class ComponentInfo 
{
    protected Indexable component;
    protected FieldIDMapper fieldIDMap;
    protected ISIDMapper ISIDMap;
    protected RecordDescriptor rDesc;
    
    /**
     * Construct a ComponentInfo from DataCollection, FieldIDMapper
     * and ISIDMapper.
     * 
     * @param cds     -- DataSource or DataBlock to be used as Component
     * @param fidmap  -- Map fields from component to composite; null is ok
     *                   and implies an identity mapping
     * @param isidmap -- Map index spaces. null is ok and implies identity.
     */
    public ComponentInfo(Indexable cds, FieldIDMapper fidmap, 
                         ISIDMapper isidmap) 
    {
        this.component = cds;
        this.fieldIDMap = fidmap;
        this.ISIDMap = isidmap;
        this.rDesc = cds.getRecordDescriptor();
    }

    public Indexable getComponent() {
        return this.component;
    }

    public FieldIDMapper getFieldIDMap() {
        return this.fieldIDMap;
    }

    public ISIDMapper getISIDMap() {
        return this.ISIDMap;
    }
    
    public boolean equals(ComponentInfo comp2) 
    {
        return ( comp2.getComponent().equals(component) 
              && ( this.fieldIDMap == null && comp2.fieldIDMap == null 
                    || this.fieldIDMap != null && this.equals( comp2 )
                    || comp2.fieldIDMap != null && comp2.equals( this ))
              && comp2.getISIDMap().equals(ISIDMap) );
    }
    
    /** rename an attribute at position given by index*/
    public void setAttributeName ( int index, String name ) 
    {
        if ( index < 0 || index >= this.rDesc.size() ) 
        {
            new IllegalArgumentError( SDBError.Recoverable, 
                   "Invalid attribute index to rename an attribute. " 
                    + name + "@" + index );
            return;
        }

        AttributeDescriptor[] attribs = new AttributeDescriptor[ this.rDesc.size() ];
        for ( int i=0 ; i< this.rDesc.size(); i++ )
            attribs[i] = this.rDesc.getAttribute(i);
        
        attribs[ index ] = new AttributeDescriptor( attribs[ index ].getStorageType(), name);
        
        this.rDesc = new RecordDescriptor(attribs);        
    }
    
    /** rename all (or part of) the attributes by giving an array of names*/
    public void rename(String[] names)
    {
        int len = this.getRecordDescriptor().size();
        
        for ( int i=0; i<len; i++)
        {
           if (names[i] != null) {
               this.setAttributeName(i,names[i]);           
           }        
        }       
    }
    
    /** return the current recordDescriptor for this componentInfo*/
    public RecordDescriptor getRecordDescriptor ()
    {
        return this.rDesc;
    }
    
    /** If this component contains the given IndexSpaceID, returns true */
    public boolean contains(IndexSpaceID isid) 
    {
        return this.ISIDMap.contains(isid);    
    }

}
