/*
 * DCIterator
 *    This class extends the notion of ISIterator by allowing the
 *    iterator to "know" which DataCollection is the target of the
 *    iteration. I created this for use in the GraniteServer class,
 *    but it will be useful for automatic cache creation.
 *
 * Created on May 9, 2005
 *
 * ------------------- Edit history ----------------------------------
 * 05/09/05 rdb: Derive from ISIterator. Add data collection link.
 */
package edu.unh.sdb.datasource;

/**
 * @author rdb
 *
 */
public class DCIterator extends ISIterator
{
   DataCollection baseDC = null;

   /**
    * @param db
    */
   public DCIterator( DataCollection dc, ISBounds db )
   {
      super( db );
      this.baseDC = dc;
   }

   /**
    * @param db
    * @param ord
    */
   public DCIterator( DataCollection dc, ISBounds db, AxisOrdering ord )
   {
      super( db, ord );
      this.baseDC = dc;
   }

   /**
    * @param db
    * @param steps
    */
   public DCIterator( DataCollection dc, ISBounds db, int[] steps )
   {
      super( db, steps );
      this.baseDC = dc;
   }

   /**
    * @param db
    * @param ord
    * @param steps
    */
   public DCIterator( DataCollection dc, ISBounds db, AxisOrdering ord, int[] steps )
   {
      super( db, ord, steps );
      this.baseDC = dc;
   }
   //+++++++++++++++++ Accessor methods ++++++++++++++++++++++++++++++++++++
   /**
    * Get the DataCollection associated with this Iterator
    */
   public DataCollection getDataCollection()
   {
      return this.baseDC;
   }

   /**
    * Set the DataCollection associated with this Iterator
    */
   public void getDataCollection( DataCollection newDC )
   {
      this.baseDC = newDC;
   }
}
