/*
    DSApproximator.java
    Copyright 2004 University of New Hampshire
    Durham, NH 03824
    Programmer: LI YE
*/
                                                                                
/**
  * This DSApproximator class is to interpolate the given data block to the
  * expected data block.
  */
//--------------------Edit history (since 01/03/05) ----------------------
//
// 11/13/05 rdb: Removed all uses of CompositeBlock
// 10/21/05 rdb: Deleted unused variables to avoid compiler warnings
//               Deleted approximateDatum(...).
//               Revised approximateDatum( ..., RecordSpec ) so it can take null RS
//                    and replaced call to non-RecordSpec version with this one.
//                    All the code was essentially replicated before. This method
//                    should be replaced anyway since we shouldn't be using arrays
//                    of Datums.
// 01/06/05 rdb: create Datum objects from RecordDescriptor
// 01/03/05 rdb Changed (private) approximateData method. It had done interpolation
//             using ints and produced a FloatDatum from the int results. I can't
//             imagine that is correct. I changed so it uses float arithmetic
//
//--------------------------------------------------------------------------

package edu.unh.sdb.datasource;

//import java.io.*;
import edu.unh.sdb.common.*;
import edu.unh.sdb.lattice.*;


public class DSApproximator {
   int[] srcShape; 
   int[] dstShape; 
   int[] gaps; 
   int[] molds; 
   boolean[] moldsflag;
   int[] dimensions; 
   float[] srcRates;
   float[] dstRates;
   int dim;
   ISBounds srcBounds;
   ISBounds dstBounds; 
   Datum[] data;
   IndexSpaceID[] vertices;
   //rdb CompositeBlock dstblock;
   DataBlock dstblock;

   /** Constructor without arguments */
   public DSApproximator()
   { }

   public Indexable approximate( Indexable src, ISBounds dstBounds )
   { 
      this.dim = src.dim();
      if ( this.dim != dstBounds.dim() ) 
      {
         new IllegalArgumentError( SDBError.Fatal, 
                    "The dimensions of srcBounds and dstBounds do not match.");
         return null;
      }
      ISBounds srcBounds = src.getBounds();
      int dB = dstBounds.volume();
      int sB = srcBounds.volume();
//       System.out.println("Starting of equal");  //----
      //if (dstBounds.volume() == src.getBounds().volume()) {
      if ( dB == sB ) 
      {
        /* this.dstblock = new CompositeBlock(dstBounds, 
                                         src.getRecordDescriptor()); 
         src.subblock(this.dstblock,srcBounds,this.dstblock.getBounds());
         */
         //return this.dstblock;
         return src;
      }

      this.data = new Datum[ 1<<this.dim ];
 //      System.out.println("Starting of interpolate");  //----
      if ( dB > sB )
         return interpolate( src, dstBounds );

  //     System.out.println("Starting of discard");  //----
      if (dB < sB)
         return discard( src, dstBounds );

      return null;
   }
  
   private Indexable discard( Indexable src, ISBounds approximatedBounds )
   { 
    //System.out.println("start discard"); //---------
      int  j, k, d;
      ISBounds sBounds;
      d = src.dim();
      sBounds = src.copyBounds();
      //approximatedBounds.moveToOrigin();
      //System.out.println(((CompositeBlock)src).toString()); //--------
      //rdb: Don't use CompositeBlock
      //rdb this.dstblock = new CompositeBlock(approximatedBounds, 
      //rdb                                    src.getRecordDescriptor());
      this.dstblock = src.getRecordDescriptor().createDataBlock( approximatedBounds );
      //System.out.println("dstblock bounds: "+this.dstblock.getBounds().toString()); //--------
      int[] steps = new int[d];
      for ( int i = 0; i < d; i++ ) 
      {
         j = sBounds.copyUpper().getCoord(i) - sBounds.copyLower().getCoord(i);
         k = approximatedBounds.copyUpper().getCoord( i ) 
                                - approximatedBounds.copyLower().getCoord( i );
         if ( k == 0 )
            steps[i] = j+1;
         else
            steps[i] = j/k;
         //System.out.println("steps: "+steps[i]); //--------
      }
      FloatDatum data = new FloatDatum( 1 ); 
      ISIterator dstIter = new ISIterator( this.dstblock.getBounds() );
      ISIterator srcIter = new ISIterator( sBounds, steps );
      //for (srcIter.init(), dstIter.init(); srcIter.valid(); srcIter.next()){
      for ( srcIter.init(), dstIter.init(); dstIter.valid(); dstIter.next() ) 
      {
         //data = src.datum(srcIter);
         src.datum( data, srcIter );
         this.dstblock.setDatum( dstIter, data );
         srcIter.next();
      }
      return this.dstblock;
   } 


   /* ***********************************************************
    * rdb: Comment away unused function 
    * help function for the discard function. change ISBounds into GBounds
    * 
   private GBounds getGBounds(ISBounds bounds)
   {
      int d = bounds.dim();
      int[] lower = bounds.copyLower().refCoords();
      int[] upper = bounds.copyUpper().refCoords();
      float[] flower = new float[d];
      float[] fupper = new float[d];
      for ( int i = 0; i < d; i++ ) 
      {
         flower[i] = (float)lower[i];
         fupper[i] = (float)upper[i];
      }
     
      return new GBounds( new Point(flower), new Point(fupper) );
   }
   /**************************************************************/
 
   private Indexable interpolate( Indexable src, ISBounds approximatedBounds )
   {   
    //System.out.println("start interpolate");
      int length;
      //ISBounds sBounds, dBounds;
      ISBoundsIterator srcIter, dstIter;
      ISIterator iter;
      int[] dstUpper, dUpper;
      this.init( src.copyBounds(), approximatedBounds );
      //System.out.println("datablock for interpolate");
      //System.out.println(src.toString());
      //rdb this.dstblock = new CompositeBlock(this.dstBounds, 
      //rdb                                    src.getRecordDescriptor()); 
      this.dstblock = src.getRecordDescriptor().createDataBlock( this.dstBounds );
      
      srcIter = new ISBoundsIterator( this.srcBounds, this.srcShape, this.gaps );
      dstIter = new ISBoundsIterator( this.dstBounds, this.dstShape, this.gaps );
      // System.out.println("srcbounds: " + srcIter.toString());
      //System.out.println("dstbounds: " + dstIter.toString());
      dstUpper = this.dstBounds.copyUpper().refCoords();
      for ( srcIter.init(), dstIter.init(); srcIter.valid(); srcIter.next() ) 
      {
         //sBounds = srcIter.copyBounds();
         //dBounds = dstIter.copyBounds();
         //dUpper = dBounds.getUpper().getCoords();
         dUpper = dstIter.copyUpper().refCoords();
         for ( int i = this.dim-1; i >= 0; i--) 
         {
            if (( this.molds[i] == 1 ) && ( dUpper[i] == dstUpper[i] ))
            {
               // System.out.println("check molds");
               dstIter.next();
               //dBounds = dstIter.copyBounds();
            } 
            //rdb 12/1/05: ?????? following line doesn't make sense.
            //  There is no loop!
            break;
         }
        
         //System.out.println("sbounds: " + sBounds.toString());
         //System.out.println("dbounds: " + dBounds.toString());
         this.dimensions = dstIter.getSpaceDimensionsArray();
         //if (sBounds.volume() == this.data.length){
         if ( srcIter.volume() == this.data.length )
         {
            /*get IndexSpaceID and data of the corners*/
            //this.vertices = this.getCornerIndexSpaceID(dBounds);
            this.vertices = this.getCornerIndexSpaceID( dstIter );
            //this.getCornerData(src, sBounds);
            this.getCornerData( src, srcIter );
            length = this.vertices.length;
            //System.out.println("dstblock bounds: " + dstblock.getBounds().toString());
            /*insert corner data*/
            for ( int i = 0; i < length; i++ ) 
            {
               dstblock.setDatum( this.vertices[i], this.data[i] );
            } 
            /*insert interpolated points*/
            //iter = new ISIterator(dBounds);
            iter = new ISIterator( dstIter );
            for ( iter.init(); iter.valid(); iter.next() ) 
            {
               //if (!checkCorners(dBounds, iter)) {
               if ( !checkCorners( dstIter, iter )) 
               {
                  Datum value = this.approximateDatum( this.data, this.vertices,
                                 this.getPoint(iter), this.dimensions, null );
                  this.dstblock.setDatum( iter, value ); 
               }
            }
         }
         dstIter.next();
      } 
    //System.out.println("dstblock before subinterp");
    //System.out.println(dstblock.toString());
      for ( int i = 0; i < this.dim; i++)
      {
         if ( moldsflag[i] )  
         {
            int coord = this.dstblock.getBounds().copyUpper().getCoord( i );
            subinterp( i, coord );
         }
      }
      return this.dstblock; 
   }

   /* help function for the interpolate function */
   private void subinterp( int axis, int coord )
   {
      int d, size;
      int[] index1, index2, index;
      float[] values;
      FloatDatum data1, data2;
      IntegerIndexSpaceID id1, id2;
      ISBounds bd = this.dstblock.getBounds().slice( axis, coord );
      d = bd.dim();
     // System.out.println("subinterp bounds: " + bd.toString()); //----
      //System.out.println("subinterp bounds volume: " + bd.volume()); //----
      index1 = new int[d];
      index2 = new int[d];
      ISIterator iter = new ISIterator( bd );
      for ( iter.init(); iter.valid(); iter.next() ) 
      { 
         //index = (int[])iter.getCoords().clone(); 
         index = iter.refCoords(); 
         for ( int i = 0; i < d; i++ ) 
         {
            if ( i == axis ) 
            {
               index1[i] = coord-1;
               index2[i] = coord-2;
            }
            else 
            {
               index1[i] = index[i];
               index2[i] = index[i];
            }
         }
         id1 = new IntegerIndexSpaceID( index1 ); 
         id2 = new IntegerIndexSpaceID( index2 ); 
       // System.out.println("id1: " + id1.toString());
        //System.out.println("id2: " + id2.toString());
         size = this.dstblock.getNumAttributes();
         values = new float[ size ];
         data1 = new FloatDatum( values );
         data2 = new FloatDatum( values );
         this.dstblock.datum( data1, id1 );
         this.dstblock.datum( data2, id2 );
        //System.out.println("size: " + size);
        //System.out.println("data1: " + data1.toString());
        //System.out.println("data2: " + data2.toString());
         for ( int i = 0; i < size; i++ ) 
         {
            values[i] = 2*data1.getFloat( i ) - data2.getFloat( i );
         }
         Datum newDatum = this.dstblock.getRecordDescriptor().createDatum();
         newDatum.setFloats( values );
         this.dstblock.setDatum( iter.copyID(), newDatum );
      }
   }  


   /*help function for the interpolate function. change IndexSpaceID 
     into Point*/
   private Point getPoint( IndexSpaceID id )
   {
      int[] index = id.refCoords();
      float[] point = new float[ index.length ];
      for ( int i = 0; i < index.length; i++ ) 
      {
         point[i] = index[i];
      }
      return new Point( point );
   } 
      
   /*help function for the discard and interpolate functions*/
   private boolean checkCorners( ISBounds bounds, IndexSpaceID id )
   {
      int len;
      IndexSpaceID[] corners = this.getCornerIndexSpaceID(bounds);
      len = corners.length;
      for ( int i = 0; i < len; i++ )  
      {
         if ( corners[i].equals( id ) )
            return true;
      }
      return false;
   }

   private void init( ISBounds srcBounds, ISBounds approximatedBounds )
   {
      int[] srcLower, srcUpper, dstLower, dstUpper;
      IndexSpaceID offset;
      this.srcBounds = (ISBounds)srcBounds.clone();
      this.dstBounds = (ISBounds)approximatedBounds.clone();
      offset = this.dstBounds.copyLower();
      offset.negate();
      this.dstBounds.move( offset ); 
      srcLower = srcBounds.copyLower().refCoords();
      dstLower = dstBounds.copyLower().refCoords();
      srcUpper = srcBounds.copyUpper().refCoords();
      dstUpper = dstBounds.copyUpper().refCoords();
      this.srcShape = new int[ this.dim ];
      this.dstShape = new int[ this.dim ];
      this.gaps = new int[ this.dim ];
      this.molds = new int[ this.dim ];
      this.moldsflag = new boolean[ this.dim ];
      for ( int i = 0; i < this.dim; i++ ) 
      {
         this.dstShape[i] = ( dstUpper[i] - dstLower[i] ) 
                           / ( srcUpper[i] - srcLower[i]) + 1;
         this.molds[i] = ( dstUpper[i] - dstLower[i]) 
                           % ( srcUpper[i] - srcLower[i] );
    //System.out.println("molds: " + this.molds[i]);
         if ( this.molds[i] == 1 )
            this.moldsflag[i] = true;
         else
            this.moldsflag[i] = false;

         this.srcShape[i] = 2;
         this.gaps[i] = -1;
      }
   } 

   private void getCornerData( Indexable src, ISBounds sBounds )
   {
      int len;
      IndexSpaceID[] corners = this.getCornerIndexSpaceID( sBounds );
      len = corners.length;
      this.data = new Datum[ len ];
      for ( int i = 0; i < len; i++ ) 
      {
         data[i] = src.getRecordDescriptor().createDatum();
         src.datum( data[i], corners[i] );
      }
      /*** rdb moved the src.datum call from its own loop to loop above
      for ( int i = 0; i < len; i++ ) 
      {
         src.datum( data[i], corners[i] );
        //data[i] = (FloatDatum)(src.datum(corners[i])).clone();
       //System.out.println("data in getCornerData: "+data[i].toString());
      }
      /****/
   } 

   /**
    * Find corners of a bounds.
    * 
    * rdb: 12/1/05 Simplified code (without testing); really should be completely
    * rewritten. Should generate the corners directly without looping
    * through all isids in the bounds. It's tricky, however.
    * 
    * @param bounds
    * @return array of corner positions.
    */
   private IndexSpaceID[] getCornerIndexSpaceID( ISBounds bounds )
   {
      int j;
      boolean isCorner;
      IndexSpaceID[] corners = new IndexSpaceID[ 1 << this.dim ];
      int[] low, up, point;
      low = bounds.copyLower().refCoords();
      up  = bounds.copyUpper().refCoords();
      
      ISIterator iter = new ISIterator(bounds);
      j = 0;
      for ( iter.init(); iter.valid(); iter.next() ) 
      {
         point = iter.refCoords(); 
         isCorner = true;
         for ( int i = 0; i < this.dim && isCorner; i++ ) 
         {
            if (( point[i] != low[i] ) && ( point[i] != up[i] ))
               isCorner = false;
         }

         if ( isCorner ) 
         {
           corners[j] = iter.copyID(); 
           j++;
         }
      }

      return corners;
   }
 
   private float approximateFloat( float[] data, IndexSpaceID[] vertices,
                                 Point point, int[] dimensions )
   {
      float product;  //rdb changed from int
      int[] vertexCoords;
      int numVertices = vertices.length;
      float result = 0.0f;
      float[] pointCoords = new float[ vertices[0].dim() ];
            
      for ( int i = 0; i < pointCoords.length; i++)
         pointCoords[i] = point.getCoord(i);
      for ( int i = 0;i < numVertices; i++)
      {
         vertexCoords = vertices[i].refCoords();
         product = 1;
         
         for ( int j = 0; j < vertexCoords.length; j++ )
         {
            product *= 1 - Math.abs( pointCoords[j] - vertexCoords[j] )
                                               /  dimensions[j];
         }
         
         result += product * data[i];
      }
      return result;     
   }
   
   private Datum approximateDatum( Datum[] data, IndexSpaceID[] vertices,
                      Point point, int[] dimensions, RecordSpec s )
   {
      float product;  //rdb changed from int
      int[] vertexCoords;
      int numVertices = vertices.length;
      int numAttributes = data[0].size();
      float resultArray[] = new float[ numAttributes ]; // rdb changed from float
      float[] pointCoords = new float[ vertices[0].dim() ];

      int [] fieldNumbers = null;
      if ( s != null )
          fieldNumbers = s.getFieldNumbers();
      else
      {
         fieldNumbers = new int[ numAttributes ];
         for ( int i = 0; i < numAttributes; i++ )
            fieldNumbers[ i ] = i;
      }

      for ( int i = 0; i < pointCoords.length; i++)
         pointCoords[i] = point.getCoord(i);
      for ( int i = 0;i < numVertices; i++)
      {
         vertexCoords = vertices[i].refCoords();
         product = 1;
            
         for ( int j = 0; j < vertexCoords.length; j++ )
         {
            product *= 1 - Math.abs( pointCoords[j] - vertexCoords[j] )
                                                    / dimensions[j];
         }
            
         for ( int k=0; k < numAttributes; k++ )
         {
            resultArray[k] += product * data[i].getFloat( fieldNumbers[k] );
         }
      }
        
      //return new FloatDatum(resultArray);
      Datum result = (Datum) data[0].clone();
      result.setFloats( resultArray );
      return result;     
   }
}
