
/* 
    Indexable.java
    Copyright 2000 University of New Hampshire
    Durham, NH 03824
    
    ---------------------- Edit history as of 01/25/04 ---------------------
    01/25/04 rdb: Added additional subblock methods to provide more consistent
                     options. Most are implemented in DataCollection directly.
*/

package edu.unh.sdb.datasource;

import edu.unh.sdb.common.*;
//import java.io.*;

/**
 * An interface for classes that can be thought of as an index space. Such classes 
 * can return a datum corresponding to an IndexSpaceID. Currently, DataSource and
 * DataBlock are the only indexable classes.
 */


public interface Indexable {

    public  Datum datum(IndexSpaceID did);
    public  Datum datum(IndexSpaceID did, RecordSpec spec);
    
    
    /** Return by reference the Datum corresponding to the given 
     *  IndexSpaceID.
     */
    public void datum(Datum dtm, IndexSpaceID did);
    
    
    /** Get the Datum and fill its values in the passed Datum at positions
     *  specified by the corresponding FieldIDMapper.
     */
    public void datum(Datum dtm, IndexSpaceID did, FieldIDMapper mp);
    
    
    /** Get the Datum and fill its values in the passed Datum at positions
     *  specified by the corresponding RecordSpec.
     */
    public void datum(Datum dtm, IndexSpaceID did, RecordSpec recSpec);


    /** return a subblock defined by the bounds */
    public  DataBlock subblock(ISBounds db);
    
    /** return a subbloc defined by the bounds and recordSpec */
    public  DataBlock subblock(ISBounds db, RecordSpec rs );
    
    /** return a subbloc defined by the bounds and a FieldIDMapper */
    public  DataBlock subblock(ISBounds db, FieldIDMapper fmap );
        
    public void subblock( DataBlock theBlock, ISBounds dstBounds, ISBounds srcBounds);
    public void subblock( DataBlock theBlock, ISBounds dstBounds, ISBounds srcBounds, FieldIDMapper mp);
	public void subblock( DataBlock theBlock, ISBounds dstBounds, ISBounds srcBounds, RecordSpec spec);

    public int dim();
    public ISBounds copyBounds();
    public ISBounds getBounds();
    public int getNumAttributes();
    public RecordDescriptor getRecordDescriptor();
    
}