//
//  LRUCacheDataSource.java
//  CacheTest
//
//  Created by Philip Rhodes on Sat Feb 07 2004.
//  Copyright (c) 2004 University of New Hampshire. All rights reserved.
//



package edu.unh.sdb.datasource;

//import edu.unh.sdb.common.*;
//import java.io.*;


/** 
 *  The LRUCacheDataSource provides cache service for an underlying datasource. 
 */
public class LRUCacheDataSource extends BlockCacheDataSource{

    protected int numCacheBlocks;

    /** Create a LRUCacheDataSource with a single CacheBlock connected to the ChunkedDataSource
      * argument. The CacheBlock dimensions match the datasource chunk size. The memory 
      * for the cache itself is not allocated until this datasource is activated. 
      */
	public LRUCacheDataSource(String name, ChunkedDataSource ds, int numCacheBlocks){
	
		super(name, ds, ds.getBounds(), ds.getChunkDimensions());
        
        this.numCacheBlocks = numCacheBlocks;
	}

    /** Create a LRUCacheDataSource with a single CacheBlock connected to the DataSource
      * argument. The CacheBlock dimensions are specified by the last argument. The memory 
      * for the cache itself is not allocated until this datasource is activated. 
      */
	public LRUCacheDataSource(String name, DataSource ds, IndexSpaceID blockSizes, int numCacheBlocks){
	
		super(name, ds, ds.getBounds(), blockSizes);
        
        this.numCacheBlocks = numCacheBlocks;
	}



	/** Calls activate() for the underlying datasource, and creates the actual cache,
      * which allocates memory for the CacheBlocks.
	  */	
	public void activate(){
	
	    super.activate();
        
        theCache=new LRUCache(this.theSource, this.iterationSpace, this.blockDimensions, ((PhysicalDataSource)this.theSource).getAxisOrdering(), this.numCacheBlocks);
        
    }
    
    public void setIterationBounds(ISBounds bounds){
    
      this.iterationSpace = bounds.copy();
    }
	
	
	/** For debugging. Runs basic tests on the LRUCacheDataSource class. */
    static public void main(String[] args){

//        System.out.println("\n******************************\n***LRUCacheDataSource Tests***\n******************************");
//        
//        AttributeDescriptor[] attrDescriptors=new AttributeDescriptor[]{
//                                                    new AttributeDescriptor(StorageType.Float,"Carbon"),
//                                                    new AttributeDescriptor(StorageType.Float,"Nitrogen"),
//                                                    new AttributeDescriptor(StorageType.Float,"Oxygen"),
//                                                    new AttributeDescriptor(StorageType.Float,"Bogosity")
//                                                 };
//                                                 
//        RecordDescriptor recordDesc=new RecordDescriptor(attrDescriptors); 
// 
//        IndexSpaceID dimensions;
//        AxisOrdering axisOrdering;
//        ISBounds bounds;
//        BinaryFileDataSource ds1;
//        LRUCacheDataSource cds;
//        
//        int i,j,k;
//        Datum d;
//        int numAttributes;
//
//       
//
//        // One D Test
//        System.out.println("\n\n***One Dimensional Test***\n");
//        
//        axisOrdering=new AxisOrdering(1);
//        bounds=new ISBounds(new IntegerIndexSpaceID(0),new IntegerIndexSpaceID(63));
//    
//        IndexSpaceID blockSizes=new IntegerIndexSpaceID(8);
//        IndexSpaceID numBlocks=new IntegerIndexSpaceID(2);
//        
//        ds1=new PointOrderBFDS("foo",recordDesc, bounds,axisOrdering,"/home/graphics/sdb/testdata/file.6.4.bin", ByteOrder.BIG_ENDIAN );
//        
//        cds=new LRUCacheDataSource("bar",ds1,blockSizes, 10);
//        
//        cds.activate();
//        
//        
//    
//        System.out.println("ds1.datum(<0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0)));    
//        System.out.println("ds1.datum(<0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0)));    
//
//        System.out.println("ds1.datum(<7>) should be {107.1, 107.2, 107.3, 107.4}-->"+cds.datum(new IntegerIndexSpaceID(7)));    
//        System.out.println("ds1.datum(<8>) should be {108.1, 108.2, 108.3, 108.4}-->"+cds.datum(new IntegerIndexSpaceID(8)));    
//        
//        System.out.println("ds1.datum(<16>) should be {116.1, 116.2, 116.3, 116.4}-->"+cds.datum(new IntegerIndexSpaceID(16)));    
//
//        System.out.println("ds1.datum(<62>) should be {162.1,162.2,162.3,162.4}-->"+cds.datum(new IntegerIndexSpaceID(62)));    
//        System.out.println("ds1.datum(<54>) should be {154.1,154.2,154.3,154.4}-->"+cds.datum(new IntegerIndexSpaceID(54)));    
//
//        System.out.println();
//        System.out.println("       Round Two");
//
//
//
//        System.out.println("ds1.datum(<0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0)));    
//        System.out.println("ds1.datum(<0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0)));    
//
//        System.out.println("ds1.datum(<7>) should be {107.1, 107.2, 107.3, 107.4}-->"+cds.datum(new IntegerIndexSpaceID(7)));    
//        System.out.println("ds1.datum(<8>) should be {108.1, 108.2, 108.3, 108.4}-->"+cds.datum(new IntegerIndexSpaceID(8)));    
//        
//        System.out.println("ds1.datum(<16>) should be {116.1, 116.2, 116.3, 116.4}-->"+cds.datum(new IntegerIndexSpaceID(16)));    
//
//        System.out.println("ds1.datum(<62>) should be {162.1,162.2,162.3,162.4}-->"+cds.datum(new IntegerIndexSpaceID(62)));    
//        System.out.println("ds1.datum(<54>) should be {154.1,154.2,154.3,154.4}-->"+cds.datum(new IntegerIndexSpaceID(54)));    
//   
//        // Two-D  Tests
//        System.out.println();
//        System.out.println("Two Dimensional Tests");
//        
//        axisOrdering=new AxisOrdering(2);
//        bounds=new ISBounds(new IntegerIndexSpaceID(0,0),new IntegerIndexSpaceID(7,7));
//    
//        blockSizes=new IntegerIndexSpaceID(4,4); //blocks are 4x4
//        numBlocks=new IntegerIndexSpaceID(2,2);
//        
//        ds1=new PointOrderBFDS("foo",recordDesc, bounds,axisOrdering,"/home/graphics/sdb/testdata/file.6.4.bin", ByteOrder.BIG_ENDIAN );
//        cds=new LRUCacheDataSource("bar",ds1,blockSizes, 10);
//        
//        cds.activate();
//
//        System.out.println("ds1.datum(<0,0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0,0)));    
//        System.out.println("ds1.datum(<0,0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0,0)));  
//
//        System.out.println("ds1.datum(<0,3>) should be {103.1, 103.2, 103.3, 103.4}-->"+cds.datum(new IntegerIndexSpaceID(0,3)));    
//        System.out.println("ds1.datum(<0,4>) should be {104.1, 104.2, 104.3, 104.4}-->"+cds.datum(new IntegerIndexSpaceID(0,4)));    
//
//        System.out.println("ds1.datum(<0,7>) should be {107.1, 107.2, 107.3, 107.4}-->"+cds.datum(new IntegerIndexSpaceID(0,7)));    
//        System.out.println("ds1.datum(<1,0>) should be {108.1, 108.2, 108.3, 108.4}-->"+cds.datum(new IntegerIndexSpaceID(1,0)));    
//
//        System.out.println("ds1.datum(<2,0>) should be {116.1, 116.2, 116.3, 116.4}-->"+cds.datum(new IntegerIndexSpaceID(2,0)));    
//        System.out.println("ds1.datum(<3,0>) should be {124.1, 124.2, 124.3, 124.4}-->"+cds.datum(new IntegerIndexSpaceID(3,0)));
//        System.out.println("ds1.datum(<4,0>) should be {132.1, 132.2, 132.3, 132.4}-->"+cds.datum(new IntegerIndexSpaceID(4,0)));
//        
//        System.out.println("ds1.datum(<7,6>) should be {162.1,162.2,162.3,162.4}-->"+cds.datum(new IntegerIndexSpaceID(7,6)));    
//        System.out.println("ds1.datum(<6,6>) should be {154.1,154.2,154.3,154.4}-->"+cds.datum(new IntegerIndexSpaceID(6,6)));    
//   
//        System.out.println();
//        System.out.println("       Round Two");
//
//
//
//        System.out.println("ds1.datum(<0,0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0,0)));    
//        System.out.println("ds1.datum(<0,0>) should be {100.1, 100.2, 100.3, 100.4}-->"+cds.datum(new IntegerIndexSpaceID(0,0)));  
//
//        System.out.println("ds1.datum(<0,3>) should be {103.1, 103.2, 103.3, 103.4}-->"+cds.datum(new IntegerIndexSpaceID(0,3)));    
//        System.out.println("ds1.datum(<0,4>) should be {104.1, 104.2, 104.3, 104.4}-->"+cds.datum(new IntegerIndexSpaceID(0,4)));    
//
//        System.out.println("ds1.datum(<0,7>) should be {107.1, 107.2, 107.3, 107.4}-->"+cds.datum(new IntegerIndexSpaceID(0,7)));    
//        System.out.println("ds1.datum(<1,0>) should be {108.1, 108.2, 108.3, 108.4}-->"+cds.datum(new IntegerIndexSpaceID(1,0)));    
//
//        System.out.println("ds1.datum(<2,0>) should be {116.1, 116.2, 116.3, 116.4}-->"+cds.datum(new IntegerIndexSpaceID(2,0)));    
//        System.out.println("ds1.datum(<3,0>) should be {124.1, 124.2, 124.3, 124.4}-->"+cds.datum(new IntegerIndexSpaceID(3,0)));
//        System.out.println("ds1.datum(<4,0>) should be {132.1, 132.2, 132.3, 132.4}-->"+cds.datum(new IntegerIndexSpaceID(4,0)));
//        
//        System.out.println("ds1.datum(<7,6>) should be {162.1,162.2,162.3,162.4}-->"+cds.datum(new IntegerIndexSpaceID(7,6)));    
//        System.out.println("ds1.datum(<6,6>) should be {154.1,154.2,154.3,154.4}-->"+cds.datum(new IntegerIndexSpaceID(6,6)));    
   
    }
		
}