/* 
	Log.java
	Copyright 2000 University of New Hampshire
	Durham, NH 03824
    
  ---------------------- Edit history as of 10/21/05 -------------------------
  10/21/05 rdb: Removed unused variables
  ----------------------------------------------------------------------------

*/

package edu.unh.sdb.datasource;
import java.io.*;
import java.util.*;

/** This class provides basic logging functionality for errors and 
    debugging. It is meant to be used statically, that is, no Log
    object is ever instantiated.
 */
public abstract class Log{
    public static final int LogToFile=1;
    public static final int LogToStdErr=2;
    
    static private FileWriter log;
    static private int policy=LogToStdErr;
    
    
    /** Initialize the log by specifying a filename (or null), and 
        also a logging policy. The logging policy is formed from the
        bitwise OR of one or more of the policy specifiers: LogToFile
        and LogToStdErr.
        
        The log file is actually appended to. Prior log informtation
        will be preserved. 
        
        This function  prints a banner to the log that contains
        the current date, thereby distinguishing the log data for 
        different runs. 
    **/
	public static void init(String filename,int policy){
	    
	    Log.policy=policy;
	    
	    if((policy & LogToFile) !=0){
	        try{
	            log=new FileWriter(filename,true); //append
	        } catch (IOException e){
	           System.err.println(e);
	        }
	    }
	    
        log( "*********************************************************" );
        log( "  *************"+new Date()+"************" );
        log( "*********************************************************" );

	}

    /** Write a message to the log. */
    public static void log(String message){
    
        try{
            if((policy & LogToFile) !=0){
                log.write(message+"\r\n");
                log.flush();
            }
        } catch (IOException e){
            System.err.println("Log: Couldn't log error to file.");
        }
        
        if((policy & LogToStdErr) !=0){
            System.err.println(message);
        }
    }
} 