//
//  ObliquePlaneIterator.java
//  ObliquePlaneIteratorTest
//
//  Created by Philip Rhodes on 3/21/05.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//
//------------------- Edit history --------------------------------------
// 10/21/05 rdb: removed unused variables
//-----------------------------------------------------------------------

package edu.unh.sdb.datasource;

import edu.unh.sdb.common.*;
//import java.util.*;


public class ObliquePlaneIterator extends DataBlock implements ObliquePlaneIteratorInterface  {

   protected BasicBlock theBlock;
   protected DataSource theSource;
   protected int spaceDimensionality;

   protected int currentStep=0;  // current step of the iteration
   protected int numSteps;       // total number of steps in the iteration
   protected boolean valid;      // set to false at end of iteration

   protected double[] currentPoint, // currentPoint is the 3D point at the center of the plane used during data access,
                     startPoint,    // startpoint is its initial value
                     prefetchPoint; // currentPoint is the 3D point at the center of the plane used during prefetching,


   protected float[] normal;
   protected double[] shiftTotals;
   protected int projectionAxis;               // axis orthogonal to the principal plane we iterate over
   
   
   protected ISIterator planeIterator;         // iterates over a subset of a principal plane during data access
   protected ISBounds planeBounds;             // bounds for planeIterator
   protected ISBounds startBounds;             // Initial value for planeBounds
   
   
   protected ISBoundsIterator fetchIterator;    // iterates over a subset of a principal plane for fetching
   protected ISBounds fetchBounds;                 // bounds for planeIterator
   
   
   
   protected int [] blockDimensionsArray;

   protected PlaneMapper planemapper;          // maps 2 coords to a third coord via the plane equation
   protected int plane0;                        // The 3D axis that maps to plane axis 0
   protected int plane1;                        // The 3D axis that maps to plane axis 1
   
   protected RegularISPartitioning planePartitioning;
   protected ISBounds dilatedFetchBounds;
   
   protected IntegerIndexSpaceID spaceLocation;          // scratch space, used to index the DataSource
   protected IntegerIndexSpaceID mainSpaceLocation;          // scratch space, used to index the DataSource by main thread
   protected IntegerIndexSpaceID blockLocation;          // scratch space, used to index the DataBlock
   protected IntegerIndexSpaceID block_plane_offset;     // scratch space, represents the delta between planeBounds and the DataBlock index space
   
   protected IndexSpaceID scratchID2D;
   protected IndexSpaceID mainScratchID2D;
   protected IndexSpaceID scratchID3D;
   protected double[] absNormal;
   protected int[] normalSigns;
   protected int[] shiftDirections;             // 1 or -1 indicating the direction that the 2D plane should be shifted in each axis
   protected int prefetchInterval;
   protected boolean threaded = true;
   protected IOThread prefetchThread= null;
   
   
   // scratch space for pdatum
   protected int [] sQBLowerInts;
   protected int [] sQBUpperInts;
   protected int [] datumCoords1;
   protected ISBounds sourceQueryBounds;
   
   protected CacheContext mainCacheContext; 
   
   

   protected RegularISPartitioning cachePartitioning;


   protected Datum datum;

   //protected DataBlockCollection cacheBlockList;
   protected DSBlockStencil stencil;

   //Scratch space for getPlane3Dvertices
   protected IndexSpaceID externalScratchID;
   
   protected boolean SPBlockShape=true; // blocks are shaped according to normal vector
   
   protected boolean stoppedPrefetching=false;

   /** Construct an ObliquePlaneIterator that represents data using a datablock 
    *  with shape specified by the first argument. The initial configuration of
    *  the plane is defined by the starting point and normal vector. The starting
    *  point is conceptually in the middle of the quadrilateral plane subset. The
    *  iteration will continue from this initial configuration in the direction of
    *  the normal vector for the specified number of steps, reading data from 
    *  the specified datasource at each step.
    */   
   public ObliquePlaneIterator(IndexSpaceID planeShape, IndexSpaceID startpoint, float  normal[], int numsteps, DataSource source){
      
      this(planeShape, startpoint, normal, numsteps, true, source);
   }


   /** Construct an ObliquePlaneIterator that represents data using a datablock 
    *  with shape specified by the first argument. The initial configuration of
    *  the plane is defined by the starting point and normal vector. The starting
    *  point is conceptually in the middle of the quadrilateral plane subset. The
    *  iteration will continue from this initial configuration in the direction of
    *  the normal vector for the specified number of steps, reading data from 
    *  the specified datasource at each step.
    */
   public ObliquePlaneIterator(IndexSpaceID planeShape, IndexSpaceID startpoint, float  normal[], int numsteps, boolean threaded, DataSource source){

      super(new ISBounds(planeShape), source.getRecordDescriptor());

      
      this.threaded = threaded;
      theBlock = new BasicBlock(new ISBounds(planeShape), source.getRecordDescriptor());
      theSource = source;
      this.numSteps = numsteps;
      int d=this.spaceDimensionality = this.theSource.dim();

      this.externalScratchID = new IntegerIndexSpaceID(new int[d-1]);

      this.currentPoint = new double[d];
      this.startPoint = new double[d];
      this.prefetchPoint = new double[d];

      this.absNormal = new double[d];
      this.normalSigns = new int[d];
      this.shiftDirections = new int[d - 1];



      this.shiftTotals = new double[d-1];

      this.normal = normal.clone();
      
      setEffectiveZeros(this.normal, numSteps);
      
      this.projectionAxis = findMax(normal); // Choose the principal plane to iterate over.
      reduceByCoord(this.normal, this.projectionAxis);

      int j = 0;
      for(int i=0; i<d; i++){
         // for the plane equation, the startpoint is in the middle of an element
         this.startPoint[i] = startpoint.getCoord(i) + 0.5;  
         this.absNormal[i] = Math.abs(this.normal[i]);
         
         this.normalSigns[i] = 0;
         this.normalSigns[i] = (this.normal[i] > 0) ? 1 : this.normalSigns[i];
         this.normalSigns[i] = (this.normal[i] < 0) ? -1 : this.normalSigns[i];

         if(i != this.projectionAxis){

             this.shiftDirections[j] = ((this.normal[i] > 0) ? 1 : -1);
             j++;
         }
      }


      switch(this.projectionAxis){
         case 0:
            this.planemapper = new PlaneMapper_i();

            this.plane0=1;
            this.plane1=2;
            break;

         case 1:
            this.planemapper = new PlaneMapper_j();

            this.plane0=0;
            this.plane1=2;
            break;

         case 2:
            this.planemapper = new PlaneMapper_k();

            this.plane0=0;
            this.plane1=1;
            break;
      }

      this.startBounds = ISBounds.createCentered(planeShape, startpoint.project(this.projectionAxis));
      this.planeBounds = (ISBounds) startBounds.clone();
      this.fetchBounds = (ISBounds) startBounds.clone();
      
      this.planeIterator = new ISIterator(planeBounds);
      this.fetchIterator = new ISBoundsIterator(planeBounds, new int[] {1,1});
      
      this.spaceLocation = new IntegerIndexSpaceID(new int[d]);
      this.mainSpaceLocation = new IntegerIndexSpaceID(new int[d]);
      this.blockLocation = new IntegerIndexSpaceID(new int[d - 1]);
      this.scratchID2D = new IntegerIndexSpaceID(new int[d - 1]);
      this.mainScratchID2D = new IntegerIndexSpaceID(new int[d - 1]);
      
      this.scratchID3D = new IntegerIndexSpaceID(new int[d]);
      this.block_plane_offset = new IntegerIndexSpaceID(planeBounds.getLowerArray());
      this.datum = this.theSource.createDatum();



      // scratch space for pdatum
      this.sQBLowerInts = new int[d];
      this.sQBUpperInts = new int[d];
      this.datumCoords1 = new int[d];
      this.sourceQueryBounds = new ISBounds(d);
      
      this.mainCacheContext = new CacheContext(d);

      this.stoppedPrefetching=false;


   }


   /** Any component of the normal that is sufficiently small will be set
    *  to zero. A component is sufficiently small if setting it to zero will
    *  make no functional difference to the iteration.
    */
   public static void setEffectiveZeros(float [] normal, int numSteps){
   
      int i, d = normal.length;
      float epsilon = 0.5f/numSteps;
      
      for(i=0; i < d; i++){
      
         normal[i] = (Math.abs(normal[i]) < epsilon) ? 0 : normal[i];      
      }
   
   }


   /** Perform any necessary initialization, and set the iterator value
   *  to the first value in the iteration.
   */
   public void init(){

      int i, d = this.spaceDimensionality;
      //int j=0;
      
      for(i=0; i<d; i++){

         this.currentPoint[i] = this.startPoint[i];
            
//         if(i != this.projectionAxis){
//
//            this.shiftTotals[j] = 0.5;
//            j++;
//         }

      }


      currentStep=0;

      this.planeBounds = (ISBounds) startBounds.clone();
      this.planeIterator.setBounds(planeBounds);
      this.fetchIterator.setBounds(planeBounds);


     if(currentStep < numSteps){

         fillBlock(true);
         valid=true;
     } else {

         valid = false;
     }
     
     this.stoppedPrefetching=false;
   }

    /** Advance the iterator by one step. If the iterator cannot be advanced
     *  to produce a valid value, the value of the iterator is undefined, but
     *  the next() method will return false, as will a subsequent call to
     *  valid().
     */
    public boolean next(){

        int j=0, d = this.spaceDimensionality;
        boolean shift=false;

        if(this.valid) {

            this.currentStep++;

            if(currentStep < numSteps){

                for(int i=0; i<d; i++){

                    // increment 3D point, advancing plane through space
                    this.currentPoint[i] += this.normal[i];

                    // Shift the projection plane, if necessary, for axes other than
                    // the projection axis.
                    if(i != this.projectionAxis){

                        // j is used to index structures relating to the 2D projection of the plane.
                        // Inside this block, i and j should be referring to the same conceptual axis,
                        // even though their values may differ.

                        this.shiftTotals[j] += this.absNormal[i];

                        if(this.shiftTotals[j] >= 1.0){

                            // We'll have to adjust the 2D plane iterator after we're done with the loop
                            shift=true;

                            // move plane bounds by +  or -1
                            this.planeBounds.move(j, shiftDirections[j]);

                            // move block_plane_offset by +  or -1
                            this.block_plane_offset.add(j, shiftDirections[j]);

                            this.shiftTotals[j] -= 1.0;
                        }

                        j++;
                    }
                }


                if(shift) {

                    //System.out.println("Shift!!");
                    this.planeIterator.setBounds(this.planeBounds);
//                    this.fetchIterator.setBounds(this.planeBounds);
                }

                fillBlock(false);

            } else {

                this.valid = false;
            }
        }

        return this.valid;
    }

    /** Return true if the iterator currently has a valid value, false otherwise.*/
    public boolean valid(){

        return this.valid;
    }

   /** Advance the iterator to the last valid value in the iteration. (optional operation)*/
   public void end(){

      int i, d = this.spaceDimensionality;

      this.currentStep = this.numSteps - 1;
      this.planeBounds.set(startBounds);

      int j=0;
      for(i=0; i<d; i++){

         this.currentPoint[i] = this.startPoint[i] + this.normal[i] * this.currentStep;

         if(i != this.projectionAxis){
            // move plane bounds by +  or -1
            this.planeBounds.move(j, (int)(shiftDirections[j] * absNormal[i] *this.currentStep));

            // move block_plane_offset by +  or -1
            this.block_plane_offset.add(j, (int)(shiftDirections[j] * absNormal[i] *this.currentStep));
            j++;
         }
      }


      this.planeIterator.setBounds(this.planeBounds);
      this.fetchIterator.setBounds(this.planeBounds);


      fillBlock(false);

      this.valid = true;
    }



   /** This method assumes that the underlying datasource is a MultiBlockCache.
   *  The method informs the cache of the data it will need in order to fill the
   *  plane block.
   */
   protected void fetch(double [] centerPoint, ISBoundsIterator planeIterator, boolean fetchCurrent, boolean fetchNext){

//      synchronized (this) {

      try{


         MultiBlockCacheDataSource mbcds = (MultiBlockCacheDataSource) this.theSource;
         //int [] tempcoords = new int[3];
         
         this.stencil.clear();

//         int [] coords2D = this.scratchID2D.refCoords();
//         int [] partitionCoords = this.scratchID3D.refCoords();
//         int currentCoord;
//         int min, max;

         // The planeIterator is a 2D iterator that iterates over the 2D planebounds. It's
         // an ISBoundsIterator with shape and anchor matching the cache partitioning. So, for
         // each region of the plane mapping to a single cache block, we get the vertices and
         // map them to a point on the plane in 3D. This 3D point is then mapped to a the
         // cache partitioning. All such 3D points from a single region will have the same
         // coordinates for two axes.  For the third, (the projection axis), the value may
         // differ. We keep track of the min and max values for this third coordinate, and
         // fetch all cache blocks in that range.

         //System.out.println("prefetch bounds = " + this.fetchIterator.getSpaceBounds());
         
         
//         int [] dilatedPlaneDimensions = this.dilatedFetchBounds.getDimensionsArray();
//         int dilatedPlaneVolume = this.dilatedFetchBounds.volume();
         
         

         //int boundaryCode; 
         
         for(planeIterator.init(); planeIterator.valid(); planeIterator.next()){


            handleInteriorPartition(centerPoint, planeIterator, fetchCurrent, fetchNext, mbcds);
//            boundaryCode = planeIterator.boundaryCode();
//         
//            if(boundaryCode == 0x05 || boundaryCode == 0x09 || boundaryCode == 0x06 || boundaryCode == 0x0A)
//               handleInteriorPartition(centerPoint, planeIterator, fetchCurrent, fetchNext, mbcds);   // 4 corners
//            else if( (boundaryCode & 0x01) !=0)
//               handleEdgePartition(centerPoint, planeIterator, fetchCurrent, fetchNext, mbcds, plane0, false); // left edge (lower i)
//            else if ( (boundaryCode & 0x02) !=0)
//               handleEdgePartition(centerPoint, planeIterator, fetchCurrent, fetchNext, mbcds, plane0, true);  // right edge
//            else if ( (boundaryCode & 0x04) !=0)
//               handleEdgePartition(centerPoint, planeIterator, fetchCurrent, fetchNext, mbcds, plane1, false); // top
//            else if ( (boundaryCode & 0x08) !=0)
//               handleEdgePartition(centerPoint, planeIterator, fetchCurrent, fetchNext, mbcds, plane1, true);  // bottom
//            else
//               handleInteriorPartition(centerPoint, planeIterator, fetchCurrent, fetchNext, mbcds);  // interior
               
               
         }  // planeIterator
         

         //System.out.println("End prefetch\n");
         

         mbcds.prefetch(this.stencil);
         
         
      } catch (ClassCastException e){

         // Do nothing.
      }
//      } // synchronized
   }


   void handleEdgePartition(double [] centerPoint, ISBoundsIterator planeIterator, boolean fetchCurrent, boolean fetchNext, MultiBlockCacheDataSource mbcds, int axis, boolean upper){
   
      int [] partitionCoords = this.scratchID3D.refCoords();
      int [] coords2D = this.scratchID2D.refCoords();
      int min, max;
      int currentCoord;
//System.out.println("handleEdgePartition");
      planeIterator.getVertex(0, coords2D);
      this.planemapper.map(this.normal, centerPoint, this.spaceLocation, this.scratchID2D);
      this.scratchID3D = this.cachePartitioning.map(this.spaceLocation, this.scratchID3D);

      max = min = partitionCoords[this.projectionAxis]; // from scratchID3D


      for(int v=1; v<4; v++){

         planeIterator.getVertex(v, coords2D);
         this.planemapper.map(this.normal, centerPoint, this.spaceLocation, this.scratchID2D);
         this.scratchID3D = this.cachePartitioning.map(spaceLocation, this.scratchID3D);
         currentCoord = partitionCoords[this.projectionAxis]; // from scratchID3D


         if (max < currentCoord) {

            max = currentCoord;
         }

         if (min > currentCoord) {

            min = currentCoord;
         }
      }
      
      
      int edgeblock;
      
      if( upper) {
      
         edgeblock = (this.absNormal[axis] > 0) ? max : min;
      } else {
      
         edgeblock = (this.absNormal[axis] > 0) ? min : max;
      }
      
      
      for(int p=min; p<=max; p++){

         partitionCoords[this.projectionAxis] = p; // affects this.scratchID3D  

         if(fetchCurrent){
         
            mbcds.addToStencil(this.stencil, this.scratchID3D); 
         }
         
         
         if(fetchNext){
         
            if(p == edgeblock)
               edgeTemplate(this.scratchID3D, this.projectionAxis, axis, upper, mbcds); //IndexSpaceID cid, int projectionAxis, int axis, boolean upper, MultiBlockCacheDataSource mbcds
            else
               defaultTemplate(this.scratchID3D, mbcds);
         }
      }
   
   
   }


   void handleInteriorPartition(double [] centerPoint, ISBoundsIterator planeIterator, boolean fetchCurrent, boolean fetchNext, MultiBlockCacheDataSource mbcds){
   
      int [] partitionCoords = this.scratchID3D.refCoords();
      int [] coords2D = this.scratchID2D.refCoords();
      int min, max;
      int currentCoord;
//System.out.println("handleInteriorPartition");
      planeIterator.getVertex(0, coords2D);
      this.planemapper.map(this.normal, centerPoint, this.spaceLocation, this.scratchID2D);
      this.scratchID3D = this.cachePartitioning.map(this.spaceLocation, this.scratchID3D);

      max = min = partitionCoords[this.projectionAxis]; // from scratchID3D


      for(int v=1; v<4; v++){

         planeIterator.getVertex(v, coords2D);
         this.planemapper.map(this.normal, centerPoint, this.spaceLocation, this.scratchID2D);
         this.scratchID3D = this.cachePartitioning.map(spaceLocation, this.scratchID3D);
         currentCoord = partitionCoords[this.projectionAxis]; // from scratchID3D


         if (max < currentCoord) {

            max = currentCoord;
         }

         if (min > currentCoord) {

            min = currentCoord;
         }
      }
      
      for(int p=min; p<=max; p++){

         partitionCoords[this.projectionAxis] = p; // affects this.scratchID3D  

         if(fetchCurrent){
         
            mbcds.addToStencil(this.stencil, this.scratchID3D); 
         }
         
         
         if(fetchNext){
         
            defaultTemplate(this.scratchID3D, mbcds);
         }
      }
   
   
   }
   
   
   void defaultTemplate(IndexSpaceID cid, MultiBlockCacheDataSource mbcds){
   
      IndexSpaceID id = cid.copyID();
      int [] coords = id.refCoords();
      
      // 0,0,0
      // 0,0,1
      coords[2] += this.normalSigns[2]; 
      mbcds.addToStencil(this.stencil, id); // 0 misses with 1,1,1

      // 0,1,1
      coords[1] += this.normalSigns[1];
      mbcds.addToStencil(this.stencil, id); // 2 misses
      
      // 0,1,0
      coords[2] -= this.normalSigns[2];
      mbcds.addToStencil(this.stencil, id);
      
      // 1,0,0
      coords[1] -= this.normalSigns[1];
      coords[0] += this.normalSigns[0];
      mbcds.addToStencil(this.stencil, id);

      // 1,0,1
      coords[2] += this.normalSigns[2];
      mbcds.addToStencil(this.stencil, id);
      
      // 1,1,1
      coords[1] += this.normalSigns[1];
      mbcds.addToStencil(this.stencil, id);

      // 1,1,0
      coords[2] -= this.normalSigns[2];
      mbcds.addToStencil(this.stencil, id);
   }


   void edgeTemplate(IndexSpaceID cid, int projectionAxis, int axis, boolean upper, MultiBlockCacheDataSource mbcds){
   
      // use this.cachepartitioning
      IndexSpaceID id = cid.copyID();
      int [] coords = id.refCoords();
      int axisBoundary;
      int pAxisBoundary;
      boolean addExtra = false;

      // need partition boundary for axis
      int remainingAxis = 3 - (projectionAxis + axis);  
      
              
      //Let's handle the case where normalSigns[axis] == 0
      
      if(this.normalSigns[axis] == 0){
      
         coords[projectionAxis] += this.normalSigns[projectionAxis]; 
         mbcds.addToStencil(this.stencil, id);      

         if(this.normalSigns[remainingAxis] != 0){
      
             coords[remainingAxis] += this.normalSigns[remainingAxis];
             mbcds.addToStencil(this.stencil, id);
             
             coords[projectionAxis] -= this.normalSigns[projectionAxis];
             mbcds.addToStencil(this.stencil, id);
         }
         
         return;
      }
      
      // assert this.normalSigns[projectionAxis] != 0


      pAxisBoundary = this.cachePartitioning.getPartitionBoundary(id, projectionAxis, this.normalSigns[projectionAxis] > 0);
      axisBoundary = this.cachePartitioning.getPartitionBoundary(id, axis, this.normalSigns[axis] > 0);
      
      // find axis value at which ray intersects pAxisboundary  
      float a = ( pAxisBoundary - coords[projectionAxis]  ) * normal[axis] / normal[projectionAxis] + coords[axis];


      
      if( (upper && this.normalSigns[axis] > 0 ) || ( !upper && this.normalSigns[axis] < 0) ){
                  
         // Case A
         
         
         if(upper){
         
            addExtra = a >= axisBoundary;
         } else {
         
            addExtra = a <= axisBoundary;
         }
         
         
         
         coords[projectionAxis] += this.normalSigns[projectionAxis]; 
         mbcds.addToStencil(this.stencil, id);
         
         coords[axis] += this.normalSigns[axis]; 
         mbcds.addToStencil(this.stencil, id);
         
         coords[projectionAxis] -= this.normalSigns[projectionAxis]; 
         
         if (addExtra){
         
            mbcds.addToStencil(this.stencil, id);
         }

         if(this.normalSigns[remainingAxis] != 0){
            // do it again, but offset in remainingAxis
            
            // first, back to beginning
            coords[axis] -= this.normalSigns[axis];
            // Now, offset in remainingAxis
            coords[remainingAxis] += this.normalSigns[remainingAxis];

            // Same pattern again

            coords[projectionAxis] += this.normalSigns[projectionAxis]; 
            mbcds.addToStencil(this.stencil, id);
            
            coords[axis] += this.normalSigns[axis]; 
            mbcds.addToStencil(this.stencil, id);
            
            coords[projectionAxis] -= this.normalSigns[projectionAxis]; 
            
            if (addExtra){
            
               mbcds.addToStencil(this.stencil, id);
            }
         }
         
      
      } else {

         // Case B
         

         if(upper){
         
            addExtra = a >= axisBoundary;
         } else {
         
            addExtra = a <= axisBoundary;
         }

         
      
         coords[axis] += this.normalSigns[axis]; 
         mbcds.addToStencil(this.stencil, id);
         
         coords[projectionAxis] += this.normalSigns[projectionAxis]; 
         mbcds.addToStencil(this.stencil, id);
         
         coords[axis] -= this.normalSigns[axis]; 
         
         if (addExtra){
         
            mbcds.addToStencil(this.stencil, id);
         }
         
      
         if(this.normalSigns[remainingAxis] != 0){
            // do it again, but offset in remainingAxis
            
            // first, back to beginning
            coords[projectionAxis] -= this.normalSigns[projectionAxis];
            // Now, offset in remainingAxis
            coords[remainingAxis] += this.normalSigns[remainingAxis];
         
            // Same pattern again 
            coords[axis] += this.normalSigns[axis]; 
            mbcds.addToStencil(this.stencil, id);
            
            coords[projectionAxis] += this.normalSigns[projectionAxis]; 
            mbcds.addToStencil(this.stencil, id);
            
            coords[axis] -= this.normalSigns[axis]; 
            
            if (addExtra){
            
               mbcds.addToStencil(this.stencil, id);
            }
         }
      
      }
   }

   /** This method assumes that the underlying datasource is a MultiBlockCache.
   *  The method informs the cache of the data it will need in order to fill the
   *  plane block.
   */
   protected void fetchWorks(double [] centerPoint, ISBoundsIterator planeIterator, boolean fetchCurrent, boolean fetchNext){

//      synchronized (this) {

      try{


         MultiBlockCacheDataSource mbcds = (MultiBlockCacheDataSource) this.theSource;
         int [] tempcoords = new int[3];
         
         this.stencil.clear();

         int [] coords2D = this.scratchID2D.refCoords();
         int [] partitionCoords = this.scratchID3D.refCoords();
         int currentCoord;
         int min, max;

         // The fetchIterator is a 2D iterator that iterates over the 2D planebounds. It's
         // an ISBoundsIterator with shape and anchor matching the cache partitioning. So, for
         // each region of the plane mapping to a single cache block, we get the vertices and
         // map them to a point on the plane in 3D. This 3D point is then mapped to a the
         // cache partitioning. All such 3D points from a single region will have the same
         // coordinates for two axes.  For the third, (the projection axis), the value may
         // differ. We keep track of the min and max values for this third coordinate, and
         // prefetch all cache blocks in that range.

         //System.out.println("prefetch bounds = " + this.fetchIterator.getSpaceBounds());
         for(planeIterator.init(); planeIterator.valid(); planeIterator.next()){

            //System.out.println("Prefetch: " + this.fetchIterator);


            planeIterator.getVertex(0, coords2D);
            this.planemapper.map(this.normal, centerPoint, this.spaceLocation, this.scratchID2D);
            this.scratchID3D = this.cachePartitioning.map(this.spaceLocation, this.scratchID3D);

            max = min = partitionCoords[this.projectionAxis]; // from scratchID3D


            for(int v=1; v<4; v++){

//               int prevCoord0 = partitionCoords[plane0];
//               int prevCoord2 = partitionCoords[plane1];


               planeIterator.getVertex(v, coords2D);
               this.planemapper.map(this.normal, centerPoint, this.spaceLocation, this.scratchID2D);
               this.scratchID3D = this.cachePartitioning.map(spaceLocation, this.scratchID3D);
               currentCoord = partitionCoords[this.projectionAxis]; // from scratchID3D

//               assert(prevCoord0 ==  partitionCoords[plane0]);
//               assert(prevCoord2 ==  partitionCoords[plane1]);

               if (max < currentCoord) {

                  max = currentCoord;
               }

               if (min > currentCoord) {

                  min = currentCoord;
               }
            }


            for(int i=0; i<3; i++){
            
               tempcoords[i] = partitionCoords[i];
            }
            
            

//            for(int p=min; p<=max+1; p++){
            for(int p=min; p<=max; p++){

               partitionCoords[this.projectionAxis] = p; // affects this.scratchID3D  

               if(fetchCurrent && p<= max){
               
                  mbcds.addToStencil(this.stencil, this.scratchID3D); 
               }
               
               if(fetchNext){
                  for(int i=0; i<4; i++){
                  
                     switch(this.projectionAxis) {
                        case 0:
                        
                           partitionCoords[0] = p ;
                           partitionCoords[1] = (tempcoords[1] + this.normalSigns[1] * (i & 0x1)) ;
                           partitionCoords[2] = (tempcoords[2] + this.normalSigns[2] * ((i >> 1) & 0x1));
                           
//                           System.out.println("Prefetching:"+ Arrays.toString(this.scratchID3D.refCoords()));
                           mbcds.addToStencil(this.stencil, this.scratchID3D);
                           
                           partitionCoords[0] = p + this.normalSigns[0];

//                           System.out.println("Prefetching:"+ Arrays.toString(this.scratchID3D.refCoords()));
                           mbcds.addToStencil(this.stencil, this.scratchID3D);
                           break;

                        case 1:
                        
                           partitionCoords[0] = (tempcoords[0] + this.normalSigns[0] * (i & 0x1)) ;
                           partitionCoords[1] = p;
                           partitionCoords[2] = (tempcoords[2] + this.normalSigns[2] * ((i >> 1) & 0x1));
                           
//                           System.out.println("Prefetching:"+ Arrays.toString(this.scratchID3D.refCoords()));
                           mbcds.addToStencil(this.stencil, this.scratchID3D);
                           
                           partitionCoords[1] = p + this.normalSigns[1];

//                           System.out.println("Prefetching:"+ Arrays.toString(this.scratchID3D.refCoords()));
                           mbcds.addToStencil(this.stencil, this.scratchID3D);
                           break;

                        case 2:
                        
                           partitionCoords[0] = (tempcoords[0] + this.normalSigns[0] * (i & 0x1)) ;
                           partitionCoords[1] = (tempcoords[1] + this.normalSigns[1] * ((i >> 1) & 0x1));
                           partitionCoords[2] = p;
                           
//                           System.out.println("Prefetching:"+ Arrays.toString(this.scratchID3D.refCoords()));
                           mbcds.addToStencil(this.stencil, this.scratchID3D);
                           
                           partitionCoords[2] = p + this.normalSigns[2];

//                           System.out.println("Prefetching:"+ Arrays.toString(this.scratchID3D.refCoords()));
                           mbcds.addToStencil(this.stencil, this.scratchID3D);
                           
                           break;
                     }
                  }
               }
            }

            
           
            
         }  // planeIterator

         //System.out.println("End prefetch\n");
         

         mbcds.prefetch(this.stencil);
         
         
      } catch (ClassCastException e){

         // Do nothing.
      }
//      } // synchronized
   }





   /** This method assumes that the underlying datasource is a MultiBlockCache.
   *  The method informs the cache of the data it will need in order to fill the
   *  plane block.
   */
   protected void fetchNext(int step){
   
//      System.out.println("\nStarting fetchNext()");
      
      setFetchPlane(this.fetchIterator, this.prefetchPoint, step);
   
      fetch(this.prefetchPoint, this.fetchIterator, false, true);
      
//      System.out.println("After fetchNext(), stencil.size()="+stencil.size());
//      System.out.println("\n");
   }

   /** This method assumes that the underlying datasource is a MultiBlockCache.
   *  The method informs the cache of the data it will need in order to fill the
   *  plane block.
   */
   protected void fetchCurrent(int step){
   
//      System.out.println("\nStarting fetchCurrent()");
   
      setFetchPlane(this.fetchIterator, this.prefetchPoint, step);
      
      fetch(this.prefetchPoint, this.fetchIterator, true, false);

//      System.out.println("After fetchCurrent(), stencil.size()="+stencil.size());
//      System.out.println("\n");
   }

   /** This method assumes that the underlying datasource is a MultiBlockCache.
   *  The method informs the cache of the data it will need in order to fill the
   *  plane block.
   */
   protected void fetchCurrentandNext(int step){
   
//      System.out.println("\nStarting fetchCurrentandNext()");
         
      setFetchPlane(this.fetchIterator, this.prefetchPoint, step);
      
      fetch(this.prefetchPoint, this.fetchIterator, true, true);

//      System.out.println("After fetchCurrentandNext(), stencil.size()="+stencil.size());
//      System.out.println("\n");
   }
   




   protected ISBoundsIterator createPrefetchIterator(ISBounds planeBounds, ISBounds iterationSpace, IndexSpaceID blockDimensions) {


      int [] planePartitionDimensions = blockDimensions.project(this.projectionAxis).getCoords();
      IntegerIndexSpaceID planeAnchorPoint = (IntegerIndexSpaceID) iterationSpace.getLower().project(this.projectionAxis);

      return new ISBoundsIterator(planeBounds, planeAnchorPoint, planePartitionDimensions);
      //return new ISBoundsIterator(planeBounds, planePartitionDimensions);
   }


   /** Set the iteration space of the 2D plane iterator to correspond to 
    *  the given 3D iteration step. Also set the point array so it correctly
    *  defines the plane (in conjunction with this.normal) for the given 
    *  step of the 3D iteration. This iterator will be used to build
    *  a stencil for fetching required cache blocks.
    */
   protected void setFetchPlane(ISBoundsIterator planeIterator, double[] point, int step){
   
      this.fetchBounds.set(startBounds);
      
      int d = point.length;
      int[] lower = this.fetchBounds.getLowerArray();
      int[] upper = this.fetchBounds.getUpperArray();
      
      int shiftDistance;
      
      int j=0;
      for(int i=0; i < d; i++){
      
         point[i] =  this.startPoint[i] + this.normal[i] * (step);
      
         if(i != this.projectionAxis){
         
            shiftDistance = (int)(shiftDirections[j] * absNormal[i] * (step));
         
            lower[j] += shiftDistance;
            upper[j] += shiftDistance;
            
            j++;
         }
   
      }
      
      
      planeIterator.setBounds(this.fetchBounds);
      
      
      this.planePartitioning = this.cachePartitioning.project(this.projectionAxis);
      
      this.dilatedFetchBounds = this.planePartitioning.map(this.fetchBounds);
   }




   /** This method fills the plane block with data read. */
   protected void fillBlock(boolean initial){

//      System.out.println("Fillblock (step "+this.currentStep+"):"); 
      boolean stopPrefetching=false;
      try{

         //MultiBlockCacheDataSource mbcds = (MultiBlockCacheDataSource) this.theSource;

         if(initial){ 
            if(this.threaded){
               fetchCurrentandNext(this.currentStep); 
            } else {
               fetchCurrent(this.currentStep);
            }
         } else 
            if(this.threaded){
               if (  ((this.currentStep % this.prefetchInterval) == this.prefetchInterval/2) ){
                  
                  int prefetchStep;
                  
                  if(this.currentStep + this.prefetchInterval > this.numSteps){
                  
                     prefetchStep = this.numSteps - this.prefetchInterval;
                     stopPrefetching = true;
                  } else {
                     
                     prefetchStep = this.currentStep;
                  }
                  
                  if(!this.stoppedPrefetching){
                  
                     waitOnIOThread();
                     
                     //this.prefetchThread = new IOThread(this, this.currentStep);
                     
                     
                     this.prefetchThread = new IOThread(this, prefetchStep);
                     this.prefetchThread.start();
                     
//                     this.fetchNext(prefetchStep);
                  }
                  
                  this.stoppedPrefetching =  this.stoppedPrefetching || stopPrefetching;
                  //waitOnIOThread();   // Remove for overlapped IO (default)
                }
            } else {
            
               fetchCurrent(this.currentStep);
               
            }
      } catch (ClassCastException e){

         // Do nothing.
      }
      
      if(this.prefetchThread != null && (this.currentStep  >= (this.prefetchThread.step + this.prefetchInterval - 1))){

         waitOnIOThread();  
      }
      
//      for(this.planeIterator.init(); this.planeIterator.valid(); this.planeIterator.next()){ 
//
//         this.planemapper.map(this.normal, this.currentPoint, this.mainSpaceLocation, this.planeIterator);
//
//         //System.out.println("Calling the pdatum method.");
//         //((MultiBlockCacheDataSource)theSource).pdatum(this.datum, this.mainSpaceLocation, sQBLowerInts, sQBUpperInts, datumCoords1, sourceQueryBounds );
//         
//         ((MultiBlockCacheDataSource)theSource).pdatum(this.datum, this.mainSpaceLocation, mainCacheContext );
//
//
//         this.mainScratchID2D = this.planeIterator.relativeLocation( this.mainScratchID2D );
//         this.theBlock.setDatum( this.mainScratchID2D, this.datum );
//
//      }

         
         int r0 = this.planeBounds.getLowerArray()[0];
         int c0 = this.planeBounds.getLowerArray()[1];
         int r1 = this.planeBounds.getUpperArray()[0];
         int c1 = this.planeBounds.getUpperArray()[1];
         
         int [] coords = mainScratchID2D.refCoords();
         
         for(int j=c0; j<=c1; j++){
            for(int i=r0; i<=r1; i++){

               coords[0] = i;  coords[1] = j;
               
               this.planemapper.map(this.normal, this.currentPoint, this.mainSpaceLocation, mainScratchID2D);

               //System.out.println("Calling the pdatum method.");
               //((MultiBlockCacheDataSource)theSource).pdatum(this.datum, this.mainSpaceLocation, sQBLowerInts, sQBUpperInts, datumCoords1, sourceQueryBounds );
               
               ((CacheDataSource) theSource).pdatum(this.datum, this.mainSpaceLocation, mainCacheContext );


               //this.mainScratchID2D = this.planeIterator.relativeLocation( this.mainScratchID2D );
               coords[0] = i-r0;  coords[1] = j-c0;
               this.theBlock.setDatum( this.mainScratchID2D, this.datum );
            }
         }

    }

   protected void removePrefetchThread(){
   
      this.prefetchThread = null;
   }
   
	/** If any IO thread is running, wait for it to terminate.
	 *  
	 */
	void waitOnIOThread(){
	
		if(this.prefetchThread != null && this.prefetchThread.isAlive()){
		
			try{
			
				this.prefetchThread.join();
			} catch(InterruptedException e){
				
				new InternalFaultError(SDBError.Recoverable,"join() was interrupted in waitOnIOThread()");
			}
		}
	}
   
   public void deactivate(){
   
      waitOnIOThread();
      this.theSource.deactivate();
   }
   
   protected class IOThread extends Thread{
   
      protected int step;
      protected ObliquePlaneIterator iter;

      
      IOThread(ObliquePlaneIterator iter,  int step){
      
         super("ObliquePlaneIterator IO Thread");
         
         this.step = step;
         this.iter = iter;
      }
      
      public void run(){
      
//         System.out.println("IOThread (step "+this.step+"):");
         
         try {
            iter.fetchNext(step);
            
         } catch(Exception e){
            
            System.out.println("OPI IO thread received an exception during fetchNext("+step+").");
            System.out.println(e);
            
            System.exit(1);
         }
         iter.removePrefetchThread();
      }
      
   }




    private abstract class PlaneMapper{

        abstract void map(float n[], double p[], IndexSpaceID spaceID, IndexSpaceID planeID);

        abstract void map(float n[], double p[], int [] spaceCoords, int [] planeCoords);
    }

    private class PlaneMapper_i extends PlaneMapper{

        void map(float n[], double p[], IndexSpaceID spaceID, IndexSpaceID planeID){


            int [] spaceCoords = spaceID.refCoords();
            int [] planeCoords = planeID.refCoords();

            spaceCoords[0] = (int) (  - ( -p[0]*n[0] + n[1]*(planeCoords[0] - p[1]) + n[2]*(planeCoords[1] - p[2]) ) / n[0] );
            spaceCoords[1] = planeCoords[0];
            spaceCoords[2] = planeCoords[1];
        }

        void map(float n[], double p[], int [] spaceCoords, int [] planeCoords){

            spaceCoords[0] = (int) (  - ( -p[0]*n[0] + n[1]*(planeCoords[0] - p[1]) + n[2]*(planeCoords[1] - p[2]) ) / n[0] );
            spaceCoords[1] = planeCoords[0];
            spaceCoords[2] = planeCoords[1];
        }
    }


    private class PlaneMapper_j extends PlaneMapper{

        void map(float n[], double p[],  IndexSpaceID spaceID, IndexSpaceID planeID){


            int [] spaceCoords = spaceID.refCoords();
            int [] planeCoords = planeID.refCoords();

            spaceCoords[0] = planeCoords[0];
            spaceCoords[1] = (int) (  - ( n[0]*(planeCoords[0] - p[0]) - n[1]*p[1] + n[2]*(planeCoords[1] - p[2]) ) / n[1] );
            spaceCoords[2] = planeCoords[1];
        }

        void map(float n[], double p[], int [] spaceCoords, int [] planeCoords){

            spaceCoords[0] = planeCoords[0];
            spaceCoords[1] = (int) (  - ( n[0]*(planeCoords[0] - p[0]) - n[1]*p[1] + n[2]*(planeCoords[1] - p[2]) ) / n[1] );
            spaceCoords[2] = planeCoords[1];
        }

    }


    private class PlaneMapper_k extends PlaneMapper{

        void map(float n[], double p[],  IndexSpaceID spaceID, IndexSpaceID planeID){

            int [] spaceCoords = spaceID.refCoords();
            int [] planeCoords = planeID.refCoords();

            spaceCoords[0] = planeCoords[0];
            spaceCoords[1] = planeCoords[1];
            spaceCoords[2] = (int) (  - (n[0]*(planeCoords[0] - p[0]) + n[1]*(planeCoords[1] - p[1]) - n[2]*p[2] ) / n[2] );
        }

        void map(float n[], double p[], int [] spaceCoords, int [] planeCoords){

            spaceCoords[0] = planeCoords[0];
            spaceCoords[1] = planeCoords[1];
            spaceCoords[2] = (int) (  - (n[0]*(planeCoords[0] - p[0]) + n[1]*(planeCoords[1] - p[1]) - n[2]*p[2] ) / n[2] );
        }
    }


    /** Return the index of the element with maximum magnitude */
    protected static int findMax(float vector[]){

        int i, d=vector.length;
        float max = Math.abs(vector[0]);
        float a;
        int maxindex = 0;

        for(i=0; i<d; i++){

            a = Math.abs(vector[i]);
            if(a > max){
                max = a;
                maxindex = i;
            }

        }

        return maxindex;
    }


    /** Return the axis along which the iterator will project the plane. */
    protected static int chooseProjectionAxis(float vector[]){

       return findMax( vector);
    }

    /** Return the axis along which the iterator will project the plane. */
    public  int getProjectionAxis(){

       return this.projectionAxis;
    }



    /** Divide all elements of the vector by the absolute value of the
      * coordinate at the specified axis.
      */
    public static void reduceByCoord(float vector[], int axis){

        int i, d=vector.length;
        float coord =  Math.abs(vector[axis]);

        for(i=0; i<d; i++){

            vector[i] /= coord;
        }
    }

   /** Return the datasource being iterated over.*/
   public DataSource getDataSource(){

      return this.theSource;
   }
   
   /** Indicate to this iterator whether the underlying cache has blocks shaped according to 
    *  the plane normal.
    */
   public void setSPCacheBlocks(boolean spBlockShape){
   
      this.SPBlockShape = spBlockShape;
   }
   



   /** Set the datasource being iterated over.*/
   public void setDataSource(DataSource source){

      this.theSource = source;



      if (this.theSource instanceof BlockCacheDataSource){

         BlockCacheDataSource bcds = (BlockCacheDataSource) this.theSource;

         IndexSpaceID blockDimensions = bcds.getCacheBlockDimensions();

         this.blockDimensionsArray = blockDimensions.getCoords();

         ISBounds iterationSpace = bcds.getIterationSpace();

         //RegularISPartitioning partitioning = new RegularISPartitioning(blockDimensions, iterationSpace );
         
//         this.cachePartitioning = (RegularISPartitioning) bcds.getPartitioning();
         this.cachePartitioning = (RegularISPartitioning) bcds.getCachePartitioning();

         this.stencil = new DSBlockStencil(this.cachePartitioning, this.theSource.getRecordDescriptor());

         //int volume = (int) ((DSPartitionedStencil)this.stencil).volume();

         this.planePartitioning = this.cachePartitioning.project(this.projectionAxis);



         this.fetchIterator = createPrefetchIterator(this.planeBounds, iterationSpace, blockDimensions);
         
         this.prefetchInterval = computePrefetchInterval(blockDimensionsArray, this.normal);


      }
   }

   /** Compute the interval for prefetching. This is the number of steps we can 
    * proceed through the iteration without having to prefetch. 
    */
   private int computePrefetchInterval(int [] blockDimensionsArray, float [] normal){
   
//      float length2=0;
//      float normalLength = 0;
//   
//      for(int i=0; i<normal.length; i++){
//         
//         if(normal[i] != 0){ // this should be safe because of setEffectiveZeros()
//         
//            length2 += blockDimensionsArray[i] * blockDimensionsArray[i];
//         }
//         
//         normalLength += normal[i] * normal[i];
//      }
      
      //return (int) (Math.sqrt(length2)-1 );
      //return (int) (Math.sqrt(length2) / Math.sqrt(normalLength));
      
      int steps;
      int min = Integer.MAX_VALUE;
      
      for(int i=0; i<normal.length; i++){
      
         if(normal[i] != 0){ // this should be safe because of setEffectiveZeros()
         
            steps = (int) (blockDimensionsArray[i] / Math.abs(normal[i]));
            
            min = (steps < min) ? steps : min;
         }
      }


//      int i1 = (int) (Math.sqrt(length2) / Math.sqrt(normalLength));

      
      
      return min; // should be min
      //return 2;
   }

   /** Return a copy of the normals array.*/
   public float [] getNormalArray(){

      return this.normal.clone();
   }


   /** Return a projection of the normals array onto the projection Axis. */
   public float [] getProjectedNormalArray(){

      float [] pnorm = new float[2];
      int j=0;

      for(int i=0; i<this.spaceDimensionality; i++){

         if(i != this.projectionAxis){

            pnorm[j] = this.normal[i];
            j++;
         }
      }

      return pnorm;
   }


   /** Return an array indicating the 2D shape of the iteration plane.*/
   public int [] getPlaneShapeArray(){

      return this.planeBounds.getDimensionsArray();
   }

   /** Return the volume of the iteration plane.*/
   public int  getPlaneVolume(){

      return this.planeBounds.volume();
   }





   public IndexSpaceID [] getPlane3DVertices(IndexSpaceID [] vertices) {
   

      if (vertices == null){
      
         vertices = new IndexSpaceID[4];
         
         for(int vi=0; vi < 4; vi++){
         
            vertices[vi] = new IntegerIndexSpaceID(new int[3]);
         }
      }
      
      
      for(int vi=0; vi < 4; vi++){
      
         this.planeBounds.getVertex(vi, externalScratchID);

         this.planemapper.map(this.normal, this.currentPoint, vertices[vi], externalScratchID);
      }
   
   
      return vertices;
   
   }

   /**
    * Returns an ISBounds representing the dimensions of the space being
    * iterated over. This space is calculated as the bounding box of the
    * path of the plane through space, so not all elements in this space are
    * necessarily visited by the iterator.
    */
   public final ISBounds getSpaceBounds(){


      int d = this.spaceDimensionality;
      int min[] = new int[d];
      int max[] = new int[d];

      double [] point = this.startPoint.clone();

      int j;

      // This needs to generalized to n-d, but we'll stick with 2D for now
      // because of a looming paper deadline

      int corner2D[] = new int[2];
      int corner3D[] = new int[d];

      int [] lower = startBounds.copyLowerArray();
      int [] upper = startBounds.copyUpperArray();


      // The idea is to start with the initial plane position in 3D, and
      // find max and min values for the coordinates. That is, we're effectively
      // computing an iso-aligned bounding box for the initial plane. Then,
      // we repeat the process for the ending plane position, updating max
      // and min values.  This allows us to construct an iso-aligned bounding
      // box for the entire iteration volume.

      // We'll do the first corner outside the loop,
      // so we can properly initialize the max and min arrays.

      // build a 2D corner
      corner2D[0] = lower[0];
      corner2D[1] = lower[1];

      // map it to 3D
      this.planemapper.map(this.normal, point, corner3D, corner2D);


      // init the max and min
      for(j=0; j<d; j++){

         max[j] = min[j] = corner3D[j];

      }


      // for each remaining corner...
      for(int i=1; i<4; i++){

         // build a 2D corner
         corner2D[0] = ( (i & 0x01) == 0) ? lower[0] : upper[0];
         corner2D[1] = ( (i & 0x02) == 0) ? lower[1] : upper[1];

         // map it to 3D
         this.planemapper.map(this.normal, point, corner3D, corner2D);


         // Update max and min
         for(j=0; j<d; j++){

            max[j] = (max[j] < corner3D[j]) ? corner3D[j] : max[j];

            min[j] = (min[j] > corner3D[j]) ? corner3D[j] : min[j];

         }
      }


      // Now we (conceptually) advance the plane to its ending position.

      j=0;
      for(int i=0; i<d; i++){

         point[i] = (int) (this.startPoint[i] + this.normal[i] * (this.numSteps));

         if(i != this.projectionAxis){

            // move plane bounds by +  or -1

            lower[j] += (int)(shiftDirections[j] * absNormal[i] * (this.numSteps));
            upper[j] += (int)(shiftDirections[j] * absNormal[i] * (this.numSteps));

            j++;
         }
      }



      // And update the max and mins again.
      for(int i=0; i<4; i++){

         // build a 2D corner
         corner2D[0] = ( (i & 0x01) == 0) ? lower[0] : upper[0];
         corner2D[1] = ( (i & 0x02) == 0) ? lower[1] : upper[1];

         // map it to 3D
         this.planemapper.map(this.normal, point, corner3D, corner2D);


         // Update max and min
         for(j=0; j<d; j++){

            max[j] = (max[j] < corner3D[j]) ? corner3D[j] : max[j];

            min[j] = (min[j] > corner3D[j]) ? corner3D[j] : min[j];

         }
      }


      return new ISBounds(min, max).intersectWith(this.theSource.getBounds());

   }

   /**
    * Returns an array of ints representing the dimensions of the space being
    * iterated over. This space is calculated as the bounding box of the
    * path of the plane through space, so not all elements in this space are
    * necessarily visited by the iterator.
    */
   public final int[] getSpaceShapeArray()
   {

      return this.getSpaceBounds().getDimensionsArray();
   }

   /** Return the current center point of the plane in space.*/
   public final float [] getCenterPoint(){
   
      return getCenterPoint(new float[this.spaceDimensionality]);
   
   }

   /** Return the current center point of the plane in space. The coordinates
    *  are copied into the argument array and returned. */
   public final float [] getCenterPoint(float [] point){
   
      int d = this.spaceDimensionality;
      
      if (point == null) {
      
         point = new float[d];
      }
      
      for(int i=0; i<d; i++){
      
         point[i] = (float) this.currentPoint[i];
      }
      
      return point;
   }

    /*--------------------------------------------
     * DataBlock methods
     *____________________________________________*/

    /** Return the number of elements in the datablock.
     *  @deprecated Use volume()
     */
    public int size(){

        return theBlock.volume();
    }

    /** Return the number of elements in the datablock. */
    public int volume(){

        return theBlock.volume();
    }

    /** Given an IndexSpaceID, return the corresponding datum.*/
    public  Datum datum(IndexSpaceID did){

        return theBlock.datum(did);
    }

    /** Given an array of coordinates, copy the corresponding datum
        into the first parameter.*/
    public  void datum(Datum d,int[] coordArray){

        theBlock.datum(d, coordArray);
    }

    /** Given an IndexSpaceID, return the corresponding datum with attributes
        specified in the RecordSpec. */
    public  Datum datum(IndexSpaceID did, RecordSpec spec){

        return theBlock.datum(did, spec);
    }

    /** Get the Datum and fill its values in the passed Datum at positions
     *  specified by the corresponding RecordSpec.
     */
    public void datum(Datum dtm, IndexSpaceID did, RecordSpec recSpec){

        theBlock.datum(dtm, did, recSpec);
    }

    /** Get the Datum and fill its values in the passed Datum at positions
     *  specified by the corresponding FieldIDMapper.
     */
    public void datum(Datum dtm, IndexSpaceID did, FieldIDMapper mp){

        theBlock.datum(dtm, did, mp);
    }

    /** Return by reference the Datum corresponding to the given
     *  IndexSpaceID.
     */
    public void datum(Datum dtm, IndexSpaceID did){

        theBlock.datum(dtm, did);
    }


    /** Set the location specified by the IndexSpaceID to the given datum.*/
    public void setDatum(IndexSpaceID isid, Datum d){

        theBlock.setDatum(isid, d);
    }


    /** Return a new DataBlock that is a subset of this datablock. The Data
        is shared between the two DataBlocks.*/
    public  DataBlock subblock(ISBounds db){

        return theBlock.subblock(db);
    }



    /** Reads a subblock defined by srcBounds into the region of the Datablock argument
        defined by dstBounds. */
    public void subblock(DataBlock theBlock, ISBounds dstBounds, ISBounds srcBounds){

        this.theBlock.subblock(theBlock, dstBounds, srcBounds);
    }


    /** Reads a subblock defined by srcBounds into the region of the Datablock argument
        defined by dstBounds. The FieldIDMapper argument determines the mapping between
        datums in this DataBlock and datums in the argument DataBlock. */
    public void subblock(DataBlock theBlock, ISBounds dstBounds, ISBounds srcBounds, FieldIDMapper mp){

        this.theBlock.subblock(theBlock,dstBounds, srcBounds, mp);
    }

    /** return the Values (data) stored in this basicBlock
     */
    public  Values getValues()
     {
         return this.theBlock.getValues();
     }



    /** Reads a subblock defined by srcBounds into the region of the Datablock argument
        defined by dstBounds. The RecordSpec argument specifies a subset of the attributes
        of this DataSource to be written into the DataBlock datums.*/
	public void subblock(DataBlock theBlock, ISBounds dstBounds, ISBounds srcBounds, RecordSpec spec){

        this.theBlock.subblock(theBlock, dstBounds, srcBounds, spec);
    }

    public String toString(){

        return this.theBlock.toString();
    }
}
