/*	
	OverlapBlockedDataSource.java
	Copyright 2000 University of New Hampshire
	Durham, NH 03824

        Programmer: Li Ye
------------------------ Edit history as of 01/30/05 -----------------------
09/09/05 rdb: Avoided some warnings by changing static references to new              
02/07/05 rdb: initialize -> initializeComponents and deleted a bunch of ctor code
                 that is already also done by parent.
01/30/05 rdb: Deleted methods now done by parents.
                  datum and subblock that return results.
*/

package edu.unh.sdb.datasource;
//import java.io.*; 
import edu.unh.sdb.common.*;

/**
 * OverlapBlockedDataSource does a spatial join on several 
 * blocks of DataSources into one DataSource 
 */

public class OverlapBlockedDataSource extends CompositeDataSource {

    private OverlapBlockManager overlapmanager;
    
    /**Construct a OverlapBlockedDataSource from an array of ComponentInfo.*/
    public OverlapBlockedDataSource(String dsname, ComponentInfo [] compinfo)
    {
       super(dsname,compinfo);
       // done by parent this.initialize(compinfo);
    }
    
    /**Construct a OverlapBlockedDataSource from parallel arrays of DataSource,
       FieldIDMapper and ISIDMapper
      */
    OverlapBlockedDataSource(String aname, DataSource[] sources, 
                             FieldIDMapper[] FIDmap, ISIDMapper[] ISIDmap)
    {
       super(aname,sources,FIDmap,ISIDmap);
       // rdb deleted a whole lot of code that is already executed by parent
    }
    
    /** Check the validity of the given ComponentInfo array */
    protected boolean check(ComponentInfo[] cinfo)
    {
       OverlapBlockManager overlapmanager = new OverlapBlockManager(cinfo);
       return overlapmanager.check(cinfo);
    }

    /** overrides parent abstract definition. Initialize info base don
     *  components. */
    protected void initializeComponents( ComponentInfo [] compinfo )
    {
       if (BlockManager.isRegular(compinfo)) {
          this.overlapmanager = new RegularGridBlockManager( compinfo );
       }

       else 
          this.overlapmanager = new OverlapBlockManager(compinfo);
       this.recordDesc = overlapmanager.getRecordDescriptor();
       this.bounds = overlapmanager.getBounds();
    }

    /** Return the OverlapBlockManager, from which you can get information 
      * for each Component.
      */
    public OverlapBlockManager getOverlapBlockManager()
    {
       return this.overlapmanager;
    }

    
    /** Activate() method call its component DataSource to activate itself */
    public void activate()
    {
       int i;
       ComponentInfo[] components=this.overlapmanager.getComponentInfo();
       for (i=0;i<components.length;i++) {
          if (components[i].getComponent() instanceof DataSource) {
             DataSource c = (DataSource)components[i].getComponent();
             c.activate();
          }
       }
    }
	    
    /** Get a block of data of the DataSource. The DataBlock is passed by 
      * reference
      */
    public void subblock(DataBlock blk, ISBounds blkisb, ISBounds dsisb, 
                         RecordSpec spec)
    { 
       overlapmanager.subblock(blk, blkisb, dsisb, spec);
    }
    
    /** Get a block of data of the DataSource, pass by reference
      * before this method is called, check to make sure the 
      * Dimensionality and dsisb is inside the DataSource's bounds
      */
    public void subblock(DataBlock blk, ISBounds blkisb, ISBounds dsisb)
    {
       overlapmanager.subblock(blk, blkisb, dsisb);
    }
    
    /** Pass by reference subblock method taking FieldIDMapper */
    public void subblock(DataBlock blk, ISBounds blkisb, ISBounds dsisb, 
                         FieldIDMapper mp) 
    {
       overlapmanager.subblock(blk, blkisb, dsisb, mp);
    
    }

    /** get the Datablock and fit in a Datablock which may have different size*/
    public DataBlock subblock(ISBounds blk, ISBounds resolution)
    {
       ISIDMapper mapper;
       CompositeBlock ret;
       ISIterator iter;
       if (resolution.volume() == blk.volume()){
          return this.subblock(blk);
       }
        
       ISBounds isb1 = resolution.copy();
       isb1.moveToOrigin();
       ret = new CompositeBlock(isb1,this.recordDesc);
       iter = new ISIterator(isb1);                           
       mapper = new ISIDMapper(blk,isb1);
        
       if (resolution.volume() > blk.volume()){
          CompositeBlock data_blk = (CompositeBlock)this.subblock(blk);
          while (iter.valid()){
             ret.setDatum(iter,data_blk.datum(mapper.toComponent(iter)));
             iter.next();
          }
          return ret;
       }

       if (resolution.volume() < blk.volume()){
          while (iter.valid()){
             ret.setDatum(iter,this.datum(mapper.toComponent(iter)));
             iter.next();
          }
          return ret;
       }

       return null;
    }

    /** Return by reference the Datum corresponding to the given 
     *  IndexSpaceID.
     */
    public void datum(Datum dtm, IndexSpaceID isid)
    {
       overlapmanager.datum(dtm, isid);
       return;
    }
       
    /** Get the Datum and fill its values in the passed Datum at positions
     *  specified by the corresponding FieldIDMapper.
     */
    public void datum(Datum dtm, IndexSpaceID isid, FieldIDMapper mp)
    {
       overlapmanager.datum(dtm, isid, mp);
       return;
    }
    
    
    /** Get the Datum and fill its values in the passed Datum at positions
     *  specified by the corresponding RecordSpec.
     */
    public void datum(Datum dtm, IndexSpaceID isid, RecordSpec recSpec)
    {
       overlapmanager.datum(dtm, isid, recSpec);
       return;
    }
    
    
    /** Return a ARBlock, which is a AR representation of the requested
     *  area of specified ISBounds
     */
    public ARBlock ARRep(ISBounds bounds) 
    {
       return overlapmanager.ARRep(bounds);
    }


    public String toString()
    {
       StringBuffer result=new StringBuffer();
       result.append(super.toString());
       result.append("\n");
        
       result.append("DataSource Type: Blocked DataSource.\n");
        
       result.append("Number of Components:");
       result.append(" " + this.overlapmanager.size());
       result.append("\n");
       return new String(result);
    }
}


