   /************************************************************
      RemoteUDTClient.java
      Copyright 2005 University of New Hampshire
      Durham, NH 03824
   
   ************************************************************/

// ------------------ Edit history (as of 01/10/05) --------------------
// 02/12/12 rdb: Added impl of subBasicBlock( ISB, ISB, ISB ) (which
//               just call psubblockUDP), because this is now an abstract
//               class in parent.
// 06/06/08 rdb: commented out many variables that are not being used.
//               replaced instance variable reference to static methods nad
//                  variables to "correct" static references via class name.
// 01/19/05 rdb: replaced getFloatAttributes() with getFloats()
//                    and getByteAttributes() with getBytes()
//          rdb: still lots of type limitations
// 01/10/05 rdb: removed direct allocation of children of Datum
// 03/20/05 srk: added the remote functionality using UDP based
//                  file transfer support.
// 05/05/05 srk: Some fine tuning done.
//
// ---------------------------------------------------------------------
   
   package edu.unh.sdb.datasource;
   
   import edu.unh.sdb.common.*;
   
   import java.io.*;
   //import java.nio.*;
   
   import java.rmi.*;
   import java.net.*;
   //import java.util.*;
   
   /** This class will read data  from a remote datasource using UFTP logic.
   */
   public class RemoteUDTClient extends PhysicalDataSource{
   
   	  static {
   	      System.loadLibrary("RemoteUDTClient");
   	      System.out.println("RemoteUDTClient library loaded !!!!!!");
   	  }
   
   		private static native int NativeReceiveData(byte[] dstByteArray, String servIP, int dataPort, int size);
   
       private Server fileServer =null;
       private String fileServerURL = null;
   
       //private DataInputStream input;
   
       private String remoteXmlFile;
       private String remoteSourceName;
   
       private ISBounds currentSubBounds = null;
   
       private byte[] store = null; // used for stencils
   
       //for performance tests
       //private long t1 =0;
       //private long t2 =0;
       private long time =0;
   
      //port numbers.
      public static int serverDataPort = 1111;//UDP Data source port.
      public static int serverControlPort = 2223;//TCP Control source/sink port.
      //public static int serverHelloPort = 6789;//Server Hello Port.
      public static int serverAlarmPort = 3333;//TCP Server Alarm port for abnormal exit.
      public static int clientDataPort = 5555;//UDP Data sink port.
      public static int clientControlPort = 4445;//TCP Control sink/source port.
   	 
   
      //sockets used.
      public static ServerSocket welcomeSocket;
      public static Socket connectionSocket;
      public static Socket clientSocket;
      public static Socket clientAlarmSocket;
      public static InetAddress ip;
      public static String ipStr;
   
      //some global variables used.
      public static String fileName;// = "8gigbytes.xfdl";//name of file being transferred.
      public static int fileSize;//size of file being transferred.
      public static int globalVal;//variable used by the alarm thread to check the client activity.
      public static long start, end;//variables to hold start & end times of file transfer.
      public static int statusFlag = 0;//flag that tells which stage of main thread is running.
      public static byte[] dstByteArray;//byte array into which the requested subblock will get into.
      public static int[] lower, upper;//integer arrays used for storing the bounds.
      public static int[] lowerSpaceArray, upperSpaceArray;//integer arrays used for storing the space bounds.
      public static int[] lowerPartitioningArray, upperPartitioningArray;//arrays for storing partitioning bounds.
      public static String bounds;//variable used to hold a string of lower & upper bounds.
   
   
      private boolean sentStencilConstructorInfo = false;
      
      /** List of remote servers.
      */
      //public static byte ip[] = new byte[]{(byte) 130,(byte) 74,(byte) 96,(byte) 49};//scorpion.cs.olemiss.edu (server).
      //public static byte ip[] = new byte[]{(byte) 130,(byte) 74,(byte) 87,(byte) 188};//My machine somewhere (server).
      //public static byte ip[] = new byte[]{(byte) 127,(byte) 0,(byte) 0,(byte) 1};//Localhost (server).
      //public static byte ip[] = new byte[]{(byte) 130,(byte) 74,(byte) 96,(byte) 30};//jocasta.mine.nu (server).
      //public static byte ip[] = new byte[]{(byte) 130,(byte) 74,(byte) 120,(byte) 71};//sunset.olemiss.edu (server).
      //public static byte ip[] = new byte[]{(byte) 130,(byte) 74,(byte) 96,(byte) 5};//onyx.cs.olemiss.edu (server).
      //public static byte ip[] = new byte[]{(byte) 132,(byte) 177,(byte) 4,(byte) 119};//zheng.cs.unh.edu (server).
      //public static byte ip[] = new byte[]{(byte) 132,(byte) 177,(byte) 4,(byte) 137};//scott.cs.unh.edu (server).
   
   
      /** RemoteUDTClient class constructor.
        */
      public RemoteUDTClient(String name, RecordDescriptor recordDesc, ISBounds theBounds,
               AxisOrdering ordering, String URL, String serverURL)
      {
         super(name,recordDesc,theBounds,ordering,URL );// call inherited constructor
         this.fileServerURL = serverURL;
         remoteSourceName = URL;
         System.out.println("==== In RemoteUDTClient Constructor ====");
         System.out.println("remoteSourceName: "+remoteSourceName);
         System.out.println("serverURL: "+serverURL);
         System.out.println("========================================");
      }
   
      /** Sets the filename for transfer.
       */
      public static void setFileName(String s)
      {
         fileName = s;
      }
   
       /** Open the server socket on the client side.
       */
       public void activate()
       {
         if(!this.isActivated)
         {
            super.activate();
            try
            {
               URI remoteSourceURI = new URI(remoteSourceName);
               String hostname = remoteSourceURI.getHost();
               String path = remoteSourceURI.getPath();
               
               ip = InetAddress.getByName(hostname);//getting the ip address from URL.
   						ipStr = ip.getHostAddress();
   
               welcomeSocket = new ServerSocket();//Control channel from client to Server.
               welcomeSocket.setSoTimeout(10 * 60 * 1000);// 10 minute timeout
               welcomeSocket.bind(new InetSocketAddress(clientControlPort));
   
   
               Thread.sleep(1000);
               System.out.println("Requesting Server Thread");
               SendControlCode((byte)'A');//get the server to create a Server thread
   
               this.sentStencilConstructorInfo = false;  // must send basic stencil info if  we use stencils.
               System.out.println("path="+path);
               ClientToServerControlInfo(path);
            }
            catch ( Exception exc)
            {
               new SDBError(SDBError.Fatal, "Exception in activate():" + exc.toString());
            }
            
            
         }//end of if.
       }//end of activate.
   
      /** Close the server socket on the client side.
      */
      public void deactivate()
      {
         super.deactivate();
         ////System.out.println("Closing the welcome socket.");      
         try
         {
            SendControlCode((byte)'D');//get the server to exit the infinite while loop.
            System.out.println("Deactivating server");
            //System.out.println("Activated flag: " + this.isActivated);
            RemoteUDTClient.welcomeSocket.close();
         }
         catch ( IOException exc)
         {
            new SDBError(SDBError.Warning, exc.toString());
          }
      }
   
      /**Method to tell server the bounds for basic block construction.
      */
      public static void setBasicBlockBounds(String s) throws IOException
      {
         ClientToServerControlInfo(s);
      }
   
      /** set the xml file for the remote datasource.
       */
      public void setRemoteDataSource( String xmlFile, String dsName )
      {
         this.remoteXmlFile = xmlFile;
         this.remoteSourceName = dsName;
      }
   
   
       /** Return the Datum corresponding to the given IndexSpaceID.
       */
       public    Datum datum( IndexSpaceID did )
       {
           Datum returnVal = this.getRecordDescriptor().createDatum();
   
           this.datum( returnVal, did);
           return returnVal;
       }
   
   
       /** Get the Datum and fill its values in the passed Datum at positions
        *  specified by the corresponding IndexSpaceID coordinates. No argument
        *  validation is performed. Package Access Only.
        */
       void pdatum(Datum datum, IndexSpaceID did){
   
         bounds = "@,3,0,0,0";//setting bounds so as not to change the client code.
         pdatumUDP(datum, did);//call to the remote datum fetch method.
       }
   
   
       /** Get the Datum and fill its values in the passed Datum at positions
        *  specified by the corresponding RecordSpec.
        */
       public void datum(Datum datum, IndexSpaceID did, RecordSpec recSpec){
       }
   
   
       /** Get the Datum and fill its values in the passed Datum at positions
        *  specified by the corresponding FieldIDMapper.
        */
       public void datum(Datum datum, IndexSpaceID did, FieldIDMapper mp){
       }
   
   
       /** Return the Datum corresponding to the given IndexSpaceID. The
           Datum will only contain attributes specified by the RecordSpec
           argument.
       */
       public Datum datum(IndexSpaceID did, RecordSpec spec){
           Datum returnVal= null;
           return returnVal;
       }
   
   
      /** this method will extract data from datasource on remote server using UFTP logic.
        */
      protected  void pdatumUDP(Datum datum, IndexSpaceID did)
      {
         int dataType = datum.getStorageType();
         //int offset;
   
         switch(dataType)
         {
            case StorageType.Float:
   
                  System.out.println("Float not implemented");
                  break;
            case StorageType.Short:
   
                  System.out.println("Short not implemented");
                  break;
   
            case StorageType.Int:
   
                  System.out.println("Int not implemented");
                  break;
   
            case StorageType.Double:
   
                  System.out.println("Double not implemented");
                  break;
   
            case StorageType.Byte:
   
                  ReceiveData();
                  datum.setBytes(dstByteArray);
                   break;
   
            default:
   
                   new InternalFaultError(SDBError.Fatal,"Unimplemented storage type in RemoteBFDS.pssublockUDP()");
                   break;
         }
      }
   
       /** Reads a subblock defined by srcBounds into the region of the Datablock argument
           defined by dstBounds. The FieldIDMapper argument determines the mapping between
           datums in the datasource and datums in the DataBlock. Note that the DataBlock is
           assumed to already have a valid data array. the dataBlock can be basicBlock or
           compositeBlock
       */
       public void subblock( DataBlock dstBlock,
                             ISBounds dstBounds, ISBounds srcBounds,
                             FieldIDMapper mp)
       {
          if ( dstBlock instanceof BasicBlock)
             subBasicBlock( (BasicBlock)dstBlock,dstBounds,srcBounds,mp);
          else if ( dstBlock instanceof CompositeBlock)
             subCompositeBlock( (CompositeBlock)dstBlock, dstBounds, srcBounds,mp);
          else  // what's the next
          {
             new SDBError( SDBError.Recoverable, "RemoteBFDS.subbblock:: " +
             " Unimplemented method. Unknown DataBlock type" );
          }
       }
   
   
       // compositeBlock implementation
       void subCompositeBlock( CompositeBlock dstBlock,
                                       ISBounds dstBounds, ISBounds srcBounds,
                                       FieldIDMapper mp)
       {
          new SDBError( SDBError.Recoverable, "RemoteBFDS.subCompositeBlock:: " +
                       " Unimplemented method!" );
       }
   
   
       void subCompositeBlock( CompositeBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds )
       {
          new SDBError( SDBError.Recoverable, "RemoteBFDS.subCompositeBlock:: " +
                       " Unimplemented method!" );
       }
   
   
       void subCompositeBlock( CompositeBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds,
                                     RecordSpec recSpec )
       {
          new SDBError( SDBError.Recoverable, "RemoteBFDS.subCompositeBlock:: " +
                       " Unimplemented method!" );
       }
   
   
       /** Reads a subblock defined by srcBounds into the region of the Datablock argument
           defined by dstBounds. The RecordSpec argument specifies a subset of the DataSource
           attributes to be written into the DataBlock datums. Note that the DataBlock is
           assumed to already have a valid data array. the DataBlock can be BasicBlock or
           CompositeBlock
       */
       public void subblock( DataBlock dstBlock,
                             ISBounds dstBounds, ISBounds srcBounds,
                             RecordSpec recSpec)
       {
          if ( dstBlock instanceof BasicBlock)
             subBasicBlock( (BasicBlock)dstBlock,dstBounds,srcBounds, recSpec);
          else // should work for compositeBlock
             subCompositeblock( (CompositeBlock) dstBlock,dstBounds,srcBounds, recSpec);
       }
   
   
       /** Unimplemented
        * @param dstBlock
        * @param dstBounds
        * @param srcBounds
        * @param recSpec
        */
       void subBasicBlock( BasicBlock dstBlock,
                                   ISBounds dstBounds, ISBounds srcBounds,
                                   RecordSpec recSpec)
       {
          new SDBError( SDBError.Recoverable, "RemoteBFDS.subBasicBlock:: " +
                       " Unimplemented method!" );
       }
   
       void subBasicBlock( BasicBlock dstBlock, ISBounds dstBounds, ISBounds srcBounds,
                                     FieldIDMapper mp )
   
       {
          new SDBError( SDBError.Recoverable, "RemoteBFDS.subBasicBlock:: " +
                       " Unimplemented method!" );
       }
   
      /** Return true if the stencil constructor info should be sent to the server.
       *  Return false otherwise. In general, a new stencil, or a stencil that has
       *  had its partitioning modified should be resent. This implementation is 
       *  a weak one, meant to be augmented in future.
       *
       *  This method assumes that if the value true is returned, then the stencil
       *  will actually be sent. 
       */
      private boolean shouldSendStencilConstructorInfo(DSStencil stencil){
      
         // For now, we don't use more than one stencil per session, so this
         // code will do the job.
      
         if (this.sentStencilConstructorInfo){
         
            return false;
         } else {
         
            this.sentStencilConstructorInfo = true;
            return true;
         }
      }
   
   
      private void sendStencilConstructorInfo(DSStencil stencil){
      
         try{
      
            String stencilInfo = buildStencilConstructorInfo(stencil);
            
            //System.out.println("Sending Stencil Constructor Info");
            ClientToServerControlInfo(stencilInfo);
         } catch (IOException e){
         
            e.printStackTrace();
            new SDBError(SDBError.NeverThrow, e.toString());
         }
      }
   
   
      public void subblock(DSBlockStencil stencil)
      {        
         DSBlockStencil pStencil = stencil;
   
         this.store = pStencil.getPartitionListBytes(this.store);
   
   		int partitionListLength = pStencil.getPartitionListBytesLength();
   
         //creating and populating a byte array with length preceeding the partition list.
         String partitionListLengthString = "@," + partitionListLength;
         
         
         if(shouldSendStencilConstructorInfo(stencil)){
         
            sendStencilConstructorInfo(stencil);
         }
   
         //Call to receive multiple datablocks from server.
         try{
            ClientToServerControlInfo(partitionListLengthString);//partition list length info.
            
            ClientToServerStencilPartitionList(store, partitionListLength);//sending the partition list to server.
            
            getStencilBlocks(pStencil);
            
            }
         catch ( IOException exception )
            {
            exception.printStackTrace();
            }
   
   			//System.out.println("Exited Subblock()");
      }

      // composite implementation
       private void subCompositeblock( CompositeBlock dstBlock,
                                       ISBounds dstBounds, ISBounds srcBounds,
                                       RecordSpec recSpec )
       {
          new SDBError( SDBError.Recoverable, "RemoteBFDS.subCompositeblock:: " +
                       " Unimplemented method!" );
       }
   
   
      /** Reads a subblock defined by srcBounds into the region of the Datablock argument
        defined by dstBounds. The BasicBlock is assumed to be created and  a valid data array is already allocated.
        NOTE: dstBlock can be  BasicBlock or CompositeBlock. data is copied from file to BasicBlock or CompositeBlock
      based on its data type.
      */
      public void subblock( DataBlock dstBlock,
                          ISBounds dstBounds, ISBounds srcBounds)
      {
         buildBounds(srcBounds);
   
         if ( dstBlock instanceof BasicBlock){
       
            psubblockUDP((BasicBlock)dstBlock,dstBounds,srcBounds ); // PJR //changed the method name to psubblockUDP.
         }  
         else if ( dstBlock instanceof CompositeBlock ){
               
               new SDBError( SDBError.Fatal, "RemoteBFDS.subblock:: " +
                       " Unimplemented method!" );
         }
      }
   
   
      /** 
       *  Note:  The BasicBlock is assumed to be created and  a valid data 
       * array is already allocated.
       * 
       * This has been implemented as psubblockUDP
       */
      protected void subBasicBlock( BasicBlock dstBlock,
                          ISBounds dstBounds,
                          ISBounds srcBounds )
      {
         psubblockUDP( dstBlock, dstBounds, srcBounds );
      }

      /** this method will extract data from datasource on remote server using UFTP logic.
        */
       protected void psubblockUDP(BasicBlock dstBlock, ISBounds dstBounds, ISBounds srcBounds)
       {
         int dataType = dstBlock.getStorageType();
         //int offset;
         this.currentSubBounds = srcBounds;
         //why are we calling this method second time besides the one in subblock function?
         buildBounds(srcBounds);
         System.out.println("==========================================");
         System.out.println(bounds);
         System.out.println("==========================================");
         try {
         	 System.out.println("Sending bounds to server ... ");
            ClientToServerControlInfo(bounds);
            
         } catch (IOException e){
         
            e.printStackTrace();
         }
   
         switch(dataType)
         {
         case StorageType.Float:

            ReceiveData();
            RemoteUDTClient.bytesToFloats( dstBlock.getFloats(),
                                           dstByteArray,
                                           dstByteArray.length);
            break;

         case StorageType.Short:

            System.out.println("Short not implemented");
            break;

         case StorageType.Int:

            System.out.println("Int not implemented");
            break;

         case StorageType.Double:

            System.out.println("Double not implemented");
            break;

         case StorageType.Byte:
            System.out.println("About to receive data ... ");
            ReceiveData();
            System.out.println("Data received ...Done ");
            System.arraycopy(dstByteArray,0,dstBlock.getBytes(),0,dstByteArray.length);
            break;

         default:
            new InternalFaultError(SDBError.Fatal,"Unimplemented storage type in RemoteBFDS.pssublockUDP()");
         break;
         }
      }
   
          /** this method will extract data from datasource on remote server using UFTP logic.
           */
          protected void getStencilBlocks(DSBlockStencil stencil)
          
          {
            int dataType = stencil.getStorageType();
            int size = stencil.size();
            int offset;
              
            switch(dataType)
            {
   
               case StorageType.Float:
   
                        System.out.println("Float not implemented");
                        break;
   
                 case StorageType.Short:
   
                        System.out.println("Short not implemented");
                        break;
   
                 case StorageType.Int:
   
                        System.out.println("Int not implemented");
                        break;
   
               case StorageType.Double:
   
                        System.out.println("Double not implemented");
                        break;
   
               case StorageType.Byte:
   
   //                     System.out.println("Calling ReceiveData()");
   //                     ReceiveData();
   //                     System.out.println("Back From ReceiveData()");
                        offset = 0;
                        
                        ////System.out.println("dstByteArrayLength: " + dstByteArray.length);
                        ////System.out.println("Number of Blocks: " + dstBlock.size());
                        ////System.out.println("Length of each block: " + blockLength);
   
                        //logic to put the data in dstByteArray into dstBlocks.
                        //assuming all the blocks are of same size.
                        
                        //System.out.println("block collection has size="+ dstBlock.size());
                        
                        
                        for(int i=0; i<size; i++)
                        {
                        
   								//System.out.println("Calling ReceiveData()");
   								ReceiveData();
   								//System.out.println("Back From ReceiveData()");
                       
                         	int blockLength = dstByteArray.length/size;//size of dstByteArray/number of blocks.
                         	
   //                        System.arraycopy(dstByteArray,offset,dstBlock.get(i).getBytes(),0,blockLength);//copying the dataStore into "tmp" array.
                           System.arraycopy(dstByteArray,0,stencil.getIndexedBlock(i).getBytes(),0,blockLength);//copying the dataStore into "tmp" array.
   
   
   //                        offset += blockLength;
                        }
                        ////System.out.println("Done fetching block(s).");
                           break;
   
               default:
                        new InternalFaultError(SDBError.Fatal,"Unimplemented storage type in RemoteBFDS.pssublockUDP()");
                        break;
             }
      }
   
   
      /** this method will extract data from datasource on remote server using UFTP logic.
           */
          protected void getStencilBlocks(DataBlockCollection dstBlock)
          
          {
            int dataType = dstBlock.getStorageType();
            int offset;
   
            switch(dataType)
            {
   
               case StorageType.Float:
   
                        System.out.println("Float not implemented");
                        break;
   
                 case StorageType.Short:
   
                        System.out.println("Short not implemented");
                        break;
   
                 case StorageType.Int:
   
                        System.out.println("Int not implemented");
                        break;
   
               case StorageType.Double:
   
                        System.out.println("Double not implemented");
                        break;
   
               case StorageType.Byte:
   
   //                     System.out.println("Calling ReceiveData()");
   //                     ReceiveData();
   //                     System.out.println("Back From ReceiveData()");
                        offset = 0;
                        
                        ////System.out.println("dstByteArrayLength: " + dstByteArray.length);
                        ////System.out.println("Number of Blocks: " + dstBlock.size());
                        ////System.out.println("Length of each block: " + blockLength);
   
                        //logic to put the data in dstByteArray into dstBlocks.
                        //assuming all the blocks are of same size.
                        
                        //System.out.println("block collection has size="+ dstBlock.size());
                        
                        for(int i=0;i<dstBlock.size();i++)
                        {
                        
   								//System.out.println("Calling ReceiveData()");
   								ReceiveData();
   								//System.out.println("Back From ReceiveData()");
                       
                         	int blockLength = dstByteArray.length/dstBlock.size();//size of dstByteArray/number of blocks.
                         	
   //                        System.arraycopy(dstByteArray,offset,dstBlock.get(i).getBytes(),0,blockLength);//copying the dataStore into "tmp" array.
                           System.arraycopy(dstByteArray,0,dstBlock.get(i).getBytes(),0,blockLength);//copying the dataStore into "tmp" array.
   
   
   //                        offset += blockLength;
                        }
                        ////System.out.println("Done fetching block(s).");
                           break;
   
               default:
                        new InternalFaultError(SDBError.Fatal,"Unimplemented storage type in RemoteBFDS.pssublockUDP()");
                        break;
             }
      }
   
   
       /** Builds the bounds string to be sent to the RemoteUDPServer.
         */
       public void buildBounds(ISBounds srcBounds)
       {
         lower = srcBounds.getLowerArray();//reference to the lower array of srcBounds.
         upper = srcBounds.getUpperArray();//reference to the upper array of srcBounds.
   
         int k = lower.length + upper.length;//variable to hold the sum of dimensions.
         //System.out.println("bounds: " + k);
   
         bounds = "#,"+ k + ",";
   
         //lower bounds
         for(int i=0;i<lower.length;i++)
         {
            bounds += lower[i] + ",";
         }
   
         //upper bounds
         for(int i=0;i<upper.length-1;i++)
         {
                  bounds += upper[i] + ",";
         }
   
         bounds += upper[upper.length-1];//string variable "bounds" is used to let server know of sum of dimensions.
   
         ////System.out.println("Bounds string: " + bounds);
      }
   
      /** Builds the stencil space bounds & partitioning dimensions string to be sent to the RemoteBlockServer.
       */
          private static String buildStencilConstructorInfo(DSStencil stencil)
   
          {
          
             DSPartitionedStencil pStencil = (DSPartitionedStencil) stencil;
             
             int [] partitioningElementDimensions = pStencil.getPartitionDimensions().getCoords();
             
             ISBounds spaceBounds = pStencil.getSpaceBounds();
            
            
            //Space dimensions.
            lowerSpaceArray = spaceBounds.getLowerArray();//reference to the lower array of srcBounds.
            upperSpaceArray = spaceBounds.getUpperArray();//reference to the upper array of srcBounds.
   
            int dim = lowerSpaceArray.length; //space dimensionality
            //System.out.println("bounds: " + k);
   
            bounds = "$,"+ dim + ",";//$ is the control character used for Stencils.
            
            
            //lower space bounds
            for(int i=0;i<dim;i++)
            {
               bounds += lowerSpaceArray[i] + ",";
            }
            
            //upper space bounds
            for(int i=0;i<dim;i++)
            {
                     bounds += upperSpaceArray[i] + ",";
            }
            
            //partitioning dimensions
            for(int i=0;i<dim;i++)
            {
                     bounds += partitioningElementDimensions[i] + ",";
            }
            
    
            return(bounds);
      }
   
       /** read an array of bytes from the socket. It makes sure the number of bytes being read ==
        *  the length. The function will block until length number of bytes being read
        */
       private void readBytes(byte[] data, int length )
       {
       	NativeReceiveData(data, ipStr, serverDataPort, length);
       }
   
   
       /** read an array of floats from the socket. It reads bytes from the socket buffer
      and put it into a small temp byte buffer with its size as a multiple of 4. Then
      we convert the bytes into floats and copy them into dst float array.
      This function will block until the dstArray is full
       */
       private void readFloats(float[] dstArray)
       {
         //int size = dstArray.length;
         int byteSize = dstArray.length * 4;
         byte tmpBuffer[] = new byte[byteSize];
         readBytes(tmpBuffer, byteSize);      
         RemoteBFDS.bytesToFloats( dstArray, 0, tmpBuffer, byteSize);
       }
   
       public long transmitTime()
       {
           return time;
       }
   
   
      /** private method, convert bytes into floats.
      It takes numBytes from bytes array starting from index 0, convert
      them into floats, and put them into floatDst array starting at index offset
       */
       static void bytesToFloats( float[] floatDst,byte[] bytes, int numBytes )
       {
           int bits;
           int i=0,j=0;
           while(i< numBytes){
   
          bits=   (  (bytes[i++] & 0xFF)  << 24 ) |
         (  (bytes[i++] & 0xFF)  << 16 ) |
         (  (bytes[i++] & 0xFF)  << 8  ) |
         (  (bytes[i++] & 0xFF));
   
          floatDst[j] = Float.intBitsToFloat(bits);
          //System.out.println(" convert a float " + floatDst[j] );
          j++;
           }
       }
   
   
       /** private method, convert bytes into floats.
      It takes numBytes from bytes array starting from index 0, convert
      them into floats, and put them into floatDst array starting at index offset
       */
       static void bytesToFloatsOldVersion( float[] floatDst, int offset, byte[] bytes, int numBytes )
       {
           int bits;
           int j = offset;
           int i=0;
           while(i< numBytes){
   
          bits=   (  (bytes[i++] & 0xFF)  << 24 ) |
         (  (bytes[i++] & 0xFF)  << 16 ) |
         (  (bytes[i++] & 0xFF)  << 8  ) |
         (  (bytes[i++] & 0xFF));
   
          floatDst[j] = Float.intBitsToFloat(bits);
          //System.out.println(" convert a float " + floatDst[j] );
          j++;
           }
       }
   
   
       /** Reads a subblock defined by srcBounds into the region of the Datablock argument
           defined by dstBounds. Note that the DataBlock is assumed to already have a valid data
           array.
           NOTE: DataBlock must be a CompositeBlock
       */
       /**** Commented by Sridhar on March 1, 2006
       private void subCompositeblock(CompositeBlock dstBlock,
                                      ISBounds dstBounds, ISBounds srcBounds)
       {
          new SDBError( SDBError.Recoverable, "RemoteBFDS.subCompositeBlock:: " +
                       " Unimplemented method!" );
       }
       ****/
   
       /** Returns a spatial subset of the DataSource as delimited by the ISBounds argument.
           The DataSource data is copied when filling the DataBlock.
           This method returns a BasicBlock.  Need more discussion on this method
       */
       public DataBlock subblock(ISBounds db)
       {
           BasicBlock basicBlock = new BasicBlock( db, this.recordDesc );
           this.subBasicBlock( basicBlock, basicBlock.getBounds(), db );
           return basicBlock;
       }
   
   
       /** For debugging only. */
       private String toBinaryString(int value, int fieldWidth)
      {
           int mask=(int) (Math.pow(2,fieldWidth)-1);
           value=value & mask;
   
           String binaryString=Integer.toBinaryString(value);
           int i,numPadChars=fieldWidth-binaryString.length();
   
           for(i=0;i<numPadChars;i++){
               binaryString="0"+binaryString;
           }
   
         return binaryString;
       }
   
   
       /** request server for for subblock
        */
       private class ServerSubblock extends Thread {

          ServerSubblock()
          {
             super();
          }
          public void  run()
          {
             try
             {
                fileServer.subblock( currentSubBounds);
             }
             catch(RemoteException e)
             {
                System.out.println(" In client thread, can't find remote object " + e.toString() );
             }
          }
       }
   
   
   /** File transfer client program.
   */
   public static synchronized void ReceiveData() //throws Exception
   {
   
      try
      {
        		//start = System.currentTimeMillis();//start of transfer.
   				System.out.println("Waiting for the data size from server ... ");
           ServerToClientControlInfo();//tcp control information regarding the file size.
           System.out.println("Back from  ServerToClientControlInfo(File Size) in ReceiveData().");            
           System.out.println("Data Size: "+fileSize);
           dstByteArray = new byte[fileSize];
           System.out.println("Calling native receive data method ... ");
        		start = System.currentTimeMillis();//start of transfer.
           NativeReceiveData(dstByteArray, ipStr, serverDataPort, fileSize);
           end = System.currentTimeMillis();//end of transfer.
           System.out.println("Done with native data receiving ... ");
           //end = System.currentTimeMillis();//end of transfer.
           System.out.println("%%% Transfer_Time " + (end - start));
      }//end of outer try.   
      catch ( Exception exception )
      {
         exception.printStackTrace();
      }//end of catch for Exception exception.
   }//end of Client.
   
   
   /** Server to Client Control Information regarding
      file size & packet size.
   */
   public static void ServerToClientControlInfo() throws IOException
   {
      String ctrl_info = null;
      clientSocket = new Socket(ip, serverControlPort);
      
      BufferedReader inFromServer = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
      
      try {
         clientSocket.setSoTimeout(10 * 60 * 1000);
         
         ctrl_info = inFromServer.readLine();
         
      } catch (SocketTimeoutException e) {
   
         new SDBError(SDBError.Fatal, "clientSocket Timed Out");
      }
   
      //control information about File name.
      if(ctrl_info.charAt(0) == 'F')
      {
         //fileName = ctrl_info.substring(11,ctrl_info.length());
      }
      //control information about File size.
      else if(ctrl_info.charAt(0) == 'S')
      {
         fileSize = Integer.parseInt(ctrl_info.substring(6,ctrl_info.length()));
      }
      //control information used to synchronize client with server.
      else
      {
         //empty else for doing nothing used to make the server wait
         //for the start of file transfer until the client initializes.
      }
      clientSocket.close();
   }
   
   
   /** TCP Control Traffic to Server about missing packets.
   */
   public static void ClientToServerControlInfo(String s) throws IOException
   {
   
   
      String ctrl_packet;
      
      try {
      
         connectionSocket = welcomeSocket.accept();
         
      } catch (SocketTimeoutException e) {
   
         new SDBError(SDBError.Fatal, "WelcomeSocket Timed Out");
      }
      
      DataOutputStream outToServer = new DataOutputStream(connectionSocket.getOutputStream());
      ctrl_packet = s + '\n';
      outToServer.writeBytes(ctrl_packet);
      connectionSocket.close();
      //System.out.println("welcomeSocket.isOpen()="+welcomeSocket.isOpen());
   }
   
   
   /** TCP Control Traffic to Server about Stencil's partition list.
   */
   public static void ClientToServerStencilPartitionList(byte[] b, int length) throws IOException
   {
   
      try {
      
         connectionSocket = welcomeSocket.accept();
         
      } catch (SocketTimeoutException e) {
   
         new SDBError(SDBError.Fatal, "WelcomeSocket Timed Out");
      }
      
      OutputStream outToServer = connectionSocket.getOutputStream();
      outToServer.write(b,0,length);//send the getPartitionListBytes to the Server.
      outToServer.flush();
      
      connectionSocket.close();
   }
   
   
   /** Send the given byte to the master thread on the server.
   */
   public static void SendControlCode(byte b) throws IOException
   {
      //String ctrl_info;
      clientAlarmSocket = new Socket(ip, serverAlarmPort);
      
      
      OutputStream os = clientAlarmSocket.getOutputStream();
      InputStream is = clientAlarmSocket.getInputStream();
      
      os.write(b);
      os.flush();
      
      try {
      
         clientAlarmSocket.setSoTimeout(10 * 60 *1000);
         
         int ack =is.read();
         if(ack != 0){
         
            System.out.println("Non zero acknowledge from server in SendControlCode()");
          }
      } catch (SocketTimeoutException e) {
   
         new SDBError(SDBError.Fatal, "clientAlarmSocket Timed Out");
      }
      
      clientAlarmSocket.close();
      
      
   }
   
      /** Thread class to start the Client.
      */
      public static class ClientStarter extends Thread
      {
         public void run()
         {
            try
            {
               ReceiveData();//call to the method ReceiveData().
            }
            catch ( Exception exception )
            {
               exception.printStackTrace();
            }
         }//end of run.
      }
   
   
      /** Clear the filesystem cache on the server. */
       public void clearCache()
       {
           
           try
           {
               System.out.println("Clearing Server Cache");
               SendControlCode((byte)'C');//get the server to create a Server thread
   
           } 
           catch(IOException e)
           {
               System.out.println(e);
           }
       }
   		/** RemoteUDTClient main.
   		*/
   		public static void main(String[]  args)
   		{
   		
   //		   RemoteUDTClient dsrc =(RemoteUDTClient) DataSource.createRemoteUDT(args[0],args[0]+".xfdl",args[1]);
   //		   
   //		   lower = new int[]{Integer.parseInt(args[2]),Integer.parseInt(args[3]),Integer.parseInt(args[4])};
   //		   upper = new int[]{Integer.parseInt(args[5]),Integer.parseInt(args[6]),Integer.parseInt(args[7])};
   //		
   //		   dsrc.activate();//sets up the clienttoserver control socket.
   //		
   //		   RecordDescriptor rec = dsrc.getRecordDescriptor();
   //		
   //		   //constructing bounds string to pass on to server.
   //		   int k = lower.length + upper.length;//sum of dimensions.
   //		   bounds = "#,"+ k + ",";
   //		   for(int i=0;i<lower.length;i++)
   //		   {
   //		      bounds += lower[i] + ",";
   //		   }
   //		   for(int i=0;i<upper.length-1;i++)
   //		   {
   //		      bounds += upper[i] + ",";
   //		   }
   //		   bounds += upper[upper.length-1];
   //			
   //		   ISBounds tbounds = new ISBounds(lower,upper);
   //		   BasicBlock theBlock = new BasicBlock( tbounds , rec );
   //		
   //		   try
   //		   {  dsrc.psubblockUDP(theBlock, tbounds, tbounds );//calling psubblock method to make network call.
   //		   }
   //		   catch ( Exception exception )
   //		   {
   //		      exception.printStackTrace();
   //		    }
   //		
   //		   //System.out.println( theBlock);
   //			 dsrc.deactivate();
   		   /*****
   		   int[] lower1 = { 2,2,2};
   		   int[] upper1 = {4,4,4};
   		   ISBounds tbounds1 = new ISBounds( new IntegerIndexSpaceID(lower1), new IntegerIndexSpaceID(upper1) );
   		
   		   BasicBlock theBlock1 = new BasicBlock( tbounds1 , rec );
   		   try
   		   {
   		      dsrc.psubblockUDP(theBlock1, tbounds, tbounds1 );//calling psubblock method to make network call.
   		   }
   		   catch ( Exception exception )
   		   {
   		      exception.printStackTrace();
   		    }
   		   System.out.println( theBlock1);
   		   ****/
   		
   		   /*
   		   System.exit(0);
   		   */
   		}
   }
