package edu.unh.sdb.datasource;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.Map;

import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.ExpectedExceptions;
import org.testng.annotations.Test;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.FloatValues;
import edu.unh.sdb.common.Values;
import edu.unh.sdb.lattice.RegularPartitioning;

public class SparseStorageTest {
  
	private SparseStorage sparseStorage = null;
	private DataSource sparseDS = null;
	private DataSource origDS = null;
 
	
  @BeforeMethod
  public void init() {
	  
	  sparseStorage = new SparseStorage("sparseDS", "SolarWindData/SolarWind-vx/vx-1800-sparse.xfdl");
      sparseDS = sparseStorage.getSparseDS();
      origDS = DataSource.create("orig-ds", "SolarWindData/SolarWind-vx/vx-1800-res3.xfdl");
  }
  
  @Test
  public void getPartTestSparseDataExistingLocation(){
	  
	  // Location with containing partition bounds // 0, 0, 1
	  IntegerIndexSpaceID location = new IntegerIndexSpaceID(0, 12, 54);
	  ISBounds bounds = sparseStorage.getPart(location); // returns the sparse Block bounds of this location
	  Assert.assertNotNull(bounds);
	  System.out.println("Bounds: " + bounds );
	  
	  
	  
	  
	 
  }
  
  @Test
  public void getPartsSparseDataExistingBounds(){
	  int[] lower =  {1, 1, 39};// { 0, 9, 31 } ; 
	  int[] upper =  {2, 2, 40 } ; // { 0, 9, 35 };
	  ISBounds existingBounds = new ISBounds(lower, upper);
	  Map<Integer, ISBounds > partsMap = sparseStorage.getParts(existingBounds);
	  Assert.assertNotNull(partsMap);
	   
	   
	   
  }
  
  
  
  @Test(expectedExceptions = NullPointerException.class)
  public void getPartTestSparseDataNonExistingLocation(){
	  // Location that does not exist in sparse data
	  IntegerIndexSpaceID non_existing_Loc = new IntegerIndexSpaceID(0, 12, 7);
	  sparseStorage.getPart(non_existing_Loc); // returns the sparse Block bounds of this location
	 
	  
  }
  
  
  // Not actual tests : 
  
  @Test
  public void regularPartitioningTest(){
	  IntegerIndexSpaceID blockSizes = new IntegerIndexSpaceID(3, 3, 3);
	  DataSource ds = DataSource.create("dfdf", "ARData/Solar_vx/vx-64x64x64-bin.xfdl");
	  RegularISPartitioning partitioning = new RegularISPartitioning( 
            blockSizes, 
              ds.getBounds(),
              ds.getFDLDescriptor().getAxisOrdering());
	  
	  IntegerIndexSpaceID id = new IntegerIndexSpaceID(20, 6, 0);
	  System.out.println(" partitioning.getPartitionIndex(new IntegerIndexSpaceID(3, 0, 0) " +
			  		partitioning.getPartitionIndex(id));
	  System.out.println(" Par Bounds" + partitioning.getContainingPartitionBounds(id));
	  //System.out.println( "fgwdgwegw " + partitioning.get)
  }
  
  
  
  @Test
  public void blockTest() throws FileNotFoundException{
//	  
	  int[] lower = { 0, 0 ,0 } ; 
	  int[] upper = { 0, 1, 2 } ; 
	  sparseDS.activate(); 
	  DataBlock block_1 = sparseDS.subblock(new ISBounds(lower, upper));
	  
	  Values values = block_1.getValues();
	  float[] floats = values.getFloats();
	  for(int i = 0; i < floats.length ; i ++ ){
	  System.out.print("floats : " + floats[i] + "\t");
	  
	  }
	  
	  System.out.println();
//	  
//	  int[] lower_2 = { 2, 4, 6 } ; 
//	  int[] upper_2 = { 3, 6, 9 } ;
//	  DataBlock block_2 = sparseDS.subblock(new ISBounds(lower_2, upper_2));
	  
	  
	//  System.out.println("origDS.getFDLDescriptor().getDataFileURL()" + sparseDS.getFDLDescriptor().getDataFileURL());
	  long pos = 0 ; 
	  try {
		  RandomAccessFile sparse_raf = new RandomAccessFile(sparseDS.getFDLDescriptor().getDataFileURL(), "rw");
		  sparse_raf.seek(pos);
		  
		  int totalsize = 16 * 6 ; // 1 datum - 4 float vals, 1 float = 4 byte, total 6 datums so 16 x 6 
		  float[] floatVals = new float[24];
		  int add = 0 ;
		  for(int i = 0; i < totalsize ; i= i + 4){
		  byte[] buffer = new byte[4];
		  sparse_raf.seek(pos);
		  pos = pos + 4 ; 
		  int read = sparse_raf.read(buffer);
		  ByteBuffer byteBuffer = ByteBuffer.wrap(buffer);
		  floatVals[add] = byteBuffer.getFloat() ;
		  add++;
		  
		  //System.out.print("floatVals: " + floatVals + "\t");
		  }
		  
		  FloatValues vals = new FloatValues(floatVals);
		  DataBlock dbBlock = origDS.createDataBlock(new ISBounds(new IntegerIndexSpaceID(1, 2 ,3)));

		  ISIterator iter2 = new ISIterator( dbBlock.getBounds() );
		  int start = 0 ;
		  for( ; iter2.valid(); iter2.next()){
			  Datum dat = origDS.createDatum() ; 
			  dat.setFloats(floatVals, start , dbBlock.getAttributeNames().length);
			  dbBlock.setDatum(iter2, dat);
			  start = start + dbBlock.getAttributeNames().length ; 
		  }
		  
		  System.out.println("ffff");
		 
		 
		  
		 
	  
	  } catch (IOException e) {
		
		e.printStackTrace();
	}
	  
	 
		  
	  }
	  
	  
	  
  
  
  @Test
  public void byteTest(){
	  
	  int noOfAttr = sparseDS.getNumAttributes(); 
	  int type = sparseDS.getStorageType();
	  
  
}
  

  @Test 
  public void  testDatum(){
	 Datum dat =  sparseDS.datum(new IntegerIndexSpaceID(2, 2 ,2));
	 System.out.println("Datum :  " + dat );
	 
	 Datum dat2 =  sparseDS.datum(new IntegerIndexSpaceID(0, 1 ,2));
	 System.out.println("Datum :  " + dat2 );
	  
	  
  }

}
