//
//  StatsDataSource.java
//  ObliqueCacheTest
//
//  Created by Philip Rhodes on 2/26/07.
//  Copyright 2007 Philip J Rhodes. All rights reserved.
//

package edu.unh.sdb.datasource;

import edu.unh.sdb.common.*;

public class StatsDataSource extends PhysicalDataSource{

   private long numSubblockCalls = 0;
   private long numDatumCalls = 0;
   private long numStencilCalls = 0;
   private long totalDataVolume = 0;
   


   public StatsDataSource(String           name,  
                              RecordDescriptor rd,        
                              ISBounds         isb, 
                              AxisOrdering     ordering, 
                              String           URL ){
                              
      super(   name, rd, isb,  ordering, URL );
   
   
   }

   public StatsDataSource( FDLDescriptor fdld ){
                              
      super("StatsDataSource", fdld.getRecordDescriptor(), fdld.getBounds(),  fdld.getAxisOrdering(), fdld.getDataFileURL() );
   
   }
   
   public StatsDataSource(){
      super("StatsDataSource", new RecordDescriptor("foo",'f',1), new ISBounds(3), new AxisOrdering(3), "URL");

   }
   
    public  void activate()
    {
        this.isActivated = true;
    }

    public  void deactivate()
    {
        this.isActivated = false;
    }

   
   public void printStats(){
   
      System.out.println("%%% numSubblockCalls "+numSubblockCalls);
      System.out.println("%%% numDatumCalls "+numDatumCalls);
      System.out.println("%%% numStencilCalls "+numStencilCalls);
      System.out.println("%%% totalDataVolume "+totalDataVolume);
   }



   /** Given an IndexSpaceID and FieldIDMapper, copy the values of the Datum at 
    *  that position to the argument Datum. 
    * TODO Define error conditions
    */
   public void datum( Datum dtm, IndexSpaceID did, FieldIDMapper mp ){
   
   
      numDatumCalls++;
      totalDataVolume += dtm.byteSize();
   
   }


   void subBasicBlock( BasicBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds,
                                  RecordSpec recSpec ){
         
      numSubblockCalls++;
      totalDataVolume += srcBnds.volume() * this.byteSize();
                                  
   }
   
   void subBasicBlock( BasicBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds ){
      
      numSubblockCalls++;
      totalDataVolume += srcBnds.volume() * this.byteSize();
   }
   
   
   void subBasicBlock( BasicBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds,
                               FieldIDMapper mp ){

      numSubblockCalls++;
      totalDataVolume += srcBnds.volume() * this.byteSize();
                               
   }
   
   
   void subCompositeBlock( CompositeBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds ){
   
      numSubblockCalls++;
      totalDataVolume += srcBnds.volume() * this.byteSize();
   
   }
   
   void subCompositeBlock( CompositeBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds, 
                               RecordSpec recSpec ){
      numSubblockCalls++;
      totalDataVolume += srcBnds.volume() * this.byteSize();

   }
   
   
   void subCompositeBlock( CompositeBlock dstBlk, ISBounds dstBnds, ISBounds srcBnds,  
                               FieldIDMapper mp ){
      numSubblockCalls++;
      totalDataVolume += srcBnds.volume() * this.byteSize();

   }

//   public void subblock(DSStencil stencil, DataBlockCollection dstBlocks) {
//      
//      numStencilCalls++;
//      
//      numSubblockCalls += stencil.size();
//      
//   }


   
   public void subblock(DSBlockStencil stencil) {
   
      boolean genPOV = false; // should we generate POV commands?
      int size = stencil.size();
      
      if(size != 0)
         numStencilCalls++;
      else return;
      
      numSubblockCalls += size;
      
      
      ISBounds sBounds = new ISBounds(this.dimensionality);
     
      IndexSpaceID lower;
      IndexSpaceID upper;
      
      if(genPOV){
      
         if(numStencilCalls == 1){ // display a single cache block
         
            stencil.getIndexedBounds(sBounds, 0);
         
            int [] dimensions = sBounds.getDimensionsArray();
            int [] lowerA = new int [] {900, 1200, 1100};
            int [] upperA = new int[3];
            
            for (int i=0; i< 3; i++){
            
               upperA[i] = lowerA[i] + dimensions[i];
            }
            
            System.out.print("cacheblock(");
            
            for(int j=0; j<this.dimensionality; j++){
            
               System.out.print(lowerA[j]+", ");
            }

            System.out.print(upperA[0]);
            for(int j=1; j<this.dimensionality; j++){
            
               System.out.print(", " +upperA[j]);
            }
            System.out.println(")");
         }
      
      
      
         if( (numStencilCalls % 2) ==0)
            System.out.println("color1()");
         else 
            System.out.println("color2()");
      }
      
      for(int i=0; i<size; i++){
      
         stencil.getIndexedBounds(sBounds, i);
         
         totalDataVolume += sBounds.volume() * this.byteSize();

         if(genPOV){ // generate commands for POVray
            lower = sBounds.getLower();
            upper = sBounds.getUpper();
            
            System.out.print("cacheblock(");
            
            for(int j=0; j<this.dimensionality; j++){
            
               System.out.print(lower.getCoord(j)+", ");
            }

            System.out.print(upper.getCoord(0));
            for(int j=1; j<this.dimensionality; j++){
            
               System.out.print(", " +upper.getCoord(j));
            }
            System.out.println(")");
         }
//         sBounds2 = sBounds.copy();
//         
//         if (! sBounds.intersectWith(this.bounds).equals(sBounds2)) {
//         
//            System.out.println("****Stencil block was clipped by datasource bounds.");
//         }
      }
      
      
   }

}