//
//  ThreadedSliceCache.java
//  TextureTest
//
//  Created by Philip Rhodes on Thu Nov 06 2003.
//  Copyright (c) 2003 University of New Hampshire. All rights reserved.
//
/*
    --------------------- Edit history as of 02/16/05 ------------------------
    10/21/05 rdb: removed unused variables
    02/16/05 rdb: API changes to new granite
 */

package edu.unh.sdb.datasource;

import edu.unh.sdb.common.*;

/** This cache is meant for use with slice iteration, and performs active prefetching 
 *  using a separate IO thread.
 */
public class ThreadedSliceCache extends BlockCache{

	protected CacheDataBlock [] blockArray;
	protected int sliceAxis;
	protected int blockSliceDimension;
	protected ISBounds sourceQueryBounds;
	protected int [] sQBLowerInts;
	protected int [] sQBUpperInts;			
    protected int [] datumCoords;
	protected int [] upperQueryCoords;       
    protected int numBlocks;
	protected int lastBlockNumber;
	protected SliceCacheIOThread sliceCacheIOThread=null;
	protected ISBounds subblockQueryBounds;
	protected ISBounds queryBlockDstBounds;

	
	
	public ThreadedSliceCache(Indexable source, int sliceAxis, IndexSpaceID blockDimensions, ISBounds iteratorSpaceBounds, AxisOrdering cacheBlockOrdering){

		super(source, iteratorSpaceBounds, blockDimensions, 2); 
		
        int d=this.dimensionality;
        
		this.sliceAxis=sliceAxis;
		
		RecordDescriptor rd=source.getRecordDescriptor();
		//int [] spaceDimensions=sourceBounds.getDimensions(new int[d]);
		
		this.blockSliceDimension = blockDimensions.getCoord(this.sliceAxis);
		
		this.blockArray = new CacheDataBlock[2];
		
		this.blockArray[0]=new CacheDataBlock(this.cacheBlockBounds, rd, cacheBlockOrdering);
		this.blockArray[1]=new CacheDataBlock(this.cacheBlockBounds, rd, cacheBlockOrdering);		
		
		this.numBlocks = this.iterationSpaceDimensions[sliceAxis] / this.blockSliceDimension;
		this.lastBlockNumber = -1;
		
        // we should get the bounds of iteration space, not the datasource.
//		this.sQBLowerInts = sourceBounds.getLowerArray(new int[this.dimensionality]);
//		this.sQBUpperInts = sourceBounds.getUpperArray(new int[this.dimensionality]);

//		this.sQBLowerInts = cacheBlockBounds.getLowerArray(new int[this.dimensionality]);
//		this.sQBUpperInts = cacheBlockBounds.getUpperArray(new int[this.dimensionality]);

//		this.sQBLowerInts = new int[this.dimensionality];
//		this.sQBUpperInts = cacheBlockBounds.getDimensions(new int[d]);

		this.sQBLowerInts = this.iterationSpaceBounds.copyLowerArray(new int[this.dimensionality]);
		this.sQBUpperInts = this.iterationSpaceBounds.copyUpperArray(new int[this.dimensionality]);
        
//        for(i=0; i<d; i++){
//            this.sQBUpperInts[i]--;
//        }


		this.sourceQueryBounds = new ISBounds(d);
		this.subblockQueryBounds = new ISBounds(d);
		this.queryBlockDstBounds = new ISBounds(d);
        
		this.datumCoords = new int[d];
		this.upperQueryCoords = new int[d];
	}


    /** Retrieves the requested datum from the Cache, or directly from the datasource, refreshing
      * the cache with a new block.
      */
    void lookup(Datum theDatum, IndexSpaceID id){

        CacheDataBlock theBlock;

		id.getCoords(this.datumCoords);
        
		int datumSliceAxisCoord = this.datumCoords[this.sliceAxis];
		int blockNumber = (datumSliceAxisCoord - lowerIterationSpaceCoords[this.sliceAxis]) 
                                                    / this.blockSliceDimension;

        int arrayIndex = blockNumber % 2; 
		//int sliceAxisBlockDatumCoord = datumSliceAxisCoord - (blockNumber * this.blockSliceDimension + lowerIterationSpaceCoords[this.sliceAxis]);

		//this.datumCoords[this.sliceAxis]=sliceAxisBlockDatumCoord;

        for ( int i=0; i<this.dimensionality; i++ ){
        
            if(i == this.sliceAxis){
            
                this.datumCoords[i] = datumSliceAxisCoord - (blockNumber * this.blockSliceDimension + lowerIterationSpaceCoords[this.sliceAxis]);
                
            } else {
            
                this.datumCoords[i] -= this.lowerIterationSpaceCoords[i];
            }
        }


		
		theBlock=this.blockArray[arrayIndex];

        if(theBlock.getTag() == blockNumber){
            //System.out.println("*Hit*");
            //AppMonitor.recordCacheHit();
            theBlock.datum(theDatum, this.datumCoords);
        } else {
            //System.out.println("*Miss*");
            //AppMonitor.recordCacheMiss();
			if(	this.sliceCacheIOThread != null && 
				this.sliceCacheIOThread.blockNumber==blockNumber &&
				!this.sliceCacheIOThread.isDone ){
				
				try{
//System.out.println("Calling join.");
					this.sliceCacheIOThread.join();
				} catch(InterruptedException e) {
					
					new SDBError(SDBError.Recoverable, "**** Main thread was interrupted");
					this.lookup(theDatum, id);
					return;
				}
			} else {	
				this.load(blockNumber);
//System.out.println("*****Called Load because of Cache Miss for blockNumber="+blockNumber);
			}
			
            theBlock.datum(theDatum,this.datumCoords);
			
        }
		
		
		if(blockNumber != this.lastBlockNumber){

			this.lastBlockNumber=blockNumber;
			
			this.sliceCacheIOThread = new SliceCacheIOThread(this, (blockNumber + 1) % (this.numBlocks));
			this.sliceCacheIOThread.start();
		}
    }
	

    
    void lookup(BasicBlock queryBlock, ISBounds dstBounds, ISBounds queryBounds){
    

        CacheDataBlock theBlock;

		queryBounds.copyLower().getCoords(this.datumCoords);
        queryBounds.copyUpper().getCoords(this.upperQueryCoords);
		
		int lowerSliceAxisCoord = this.datumCoords[this.sliceAxis];
        int upperSliceAxisCoord = this.upperQueryCoords[this.sliceAxis];
 
		int blockNumber = (lowerSliceAxisCoord - lowerIterationSpaceCoords[this.sliceAxis]) / this.blockSliceDimension;
        int upperBlockNumber = (this.upperQueryCoords[this.sliceAxis] - lowerIterationSpaceCoords[this.sliceAxis]) / this.blockSliceDimension;
        
        int arrayIndex = blockNumber % 2; 
        int upperArrayIndex = upperBlockNumber % 2;

        
        
        // Check for queries that straddle cache blocks
        if(arrayIndex != upperArrayIndex){
            
            if(upperBlockNumber - blockNumber == 1){
            
                // The query spans two cache blocks, so
                // we'll break it into two parts and satisfy it in pieces.
                
                SliceCacheISBounds.lowerTSCUtil(queryBounds, dstBounds, queryBlockDstBounds, subblockQueryBounds, this.sliceAxis, upperBlockNumber * this.blockSliceDimension - 1, this.blockSliceDimension);

                theBlock=this.blockArray[arrayIndex]; 
                
                theBlock.psubblock(queryBlock, queryBlockDstBounds, subblockQueryBounds);
                
                
                SliceCacheISBounds.upperTSCUtil(queryBounds, dstBounds, queryBlockDstBounds, subblockQueryBounds, this.sliceAxis, upperBlockNumber * this.blockSliceDimension - 1, this.blockSliceDimension);
            
                theBlock=this.blockArray[upperArrayIndex];
                
                this.waitOnIOThread();
                
                theBlock.psubblock(queryBlock, queryBlockDstBounds, subblockQueryBounds);
                
                return;
                
            } else {
            
                // We must have a query that spans more than two cache blocks.
                // We'll punt, and load directly from the data source.
                // No new cache blocks will be loaded.
                
                // We'd better wait for any threads out there, though.
                this.waitOnIOThread();
                
                theSource.subblock(queryBlock, dstBounds, queryBounds);

            }
            
            return;
        }
        
        
        
        
        // Below, we handle the case where the query is entirely contained in one cache block
                
        for ( int i=0; i<this.dimensionality; i++ ){
        
            if(i == this.sliceAxis){
            
                this.datumCoords[i] = lowerSliceAxisCoord - (blockNumber * this.blockSliceDimension + this.lowerIterationSpaceCoords[this.sliceAxis]);
                this.upperQueryCoords[i] = this.datumCoords[i] + (upperSliceAxisCoord - lowerSliceAxisCoord);
                
            } else {
            
                this.datumCoords[i] -= this.lowerIterationSpaceCoords[i];
                this.upperQueryCoords[i] -= this.lowerIterationSpaceCoords[i];
            }
        }
		
        
		subblockQueryBounds.assign(this.datumCoords, this.upperQueryCoords);
        
        
		theBlock=this.blockArray[arrayIndex];
        
        

        if(theBlock.getTag() == blockNumber){


            theBlock.psubblock(queryBlock, dstBounds, subblockQueryBounds);


        } else {

			if(	this.sliceCacheIOThread != null &&
				!this.sliceCacheIOThread.isDone ){
                
                // We know there's a thread running, but it might not 
                // be working on the partition we need.
            
                if(this.sliceCacheIOThread.blockNumber==blockNumber){
                
//System.out.println("Waiting for read to finish.");

                    // The running thread is loading the partition we need,
                    // so we can just wait for it to finish.
                    try{

                        this.sliceCacheIOThread.join();
                    } catch(InterruptedException e) {
                        
                        new SDBError(SDBError.Recoverable, "**** Main thread was interrupted***");
                        this.lookup(queryBlock, dstBounds, queryBounds);
                        return;
                    }
                    
                } else {
                
                    if( this.sliceCacheIOThread.blockNumber % 2 == blockNumber % 2){
                    
                    // The running thread is not working on the partition we need, 
                    // but it is using the same cache block we must write to.  So,
                    // we'll interrupt the thread, and do the read directly.
//System.out.println("Killing thread and performing direct read.");                    
                        //this.sliceCacheIOThread.stop();
                        this.sliceCacheIOThread.interrupt();

                        this.load(blockNumber);
                    } else {

                    
                    // The running thread is not working on the partition we need, 
                    // but it is NOT using the cache block we must write to.  So,
                    // we'll wait for the thread to finish, and do the read directly.
                    
//System.out.println("Waiting on thread and performing direct read.");                    
                        this.waitOnIOThread();
                        this.load(blockNumber);
                    }
                }
            } else {
//System.out.println("Performing direct read.");
				this.load(blockNumber);
            
            
            }



        
//			if(	this.sliceCacheIOThread != null && 
//				this.sliceCacheIOThread.blockNumber==blockNumber &&
//				!this.sliceCacheIOThread.isDone ){
//System.out.println("Waiting for read to finish.");
//				try{
//
//					this.sliceCacheIOThread.join();
//				} catch(InterruptedException e) {
//					
//					new SDBError(SDBError.Recoverable, "**** Main thread was interrupted");
//					this.lookup(queryBlock, dstBounds, queryBounds);
//					return;
//				}
//			} else {	
//System.out.println("Performing direct read.");
//				this.load(blockNumber);
//
//			}
			
            theBlock.psubblock(queryBlock, dstBounds, subblockQueryBounds);
			
        }
		
		// This is where new IO threads are started.
		if(blockNumber != this.lastBlockNumber){

			this.lastBlockNumber=blockNumber;
			
			this.sliceCacheIOThread = new SliceCacheIOThread(this, (blockNumber + 1) % (this.numBlocks));
			this.sliceCacheIOThread.start();
		}
	}



	
	
	protected synchronized void load(int blockNumber){

		CacheDataBlock theBlock=this.blockArray[blockNumber % 2];
		
		this.sQBLowerInts[this.sliceAxis] = this.lowerIterationSpaceCoords[this.sliceAxis] + blockNumber * this.blockSliceDimension;
		this.sQBUpperInts[this.sliceAxis] = this.lowerIterationSpaceCoords[this.sliceAxis] + (blockNumber + 1) * this.blockSliceDimension - 1;			
		
		this.sourceQueryBounds.assign(this.sQBLowerInts, this.sQBUpperInts);

       // The iterationSpaceBounds should already have been clipped against
       // the ds bounds, so clipping against it should be safe.
       this.sourceQueryBounds.intersectWith( iterationSpaceBounds );  


        
//System.out.println("Calling psubblock()");
		theSource.subBasicBlock(theBlock, this.cacheBlockBounds, this.sourceQueryBounds);
		theBlock.setTag(blockNumber);
	}

	
	/** If any IO thread is running, wait for it to terminate.
	 *  
	 */
	void waitOnIOThread(){
	
		if(this.sliceCacheIOThread != null && this.sliceCacheIOThread.isAlive()){
		
			try{
			
				this.sliceCacheIOThread.join();
			} catch(InterruptedException e){
				
				new InternalFaultError(SDBError.Recoverable,"join() was interrupted in waitOnIOThread()");
			}
		}
	}
	
	protected class SliceCacheIOThread extends Thread{
	
		protected ThreadedSliceCache c;
		protected int blockNumber;
		protected boolean isDone=false;
		
		SliceCacheIOThread(ThreadedSliceCache c, int blockNumber){
		
            super("ThreadedSliceCache IO Thread");
        
			this.c=c;
			this.blockNumber=blockNumber;
		}

        @Override
		public void run(){
		
            c.load(this.blockNumber);
            
            this.isDone = true;
        
		}
	
	}

// This is a utility class that handles bounds calculations for ThreadedSliceCache
static class SliceCacheISBounds extends ISBounds {
   private static final long serialVersionUID = 0; // for serialization inheritance

    /** This constructor should never be used.*/
    SliceCacheISBounds(){
    
        super(0);
    }

    static  final void  lowerTSCUtil(ISBounds userQueryBounds, ISBounds dstBounds, ISBounds lowerDstBounds, ISBounds subBlockQueryBounds, int splitAxis, int splitAxisCoord, int splitAxisCacheDimension){

        int i, d=userQueryBounds.lower.length;
        
        int [] ldstLower = lowerDstBounds.lower;
        int [] ldstUpper = lowerDstBounds.upper;
        
        int [] sbqbLower = subBlockQueryBounds.lower;
        int [] sbqbUpper = subBlockQueryBounds.upper;
        
        int [] dstLower = dstBounds.lower;
        int [] dstUpper = dstBounds.upper;
                
        try{
        
            for(i=0; i<d; i++){
            
                ldstLower[i] = dstLower[i];
                
                if(i == splitAxis){
                
                    ldstUpper[i] = ldstLower[i] + splitAxisCoord - userQueryBounds.lower[i];
                    
                    sbqbLower[i] = userQueryBounds.lower[i] % splitAxisCacheDimension;  
                    sbqbUpper[i] = splitAxisCacheDimension - 1;

                 } else {
                 
                    ldstUpper[i] = dstUpper[i]; //userQueryBounds.upper[i] - userQueryBounds.lower[i];
                    
                    sbqbLower[i] =  userQueryBounds.lower[i];  
                    sbqbUpper[i] =  userQueryBounds.upper[i];

                }
            }
        
        } catch (ArrayIndexOutOfBoundsException e){
            new DimensionalityError(SDBError.Fatal, "Bounds dimensionalities do not match.");
        }
    }



    static final void  upperTSCUtil(ISBounds userQueryBounds, ISBounds dstBounds, ISBounds upperDstBounds, ISBounds subBlockQueryBounds, int splitAxis, int splitAxisCoord, int splitAxisCacheDimension){
    
        int i, d=userQueryBounds.lower.length;
        
        int [] udstLower = upperDstBounds.lower;
        int [] udstUpper = upperDstBounds.upper;
        
        int [] sbqbLower = subBlockQueryBounds.lower;
        int [] sbqbUpper = subBlockQueryBounds.upper;
        
        int [] dstLower = dstBounds.lower;
                
        try{
        
            for(i=0; i<d; i++){
                
                if(i == splitAxis){
                
                    
                    udstLower[i] = dstLower[i] + (splitAxisCoord + 1) - userQueryBounds.lower[i];
                    udstUpper[i] = udstLower[i] + userQueryBounds.upper[i] - (splitAxisCoord + 1);
                    
                    sbqbLower[i] = 0;
                    sbqbUpper[i] = userQueryBounds.upper[i] - (splitAxisCoord + 1) ;
                    
                 } else {
                 
                    udstLower[i] = dstLower[i];
                    udstUpper[i] = dstLower[i] + userQueryBounds.upper[i] - userQueryBounds.lower[i];
                    
                    sbqbLower[i] = userQueryBounds.lower[i];
                    sbqbUpper[i] = userQueryBounds.upper[i]; 
                                        
                }
            }
        
        } catch (ArrayIndexOutOfBoundsException e){
            new DimensionalityError(SDBError.Fatal, "Bounds dimensionalities do not match.");
        }
    }


}

}

