//
//  ThreadedSliceCacheDataSource.java
//  TextureTest
//
//  Created by Philip Rhodes on Fri Nov 07 2003.
//  Copyright (c) 2003 University of New Hampshire. All rights reserved.
//
package edu.unh.sdb.datasource;

//import edu.unh.sdb.common.*;
//import java.io.*;

public class ThreadedSliceCacheDataSource extends BlockCacheDataSource{
	
	int sliceAxis;


	/** Creates a ThreadedSliceCacheDataSource that caches the specified DataSource. The blocks used have
	 *	dimensions specified by the blockDimensions argument. The cacheDimensions argument specifies
	 *	the dimensions of the array containing the blocks.
	 */
	public ThreadedSliceCacheDataSource(String name, DataSource ds, ISBounds iterationSpace, int sliceAxis, IndexSpaceID blockDimensions){
	
		super(name, ds, iterationSpace, blockDimensions);
		
		this.sliceAxis=sliceAxis;
	}

	/** Creates a ThreadedSliceCacheDataSource that caches the specified DataSource. The blocks used have
	 *	dimensions specified by the blockDimensions argument. The cacheDimensions argument specifies
	 *	the dimensions of the array containing the blocks.
	 */
	public ThreadedSliceCacheDataSource(String name, DataSource ds, int sliceAxis, IndexSpaceID blockDimensions){
	
		super(name, ds, ds.getBounds(), blockDimensions);
		
		this.sliceAxis=sliceAxis;
	}

	/** Calls activate() for the underlying datasource, and creates the actual cache,
	 *  which allocates memory for the CacheBlocks.
	 */	
	public void activate(){
	
	    super.activate();
        
        theCache=new ThreadedSliceCache(this.theSource, this.sliceAxis, this.blockDimensions, this.iterationSpace, ((PhysicalDataSource)this.theSource).getAxisOrdering());
	}


	/** Calls super.deactivate() for the underlying datasource, and waits for any lingering IO threads
	 *  to terminate.
	 */	
	public void deactivate(){
	
		this.halt();
		
	    super.deactivate();
	}
	
	/** Any lingering IO threads will have terminated after this method completes.
	 */	
	protected void halt(){
	
		if (theCache != null){
			((ThreadedSliceCache)theCache).waitOnIOThread();
		}
	}


}
