package edu.unh.sdb.datasource;

import edu.unh.sdb.common.Datum;
import edu.unh.sdb.common.Values;
import javax.vecmath.Vector3d;


class TransferFunctionRGB extends TransferFunction
{   
   public void classifyVoxel (DataBlock src, IndexSpaceID srcId,
      DataBlock dest, IndexSpaceID destId, Feature feature)
   {
      
   }
   
   public void shadeVoxel (DataBlock src, IndexSpaceID srcId,
      DataBlock dest, IndexSpaceID destId, Vector3d L, Vector3d H, double distFromPicturePlane)
   {
      double color = shadeVoxel (src, srcId, 0, L, H, distFromPicturePlane);
      dest.setDouble (destId, 0, color);      
   }
   
   public Type getType ()
   {
      return Type.RGB;
   }
   
   public void convert (Datum datum, byte[] texture, int index)
   {
      datum.getBytes (texture, index);
      texture[index + 3] = (byte) 255; // opaque
   }
   
   public void convert (Values values, byte[] texture)
   {
      
      byte[] data = values.getBytes ();
      
      for (int i = 0; i<data.length/3; i++)
      {
         
         texture[4*i] = data[3*i];
         texture[4*i+1] = data[3*i+1];
         texture[4*i+2] = data[3*i+2];
         texture[4*i+3] = (byte) 255;
         
      }
      
   }
   
   
   // we use Y = 0.3*R + 0.59*G + 0.11*B
   public boolean isFirstBigger (byte[] t1, int i1, byte[] t2, int i2)
   {
      double firstGray = 0.11 * t1[i1] + 0.59 * t1[i1 + 1] + 0.11 * t1[i1 + 2];
      double secondGray = 0.11 * t2[i2] + 0.59 * t2[i2 + 1] + 0.11 * t2[i2 + 2];
      return firstGray >= secondGray;
   }
   
   public Datum createDatum ()
   {
      return Datum.create (new byte[]{0, 0, 0});
   }
   
   public void trilinearInterpolation (TransferFunction.Block block, Datum datum, IndexSpaceID lower)
   {
      for (int color = 0; color < 3; color++)
      {
         double t = TransferFunction.trilinearInterpolation (block, lower,
               color);
         datum.setByte (color, (byte) Math.round (t));
      }
   }
}