
/*
  Programer: Li Ye
*/

/**
VRIndexable provides different resolution view of the data block.
It can be built on top of any data block to view the data block with
different resolution level. 
*/

package edu.unh.sdb.datasource;

import edu.unh.sdb.common.*;
//import java.io.*;

public class VRDataBlock extends DataBlock {

    protected VRIndexable vrIndexable;
 
    public VRDataBlock(DataBlock theBlock){
        super(theBlock.getBounds(), theBlock.getRecordDescriptor());    
        this.vrIndexable=new VRIndexable(theBlock);
    }
    

    /** Given an array of coordinates, copy the correspoinding datum int
        the first parameter */
    public void datum(Datum dtm, int[] coordArray){
       IntegerIndexSpaceID did = new IntegerIndexSpaceID(coordArray);
       this.vrIndexable.datum(dtm, did);
    }

    /** Return the Datum corresponding to the given IndexSpaceID. */
    public Datum datum(IndexSpaceID did){
        return this.vrIndexable.datum(did);
    }
            
    /** Return the Datum corresponding to the given IndexSpaceID. The
        Datum will only contain attributes specified by the RecordSpec
        argument.
    */
    public Datum datum(IndexSpaceID did, RecordSpec spec)
    {
        return this.vrIndexable.datum(did, spec);
    }
                                                                                
 
    /** Return by reference the Datum correspoinding to the given ndexSpaceID. 
     */
    public void datum(Datum dtm, IndexSpaceID did){
       this.vrIndexable.datum(dtm, did);
    }
 
 
    /** PBR: Return by reference the Datum corresponding to the given 
     *  IndexSpaceID. Only datum fields referred to by the FieldIDMapper
     *  are modified.
     */
    public  void datum(Datum dtm, IndexSpaceID did, FieldIDMapper mp){
       this.vrIndexable.datum(dtm, did, mp);
    }
    
    
    /** PBR: Return by reference the Datum corresponding to the given 
     *  IndexSpaceID. Only datum fields referred to by the RecordSpec
     *  are modified.
     */
    public void datum(Datum dtm, IndexSpaceID did, RecordSpec recSpec){
       this.vrIndexable.datum(dtm, did, recSpec);
    }


    /** Set the location specified by the IndexSpaceID to the given datum */
    public void setDatum(IndexSpaceID id, Datum d)
    {                              
       IndexSpaceID isid = this.vrIndexable.getMapper().toComponent(id);
       ((CompositeBlock)this.vrIndexable.getSource()).setDatum(isid,d);
    }


    public DataBlock subblock(ISBounds bounds)
    {
       return this.vrIndexable.subblock(bounds);
    }

    /** Returns a DataBlock that is an attribute projection of a spatial
        subset of the DataSource.Override parent method for inefficiency.
        @deprecated 
    */
    public  DataBlock subblock(RecordSpec rs, ISBounds isb)
    {
       return this.vrIndexable.subblock( isb, rs );
    }

    /** Returns a DataBlock that is an attribute projection of a spatial
       subset of the DataSource.Override parent method for inefficiency.
     */
    public  DataBlock subblock( ISBounds isb, RecordSpec rs )
    {
       return this.vrIndexable.subblock( isb, rs );
    }


    /** Reads a subblock defined by srcBounds into the region of the 
        Datablock argument defined by dstBounds. */
    public void subblock(DataBlock theBlock, ISBounds dstBounds, 
                         ISBounds srcBounds)
    {
       this.vrIndexable.subblock(theBlock,dstBounds, srcBounds);
    }


    /** Reads a subblock defined by srcBounds into the region of the Datablock 
        argument defined by dstBounds. The FieldIDMapper argument determines 
        the mapping between datums in the datasource and datums in the 
        DataBlock. */    
    public void subblock(DataBlock theBlock, ISBounds dstBounds, 
                         ISBounds srcBounds, FieldIDMapper mp)
    {
       this.vrIndexable.subblock(theBlock, dstBounds, srcBounds, mp);
    }
 

    /** Reads a subblock defined by srcBounds into the region of the Datablock 
        argument defined by dstBounds. The RecordSpec argument specifies a 
        subset of the DataBlock attributes to be written into the DataBlock 
        datums.*/
    public void subblock(DataBlock theBlock, ISBounds dstBounds, 
                         ISBounds srcBounds, RecordSpec spec)
    {
       this.vrIndexable.subblock(theBlock, dstBounds, srcBounds, spec);
    }

    public int volume()
    {
       //----------- not implemented ------------
       return 0;
    }
    
    /**
     * @deprecated Use volume instead
     * @return 
     */
    public int size()
    {
       return volume();
    }

    /** Increase Resolution Level by 1 */
    public boolean incr() {
       return this.vrIndexable.incr();
    }
                                                                                
    /** Decrease Resolution Level by 1 */
    public boolean decr() {
       return this.vrIndexable.decr();
                                                                                
    }
                                                                                
    public DataBlock getDataSource(){
       return (DataBlock) this.vrIndexable.getSourceIndexable();
    }
}
