/* 
	Cell.java
	Copyright 2000 University of New Hampshire
	Durham, NH 03824
	
	------------- Edit history as of 03/04/05 ---------------------------------
	03/04/05 rdb: Seems to be an error in assignment of dimensionality.
	              It is set to the number of points, rather than the dimensionality
	              of the points. 
*/

package edu.unh.sdb.lattice;

//import edu.unh.sdb.datasource.*;
import edu.unh.sdb.common.*;


/**
 * The Cell class groups several sample points into a cell. 
*/
 
public abstract class Cell
{
    protected int dimensionality;
    protected int numVertices;
    protected Datum[] data; 
    protected Point[] coords; //It should be possible that these are in the data.

//    public Cell(Point [] coords, Datum [] data, int dimensionality){
    public Cell( Point [] coords, Datum [] data )
    {    
        try
        {
            this.data = data.clone();
            this.coords = coords.clone();
            
            this.numVertices = this.coords.length;
            //rdb this.dimensionality = coords.length;
            this.dimensionality = coords[0].dim();;
            
            if ( this.coords.length != this.data.length)
            {
                new IllegalArgumentError(SDBError.Fatal,
                      "Number of coords and datums didn't match.");
            }
        } 
        catch( NullPointerException e)
        {
            if ( coords != null)
            {
                this.coords= coords.clone();
                
                this.dimensionality = coords[0].dim();
                this.numVertices = this.coords.length;
                
                this.data=null;
            } 
            else 
            {
                new IllegalArgumentError(SDBError.Fatal,
                      " Cell constructor was passed a null pointer for coords array.");
            }
        }
    }
	
	public Cell( int numAttributes, int dimensionality, int numVertices )
	{
	    this.data = new FloatDatum[numVertices];
		this.coords = new Point[numVertices];
		
		for ( int i=0; i<numVertices; i++)
		{
			this.data[i]   = new FloatDatum( numAttributes );
			this.coords[i] = new Point( dimensionality );
		}
		
		this.dimensionality=dimensionality;
	}
    
    public int dim()
    {
       return this.dimensionality;
    }
    
    public int getNumVertices()
    {
        return this.numVertices;
    }
    
    public Datum datum( int i )
    {
       return this.data[i];
    }

    public Datum [] getData()
    {
       return this.data;
    }
	
	protected void setData( Datum [] data )
	{
	   this.data = data;
	}

    public Point[] getVertexCoords()
    {
       return coords;
    }
    
	protected void setVertexCoords( Point[] vertexCoords )
	{
		this.coords = vertexCoords;
	}
	
	protected final void setDimensionality( int dim )
	{
		this.dimensionality = dim;
	}
	
	
    /** Return a String representing the object. */
    public String toString()
    {
        return this.toString("");
    }
    
    /** Return a String representing the object indented by the argument string. */
    public String toString( String indent ) 
    {
        int len, cl,dl;
        StringBuffer result = new StringBuffer( indent );
		
		cl = (this.coords == null) ? 0 : this.coords.length;
		dl = (this.data   == null) ? 0 : this.data.length;
		
		len = (cl > dl) ? cl : dl;
		
		if ( len == 0 )
		   result.append( "Empty cell" );

		for ( int i=0; i < len; i++)
		{
			if ( i < cl )
				result.append(this.coords[i]);
			else
				result.append( "<null>" );
			
			result.append(":");
			
			if ( i < dl)
				result.append( this.data[i] );
			else
				result.append( "<null>" );
			result.append( " " );
		}
		
        return new String(result);
    }
    
    abstract public boolean intersects(Cell c);

}








