//
//  CellGroupGrid.java
//  Unstructured
//
//  Created by Philip Rhodes on Mon Aug 12 2002.
//  Copyright (c) 2001 University of New Hampshire. All rights reserved.
//

package edu.unh.sdb.lattice;

import edu.unh.sdb.datasource.*; 
import edu.unh.sdb.common.*; 


public abstract class CellGroupGrid {


    protected Partitioning partitioning;
    
    protected CellGroup [] groups;
    
    protected AxisOrdering ordering;
    protected long[] axisWeights;

    protected int dimensionality;
    
	 public CellGroupGrid(){
	 
	 }
    
    public CellGroupGrid(Partitioning partitioning, AxisOrdering ordering){
    
        this.partitioning=partitioning;
        this.dimensionality=partitioning.dim();
        
        ISBounds pBounds=this.partitioning.getISBounds();
        
        if(ordering==null){
            this.ordering=new AxisOrdering(this.dimensionality);
        } else {
            this.ordering=ordering;
        }
        
        this.axisWeights=this.ordering.makeAxisWeights(pBounds);
        
		//this.makeCellGroups();
    }
 
	CellGroupGrid(AxisOrdering ordering){
	
        if(ordering!=null){
            this.ordering=ordering;
        }
				
	}
 
 
	void setPartitioning(Partitioning partitioning){

        this.partitioning=partitioning;
        this.dimensionality=partitioning.dim();
        
        ISBounds pBounds=this.partitioning.getISBounds();
		
        if(this.ordering==null){
		
            this.ordering=new AxisOrdering(this.dimensionality);
			
        } 
		                
        this.axisWeights=this.ordering.makeAxisWeights(pBounds);	
	}
 
	protected abstract void makeCellGroups();    
    
    public final CellGroup mapToCellGroup(IndexSpaceID id){
    
        return this.groups[(int)AxisOrdering.computeOffset(id, this.axisWeights)];
    }


	public Cell findEnclosingCell(IndexSpaceID id, Point p){

        CellGroup theGroup=this.mapToCellGroup(id);
		
		return theGroup.findEnclosingCell(p);
	
	}

    
    public int dim(){
        
        return this.dimensionality;
    }
	
	
	public ISBounds getGridISBounds(){
	
		return this.partitioning.getISBounds();
	}
	
	
/*	
	public void writeCells(){
	
		ISBounds gridBounds=this.partitioning.getISBounds();
		ISIterator iter=new ISIterator(gridBounds);
		RecordDescriptor rd=new RecordDescriptor( 
							new AttributeDescriptor [] {
									new AttributeDescriptor(StorageType.Int, "v0"),
									new AttributeDescriptor(StorageType.Int, "v1"),
									new AttributeDescriptor(StorageType.Int, "v2")
							}
		);
		
		ISBounds sinkBounds=new ISBounds(new IntegerIndexSpaceID(0), new IntegerIndexSpaceID(0));
		int groupIndex;
		DataSink sink;
		
		
		
		
		for(iter.init(); iter.valid(); iter.next()){
			
			
			groupIndex=AxisOrdering.computeOffset(iter, this.axisWeights);			
			
			this.groups[groupIndex].writeCells(dataFileName("Foo",iter));
			
		}
	}
	
	protected static String dataFileName(String baseFileName, IndexSpaceID id){
	
		int i, d=id.dim();
		StringBuffer buf=new StringBuffer(baseFileName);
		
		//buf.append("#");
		for(i=0; i < d; i++){
			
			buf.append(id.getCoord(i));
			buf.append("x");
		}
		
		
		return new String(buf);
	}

*/

	
	/*
	public void assignCellGroupSources(){
	
		ISBounds gridBounds=this.partitioning.getISBounds();
		ISIterator iter=new ISIterator(gridBounds);
		RecordDescriptor rd=new RecordDescriptor( 
							new AttributeDescriptor [] {
									new AttributeDescriptor(StorageType.Int, "v0"),
									new AttributeDescriptor(StorageType.Int, "v1"),
									new AttributeDescriptor(StorageType.Int, "v2")
							}
		);
		
		ISBounds sourceBounds=new ISBounds(new IntegerIndexSpaceID(0), new IntegerIndexSpaceID(0));
		int groupIndex;
		String filename;
		DataSource source;
		
		
		
		
		for(iter.init(); iter.valid(); iter.next()){
			
			filename=new String("Foo"+iter);
			
			groupIndex=AxisOrdering.computeOffset(iter, this.axisWeights);
						
			source=DataSource.createPhysicalDataSource(filename,filename);
			//source=new BinaryFileDataSource(filename, rd, sinkBounds);
			
			source.activate();
			
			this.groups[groupIndex].writeCells(sink, 0);
		
		}
	}
	*/
    
    /** Map the IndexSpaceID to a single integer that serves as
        as an offset into the physical file or network source. */
    protected int computeOffset2(IndexSpaceID dsid){
        
        int i,d=this.dim(),offset=0;
        
        if (d!=dsid.dim()){
            new DimensionalityError(SDBError.Recoverable,"ID Argument has wrong dimensionality.");
        }
        
        
        for(i=0;i<d;i++){
            offset+=dsid.getCoord(i)*axisWeights[i];
        }
        
        return offset;
    }

    /** Return a String representing the object. */
    public String toString(){
        
        return this.toString("");
    }    
    
    /** Return a String representing the object indented by the argument string. */
    public String toString(String indent){
    
        int i,numCellGroups=this.groups.length;
        String indent2=indent+Globals.toStringIndent; 
        StringBuffer result=new StringBuffer(indent+"{ CellGroupGrid: contains ");

        
        result.append(numCellGroups);
        result.append("cell Groups\n");
        
        result.append(indent2+"geometric bounds=");
        result.append(this.partitioning.getGBounds().toString(indent2)+"\n");
        result.append(indent2+"topological bounds=");
        result.append(this.partitioning.getISBounds().toString(indent2)+"\n");

        for(i=0;i<numCellGroups;i++){
            result.append(indent2+i);
            result.append(") ");
            result.append(this.groups[i].toString(indent2));
            result.append("\n");
        }
        
        result.append(indent2+"}");
        return new String(result);
    }


}
