//
//  FileArrayCellGroup.java
//  Unstructured
//
//  Created by Philip Rhodes on Sat Aug 03 2002.
//  Copyright (c) 2001 University of New Hampshire. All rights reserved.

//------------------- Edit history --------------------------------------
// 10/21/05 rdb: removed unused variables
//-----------------------------------------------------------------------


package edu.unh.sdb.lattice;

import edu.unh.sdb.common.*;
import edu.unh.sdb.datasource.*;
//import java.io.*;

/** This is an array implementation of the CellGroup Class. */
public class FileArrayCellGroup extends FileCellGroup {
    
	protected int[] ownedVIndices;
	protected int[] borrowedVIndices;
	
    protected float[] points;
	protected float[] data;
	
	
	protected int numOwnedCells;
	protected int numBorrowedCells;
	
	// String vAttrFDLName;
	// String ownedVIndicesFDLName;
	// String borrowedVIndicesFDLName;

	protected FDLDescriptor vAttrFDLD;
	protected FDLDescriptor ownedVIndicesFDLD;
	protected FDLDescriptor borrowedVIndicesFDLD;
	protected FDLDescriptor vInfoFDLD;

    public FileArrayCellGroup(GBounds bounds, String vIndicesFDLName, RecordSpec vIndicesRS, String pointInfoFDLName, RecordSpec pointRS, RecordSpec dataRS){
    
		super(bounds, vIndicesFDLName, vIndicesRS, pointInfoFDLName, pointRS,  dataRS);
		
		String vAttrFDLName = new String(pointInfoFDLName).replaceAll("data","attr");
		
		String ownedVIndicesFDLName = new String(vIndicesFDLName).replaceAll("tri","owned.tri");
		String borrowedVIndicesFDLName = new String(vIndicesFDLName).replaceAll("tri","borrowed.tri");

		//rdb this.vAttrFDLD = DataSource.createFDLDescriptor(vAttrFDLName);
        //rdb this.ownedVIndicesFDLD = DataSource.createFDLDescriptor(ownedVIndicesFDLName);
		//rdb this.borrowedVIndicesFDLD = DataSource.createFDLDescriptor(borrowedVIndicesFDLName);
		//rdb this.vInfoFDLD = DataSource.createFDLDescriptor(this.vInfoFdlName);
		this.vAttrFDLD = FDLDescriptor.create( vAttrFDLName );
		this.ownedVIndicesFDLD = FDLDescriptor.create( ownedVIndicesFDLName );
		this.borrowedVIndicesFDLD = FDLDescriptor.create( borrowedVIndicesFDLName );
		this.vInfoFDLD = FDLDescriptor.create( this.vInfoFdlName );
    }
	
	
    
    public boolean register(TriangleCell cell){
    
		return false;
    }

    public Cell findEnclosingCell(Point p){
   
        int i;
		
		
		if(backedByFile==true && inMemory==false){
		
			readCells();
		}

		
		//numAttributes=this.dataRS.size();

        int numCells=this.numOwnedCells;
		
        for(i=0;i<numCells;i++){
			
			if(TriangleCell.inside(p, this.ownedVIndices, this.points, i)){
				return new TriangleCell(this.ownedVIndices, this.points, this.data, this.numAttributes, 2, i);
			}
			
        }
		

        numCells=this.numBorrowedCells;
		
        for(i=0;i<numCells;i++){
			
			if(TriangleCell.inside(p, this.borrowedVIndices, this.points, i)){
			
				return new TriangleCell(this.borrowedVIndices, this.points, this.data, this.numAttributes, 2, i);
			}
			
        }
		
        return null;
    }
    
	
	TriangleCell getOwnedCell(int index){
	
		if(backedByFile==true && inMemory==false){
		
			readCells();
		}
	
		return new TriangleCell(this.ownedVIndices, this.points, this.data, this.numAttributes, 2, index);
	}


	void getOwnedCell(UnstructuredCell cell, int index){
	
		if(backedByFile==true && inMemory==false){
		
			readCells();
		}
	
		((TriangleCell) cell).assign(this.ownedVIndices, this.points, this.data, this.numAttributes, 2, index);
	}
	
	
	
	public final int getNumCells(){
	
		//return (this.points==null) ? 0 : this.points.length / (2*3);
		return this.numCells;
	}
		
	
	
	public int readCells(){

	
		this.backedByFile=true;
		
		/*
		DataSource pointGeomDS=DataSource.createPhysicalDataSource(vInfoFdlName, "vertexInfo");
		DataSource attrDS=DataSource.createPhysicalDataSource(vAttrFDLName, "vertexAttributes");
		DataSource ownedVIndicesDS=DataSource.createPhysicalDataSource(this.ownedVIndicesFDLName, "ownedVIndices");
		DataSource borrowedVIndicesDS=DataSource.createPhysicalDataSource(this.borrowedVIndicesFDLName, "borrowedVIndices");
		*/
		
		DataSource pointGeomDS = 
		         DataSource.create( "vertexInfo", this.vInfoFDLD );
		DataSource attrDS = 
		         DataSource.create( "vertexAttributes", this.vAttrFDLD ) ;
		DataSource ownedVIndicesDS =
		         DataSource.create( "ownedVIndices", this.ownedVIndicesFDLD );
		DataSource borrowedVIndicesDS = 
		         DataSource.create( "borrowedVIndices", this.borrowedVIndicesFDLD );
		
		
		
		//int numCells=verticesDS.volume();
		//int numAttributes=this.dataRS.size();
		int numDimensions=this.coordsRS.size();
		
		this.points=new float[numCells * 3 * numDimensions];
		this.data=new float[numCells * 3 * this.numAttributes];
				
		ISBounds ownedVIndicesBounds=ownedVIndicesDS.getBounds();
		ISBounds borrowedVIndicesBounds=borrowedVIndicesDS.getBounds();
		ISBounds ownedVerticesBounds=ownedVIndicesDS.getBounds();
		//ISBounds borrowedVerticesBounds=borrowedVIndicesDS.getBounds();
		ISBounds pointInfoBounds=pointGeomDS.getBounds();
		
		this.numOwnedCells = (ownedVIndicesBounds.copyLower().getCoord(0) == -1) ? 0 : ownedVIndicesBounds.volume();
		this.numBorrowedCells= (borrowedVIndicesBounds.copyLower().getCoord(0) == -1) ? 0 : borrowedVIndicesBounds.volume();
		
		BasicBlock ownedVIndicesBlock=new BasicBlock(ownedVerticesBounds, ownedVIndicesDS.getRecordDescriptor());
		BasicBlock borrowedVIndicesBlock=new BasicBlock(borrowedVIndicesBounds, borrowedVIndicesDS.getRecordDescriptor());
		//BasicBlock pointInfoBlock=new BasicBlock(pointInfoBounds, pointGeomDS.getRecordDescriptor());
		BasicBlock pointCoordBlock=new BasicBlock(pointInfoBounds, pointGeomDS.getRecordDescriptor());
		BasicBlock attrBlock=new BasicBlock(pointInfoBounds, attrDS.getRecordDescriptor());
		
		pointGeomDS.activate();
		ownedVIndicesDS.activate();
		borrowedVIndicesDS.activate();
		attrDS.activate();
		
		if(this.numOwnedCells != 0){
			ownedVIndicesDS.subblock(ownedVIndicesBlock, ownedVIndicesBounds);
			this.ownedVIndices = ((IntValues)(ownedVIndicesBlock.getValues()))
			                                                  .getDataReference();
		}
		
		if(this.numBorrowedCells !=0){

// Just a test of recordspec subblock functionality		
//RecordSpec rs=new RecordSpec(new int[] {1,0});
//borrowedVIndicesDS.subblock(borrowedVIndicesBlock, borrowedVIndicesBounds, rs);		
//System.out.println("Subblock didn't flop.");
		
		
		
		
		
			borrowedVIndicesDS.subblock(borrowedVIndicesBlock, borrowedVIndicesBounds);
			this.borrowedVIndices = ((IntValues)(borrowedVIndicesBlock.getValues()))
			                                                    .getDataReference();
		}

		if( !(this.numOwnedCells == 0 && this.numBorrowedCells == 0)){
			pointGeomDS.subblock(pointCoordBlock,  pointInfoBounds, pointInfoBounds);
			attrDS.subblock(attrBlock,  pointInfoBounds, pointInfoBounds);
	
			
			this.points = ((FloatValues)(pointCoordBlock.getValues())).getDataReference();		
			this.data = ((FloatValues)(attrBlock.getValues())).getDataReference();		
		}


		pointGeomDS.deactivate();
		attrDS.deactivate();
		ownedVIndicesDS.deactivate();
		borrowedVIndicesDS.deactivate();


		this.inMemory=true;
		this.numCells=this.numOwnedCells + this.numOwnedCells;
		return this.numCells;
	
	}
	
	public int writeCells(String dataFileName){
	
		new InternalFaultError(SDBError.Recoverable, "FileArrayCellGroup.writeCells(String) is not implemented.");
		
		return -1;
	}
	
	/** Destroy references to cell data. */
	public void empty(){
	
		this.ownedVIndices=null;
		this.borrowedVIndices=null;
		this.points=null;
		this.data=null;
		this.inMemory=false;
	}
	
	
    /** Return a String representing the object. */
    public String toString(){
        
        return this.toString("");
    }    
    
    /** Return a String representing the object indented by the argument string. */
    public String toString(String indent){
 
	/*   
        String indent2=indent+Globals.toStringIndent;    
        int i,numCells=this.cells.size();
        StringBuffer result=new StringBuffer(indent+"{ FileArrayCellGroup: contains ");

        result.append(numCells);
        result.append(" triangles. bounds=");
        result.append(this.bounds);
        result.append("\n");
        
        for(i=0;i<numCells;i++){
            result.append(((Cell) this.cells.get(i)).toString(indent2));
            result.append("\n");
        }
        
        result.append(indent+"}");
        
        return new String(result);
	*/
		return null;
    }
    
}
