//
//  FileSplitterCellGroup.java
//  Unstructured
//
//  Created by Philip Rhodes on Sat Aug 03 2002.
//  Copyright (c) 2001 University of New Hampshire. All rights reserved.
//
//--------------------------------------------------------------------------
// Edit history (since 01/03/05)
// 06/06/08 rdb: Made HashMap and ArrayLists generic
// 10/21/05 rdb: Removed unused variables
// 01/03/05 rdb Replaced FloatDatum( int[] ) with IntDatum( int[] ). 
//               The assumption is that IntDatum would have been used if it
//               had been available -- now it is.
//              Replaced FloatDatum( int ) with FloatDatum( (float) int )
//               in cases where the int becomes part of a 2-element datum where
//               other element has to be a float.
//
//--------------------------------------------------------------------------

package edu.unh.sdb.lattice;

import edu.unh.sdb.common.*;
import edu.unh.sdb.datasource.*;

import java.util.*;
import java.io.*;

public class FileSplitterCellGroup extends FileCellGroup{

/*
	protected class AList{
	
		int [][] list;
		int numElements;
	
		AList(int size){
		
			this.list=new int[size][2];
			this.numElements=0;
		}
	
		public void insert(int oldIndex, int newIndex){
		
			this.list[this.numElements][0]=oldIndex;
			this.list[this.numElements][1]=newIndex;
			
			this.numElements++;
		}
		
		public int lookup(int oldIndex){
			
			int i;
			
			for(i=0; i<this.numElements; i++){
				
				if(this.list[i][0]==oldIndex){
					return this.list[i][1];
				}
				
			}
			
			return -1;
		}
		
	}
*/


	protected class AList{
	
		HashMap<Integer,Integer> list;
		
		AList(int size){
		
			this.list=new HashMap<Integer,Integer>(size);
			
		}
	
		public void insert(int oldIndex, int newIndex){
		
			this.list.put(new Integer(oldIndex), new Integer(newIndex));
		}
		
		public int lookup(int oldIndex){
			
			Integer newInteger= this.list.get(new Integer(oldIndex));
			
			if(newInteger != null){
			
				return newInteger.intValue();
				
			} else { 
			
				return -1;
				
			}
		}
		
	}



    
    protected ArrayList<TriangleCell> ownedCells;
	protected ArrayList<TriangleCell> borrowedCells; 
	
	protected int numBorrowedCells;
	protected int numOwnedCells;
	
	protected int[] vIndices;
    //protected float[] points;
	//protected float[] data;
	
	protected boolean havePendingCell;
	protected boolean isOwner;
	protected CellID pendingCellOwner;
	
	protected RecordDescriptor pointInfoRD;
	protected RecordDescriptor borrowedCellRD;
	
	protected String ownedTempVIndicesName;
	protected String borrowedTempVIndicesName;
	protected OneDDataSink ownedTempVIndicesSink;
	protected OneDDataSink borrowedTempVIndicesSink;
	
	//protected DataBlock ownedTempVIndicesBlock;
	//protected DataBlock borrowedTempVIndicesBlock;
	
	//protected RecordSpec borrowedTempVIndicesRS=new RecordSpec(new int[]{2,3,4});
	
	
	protected RecordDescriptor vIndicesRD;

/*	
    public FileSplitterCellGroup(GBounds bounds, RecordDescriptor pointInfoRD){
    
		super(bounds);
		
		this.pointInfoRD=pointInfoRD;
        //this.cells=new ArrayList(16);
    }
*/

    public FileSplitterCellGroup(GBounds bounds, RecordDescriptor pointInfoRD, String vIndicesFdlName, RecordSpec vIndicesRS,  String pointInfoFdlName, RecordSpec pointRS, RecordSpec dataRS, IndexSpaceID partitionID){
    
	
		super(bounds, vIndicesFdlName, vIndicesRS, pointInfoFdlName, pointRS, dataRS);
		
		this.ownedTempVIndicesName=ownedCellsTempFileName(vIndicesFdlName, partitionID);
		this.borrowedTempVIndicesName=borrowedCellsTempFileName(vIndicesFdlName, partitionID);
		
		this.pointInfoRD=pointInfoRD;
		
		this.vIndices=new int[3];
		
		this.vIndicesRD=new RecordDescriptor( 
			new AttributeDescriptor [] {
					new AttributeDescriptor(StorageType.Int, "v0"),
					new AttributeDescriptor(StorageType.Int, "v1"),
					new AttributeDescriptor(StorageType.Int, "v2")
			}
		);


		int i, d=partitionID.dim();
		AttributeDescriptor[] cellIDAxesDescs=new AttributeDescriptor[d+4];
		
		for(i=0; i<d ; i++){
		
			cellIDAxesDescs[i]=new AttributeDescriptor(StorageType.Int, "axis"+i);
		}
		
		cellIDAxesDescs[i]=new AttributeDescriptor(StorageType.Int, "index");

		cellIDAxesDescs[i + 1] = new AttributeDescriptor(StorageType.Int, "v0");
		cellIDAxesDescs[i + 2] = new AttributeDescriptor(StorageType.Int, "v1");
		cellIDAxesDescs[i + 3] = new AttributeDescriptor(StorageType.Int, "v2");


		

		this.borrowedCellRD=new RecordDescriptor(cellIDAxesDescs);
		
//System.out.println(		this.borrowedCellRD);
//System.out.println("Creating Sink with name: "+ this.ownedTempVIndicesName);
//System.out.println("Creating Sink with name: "+ this.borrowedTempVIndicesName);

		this.ownedTempVIndicesSink=new BufferedOneDDataSink(this.ownedTempVIndicesName, vIndicesRD, 4096);
		this.ownedTempVIndicesSink.activate();
		
		this.borrowedTempVIndicesSink=new BufferedOneDDataSink(this.borrowedTempVIndicesName, borrowedCellRD, 4096);
		this.borrowedTempVIndicesSink.activate();
		
		this.havePendingCell=false;
		this.isOwner=false;
		
		this.numOwnedCells=0;
		this.numBorrowedCells=0;
		
       // this.cells=new ArrayList(16);
    }
	
	
	
    
    public boolean register(TriangleCell cell){
    
        //if(this.bounds.intersects(cell)){
        if(cell.intersects(this.bounds)){
            
			IndexSpaceID[] vertexIDs=cell.getVertexIndices();
			int [] vertexInts=new int[vertexIDs.length];
			
			for(int i=0; i< vertexIDs.length; i++){
				vertexInts[i]=vertexIDs[i].getCoord(0);
			}
			
			//rdb this.ownedTempVIndicesSink.write(new FloatDatum(vertexInts));
			this.ownedTempVIndicesSink.write(new IntDatum(vertexInts));
			
			this.numCells++;
            return true;
        } else {
            return false;
        }
    }
	
	
	
    public boolean register(TriangleCell cell,  OwnerInfo r){
    
		if(cell.containedBy(this.bounds)){
			
			// This is the owner, and there will be no other intersections
			
			r.hasOwner=true;
			
			this.havePendingCell=true;
			this.isOwner=true;
			
			IndexSpaceID[] vertexIDs=cell.getVertexIndices();
			
			for(int i=0; i< vertexIDs.length; i++){
				this.vIndices[i]=vertexIDs[i].getCoord(0);
			}
					
			return true;  // r has changed
			
		//} else  if(this.bounds.intersects(cell)){
        } else  if(cell.intersects(this.bounds)){

            
			if (r.hasOwner == false){
			
				r.hasOwner = true; 
				this.isOwner=true;
			} else {
			
				r.addBorrower(this);
			}
			
			this.havePendingCell=true;
			
			IndexSpaceID[] vertexIDs=cell.getVertexIndices();
			
			
			for(int i=0; i< vertexIDs.length; i++){
				this.vIndices[i]=vertexIDs[i].getCoord(0);
			}
						
			return true; // r has changed
        }
		
		return false;
    }
	
	
	void setOwner(CellID owner){
	
		this.pendingCellOwner=owner;
	}



	void writePendingCell(){
	
		if(this.havePendingCell){
		
			if(this.isOwner){
			
				//rdb this.ownedTempVIndicesSink.write(new FloatDatum(this.vIndices));
				this.ownedTempVIndicesSink.write(new IntDatum(this.vIndices));
				this.isOwner=false;
				this.numOwnedCells++;
			} else {
			
				//rdb FloatDatum [] datums=new FloatDatum[2];
				IntDatum [] datums=new IntDatum[2];
				
				datums[0]=this.pendingCellOwner.asDatum();
				//rdb datums[1]=new FloatDatum(this.vIndices);
				datums[1]=new IntDatum( this.vIndices );
				
//System.out.println("writePendingCell(): Writing borrowed cell.  Datum="+	new FloatDatum(datums));			
				
				
				//rdb this.borrowedTempVIndicesSink.write(new FloatDatum(datums));
				this.borrowedTempVIndicesSink.write(new IntDatum(datums));
				this.pendingCellOwner=null;
				this.numBorrowedCells++;
			}
			
			this.numCells++;
			
			this.havePendingCell=false;
		}
	}


	public void doneRegistering(){
	
		this.writePendingCell();
	
		this.ownedTempVIndicesSink.deactivate();
		this.borrowedTempVIndicesSink.deactivate();
		
		this.empty();
		
	}


    public Cell findEnclosingCell(Point p){
   
        TriangleCell theCell;
		
		if(backedByFile==true && inMemory==false){
		
			readCells();
		}

		int numOwnedCells=this.ownedCells.size();

        for ( int i=0; i<numOwnedCells; i++){
                
            theCell= this.ownedCells.get(i);
            
            if(theCell.inside(p)){
                return theCell;
            }
        }
		
		int numBorrowedCells=this.borrowedCells.size();

        for ( int i=0; i<numBorrowedCells; i++){
                
            theCell=  this.borrowedCells.get(i);
            
            if(theCell.inside(p)){
                return theCell;
            }
        }

		
		
       // System.out.println(p.getCoord(0)+"\t"+p.getCoord(1)+"\t"); //DEBUG
        return null;
    
    }
    
	public final int getNumCells(){
	
		//return this.cells.size();
		return this.numCells;
	}
	



	public int writeCells(String baseFileName){

//System.out.println("Starting FSCG.writeCells()");

		this.readCells();
		
		int numCells=this.numOwnedCells+this.numBorrowedCells;
		
		writeVIndicesFDLs(baseFileName);
		
		
		
		
		String vOwnedIndicesFileName=new String(baseFileName+".owned.tri");
		String vBorrowedIndicesFileName=new String(baseFileName+".borrowed.tri");
		String pointInfoFileName=new String(baseFileName+".data");
		String pointDataFileName=new String(baseFileName+".attr");
		
		
		// Temporary
		RecordSpec pointRS=new RecordSpec(new int [] {0,1});
		RecordSpec dataRS = new RecordSpec(new int [] {2});
		

		OneDDataSink vOwnedIndicesSink = new BufferedOneDDataSink(vOwnedIndicesFileName, this.vIndicesRD, 4096);
		OneDDataSink vBorrowedIndicesSink = new BufferedOneDDataSink(vBorrowedIndicesFileName, this.vIndicesRD, 4096);
		
		//  This sink should really have both point coords and data, but just coords for now. 
		OneDDataSink pointGeomSink = new BufferedOneDDataSink(pointInfoFileName, this.pointInfoRD.project(pointRS), 4096);
		OneDDataSink vertexDataSink = new BufferedOneDDataSink(pointDataFileName, this.pointInfoRD.project(dataRS), 4096);
		
						
		AList vertexList=new AList(3*numCells-1);
		
		
		

		pointGeomSink.activate();
		vertexDataSink.activate();
		
		
		vOwnedIndicesSink.activate();
		int numPoints=writeTriangles(this.ownedCells, vOwnedIndicesSink, pointGeomSink, vertexDataSink, vertexList, 0);
		vOwnedIndicesSink.deactivate();
		
		vBorrowedIndicesSink.activate();
		numPoints=writeTriangles(this.borrowedCells, vBorrowedIndicesSink, pointGeomSink, vertexDataSink, vertexList, numPoints);
		vBorrowedIndicesSink.deactivate();

		

		pointGeomSink.deactivate();
		vertexDataSink.deactivate();
		
		writePointInfoFDL(baseFileName, numPoints);
		
		
		ISBounds topoBounds= new ISBounds(new IntegerIndexSpaceID(8,8));
		Partitioning subGridPartitioning=new RegularPartitioning(topoBounds, this.bounds);

		
		WriteableInCoreArrayCellGroupGrid ownedSubGrid=new WriteableInCoreArrayCellGroupGrid(subGridPartitioning, null);
		WriteableInCoreArrayCellGroupGrid borrowedSubGrid=new WriteableInCoreArrayCellGroupGrid(subGridPartitioning, null);	
		
			
		int i,n=this.ownedCells.size();
		
		for(i=0; i<n; i++){
		
			ownedSubGrid.register( this.ownedCells.get(i), i);
		}

		n=this.borrowedCells.size();
		
		for(i=0; i<n; i++){
		
			borrowedSubGrid.register( this.borrowedCells.get(i), i);
		}
		
		

		
		
		
/*
{		
 n=this.ownedCells.size();

Point [] coords;

for(i=0; i<n; i++){

	coords=((Cell)this.ownedCells.get(i)).getVertexCoords();
	
	System.out.println(coords[0]+" "+coords[1]+" "+coords[2]);

}


		
n=this.borrowedCells.size();

for(i=0; i<n; i++){

	coords=((Cell)this.borrowedCells.get(i)).getVertexCoords();
	
	System.out.println(coords[0]+" "+coords[1]+" "+coords[2]);

}
		
}		
		
*/		
		
		
		
		
		
//ownedSubGrid.dump(vIndices, points, data, numAttributes);
		ownedSubGrid.write(new String(baseFileName+".owned"));
		
		// Borrowed cells should be written relative to the borrowedVIndices File?
//borrowedSubGrid.dump(vIndices, points, data, numAttributes);
		borrowedSubGrid.write(new String(baseFileName+".borrowed"));
		
		
		this.empty();
		
		return numCells;
	}
	
	
	protected int writeTriangles(	ArrayList triangles, 
									OneDDataSink vIndicesSink, 
									OneDDataSink pointGeomSink, 
									OneDDataSink vertexDataSink, 
									AList vertexList,
									int currentIndex){
	
		int newIndex;
        int numCells=triangles.size();
		TriangleCell theCell;
        IndexSpaceID [] vertexIDs;
		Datum [] cellDatums;
		Datum  pointInfoDatum;
		Point [] cellPoints;
		int [] vertexInts = new int[3];

		//RecordSpec pointRS=new RecordSpec(new int [] {0,1});
		//RecordSpec dataRS = new RecordSpec(new int [] {2});
		

        for ( int i=0;i<numCells;i++){
                
            theCell=(TriangleCell) triangles.get(i);
            vertexIDs=theCell.getVertexIndices();

			cellDatums=theCell.getData();
			cellPoints=theCell.getVertexCoords();
			
			for ( int j=0; j<cellDatums.length; j++){
			
				vertexInts[j] = vertexIDs[j].getCoord(0);
				newIndex=vertexList.lookup(vertexInts[j]);
			
				
				if(newIndex == -1){
					// combine points and data
					// pointInfoDatum=new FloatDatum(new Datum[]{cellPoints[j].makeDatum(),cellDatums[j]});

					pointInfoDatum=cellPoints[j].makeDatum();

					vertexList.insert(vertexInts[j], currentIndex);

					pointGeomSink.write(pointInfoDatum); 
					vertexDataSink.write(cellDatums[j]);
					
					
					vertexInts[j]=currentIndex++;
					
				} else {
					vertexInts[j]=newIndex;
				}
				
				
			}
			
//System.out.println(new FloatDatum(vertexInts));

			//vIndicesSink.write(new FloatDatum(vertexInts), vIndicesIterator);
			//rdb vIndicesSink.write(new FloatDatum(vertexInts));
			vIndicesSink.write(new IntDatum(vertexInts));

			
			//vIndicesIterator.next();

        }
	
	
		return currentIndex;
	}
	
	
	public int readCells(){
	
//System.out.println("Calling FSCG:readCells()");	
//this.numCells=524416;
		//tempVindicesSink.close();
		
		
		DataSource ownedVerticesDS=new PointOrderBFDS("OwnedvIndices", this.vIndicesRD,
											new ISBounds(new IntegerIndexSpaceID(0),new IntegerIndexSpaceID(this.numOwnedCells-1)),
											null,
											ownedTempVIndicesName,
											PhysicalDataSource.BIG_ENDIAN);
		
		DataSource borrowedVerticesDS=new PointOrderBFDS("BorrowedvIndices", this.borrowedCellRD,
											new ISBounds(new IntegerIndexSpaceID(0),
													new IntegerIndexSpaceID(this.numBorrowedCells-1)),
											null,
											borrowedTempVIndicesName,
											PhysicalDataSource.BIG_ENDIAN);
											
// If there are zero borrowed cells, the volume of the bounds is 2!!!!!!
	
		this.backedByFile=true;
		
		
		DataSource dataDS=DataSource.create( "pointInfo", vInfoFdlName );
		//DataSource verticesDS=DataSource.createPhysicalDataSource(vIndexFdlName, "vertex Indices");
		TriangleCell theCell;		
		
		int numOwnedCells=ownedVerticesDS.volume();
		
		if(ownedVerticesDS.getBounds().copyLower().getCoord(0) == -1){
			numOwnedCells=0;
		}

//System.out.println("FSCG:readCells(): numOwnedCells="+numOwnedCells);


		this.ownedCells=new ArrayList<TriangleCell>(numOwnedCells);
		
		dataDS.activate();
		ownedVerticesDS.activate();
		
		for ( int i=0; i<numOwnedCells; i++){
                			
            theCell=new TriangleCell(	ownedVerticesDS, dataDS, 
										this.vIndicesRS, this.coordsRS, this.dataRS,
										this.dimensionality,
										new IntegerIndexSpaceID(i));
										
			this.ownedCells.add(theCell);
        }

		//ownedVerticesDS.close();
		int numBorrowedCells=borrowedVerticesDS.volume();

        if(borrowedVerticesDS.getBounds().copyLower().getCoord(0) == -1){
			numBorrowedCells=0;
		}
		
		if(this.numBorrowedCells != numBorrowedCells){
			System.out.println("                     ****numBorrowedCells is wrong.****");
		}
		

//System.out.println("FSCG:readCells(): numBorrowedCells="+numBorrowedCells);
		
		this.borrowedCells=new ArrayList<TriangleCell>(numBorrowedCells);
			
		if (this.numBorrowedCells !=0 ){
	
			borrowedVerticesDS.activate();
			
			//CellID borrowedCellID;
			RecordSpec borrowedTempCellIDRS   =new RecordSpec(new int [] {0,1,2});
			RecordSpec borrowedTempVIndicesRS =new RecordSpec(new int [] {3,4,5});

			ISBounds iterBounds = new ISBounds( new IntegerIndexSpaceID(0), 
			                                    new IntegerIndexSpaceID(numBorrowedCells-1));
			ISIterator borrowedCellsIterator=new ISIterator(iterBounds);
			
			for ( borrowedCellsIterator.init(); 
			      borrowedCellsIterator.valid(); 
			      borrowedCellsIterator.next())
			{
			/**********
			 for( int i=0;i<numBorrowedCells;i++){
				borrowedVerticesDS.datum(d, borrowedCellsIterator);
				borrowedCellID=new CellID(d, this.dimensionality);
				this.borrowedCells.add(borrowedCellID);
	          ****************/			

				theCell=new Borrowed2DTriangleCell(borrowedVerticesDS, dataDS, 
											borrowedTempVIndicesRS, coordsRS, this.dataRS, borrowedTempCellIDRS,
											borrowedCellsIterator);
//System.out.println(theCell);
				this.borrowedCells.add(theCell);
			}		
	}	

//System.out.println("FSCG:readCells(): finished reading cells");	
		this.inMemory=true;
		//borrowedVerticesDS.close();
		return numBorrowedCells+numOwnedCells;
	
	}

	
	
	
	public void empty(){

//System.out.println("Calling FSCG:empty()");	
	
		this.borrowedCells=null;
		this.ownedCells=null;
		this.inMemory=false;
	}

	private void writeVIndicesFDLs(String baseFileName){
		
		try {
		
			// Owned Cells
			String fdlFileName=new String(baseFileName + ".owned.tri.xfdl");
			String splitVIndexFileName=new String(baseFileName + ".owned.tri");
			
			FileWriter fw=new FileWriter(fdlFileName);
			
			
			
			fw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			fw.write("<!DOCTYPE FileDescriptor SYSTEM \"/home/graphics/sdb/config/dtd/fdl.dtd\">\n");
			fw.write("<FileDescriptor filename= \""+splitVIndexFileName+"\"  filetype=\"binary\" recordsize=\"12\">\n");
			
			fw.write("<Bounds lower= \"0\"    upper= \""+(this.numOwnedCells-1)+"\"/>\n");
	
		
			fw.write("<Field fieldname= \"v0\"   fieldtype= \"int\"/>\n");
			fw.write("<Field fieldname= \"v1\"   fieldtype= \"int\"/>\n");
			fw.write("<Field fieldname= \"v2\"   fieldtype= \"int\"/>\n");
	
			fw.write("</FileDescriptor>\n");
		
			fw.close();
			
			
			// Borrowed
			
			fdlFileName=new String(baseFileName + ".borrowed.tri.xfdl");
			splitVIndexFileName=new String(baseFileName + ".borrowed.tri");
			
			fw=new FileWriter(fdlFileName);
			
			
			
			fw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			fw.write("<!DOCTYPE FileDescriptor SYSTEM \"/home/graphics/sdb/config/dtd/fdl.dtd\">\n");
			fw.write("<FileDescriptor filename= \""+splitVIndexFileName+"\"  filetype=\"binary\" recordsize=\"12\">\n");
			
			fw.write("<Bounds lower= \"0\"    upper= \""+(this.numBorrowedCells-1)+"\"/>\n");
	
		
			fw.write("<Field fieldname= \"v0\"   fieldtype= \"int\"/>\n");
			fw.write("<Field fieldname= \"v1\"   fieldtype= \"int\"/>\n");
			fw.write("<Field fieldname= \"v2\"   fieldtype= \"int\"/>\n");
	
			fw.write("</FileDescriptor>\n");
		
			fw.close();
			
			
			
		} catch (IOException e){
		
			new FileError(SDBError.Recoverable,"Couldn't write fdl file:"+e.getMessage());
		
		}
	}
	
	private String ownedCellsTempFileName(String vIndicesFDLName, IndexSpaceID partitionID){
	
		String tempName=new String(vIndicesFDLName);
		StringBuffer suffix=new StringBuffer();
		int i, d=partitionID.dim();
		
		for(i=0; i<d; i++){
		
			suffix.append(partitionID.getCoord(i));
		}
		
		tempName=tempName.replaceAll("xfdl", new String(suffix)+".owned.temp");
		tempName=tempName.replaceAll("xml", suffix+".temp"); // just in case

	
		return new String(tempName);
	}


	private String borrowedCellsTempFileName(String vIndicesFDLName, IndexSpaceID partitionID){
	
		String tempName=new String(vIndicesFDLName);
		StringBuffer suffix=new StringBuffer();
		int i, d=partitionID.dim();
		
		for(i=0; i<d; i++){
		
			suffix.append(partitionID.getCoord(i));
		}
		
		tempName=tempName.replaceAll("xfdl", new String(suffix)+".borrowed.temp");
		tempName=tempName.replaceAll("xml", suffix+".temp"); // just in case

	
		return new String(tempName);
	}


	private void writePointInfoFDL(String baseFileName, int numPoints){
		
		try {
			String fdlFileName=new String(baseFileName + ".data.xfdl");
			String splitDataFileName=new String(baseFileName + ".data");

			FileWriter fw=new FileWriter(fdlFileName);
			AttributeDescriptor attr;
			int i;
			//int  numFields=this.pointInfoRD.size();
			
			RecordDescriptor pointsRD=this.pointInfoRD.project(new RecordSpec(new int[] {0,1}));
			int numFields=pointsRD.size();
			
			fw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			fw.write("<!DOCTYPE FileDescriptor SYSTEM \"/home/graphics/sdb/config/dtd/fdl.dtd\">\n");
			//fw.write("<FileDescriptor filename= \""+splitDataFileName+"\"  filetype=\"binary\" recordsize=\""+this.pointInfoRD.byteSize()+"\">\n");
			fw.write("<FileDescriptor filename= \""+splitDataFileName+"\"  filetype=\"binary\" recordsize=\""+pointsRD.byteSize()+"\">\n");
			fw.write("<Bounds lower= \"0\"    upper= \""+(numPoints-1)+"\"/>\n");
	

			for(i=0; i < numFields; i++){
				
				//attr=this.pointInfoRD.getIndexedAttribute(i);
				attr=pointsRD.getAttribute(i);
				fw.write("<Field fieldname= \""+ attr.name()  +"\"   fieldtype= \""+attr.getTypeName()+"\"/>\n");
			
			}
				
			fw.write("</FileDescriptor>\n");
		
			fw.close();
			


// This next part is just temporary

			fdlFileName=new String(baseFileName + ".attr.xfdl");
			splitDataFileName=new String(baseFileName + ".attr");

			fw=new FileWriter(fdlFileName);
			//AttributeDescriptor attr;
			//int i;
			//int numFields=this.pointInfoRD.size();
			RecordDescriptor attrRD=this.pointInfoRD.project(new RecordSpec(new int[] {2}));
			numFields=attrRD.size();

			
			fw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			fw.write("<!DOCTYPE FileDescriptor SYSTEM \"/home/graphics/sdb/config/dtd/fdl.dtd\">\n");
			fw.write("<FileDescriptor filename= \""+splitDataFileName+"\"  filetype=\"binary\" recordsize=\""+attrRD.byteSize()+"\">\n");
			
			fw.write("<Bounds lower= \"0\"    upper= \""+(numPoints-1)+"\"/>\n");
	

			for(i=0; i < numFields; i++){
				
				attr=attrRD.getAttribute(i);
				
				fw.write("<Field fieldname= \""+ attr.name()  +"\"   fieldtype= \""+attr.getTypeName()+"\"/>\n");
			
			}
				
			fw.write("</FileDescriptor>\n");
		
			fw.close();
			
			
		} catch (IOException e){
		
			new FileError(SDBError.Recoverable,"Couldn't write fdl file: "+e.getMessage());
		
		}
	}

	
    /** Return a String representing the object. */
    public String toString(){
        
        return this.toString("");
    }    
    
    /** Return a String representing the object indented by the argument string. */
    public String toString(String indent){
    
        String indent2=indent+Globals.toStringIndent;    
        int i;
		int numOwnedCells=this.ownedCells.size();
		//int numBorrowedCells=this.borrowedCells.size();
		
        StringBuffer result=new StringBuffer(indent+"{ FileSplitterCellGroup: contains ");

        result.append(numOwnedCells);
        result.append(" owned triangles and ");
		result.append(numOwnedCells);
		result.append("  borrowed triangles.\n");

		result.append("bounds=");
        result.append(this.bounds);
        result.append("\n");
        
		result.append("Owned Triangles:");
        for(i=0;i<numCells;i++){
            result.append(((Cell) this.ownedCells.get(i)).toString(indent2));
            result.append("\n");
        }
		
		result.append("\nBorrowed Triangles:");
        for(i=0;i<numCells;i++){
            result.append(((Cell) this.borrowedCells.get(i)).toString(indent2));
            result.append("\n");
        }
        
        result.append(indent+"}");
        
        return new String(result);
    }
    
}
