//
//  FileSplitterCellGroupGrid.java
//  Unstructured
//
//  Created by Philip Rhodes on Mon Aug 12 2002.
//  Copyright (c) 2001 University of New Hampshire. All rights reserved.
//

package edu.unh.sdb.lattice;

import edu.unh.sdb.datasource.*; 
import edu.unh.sdb.common.*;


public class FileSplitterCellGroupGrid extends FileCellGroupGrid{
	
	String vIndicesFDLName;
	String pointInfoFDLName;
	String dataFileBaseName;
	
	RecordSpec vIndicesRS;
	RecordSpec pointRS;
	RecordSpec dataRS;
	
 /*   
	protected FileSplitterCellGroupGrid(){
	 
	}
 */   
    public FileSplitterCellGroupGrid(Partitioning partitioning, AxisOrdering ordering, String vIndicesFDLName, RecordSpec vIndicesRS, String pointInfoFDLName, RecordSpec pointRS, RecordSpec dataRS){
    
		super(partitioning, ordering, null);
		
		this.vIndicesFDLName=vIndicesFDLName;
		this.pointInfoFDLName=pointInfoFDLName;
		
		this.vIndicesRS=vIndicesRS;
		this.pointRS=pointRS;
		this.dataRS=dataRS;
		
        makeCellGroups();
    }
 
	
	protected void makeCellGroups(){
	
		ISBounds pBounds=this.partitioning.getISBounds();
        
        this.groups=new CellGroup[pBounds.volume()];

        GBoundsIterator partitionIterator=this.partitioning.getPartitionIterator();
        ISIterator tIter=this.partitioning.getISIterator();
        
        //rdb FDLDescriptor fdld=DataSource.createFDLDescriptor(this.pointInfoFDLName);
        FDLDescriptor fdld = FDLDescriptor.create( this.pointInfoFDLName );
		RecordDescriptor rd=fdld.getRecordDescriptor();
		
		this.dataFileBaseName=fdld.getDataFileURL();
		
int count=0;		

        for(partitionIterator.init(); partitionIterator.valid(); partitionIterator.next(),tIter.next()){
        			
			
			groups[(int)AxisOrdering.computeOffset(tIter, this.axisWeights)]=new FileSplitterCellGroup(partitionIterator.copyBounds(), 
																				rd,
																				this.vIndicesFDLName, 
																				this.vIndicesRS,
																				this.pointInfoFDLName,
																				this.pointRS, 
																				this.dataRS,
																				tIter);
//System.out.println(partitionIterator);
count++;
        }
			
			
if(count != pBounds.volume()){
	System.out.println("Problem in partition iterator.");
}
			
			
	}
    
	
	public void doneRegistering(){
	
		int i, n=this.groups.length;
		
		for(i=0; i < n; i++){
		
			((FileSplitterCellGroup) this.groups[i]).doneRegistering();
		}
	
	
	}
	
	
    
	
	public void writeCells(){
		
		ISIterator iter=this.partitioning.getISIterator();
		long groupIndex;
		int count=0, starSize=(int)(this.groups.length / 20.0 + 0.5);
											
		System.out.println("\nWriting Cells:   |0  25   50   75  100|");
		System.out.print(  "                  ");
		for(iter.init(); iter.valid(); iter.next()){
			
			groupIndex=AxisOrdering.computeOffset(iter, this.axisWeights);
			
			((FileCellGroup)this.groups[(int)groupIndex]).writeCells(dataFileName(this.dataFileBaseName,iter));
			
			if((++count) % starSize == 0) {
				System.out.print("*");
			}
		}
		System.out.println();
	}
	
	
	
	
	protected static String dataFileName(String baseFileName, IndexSpaceID id){
	
		int i, d=id.dim();
		StringBuffer buf=new StringBuffer(baseFileName);
		
		//buf.append("-");
		for(i=0; i < d; i++){
			
			buf.append(id.getCoord(i));
			buf.append("x");
		}
		
		
		return new String(buf);
	}
	
	
	
	
	
	/*
	public void assignCellGroupSources(){
	
		ISBounds gridBounds=this.partitioning.getISBounds();
		ISIterator iter=new ISIterator(gridBounds);
		RecordDescriptor rd=new RecordDescriptor( 
							new AttributeDescriptor [] {
									new AttributeDescriptor(StorageType.Int, "v0"),
									new AttributeDescriptor(StorageType.Int, "v1"),
									new AttributeDescriptor(StorageType.Int, "v2")
							}
		);
		
		ISBounds sourceBounds=new ISBounds(new IntegerIndexSpaceID(0), new IntegerIndexSpaceID(0));
		int groupIndex;
		String filename;
		DataSource source;
		
		
		
		
		for(iter.init(); iter.valid(); iter.next()){
			
			filename=new String("Foo"+iter);
			
			groupIndex=AxisOrdering.computeOffset(iter, this.axisWeights);
						
			source=DataSource.createPhysicalDataSource(filename,filename);
			//source=new BinaryFileDataSource(filename, rd, sinkBounds);
			
			source.activate();
			
			this.groups[groupIndex].writeCells(sink, 0);
		
		}
	}
	*/
    

    /** Return a String representing the object. */
    public String toString(){
        
        return this.toString("");
    }    
    
    /** Return a String representing the object indented by the argument string. */
    public String toString(String indent){
    
        int i,numCellGroups=this.groups.length;
        String indent2=indent+Globals.toStringIndent; 
        StringBuffer result=new StringBuffer(indent+"{ FileSplitterCellGroupGrid: contains ");

        
        result.append(numCellGroups);
        result.append("cell Groups\n");
        
        result.append(indent2+"geometric bounds=");
        result.append(this.partitioning.getGBounds().toString(indent2)+"\n");
        result.append(indent2+"topological bounds=");
        result.append(this.partitioning.getISBounds().toString(indent2)+"\n");

        for(i=0;i<numCellGroups;i++){
            result.append(indent2+i);
            result.append(") ");
            result.append(this.groups[i].toString(indent2));
            result.append("\n");
        }
        
        result.append(indent2+"}");
        return new String(result);
    }


}
