//
//  InCoreArrayCellGroup.java
//  Unstructured
//
//  Created by Philip Rhodes on Tue Sep 02 2003.
//  Copyright (c) 2003 University of New Hampshire. All rights reserved.
//
// ------------- Edit history since 01/10/05 ----------------------------
//
// 01/10/05 rdb: replaced use of FloatDatum with IntDatum where appropriate.
//
// ----------------------------------------------------------------------

package edu.unh.sdb.lattice;

import edu.unh.sdb.common.*;
import edu.unh.sdb.datasource.*;
//import java.io.*;

public class InCoreArrayCellGroup extends CellGroup{


	protected int[] cellIndices;
	protected int [] vIndices;
	protected float [] coords;
	protected float [] data;
	protected int numAttributes;
	
protected int lastIndex=-1;

	public InCoreArrayCellGroup( GBounds bounds, 
	                             int [] vIndices, float [] coords, float[] data, 
	                             int numAttributes, 
	                             DataBlock cellIndicesBlock, 
	                             ISIterator iter )
	{
		super(bounds);
	
		this.vIndices=vIndices;
		this.coords=coords;
		this.data=data;
		this.numAttributes=numAttributes;
		
		readCellIndices(cellIndicesBlock, iter);
	}
	
	private void readCellIndices(DataBlock cellIndicesBlock, ISIterator iter){
	
		Datum datum = new IntDatum(1);  //rdb changed from FloatDatum
		int i,n;
		
		cellIndicesBlock.datum(datum,iter);
		iter.next();
				
		n = datum.getInt(0);  // rdb: was: (int) datum.getFloat(0);
		this.cellIndices=new int[n];

		for(i=0; i<n; i++){
		
			cellIndicesBlock.datum(datum,iter);
			this.cellIndices[i] = datum.getInt(0); //rdb: was: (int) datum.getFloat(0);
			iter.next();
		}
	}

	void dump() {
					
		int i, n=this.cellIndices.length;
		TriangleCell c;
		
		for(i=0; i<n; i++){
			
			c=new TriangleCell(vIndices, coords, data, 1, dimensionality,this.cellIndices[i]);

			System.out.println(c);
		}
	}
	
	public boolean register(int cellIndex){
	
		new InternalFaultError(SDBError.Recoverable, 
		      "InCoreArrayCellGroup.register() was called.");
		return false;
    }

    public Cell findEnclosingCell(Point p){
   
        int i;
		int numCells=this.cellIndices.length;
		int cellIndex;
		
        for(i=0;i<numCells;i++){
			
			cellIndex=this.cellIndices[i];
			
			if(TriangleCell.inside(p, this.vIndices, this.coords, cellIndex)){
				return new TriangleCell(this.vIndices, this.coords, this.data, this.numAttributes, this.dimensionality, cellIndex);
			} 
        }
        return null;
    }
    
	public final int getNumCells(){
	
		return this.cellIndices.length;
	}
}
