//
//  InCoreArrayCellGroupGrid.java
//  Unstructured
//
//  Created by Philip Rhodes on Sat Sep 06 2003.
//  Copyright (c) 2003 University of New Hampshire. All rights reserved.
//
//------------------- Edit history --------------------------------------
// 10/21/05 rdb: removed unused variables
//-----------------------------------------------------------------------

package edu.unh.sdb.lattice;

import edu.unh.sdb.datasource.*; 
import edu.unh.sdb.common.*; 

public class InCoreArrayCellGroupGrid extends CellGroupGrid {


	protected int [] vIndices;
	protected float [] coords;
	protected float [] data;
	
	protected int numAttributes;
	protected FDLDescriptor fdld;
	protected GBounds gBounds;
	

	public InCoreArrayCellGroupGrid(FDLDescriptor fdld, GBounds gBounds, AxisOrdering ordering, int [] vIndices, float [] coords, float[] data, int numAttributes){
	
		super(ordering);

		this.fdld=fdld;

		this.vIndices= (vIndices == null) ? new int[1] : vIndices;
		
		this.coords=coords;
		this.data=data;
		this.numAttributes=numAttributes;
	
		this.gBounds=gBounds;
		
		this.makeCellGroups();
		
	}





	protected void makeCellGroups(){
	
		int d;	
		
		Datum datum=new FloatDatum(1);

		DataSource ds=DataSource.create( "subgrid", this.fdld );
		ISBounds fileBounds=ds.getBounds();
		ISIterator cellIndexIter=new ISIterator(fileBounds);
		BasicBlock theBlock=new BasicBlock(fileBounds, ds.getRecordDescriptor());

		ds.activate();
		ds.subblock(theBlock, fileBounds);

		theBlock.datum(datum,cellIndexIter);
		d=(int) datum.getFloat(0);
		cellIndexIter.next();
		
		int [] dimensions=new int[d];
		
		for ( int i=0; i<d; i++){
		
			theBlock.datum(datum,cellIndexIter);
			dimensions[i]=(int) datum.getFloat(0);
			cellIndexIter.next();
		}
		
		ISBounds pBounds=new ISBounds(dimensions);
		
		
		
        this.groups=new CellGroup[pBounds.volume()];

		this.setPartitioning(new RegularPartitioning(pBounds, this.gBounds));
		
		
		GBoundsIterator partitionIterator=this.partitioning.getPartitionIterator();
		//ISIterator tIter=new ISIterator(pBounds);
		ISIterator tIter=this.partitioning.getISIterator();
		
		
        for(partitionIterator.init(); partitionIterator.valid(); partitionIterator.next(), tIter.next()){
			
			groups[(int)AxisOrdering.computeOffset(tIter, this.axisWeights)]=new InCoreArrayCellGroup(
																	partitionIterator.copyBounds(), 
																	this.vIndices, 
																	this.coords,
																	this.data,
																	this.numAttributes,
																	theBlock,
																	cellIndexIter
																);
			
            
        }
			
	}


	void dump(){

		ISIterator iter=this.partitioning.getISIterator();
		
        for(iter.init(); iter.valid();  iter.next()){
			
			((InCoreArrayCellGroup)(groups[(int)AxisOrdering.computeOffset(iter, this.axisWeights)])).dump();
			
        }

	}




	public void registerCells(){

		int i,n=this.vIndices.length / 3;
		ISBounds tBoundingBox;
		GBounds gBoundingBox;
		ISIterator tIterator;

		
		for(i=0; i<n; i++){
		
			gBoundingBox=TriangleCell.boundingBox(this.vIndices, this.coords, 2, i);
			tBoundingBox=this.partitioning.map(gBoundingBox);
			tIterator=new ISIterator(tBoundingBox);
			
			for(tIterator.init(); tIterator.valid(); tIterator.next()){
			
				((InCoreArrayCellGroup)this.mapToCellGroup(tIterator)).register(i);
			}
			
		}

	}
	
	public Cell findEnclosingCell(Point p){

		IndexSpaceID id=this.partitioning.map(p);

        CellGroup theGroup=this.mapToCellGroup(id);
		
		return theGroup.findEnclosingCell(p);
	
	}
	
}
