//
//  MRLattice.java
//  MRLatticeTest
//
//  Created by Philip Rhodes on Mon Feb 03 2003.
//  Copyright (c) 2003 University of New Hampshire. All rights reserved.
//

package edu.unh.sdb.lattice; 

//import java.io.*;
import edu.unh.sdb.datasource.*;
import edu.unh.sdb.common.*;



public class MRLattice extends Lattice{

    protected Lattice [] lattices;
    //Lattice [] errorLattices;
    
    protected int level=0;
    
    

    public MRLattice(String name, String fileNames[], GBounds gBounds, Approximator a, boolean isCellLattice){
    
        super(name);
        
        int i;
        int numLattices=fileNames.length;
        
        Topology t;
        DataSource ds;
        Geometry g;
        
        this.lattices=new Lattice[numLattices];
        
        for(i=0; i<numLattices; i++){
        
            ds=DataSource.create( "foo", fileNames[i] );
			
			if(isCellLattice){
				g=new RegularCellGeometry(ds.getBounds(), gBounds);			
			} else {
				g=new RegularPointGeometry(ds.getBounds(), gBounds);
			}
			
            t=new RectilinearTopology(ds, g);
            
            this.lattices[i]=new Lattice("foo", t, g, a);
        }
    
    }
    
    
    public void activate(){
    
        int i, n=this.lattices.length;
        
        for(i=0; i<n; i++){
        
            this.lattices[i].activate();
        }
    
    }
    

    public void setLevel(int level){
    
        if(level < this.lattices.length || level < 0){
            this.level=level;
        } else {
            new IllegalArgumentError(SDBError.Recoverable, "level argument is out of range.");
        }
    }


    public int getLevel(){
    
        return this.level;
    }



    public Datum datum(Point p){
 
        return this.lattices[this.level].datum(p);
    }
    
    

}
