//
//  Partitioning.java
//  Unstructured
//
//  Created by Philip Rhodes on Tue Oct 01 2002.
//  Copyright (c) 2002 University of New Hampshire. All rights reserved.
//
//------------------- Edit history --------------------------------------
// 10/21/05 rdb: removed unused variables
//-----------------------------------------------------------------------

package edu.unh.sdb.lattice;

import edu.unh.sdb.datasource.*; 

/** A partitioning provides a mapping between an N-Dimensional geometric (real) space 
	and an integer Index Space. */

public abstract class Partitioning {

	protected int dimensionality;
	//protected ISBounds indexBounds;
    //protected GBounds geomBounds;
   
	int[] 		indexLowerBounds;
	int[] 		indexUpperBounds;
	float []	geomLowerBounds;
	float []	geomUpperBounds;
	
    public Partitioning(ISBounds indexBounds, GBounds geomBounds){

        int d=indexBounds.dim();
        
        this.dimensionality=d;
    
        this.indexLowerBounds=indexBounds.copyLowerArray(new int[d]);
		this.indexUpperBounds=indexBounds.copyUpperArray(new int[d]);
		
        this.geomLowerBounds=geomBounds.getLowerArray(new float[d]);
		this.geomUpperBounds=geomBounds.getUpperArray(new float[d]);
		
    }
    
    public final int dim(){
    
        return this.dimensionality;
    }
    
    public ISBounds getISBounds(){
        
        return new ISBounds(new IntegerIndexSpaceID(this.indexLowerBounds), new IntegerIndexSpaceID(this.indexUpperBounds));
    }
    
    public GBounds getGBounds(){
    
         return new GBounds(new Point(this.geomLowerBounds), new Point(this.geomUpperBounds));
    }
    
	
    public abstract GBoundsIterator getPartitionIterator();
	
	
	
	public ISIterator getISIterator(){
	
		return new ISIterator(this.getISBounds());
	}
	
    
    public abstract IndexSpaceID map(Point p);

    public abstract ISBounds map(GBounds b);
    
    
    /** Return a String representing the object. */
    public String toString(){
        
        return this.toString("");
    }    
    
    /** Return a String representing the object indented by the argument string. */
    public String toString(String indent){
    
        StringBuffer result=new StringBuffer(indent+"dimensionality="+this.dimensionality+"\n");
        
        result.append(indent+"indexbounds=");
        result.append(this.getISBounds()+"\n");
        result.append(indent+"geomBounds=");
        result.append(this.getGBounds());
        
        return new String(result);
    }
}
