//
//  RectilinearTopology.java
//  Unstructured
//
//  Created by Philip Rhodes on Mon Sep 30 2002.
//  Copyright (c) 2002 University of New Hampshire. All rights reserved.
//


package edu.unh.sdb.lattice; 

import edu.unh.sdb.datasource.*;
import edu.unh.sdb.common.*;


/** A Topology for Rectilinear data.

*/
public class RectilinearTopology extends Topology {

    protected DataSource ds;
    protected ISBounds bounds;
    
    int   [] topoDimensions;
    int	  [] topoOrigin;
    float [] geomDimensions;
    float [] geomOrigin;
    
    float [] geomCellDimensions;

    
    public RectilinearTopology(DataSource ds, Geometry g){

        super(ds.dim());
        
        int i;
        
        //this.bounds=ds.getBounds(); // Is this the bounds we want?
		this.bounds=g.getPartitioning().getISBounds();
        this.ds=ds;
        
        GBounds geomBounds=g.getBounds();
        int gDim=g.dim();
        
        this.topoDimensions=this.bounds.getDimensions(new int[this.bounds.dim()]);
        
        
        
        this.topoOrigin=this.bounds.copyLower().getCoords(new int[this.bounds.dim()]);
        
        this.geomDimensions=geomBounds.getDimensions().getCoords();
        this.geomOrigin=geomBounds.getLower().getCoords();
        
        
        this.geomCellDimensions=geomBounds.getDimensions().getCoords();
        
        for(i=0;i<gDim;i++){
            this.geomCellDimensions[i] = this.geomDimensions[i] / this.topoDimensions[i];
        }
        
    }
    
    
    
    public void activate(){
    
        this.ds.activate();
    }
    

    public Cell findEnclosingCell(IndexSpaceID partitionID, Point p){
    
    
        int i,v, d=this.dimensionality,numVertices=(int)Math.pow(2,d);  
        int mask;
        
        int [] upperAxes=partitionID.getCoords(new int[partitionID.dim()]);
        int [] axes=new int[upperAxes.length];
        
        Datum [] data=new FloatDatum[numVertices];
        Point [] vertices=new Point[numVertices];
        float [] vertexFloats=new float[this.geomDimensions.length];
        
        IndexSpaceID subBlockIndex;
      
        
        for(i=0;i<d;i++){
            upperAxes[i]++;
        }
    
        ISBounds cellBounds=new ISBounds(partitionID,new IntegerIndexSpaceID(upperAxes));
        
        DataBlock cellBlock = ds.subblock(cellBounds);

      
    
    
        for(v=0;v<numVertices;v++){
        
			//We must compute the coordinates of each vertex. Since we already have the upperAxes
			// array, let's use a bitmask to tell us when to use a coordinate value from that array,
			// and when to subtract one from that coordinate value.
			
            mask=1;
            for(i=0;i<d;i++){
			
                //axes[i] = ((v & mask) != 0) ? upperAxes[i] : upperAxes[i] - 1;
				//vertexFloats[i] = axes[i] * this.geomCellDimensions[i];
				
				axes[i] = ((v & mask) != 0) ? 1 : 0;
				vertexFloats[i] = (upperAxes[i]-1) * this.geomCellDimensions[i] + axes[i] * this.geomCellDimensions[i];
				
				//vertexFloats[i] = (int)Math.floor(p.getCoord(i) / this.geomCellDimensions[i]) * this.geomCellDimensions[i] + axes[i] * this.geomCellDimensions[i];
                     
                       
                mask <<= 1;
            }
			
            subBlockIndex=new IntegerIndexSpaceID(axes); //FIXME: Can we augment ISID.assign() to work
                                                         // with arrays? Fast, but dangerous.
                                                         
			//System.out.println(subBlockIndex);												
														 
            data[v]=cellBlock.datum(subBlockIndex);
            vertices[v]=new Point(vertexFloats);
			
			//System.out.println(subBlockIndex+"  "+vertices[v]);											 

			
        }
    
        //rdb remove superfluous argument: return new ArrayCell(vertices, data, d);
        return new ArrayCell(vertices, data );
    
    }
	
	
	/** return a datum corresponding to the IndexSpaceID argument. */
	public Datum datum(IndexSpaceID id){
	
		return ds.datum(id);
	}

    public String toString(){
        
        return this.toString("");
    }


	/** Return a bounds for the Topology. */
	public ISBounds getBounds(){
	
		return (ISBounds) this.bounds.clone();
	}


	/** Return a RecordDescriptor for datums returned by this topology.*/														
	public RecordDescriptor getRecordDescriptor(){
	
		return this.ds.getRecordDescriptor();
	}


    public String toString(String indent){
    
        String indent2=indent+Globals.toStringIndent;    
        StringBuffer result=new StringBuffer(indent+"{ Rectilinear Topology:\n");

        
        result.append(super.toString(indent2));
        result.append(this.ds.toString(indent2));
        
        return new String(result);
        
    }

}
