//
//  SplitCellGroupGrid.java
//  Unstructured
//
//  Created by Philip Rhodes on Mon Aug 12 2002.
//  Copyright (c) 2001 University of New Hampshire. All rights reserved.
//
//------------------- Edit history --------------------------------------
// 10/21/05 rdb: removed unused variables
//-----------------------------------------------------------------------

package edu.unh.sdb.lattice;

import edu.unh.sdb.datasource.*; 
import edu.unh.sdb.common.*;


public class SplitCellGroupGrid extends FileCellGroupGrid {

	class GroupList {
	
		private FileArrayCellGroup[] groups;
		int size;
		int freePos;
		int numGroups;
		int oldestGroup;
		
		GroupList(int size){
		
			this.size=size;
			this.groups=new FileArrayCellGroup[size];
			this.numGroups=0;
			
			this.freePos=0;
			this.oldestGroup=0;
		}
		
		
		
		void insert(FileArrayCellGroup g){
		
			this.groups[freePos]=g;
			
			if(freePos == oldestGroup){
				if(numGroups==0){
					oldestGroup=freePos;
				} else {
					this.oldestGroup=(this.oldestGroup++) % numGroups;  // Problem here
				}
			} 
				
			freePos=(freePos+1) % size;
			this.numGroups++;
		}
		
		
		
		FileArrayCellGroup getOldest(){
		
			return this.groups[oldestGroup];
		}
		
		
		void deleteOldest(){
		
			this.groups[oldestGroup]=null;
			this.oldestGroup = (this.oldestGroup + 1) % numGroups;   // Problem here
			this.numGroups--;
		}


		boolean isFull(){
		
			return this.numGroups == this.size;
		}
		
	}
    

    String basePointInfoFileName;
    String baseVIndicesFileName;
	
	RecordSpec pointRS;
	RecordSpec dataRS;
	RecordSpec vIndicesRS;

	GroupList gList;
    
    public SplitCellGroupGrid(Partitioning partitioning, AxisOrdering ordering,  String basePointInfoFileName, String baseVIndicesFileName, RecordSpec vIndicesRS, RecordSpec pointRS, RecordSpec dataRS){
    
		super(partitioning, ordering, basePointInfoFileName);
        
		this.basePointInfoFileName=basePointInfoFileName;
		this.baseVIndicesFileName=baseVIndicesFileName;
		this.pointRS=pointRS;
        this.dataRS=dataRS;
		this.vIndicesRS=vIndicesRS;
		
		this.gList=new GroupList(10);
		
		makeCellGroups();
    }
          
	
	protected void makeCellGroups(){
        
		ISBounds pBounds=this.partitioning.getISBounds();

        this.groups=new CellGroup[pBounds.volume()];
        
        GBoundsIterator partitionIterator=this.partitioning.getPartitionIterator();
        ISIterator tIter=new ISIterator(pBounds);
        

        for(partitionIterator.init(); partitionIterator.valid(); partitionIterator.next(),tIter.next()){
			
			groups[(int)AxisOrdering.computeOffset(tIter, this.axisWeights)]=new FileArrayCellGroup(partitionIterator.copyBounds(),
																						dataFileName(this.baseVIndicesFileName,tIter)+".tri.xfdl", 
																						this.vIndicesRS,
																						 
																						dataFileName(this.basePointInfoFileName,tIter)+".data.xfdl", 
																						this.pointRS,
																						this.dataRS);
        }
		

	}
	
	
	public Cell findEnclosingCell(IndexSpaceID id, Point p){	
	
		FileArrayCellGroup theGroup=(FileArrayCellGroup) this.getGroup(id);
		
		return theGroup.findEnclosingCell(p);
	
	}
	
	
	TriangleCell getOwnedCell(CellID id){

		FileArrayCellGroup group = (FileArrayCellGroup) this.getGroup(id.getPartitionID());
		
		return group.getOwnedCell(id.getIndex());
	
	}
	
	
	void getOwnedCell(UnstructuredCell cell, CellID id){
	
		FileArrayCellGroup group = (FileArrayCellGroup) this.getGroup(id.getPartitionID());
		
		group.getOwnedCell(cell,id.getIndex());
	}
	
	
	
	protected CellGroup getGroup(IndexSpaceID id){

		FileArrayCellGroup theGroup=(FileArrayCellGroup) this.mapToCellGroup(id);
    
		while(true){
			try{
			
				if(theGroup.inMemory()){
					return theGroup;
				} else {
				
					if(this.gList.isFull()){
						FileArrayCellGroup oldest=this.gList.getOldest();
						this.gList.deleteOldest();
						oldest.empty();
					} 
					theGroup.readCells(); 
					this.gList.insert(theGroup);
					
					return theGroup;
	
				}
				
			} catch(OutOfMemoryError e){
			
				FileArrayCellGroup oldest=this.gList.getOldest();
				this.gList.deleteOldest();
				oldest.empty();
System.out.println("*** OOM Emptied CellGroup");
			}
		}
	
	}
	
	
	
	public void writeCells(){
	
		ISBounds gridBounds=this.partitioning.getISBounds();
		ISIterator iter=new ISIterator(gridBounds);
        /***rdb: rd never used???
		RecordDescriptor rd=new RecordDescriptor( 
							new AttributeDescriptor [] {
									new AttributeDescriptor(StorageType.Int, "v0"),
									new AttributeDescriptor(StorageType.Int, "v1"),
									new AttributeDescriptor(StorageType.Int, "v2")
							}
		);
		/*****/
        
		//rdb sinkBounds never used??
        //ISBounds sinkBounds=new ISBounds(new IntegerIndexSpaceID(0), new IntegerIndexSpaceID(0));
		long groupIndex;
		String filename;
		//DataSink sink;

        for(iter.init(); iter.valid(); iter.next()){
			
			filename=new String(basePointInfoFileName+iter);
			
			groupIndex=AxisOrdering.computeOffset(iter, this.axisWeights);
			
			((FileCellGroup)this.groups[(int)groupIndex]).writeCells(filename);
			
		}
	}
	
   
    /** Return a String representing the object. */
    public String toString(){
        
        return this.toString("");
    }    
    
    /** Return a String representing the object indented by the argument string. */
    public String toString(String indent){
    
        int i,numCellGroups=this.groups.length;
        String indent2=indent+Globals.toStringIndent; 
        StringBuffer result=new StringBuffer(indent+"{ SplitCellGroupGrid: contains ");

        
        result.append(numCellGroups);
        result.append("cell Groups\n");
        
        result.append(indent2+"geometric bounds=");
        result.append(this.partitioning.getGBounds().toString(indent2)+"\n");
        result.append(indent2+"topological bounds=");
        result.append(this.partitioning.getISBounds().toString(indent2)+"\n");

        for(i=0;i<numCellGroups;i++){
            result.append(indent2+i);
            result.append(") ");
            result.append(this.groups[i].toString(indent2));
            result.append("\n");
        }
        
        result.append(indent2+"}");
        return new String(result);
    }


}
