//
//  UnstructuredSplitterTopology.java
//  Unstructured
//
//  Created by Philip Rhodes on Mon Sep 01 2003.
//  Copyright (c) 2003 University of New Hampshire. All rights reserved.
//

package edu.unh.sdb.lattice; 

import edu.unh.sdb.datasource.*; 
import edu.unh.sdb.common.*;

public class UnstructuredSplitterTopology extends UnstructuredTopology {

	public UnstructuredSplitterTopology(String pointInfoFDLFileName, String vIndicesFDLFileName, UnstructuredGeometry geometry, int dimensionality){
	
		super(pointInfoFDLFileName, vIndicesFDLFileName, geometry, dimensionality);

		RecordSpec vIndicesRS=new RecordSpec(new int[] {0,1,2});
		RecordSpec coordsRS=new RecordSpec(new int[] {0,1});
		RecordSpec dataRS=new RecordSpec(new int[] {2});
		
	
		this.groupGrid=new FileSplitterCellGroupGrid(	this.partitioning, 
														null, 
														vIndicesFDLFileName, 
														vIndicesRS,
														pointInfoFDLFileName,
														coordsRS,
														dataRS);

	
	}
	
	
	
    public void activate(){
	
		int i;
		int count, starSize;
		
		ISBounds triangleGroupGridBounds;
		
		ISIterator gridBoundsIterator;
	
		TriangleCell cell = new TriangleCell();
				
		RecordSpec coordsRS=new RecordSpec(new int[] {0,1});
		RecordSpec vIndicesRS=new RecordSpec(new int[] {0,1,2});
		RecordSpec dataRS=new RecordSpec(new int[] {2});
		
		OwnerInfo oi=new OwnerInfo(0);
		
		FileSplitterCellGroup cellGroup;
		
		CacheDataSource vIndicesCacheDS=new UniBlockCacheDataSource("vIndicesCacheDS", this.theTriangles, new IntegerIndexSpaceID(1024));
		CacheDataSource geomInfoCacheDS=new MultiBlockCacheDataSource("geomInfoCacheDS", this.thePoints, new IntegerIndexSpaceID(1024), new IntegerIndexSpaceID(4));
		
		
		vIndicesCacheDS.activate();
		geomInfoCacheDS.activate();
		
		ISIterator iter=new ISIterator(triListBounds);
		
		count=0;
		starSize=(int)(triListBounds.volume()/20.0 + 0.5);
		
		System.out.println("Splitting cells: |0  25   50   75  100|"); 
		System.out.print(  "                  ");
		
		for(iter.init(); iter.valid(); iter.next()){

			cell.assign(vIndicesCacheDS, geomInfoCacheDS, vIndicesRS, coordsRS, dataRS, this.dimensionality, 3, iter);
			
			triangleGroupGridBounds=this.partitioning.map(cell.boundingBox());
			gridBoundsIterator=new ISIterator(triangleGroupGridBounds);
			
			
			oi.reInit(triangleGroupGridBounds.volume());

			
			for(gridBoundsIterator.init(); gridBoundsIterator.valid(); gridBoundsIterator.next()){
			
				cellGroup = (FileSplitterCellGroup)this.groupGrid.mapToCellGroup(gridBoundsIterator);
												
				if(cellGroup.register(cell, oi)){
			
					if(oi.hasOwner && oi.ownerID==null){
						
						oi.ownerID=new CellID(gridBoundsIterator, cellGroup.getNumCells());
					}
					
					if(oi.isContained){
						break;
					}
				}
			}
		
			for(i=0; i< oi.numBorrowers; i++){
			
				((FileSplitterCellGroup)oi.borrowers[i]).setOwner(oi.ownerID);
			
			}
			
			
			for(gridBoundsIterator.init(); gridBoundsIterator.valid(); gridBoundsIterator.next()){
			
				((FileSplitterCellGroup)groupGrid.mapToCellGroup(gridBoundsIterator)).writePendingCell();
			}
	
	
			if((++count) % starSize == 0){
			
				System.out.print("*");
			}
		}
		
		((FileSplitterCellGroupGrid)this.groupGrid).doneRegistering();
		System.out.println();
	}
	
	
	public void writeCells(){
	
		((FileCellGroupGrid)groupGrid).writeCells();
	}

	
}
