/* 
	UnstructuredTopology.java
	Copyright 2000 University of New Hampshire
	Durham, NH 03824
*/

package edu.unh.sdb.lattice; 

import edu.unh.sdb.datasource.*; 
import edu.unh.sdb.common.*; 

public abstract class UnstructuredTopology extends Topology{	


    //protected int dimensionality;
    
    protected DataSource thePoints;
    protected DataSource theTriangles;
    
    protected ISBounds pointListBounds;
    protected ISBounds triListBounds;
    
    protected CellGroupGrid groupGrid;
    protected Partitioning partitioning;
    protected Geometry geometry;
    
	boolean isSplit;
	
	/*    
    public UnstructuredTopology(DataSource thePoints, DataSource theTriangles, UnstructuredGeometry geometry, int dimensionality){
    
        super(dimensionality);
        
        this.thePoints=thePoints;
        this.theTriangles=theTriangles;
        
        this.pointListBounds=thePoints.getBounds();        
        this.triListBounds=theTriangles.getBounds();
               
        this.geometry=geometry;
        this.partitioning=geometry.getPartitioning();

        this.groupGrid=new FileSplitterCellGroupGrid(this.partitioning, null);
		
        this.dimensionality=dimensionality;
        
		this.isSplit=false;
    }
*/    
    
/*	
	public UnstructuredTopology(String pointInfoFDLFileName, 
								String vIndicesFDLFileName, 
								UnstructuredGeometry geometry, 
								int dimensionality, 
								boolean isSplit){

        super(dimensionality);
    
        this.thePoints=DataSource.createPhysicalDataSource(pointInfoFDLFileName, "Vertices");
        this.theTriangles=DataSource.createPhysicalDataSource(vIndicesFDLFileName, "Triangles");
        
        
        this.pointListBounds=thePoints.getBounds();        
        this.triListBounds=theTriangles.getBounds();
        
        
        this.geometry=geometry;    
        this.partitioning=geometry.getPartitioning();
		this.isSplit=isSplit;
		RecordSpec verticesRS=new RecordSpec(new int[] {0,1});
		RecordSpec dataRS=new RecordSpec(new int[] {2});

		if(this.isSplit){
	
			
			System.out.println("Calling SplitCellGroupGrid Constructor.");
			
			this.groupGrid=new SplitCellGroupGrid(	this.partitioning, 
													null, 
													//basePointInfoFileName(vIndicesFDLFileName), 
													basePointInfoFileName(pointInfoFDLFileName), 
													baseVIndicesFileName(pointInfoFDLFileName),
													verticesRS,
													dataRS
												);
		} else {
		
			this.groupGrid=new FileSplitterCellGroupGrid(	this.partitioning, 
															null, 
															vIndicesFDLFileName, 
															pointInfoFDLFileName,
															verticesRS,
															dataRS);

		}


        this.dimensionality=dimensionality;
    }
*/    
	
	public UnstructuredTopology( String pointInfoFDLFileName, 
							    String vIndicesFDLFileName, 
							    UnstructuredGeometry geometry, 
							    int dimensionality)
	{
        super(dimensionality);
    
        this.thePoints = 
           DataSource.create( "Vertices", pointInfoFDLFileName );
        this.theTriangles = 
           DataSource.create( "Triangles", vIndicesFDLFileName);
        
        
        this.pointListBounds=thePoints.getBounds();        
        this.triListBounds=theTriangles.getBounds();
        
        
        this.geometry=geometry;    
        this.partitioning=geometry.getPartitioning();
		

        this.dimensionality=dimensionality;
    }
	
	
	public static String basePointInfoFileName(String pointInfoFDLFileName){
	
		return pointInfoFDLFileName.replaceAll(".xfdl","");
	}
	
	public static String baseVIndicesFileName(String vIndicesFDLFileName){
	
		return vIndicesFDLFileName.replaceAll(".xfdl","");
	}
	
	
    public void activate(){


    }
    

    public Cell findEnclosingCell(IndexSpaceID id,Point p){


		return this.groupGrid.findEnclosingCell(id, p);
		
/*    
        CellGroup theGroup=this.groupGrid.mapToCellGroup(id);
    
		try{
		
			return theGroup.findEnclosingCell(p);
			
		} catch(OutOfMemoryError e){
		
			return null;
		}
*/
    }
    
    
 /*   
    private void registerCells(TriangleCell [] cells, CellGroupGrid groupGrid){
    
        int i,j,k;
        ISBounds triangleGroupGridBounds;
        ISIterator iter;
        
        for(i=0;i<cells.length;i++){
            
            triangleGroupGridBounds=this.partitioning.map(cells[i].boundingBox());
        
            iter=new ISIterator(triangleGroupGridBounds);
            
            for(iter.init(); iter.valid(); iter.next()){
			
			
                groupGrid.mapToCellGroup(iter).register(cells[i]);
				
            }
        }
    }
*/


    /** This is public for debugging purposes only.  We may want to hide it in the future. */
    public CellGroupGrid getCellGroupGrid(){
    
        return groupGrid;
    }
        
    public DataSource getTriangleDS(){
    
        return theTriangles;
    }

	/** return a datum corresponding to the IndexSpaceID argument. */
	public Datum datum(IndexSpaceID id){
	
		return thePoints.datum(id);
	}

	/** Returns the bounds of the 1D pointlist. */
	public ISBounds getBounds(){
	
		
		return (ISBounds) this.pointListBounds.clone();
	}


	/** Return a RecordDescriptor for datums returned by this topology.*/														
	public RecordDescriptor getRecordDescriptor(){
	
		return this.thePoints.getRecordDescriptor();
	}




    /** Return a String representing the object. */
    public String toString(){
        
        return this.toString("");
    }    
    
    /** Return a String representing the object indented by the argument string. */
    public String toString(String indent){
    
        String indent2=indent+Globals.toStringIndent;    
        StringBuffer result=new StringBuffer(indent+"{ Unstructured Topology:\n");

        
        result.append(super.toString(indent2)+"\n");
        
        result.append(indent2+"point DataSource=\n");
        result.append(this.thePoints.toString(indent2)+"\n");
        
        result.append(indent2+"triangle DataSource=\n");
        result.append(this.theTriangles.toString(indent2)+"\n");

        result.append(indent2+"geometry=\n");
        result.append(this.geometry.toString(indent2)+"\n");
        
        result.append(indent2+"cellGroupGrid=");
		//result.append("<not dumped>\n");
        result.append(this.groupGrid);
        
        result.append(indent2+"partitioning=\n");
        result.append(this.partitioning.toString(indent2)+"\n");
        result.append(indent+"}");
        
        return new String(result);
    }

}

