/**
 * DBIF.java Copyright 2000 University of New Hampshire Durham, NH 03824
 */
//------------------- Edit history --------------------------------------
// 02/12/12 rdb: replaced deprecated call to BinaryFileDataSource.create
//               by creating and fdl and calling DataSource.create
// 06/05/08 rdb: made ArrayList and Hashtable variables generic
// 10/21/05 rdb: removed unused variables
//               made fillTypeTable and dumpTypeTable package visibility.
//                  these aren't used so were giving warnings. Making them
//                  package visible gets rid of messages.
//-----------------------------------------------------------------------

package edu.unh.sdb.mdb;

import java.util.*;
import java.sql.*;
import java.net.*;
import java.nio.ByteOrder;
import edu.unh.sdb.common.*;
import edu.unh.sdb.datasource.*;

abstract public class DBIF
{

   //type of database
   public static final int  MYSQL    = 0;
   public static final int  ORACLE   = 1;
   public static final int  POSTGRE  = 2;

   private static final int BaseType = 0x1F;           //  we allow up to 31 base
                                                       // types. (0 is invalid)

   protected URL            myUrl;

   protected String         localhost;
   protected String         dbUrl;
   protected String         driver;

   long                     dsID;
   long                     boundsID;
   long                     recordID;
   long                     typeID;
   long                     attributeProjectionID;

   WorkSpace                currentWorkSpace;
   TypeTable                t;

   Statement                s;
   Connection               c;

   boolean                  verbose  = false;

   //management of database
   int                      counter  = 0;
   public static Hashtable<String,DBIF>  DBIFList = new Hashtable<String,DBIF>();

   //-------------DBIF()---------------------------------------------------
   /**
    * default constructor
    */
   public DBIF()
   {
   }

   //------------------------DBIF(URL)------------------------------------
   /**
    * constructor, set up database conection
    */
   public DBIF( URL url )
   {

      myUrl = url;

      try
      {

         // Load the driver
         registration();

         // set up connection
         connection();

         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "can't make a connection to database " + myUrl.dbName );
      } catch ( ClassNotFoundException c )
      {
         new MetaDataBaseError( SDBError.Recoverable, "can't find mysql driver" );
      } catch ( UnknownHostException h )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "host of database is unknown" );
      } catch ( Exception e )
      {
         new MetaDataBaseError( SDBError.Recoverable, e.getMessage() );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

   }

   //-----------------register()---------------------------------
   abstract void registration() throws ClassNotFoundException,
         UnknownHostException, SQLException, Exception;

   //-----------------connect()------------------------------------
   abstract void connection() throws SQLException;

   //-------------------createDB(String dbName)--------------------
   //create a new database at the database server currently in use on localhost
   protected void createDB( String dbName )
   {
      String newdbUrl = "jdbc:mysql://localhost/";
      String newdriver = "org.gjt.mm.mysql.Driver";
      try
      {
         Class.forName( newdriver );
         Connection conn = DriverManager.getConnection( newdbUrl,
               myUrl.userName, myUrl.password );
         Statement st = conn.createStatement();
         String query = "create database " + dbName;

         st.executeUpdate( query );
         conn.close();
      } catch ( Exception e )
      {
         e.printStackTrace();
      }
   }

   //-----------------open(URL url)-----------------------------
   public static DBIF open( URL url ) throws Exception
   {
      DBIF tmp = DBIFList.get( url.dbName );

      if ( tmp == null )
      {
         switch ( url.dbType )
         {
         case MYSQL:
            tmp = new MySQLDBIF( url );
            break;
         case ORACLE:
            tmp = new OracleDBIF( url );
            break;
         }

         DBIFList.put( url.dbName, tmp );
         tmp.incrCounter();

      }

      return tmp;
   }

   //-----------------detach(URL url)--------------------------
   public static void detach( URL url )
   {
      DBIF tmp = DBIFList.get( url.dbName );

      if ( tmp != null )
      {
         tmp.decrCounter();
         if ( tmp.isCounterZero() )
            DBIFList.remove( url.dbName );
      }

   }

   //---------------------incrCounter()--------------------------------
   private void incrCounter()
   {
      counter++ ;
   }

   //---------------------decrCounter()--------------------------------
   private void decrCounter()
   {
      counter-- ;
   }

   //--------------------isCounterZero()--------------------------------
   private boolean isCounterZero()
   {
      if ( counter == 0 )
         return true;
      else return false;
   }

   //-------------------getURL()-------------------------------------
   public URL getURL()
   {
      return myUrl;
   }

   //------------------setDB(URL url)--------------------------------
   public void setDB( URL url )
   {
      long id = 0;

      if ( url.dbName != null )
      {
         try
         {
            connection();
            s = c.createStatement();

            String query = "select * from DBMapping where DBName = \""
                  + url.dbName + "\"";

            ResultSet rs = s.executeQuery( query );

            while ( rs.next() )
            {
               id = rs.getLong( "DBID" );
               if ( rs.wasNull() )
                  id = 0;
            }
            s.close();

            if ( id == 0 )
            {
               s = c.createStatement();
               query = "select max(DBID) as DBID from DBMapping";
               rs = s.executeQuery( query );

               while ( rs.next() )
               {
                  id = rs.getLong( "DBID" );
                  if ( rs.wasNull() )
                     id = 0;
               }
               id++ ;

               s.close();

               s = c.createStatement();

               query = "insert into DBMapping values(" + id + ", \""
                     + url.dbName + "\", \"" + url.userName + "\", \""
                     + url.password + "\"" + ", " + url.dbType + ")";

               s.executeUpdate( query );
               s.close();
            }
            else
            {
               s = c.createStatement();

               query = "update DBMapping set DBID = " + id + ", DBName = \""
                     + url.dbName + "\", UserName = \"" + url.userName
                     + "\", Password = \"" + url.password + "\', DBType = "
                     + url.dbType;

               s.executeUpdate( query );
            }

            c.close();
         } catch ( SQLException s )
         {
            new MetaDataBaseError( SDBError.Recoverable, "SQL error from setDB" );
         } finally
         {
            if ( c != null )
            {
               try
               {
                  c.close();
               } catch ( SQLException e )
               {
                  e.printStackTrace();
               }
            }
         }
      }
      else
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "SQL error from setDB, DBName given is null" );
      }

   }

   //-----------------getUrlByDBName(String name)------------------
   public URL getUrlByDBName( String name )
   {
      long id = 0;
      int type = 0;
      String user = "";
      String ps = "";

      try
      {
         connection();
         s = c.createStatement();

         String query = "select * from DBMapping where DBName = \"" + name
               + "\"";

         ResultSet rs = s.executeQuery( query );

         while ( rs.next() )
         {
            id = rs.getLong( "DBID" );
            if ( rs.wasNull() )
               new MetaDataBaseError( SDBError.Recoverable,
                     "SQL error from getUrlByDBName, DBName given is not available" 
                                             + id );

            name = rs.getString( "DBName" );
            user = rs.getString( "UserName" );
            ps = rs.getString( "Password" );
            type = rs.getInt( "DBType" );

            s.close();
            c.close();

            return new URL( name, user, ps, type );

         }

      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable, "SQL error from setDB" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }
      return new URL( name, user, ps, type );
   }

   //------------------loadWorkSpace(WorkSpace ws)------------------
   /**
    * read meta data from database to WorkSpace
    */
   public void loadWorkSpace( WorkSpace ws ) throws Exception
   {
      currentWorkSpace = ws;

      try
      {
         DataSourceCollection dsc = ws.getDSCollection();
         TypeTable tt = ws.getTypeTable();
         t = tt;

         connection();

         fillDSC( dsc );

         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "SQL error from loadWorkSpace" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

   }

   //------------------loadDS(String name, WorkSpace ws)------------------
   /**
    * read one DataSource from database to WorkSpace
    */
   public DataSource loadDS( String name, WorkSpace ws ) throws Exception
   {
      currentWorkSpace = ws;

      long id = 0;

      DataSource ds = null;

      try
      {

         DataSourceCollection dsc = ws.getDSCollection();
         TypeTable tt = ws.getTypeTable();
         t = tt;

         //check whether the data source name already in ws
         if ( dsc.nameExists( name ) )
            return null;

         connection();

         id = getDSID( name );

         if ( id == 0 )
            new MetaDataBaseError( SDBError.Recoverable,
                  "SQL error from loadDS, the DataSouce with the given name "
                        + name + " is not in database" );
         else
         {
            ds = getDS( (int) id, dsc );

            if ( ds != null )
               dsc.add( ds );
         }

         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable, "SQL error from loadDS" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

      return ds;

   }

   //-------------------writeWorkSpace(WorkSpace ws)----------------
   /**
    * write meta data from WorkSpace to database
    */

   public void writeWorkSpace( WorkSpace ws )
   {
      currentWorkSpace = ws;

      try
      {
         DataSourceCollection dsc = ws.getDSCollection();
         TypeTable tt = ws.getTypeTable();
         t = tt;

         connection();
         c.setAutoCommit( false );
         init();
         dumpDSC( dsc );

         c.commit();
         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "SQL Error from writeWorkSpace" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

   }

   //-------------------writeDS(String name, WorkSpace ws)----------------
   /**
    * write meta data from WorkSpace to database
    */

   public void writeDS( String name, WorkSpace ws )
   {
      currentWorkSpace = ws;
      DataSource ds;

      try
      {

         DataSourceCollection dsc = ws.getDSCollection();
         TypeTable tt = ws.getTypeTable();
         t = tt;

         if ( ( ds = dsc.getDataSource( name ) ) == null )

            new MetaDataBaseError( SDBError.Recoverable,
                  "Error from writeDS, the DataSource with name " + name
                        + " is not in workspace" );

         connection();
         c.setAutoCommit( false );
         init();

         if ( getDSID( ds.getName() ) == 0 )
         {
            add( ds );
         }
         else new MetaDataBaseError( SDBError.Recoverable,
               "SQL Error from writeDS, the DataSource with name "
                     + ds.getName() + " is already in database" );

         c.commit();
         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "SQL Error from writeWorkSpace" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

   }

   //--------------------creatTable()----------------------------------
   /**
    * create tables
    */

   public void createTable()
   {
      try
      {

         connection();
         c.setAutoCommit( false );

         s = c.createStatement();

         for ( int i = 0; i < Table.sql.length; i++ )
         {
            System.err.println( "------" + Table.sql[i] );
            s.executeUpdate( Table.sql[i] );
         }

         System.out.println( "Tables created" );

         for ( int i = 0; i < Table.baseType.length; i++ )
         {
            s.executeUpdate( Table.baseType[i] );
         }

         System.out.println( "base types are inserted to Type table" );

         s.close();
         c.commit();
         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "SQL Error from create tables" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

   }

   //-------------------dropTable()------------------------------------
   /**
    * drop all the tables from database
    */

   public void dropTable()
   {
      try
      {
         connection();
         c.setAutoCommit( false );

         s = c.createStatement();

         for ( int i = 0; i < Table.drop.length; i++ )
         {
            s.executeUpdate( Table.drop[i] );

         }

         System.out.println( "Tables dropped" );

         s.close();
         c.commit();
         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "SQL error from drop table" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

   }

   //-----------------setVerbose(boolean)---------------------------
   /**
    * the default state for verbose is false
    */

   public void setVerbose( boolean b )
   {
      verbose = b;
   }

   //------------------fillTypeTable(TypeTable tt)------------------
   /**
    * create a TypeTable based on the types in the database
    */

   void fillTypeTable( TypeTable tt ) throws SQLException
   {
      ArrayList<String> arr = new ArrayList<String>();
      ArrayList<String> arrb = new ArrayList<String>();
      ArrayList<String> arrc = new ArrayList<String>();
      ArrayList<String> arrd = new ArrayList<String>();

      String query = "select * from Type where TypeID > " + BaseType;

      if ( verbose )
         System.err.println( "query: " + query );

      s = c.createStatement();
      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         arr.add( rs.getString( Table.tTYPENAME ) );
         arrb.add( rs.getString( Table.tBASETYPE ) );
         arrc.add( rs.getString( Table.tRECORDID ) );
         arrd.add( rs.getString( Table.tTYPEID ) );
      }
      s.close();

      for ( int i = 0; i < arr.size(); i++ )
      {
         s = c.createStatement();
         int base = Integer.parseInt( arrb.get( i ) );
         int type = Integer.parseInt( arrd.get( i ) );

         //base type is a record
         if ( base == 11 )
         {
            tt.insert( getRecordDescriptor( Long.parseLong( arrc.get( i ) ), 
                                                            arr.get( i ) ) );
         }

         else
         {
            s = c.createStatement();
            //query = "select * from Range where Name = \"" +
            // (String)arr.get(i) + "\"";
            query = "select * from Range where TypeID = " + type;
            rs = s.executeQuery( query );

            while ( rs.next() )
            {
               double d1 = rs.getDouble( Table.raMINIMUM );
               double d2 = rs.getDouble( Table.raMAXIMUM );

               if ( rs.wasNull() )
                  tt.insert( new UserType( arr.get( i ), base ) );
               else tt.insert( new RangedType( arr.get( i ), base, d1,
                     d2 ) );
            }
            s.close();
         }
      }

   }

   //-------------------dumpTypeTable(TypeTable tt)------------------
   /**
    * save the user defined type which is not used by DS into Type table used by
    * DS.
    */
   void dumpTypeTable( TypeTable tt ) throws SQLException
   {

      for ( int i = 0; i < tt.size(); i++ )
      {
         if ( tt.get( i + 1 ) instanceof UserType )
         {

            UserType ut = (UserType) tt.get( i + 1 );
            addToType( ut.getName(), ut.getStorageType() );
         }
      }

   }

   //-----------------------------------------------------------------
   //used to give the database name used

   private void init() throws SQLException
   {

      s = c.createStatement();

      // Get the current DSID
      String query = "select max(DSID) as DSID from DataSource ";

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         dsID = rs.getLong( "DSID" );
         if ( rs.wasNull() )
            dsID = 0;

         if ( verbose )
            System.err.println( "current dsID " + dsID );

      }

      s.close();

      // Get the current BoundsID

      s = c.createStatement();
      query = "select max(BoundsID) as BoundsID from Bounds ";

      rs = s.executeQuery( query );

      while ( rs.next() )
      {
         boundsID = rs.getLong( "BoundsID" );
         if ( rs.wasNull() )
            boundsID = 0;

         if ( verbose )
            System.err.println( "current boundsID " + boundsID );
      }

      s.close();

      // Get the current RecordID
      s = c.createStatement();
      query = "select max(RecordID) as RecordID from Record ";

      rs = s.executeQuery( query );

      while ( rs.next() )
      {
         recordID = rs.getLong( "RecordID" );
         if ( rs.wasNull() )
            recordID = 0;

         if ( verbose )
            System.err.println( "current recordID " + recordID );
      }

      s.close();

      // Get the current typeID
      s = c.createStatement();
      query = "select max(TypeID) as TypeID from Type ";

      rs = s.executeQuery( query );

      while ( rs.next() )
      {
         typeID = rs.getLong( "TypeID" );
         if ( rs.wasNull() )
            typeID = 0;
         if ( typeID <= BaseType )
            typeID = BaseType + 1;

         if ( verbose )
            System.err.println( "current TypeID " + typeID );
      }

      s.close();

      // Get the current AttributeProjectionID
      s = c.createStatement();
      query = "select max(AttributeProjectionID) "
            + "as AttributeProjectionID from AttributeProjection";

      rs = s.executeQuery( query );

      while ( rs.next() )
      {
         attributeProjectionID = rs.getLong( "AttributeProjectionID" );
         if ( rs.wasNull() )
            attributeProjectionID = 0;

         if ( verbose )
            System.err.println( "current attributeProjectionID "
                  + attributeProjectionID );
      }

      s.close();

   }

   //-----------------------dumpDSC(DataSourceCollection dsc)--------
   /**
    * add a dataSource to database
    */
   private void dumpDSC( DataSourceCollection dsc ) throws SQLException
   {

      DataSource ds;
      init();

      dsc.rewind();
      ds = dsc.next();

      while ( ds != null )
      {
         // if the DataSource is NOT in the database
         if ( getDSID( ds.getName() ) == 0 )
            add( ds );

         ds = dsc.next();
      }

      if ( verbose )
         System.out.println( "Insert succeeded" );

      s.close();

   }

   //-------------------------add(DataSource ds)---------------------------
   /**
    * helper method, add a dataSource to database without init()
    */
   private void add( DataSource ds ) throws SQLException
   {

      // if the DataSource is NOT in the database
      if ( getDSID( ds.getName() ) == 0 )
      {
         /**
          * if(ds instanceof TextFileDataSource) { TextFileDataSource fds =
          * (TextFileDataSource)ds; addFDS(fds, new String("TFDS")); }
          * 
          * if(ds instanceof BinaryFileDataSource) { //PJR Changed
          * BinaryFileDataSource fds = (BinaryFileDataSource)ds; addFDS(fds, new
          * String("BFDS")); }
          */

         if ( ds instanceof PhysicalDataSource )
            addPDS( (PhysicalDataSource) ds );

         if ( ds instanceof AttributeJoinDataSource )
         {
            CompositeDataSource cds = (CompositeDataSource) ds;
            addCDS( cds, new String( "AJDS" ) );
         }

         if ( ds instanceof BlockedDataSource )
         {
            CompositeDataSource cds = (CompositeDataSource) ds;
            addCDS( cds, new String( "BDS" ) );
         }

         if ( ds instanceof VRDataSource )
         {
            VRDataSource vrds = (VRDataSource) ds;
            addVRDS( vrds );
         }

         if ( ds instanceof AliasDataSource )
         {
            AliasDataSource alds = (AliasDataSource) ds;
            addALDS( alds );
         }

         if ( ds instanceof ForeignReferenceDataSource )
         {
            ForeignReferenceDataSource frds = (ForeignReferenceDataSource) ds;
            addFRDS( frds );
         }

         System.err.println( "DS has been added=" + ds.getName() );

      }
      ;

   }

   //-------------------------addPDS(PhysicalDataSource ds)-------------
   /**
    * helper method, add a PhysicalDataSource
    */

   private void addPDS( PhysicalDataSource ds ) throws SQLException
   {
      String type = new String();
      long fileOffset = 0;
      int fieldLayout = 0;
      String endianness = null;

      if ( ds instanceof TextFileDataSource )
      {
         type = "TFDS";
      }

      else
      {
         if ( ds instanceof PointOrderBFDS )
         {
            type = "POBFDS";
            fieldLayout = DataCollection.pointOrder;
         }
         if ( ds instanceof AttributeOrderBFDS )
         {
            type = "AOBFDS";
            fieldLayout = DataCollection.attributeOrder;
         }

         BinaryFileDataSource bfds = (BinaryFileDataSource) ds;

         fileOffset = bfds.getFileOffset();
         endianness = bfds.getByteOrder().toString();

      }

      // update DataSource table
      addToDS( ds.getName(), ds.dim(), type );

      //update FileName table
      addToPhysicalDS( ds.getURL(), fileOffset, fieldLayout, endianness );

      //update AxisOrdering table
      AxisOrdering tmpAxis = ds.getAxisOrdering();
      //int[] axisOrdering = addToAxisOrdering( tmpAxis );
      addToAxisOrdering( tmpAxis );

      // update Bounds table
      ISBounds bounds = ds.getBounds();
      addToBounds( bounds );

      // update Record table
      RecordDescriptor rd = ds.getRecordDescriptor();
      addToRecord( rd );

   }

   //----------------addCDS(CompositeDataSource cds, String type)-----------
   /**
    * helper method, add a CompositeDataSource
    */
   private void addCDS( CompositeDataSource cds, String type )
         throws SQLException
   {

      //update datasource table

      //add component datasource first
      DataSource[] dses = cds.getComponentDataSources();

      for ( int i = 0; i < dses.length; i++ )
         add( dses[i] );

      //add ajds
      addToDS( cds.getName(), cds.dim(), type );

      int size;
      long cdsID = dsID;
      long cboundsID = boundsID + 1;
      long subID1;
      long subID2;
      long cattributeProjectionID = attributeProjectionID + 1;

      // update Bounds table
      ISBounds[] comps = cds.getCompositeBounds();
      if ( type.equals( "AJDS" ) )
         size = 1;
      else size = comps.length;

      subID1 = subID2 = cboundsID + size;

      for ( int i = 0; i < size; i++ )
         addToBounds( comps[i] );

      ISBounds[] compn = cds.getComponentBounds();
      for ( int i = 0; i < compn.length; i++ )
         addToBounds( compn[i] );

      // update Record table
      RecordDescriptor rd = cds.getRecordDescriptor();
      addToRecord( rd );

      // update AttributeProjection table
      FieldIDMapper[] fMapper = cds.getFIDMappers();
      addToAttributeProjection( fMapper );

      // update Component table
      long componentDSID;
      for ( int i = 0; i < dses.length; i++ )
      {
         componentDSID = getDSID( dses[i].getName() );
         String query = "insert into Component values(" + cdsID + ", "
               + componentDSID + ", " + cattributeProjectionID++ + ", "
               + subID1 + ", " + cboundsID + ")";

         if ( verbose )
            System.err.println( "query: " + query );

         s = c.createStatement();
         s.executeUpdate( query );
         s.close();

         if ( cboundsID + 1 < subID2 )
            cboundsID++ ;
         subID1++ ;
      }

   }

   //-------------------------addVRDS(VRDataSource ds)--------------------
   /**
    * helper method, add a VRDataSource
    */
   private void addVRDS( VRDataSource vrds ) throws SQLException
   {

      DataSource cds = vrds.getDataSource();

      add( cds );

      s = c.createStatement();

      // add to DataSource table
      String query = "insert into DataSource values(" + ++dsID + ", \""
            + vrds.getName() + "\"," + cds.dim() + ", " + ( boundsID + 1 )
            + ", \"VRDS\", null, null)";

      if ( verbose )
         System.err.println( "query: " + query );

      s.executeUpdate( query );

      s.close();

      // add to Composite table
      long cid = getDSID( cds.getName() );

      if ( cid > 0 )
      {
         query = " insert into Component values(" + dsID + ", " + cid
               + ", null, null, null)";

         if ( verbose )
            System.err.println( "query: " + query );

         s = c.createStatement();
         s.executeUpdate( query );

         s.close();
      }

      // add to Bounds table
      addToBounds( vrds.getBounds() );

   }

   //-------------addALDS(AliasDataSource alds)-----------------------------
   /**
    * add AliasDataSource to database
    */
   private void addALDS( AliasDataSource alds ) throws SQLException
   {
      addToDS( alds.getAliasName(), 0, "AL" );

      //add to DSMapping Table
      s = c.createStatement();
      String query = "insert into DSMapping values(" + dsID + ",\""
            + alds.getName() + "\", \"" + alds.getAliasName() + ", \""
            + alds.getDBName() + "\")";

      if ( verbose )
         System.err.println( "query: " + query );

      s.executeUpdate( query );
      s.close();

      //add to DBMapping Table
      setDB( currentWorkSpace.getURL( alds.getAliasName() ) );

   }

   //--------------addFRDS(ForeignReferenceDataSource frds)------------------
   /**
    * add ForeignReferenceDataSource to database
    */
   private void addFRDS( ForeignReferenceDataSource frds ) throws SQLException
   {
      addToDS( frds.getName(), 0, "FR" );

      //add to DSMapping Table
      s = c.createStatement();
      String query = "insert into DSMapping values(" + dsID + ", \""
            + frds.getName() + "\", null, \"" + frds.getDBName() + "\")";

      if ( verbose )
         System.err.println( "query: " + query );

      s.executeUpdate( query );
      s.close();

      //add to DBMapping Table
      setDB( currentWorkSpace.getURL( frds.getName() ) );
   }

   //--------------addToDS(String name, int dim, String type)----------------
   /**
    * helper method, add to DataSource table
    */
   private void addToDS( String name, int dim, String type )
         throws SQLException
   {

      s = c.createStatement();
      String query = "insert into DataSource values(" + ++dsID + ", " + "\""
            + name + "\", " + dim + ", " + ( boundsID + 1 ) + ", \"" + type
            + "\", " + ( recordID + 1 ) + ", \" no des\")";

      if ( verbose )
         System.err.println( "query: " + query );

      s.executeUpdate( query );
      s.close();
   }

   //---addToPhysicalDS(String url, int fileOffset, int fieldLayout, String
   // endianness)------
   /**
    * helper method, add to PhysicalDS table
    */
   private void addToPhysicalDS( String url, long fileOffset, int fieldLayout,
         String endianness ) throws SQLException
   {
      s = c.createStatement();
      String query = "insert into PhysicalDS values(" + dsID + ", \"" + url
            + "\", " + fileOffset + ", " + fieldLayout + ", \"" + endianness
            + "\")";

      if ( verbose )
         System.err.println( "query: " + query );

      s.executeUpdate( query );
      s.close();
   }

   //-----------------addToAxisOrdering(AxisOrdering axis)----------------
   /**
    * helper method, add to AxisOrdering table
    */
   private int[] addToAxisOrdering( AxisOrdering axis ) throws SQLException
   {
      int dim = axis.dim();
      int[] axisOrdering = new int[dim];

      for ( int i = 0; i < dim; i++ )
      {
         axisOrdering[i] = axis.getAxis( i );
         if ( i != axisOrdering[i] )
         {
            s = c.createStatement();

            String query = "insert into AxisOrdering values(" + dsID + ", " + i
                  + ", " + i + ", " + axisOrdering[i] + ")";

            if ( verbose )
               System.err.println( "query: " + query );

            s.executeUpdate( query );
            s.close();
         }
      }
      return axisOrdering;
   }

   //---------------------addToBounds(ISBounds bounds)----------------------
   /**
    * helper method, insert to Bounds table
    */
   private void addToBounds( ISBounds bounds ) throws SQLException
   {
      boundsID++ ;

      IndexSpaceID corner1 = bounds.copyLower();
      IndexSpaceID corner2 = bounds.copyUpper();

      for ( int i = 0; i < corner1.dim(); i++ )
      {
         s = c.createStatement();

         if ( verbose )
            System.err.println( "axisNumber: " + i + " low: "
                  + corner1.getCoord( i ) + " high: " + corner2.getCoord( i ) );

         String query = "insert into Bounds values(" + boundsID + ", " + i
               + ", " + corner1.getCoord( i ) + ", " + corner2.getCoord( i )
               + ")";

         if ( verbose )
            System.err.println( "query: " + query );

         s.executeUpdate( query );
         s.close();
      }
   }

   //------------addToAttributeProjection(FieldIDMapper[]
   // fMapper)-----------------
   /**
    * helper method, insert to AttributeProjection table
    */
   private void addToAttributeProjection( FieldIDMapper[] fMapper )
         throws SQLException
   {
      int[] component;
      int[] composite;

      attributeProjectionID++ ;

      for ( int i = 0; i < fMapper.length; i++ )
      {
         component = fMapper[i].getComponentFields();
         composite = fMapper[i].getCompositeFields();

         for ( int j = 0; j < component.length; j++ )
         {
            s = c.createStatement();
            String query = "insert into AttributeProjection values("
                  + attributeProjectionID + ", " + composite[j] + ", "
                  + component[j] + ")";

            if ( verbose )
               System.err.println( "query: " + query );

            s.executeUpdate( query );
            s.close();
         }
         attributeProjectionID++ ;
      }

   }

   //---------------------addToRecord(RecordDescriptor rd)---------------------
   /**
    * helper method, insert to Record table
    */
   private long addToRecord( RecordDescriptor rd ) throws SQLException
   {
      int size = rd.size();
      if ( size > 0 )
         recordID++ ;
      AttributeDescriptor ad;

      long myRecordID = recordID;

      for ( int i = 0; i < size; i++ )
      {
         ad = rd.getAttribute( i );
         int baseType = ad.getStorageType();
         long myType = baseType;
         String typeName = ad.getTypeName();
         //int type = ad.getUserTypeNumber();

         //the type itself is a user defined type
         if ( ad.isUserType() )
         {
            //add to Type table
            myType = addToType( typeName, baseType );

         }

         s = c.createStatement();
         String query = "insert into Record values(" + myRecordID + " , " + i
               + ", \"" + ad.name() + "\", " + myType + ", " + "1)";

         if ( verbose )
            System.err.println( "query: " + query );

         s.executeUpdate( query );
         s.close();

      }//end of for loop

      return recordID;

   }

   //--------------addToType(String typeName, int baseType)-----------
   /**
    * helper method, add to the Type table
    */
   private long addToType( String typeName, int baseType ) throws SQLException
   {
      long tid = 0;

      s = c.createStatement();
      String query = "select TypeID from Type where TypeName = \"" + typeName
            + "\"";

      if ( verbose )
         System.err.println( "query: " + query );

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         tid = rs.getLong( "TypeID" );
      }

      s.close();

      //the typeName is not in Type table
      if ( tid == 0 )
      {
         //the new type's base type is a record
         if ( baseType == 11 )
         {
            long record = addToRecord( (RecordDescriptor) t.get( typeName ) );
            query = "insert into Type values(" + ++typeID + ", " + "\""
                  + typeName + "\", 11, " + record + ")";
         }
         else query = "insert into Type values(" + ++typeID + ", " + "\""
               + typeName + "\", " + baseType + ", NULL )";

         if ( verbose )
            System.err.println( "query: " + query );

         s = c.createStatement();
         s.executeUpdate( query );
         s.close();

         if ( baseType != 11 )
         {
            addToRange( typeName );
         }
      }

      return typeID;
   }

   //-----------------------addToRange(String
   // typeName)-------------------------
   /**
    * add the new user type to Range table
    */

   private void addToRange( String typeName ) throws SQLException
   {
      RangedType rt = (RangedType) t.get( typeName );

      String query = "insert into Range values(" + typeID + ", 0, "
            + rt.getLowValue() + ", " + rt.getHighValue() + ", \" " + typeName
            + "\")";

      if ( verbose )
         System.err.println( "query: " + query );

      s = c.createStatement();
      s.executeUpdate( query );
      s.close();

   }

   //------------------------getDSID(String
   // name)-------------------------------
   /**
    * Get DSID from DataSource table
    */

   private long getDSID( String name ) throws SQLException
   {

      long id = 0;
      String query = "select DSID from DataSource where DSName = \"" + name
            + "\"";

      if ( verbose )
         System.err.println( "query: " + query );

      s = c.createStatement();

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         id = rs.getLong( "DSID" );
         if ( rs.wasNull() )
            id = 0;
      }

      s.close();

      return id;
   }

   //------------------fillDSC(DataSourceCollection dsc)--------------
   /**
    * fill in the DataSourceCollection
    */
   private void fillDSC( DataSourceCollection dsc ) throws SQLException,
         Exception
   {

      int totalDS = 0;

      // find out how many datasource in the DataSource table
      s = c.createStatement();

      String query = "select max(DSID) as DSID from DataSource ";

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         totalDS = rs.getInt( "DSID" );

         if ( rs.wasNull() )
            totalDS = 0;

      }

      s.close();

      totalDS++ ;

      for ( int k = 1; k < totalDS; k++ )
      {
         DataSource ds = getDS( k, dsc );

         if ( ds != null )
            dsc.add( ds );
      }

   }

   //-----------------getDS(int id, DataSourceCollection dsc)-----------------------
   //get a DS from DataBase
   private DataSource getDS( int id, DataSourceCollection dsc )
         throws SQLException, Exception
   {
      DataSource ds = null;

      String name = new String();
      int dim = 0;
      String dsType = null;
      //rdb: What is "des" for. It is read, but not used??
      String des = null;

      long dsid = 0;
      long boundsid = 0;
      long recordid = 0;

      ISBounds isBounds;
      AxisOrdering axisOrdering;
      RecordDescriptor rd;

      String query = "select * from DataSource where DSID = " + id;

      if ( verbose )
         System.err.println( "query: " + query );

      s = c.createStatement();

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         name = rs.getString( 2 );
         if ( !dsc.nameExists( name ) )
         {
            int i = 1;
            dsid = rs.getLong( i++ );
            i++ ;
            dim = rs.getInt( i++ );
            boundsid = rs.getLong( i++ );
            dsType = rs.getString( i++ );
            recordid = rs.getLong( i++ );
            des = rs.getString( i );
         }
      }

      s.close();

      if ( !dsc.nameExists( name ) )
      {
         if ( dsType.equals( "TFDS" ) || dsType.equals( "POBFDS" )
               || dsType.equals( "AOBFDS" ) )
         {
            //get AxisOrdering from AxisOrdering table
            axisOrdering = getFromAxisOrdering( dim, dsid );

            //get Bounds from Bounds table
            isBounds = getFromBounds( dim, boundsid );

            //get RecordDescriptor from Record and Type table
            rd = getRecordDescriptor( recordid, null );

            // get PDSDescriptor from PhysicalDS table
            PDSDescriptor pds = getFromPhysicalDS( dsid );

            if ( dsType.equals( "TFDS" ) )
            {
               // ds = new TextFileDataSource(name, isBounds,axisOrdering,
               // fName, pointOrder);
               ds = new TextFileDataSource( name, rd, isBounds, axisOrdering,
                     pds.url );

               if ( verbose )
                  System.err.println( "generated a TFDS " + name );
            }

            if ( dsType.equals( "POBFDS" ) || dsType.equals( "AOBFDS" ) )
            {
               /* rdb replaced call to deprecated method with equivalent calls
               ds = BinaryFileDataSource.createBinaryFileDataSource( 
                                   name, rd, isBounds, axisOrdering, pds.url, 
                                   pds.endianness, pds.fieldLayout );
                ********************/
               
               FDLDescriptor fdl = new FDLDescriptor( rd, isBounds, axisOrdering );
               fdl.setDataFileURL( pds.url );
               fdl.setByteOrder( pds.endianness );
               fdl.setFieldLayout( pds.fieldLayout );
               
               ds = DataSource.create(  name, fdl );
               
               if ( ds != null )
                  ( (BinaryFileDataSource) ds ).setFileOffset( pds.fileOffset );
            }
         }

         if ( dsType.equals( "AJDS" ) || dsType.equals( "BDS" ) )
         {
            //find out the number of components
            int size = getComponent( dsid );

            ComponentInfo[] ci = new ComponentInfo[size];
            int[] componentDSID = new int[size];
            int[] attributeProjectionID = new int[size];
            int[] compositeBoundsID = new int[size];
            int[] componentBoundsID = new int[size];

            getFromComponent( dsid, componentDSID, attributeProjectionID,
                  compositeBoundsID, componentBoundsID );

            for ( int i = 0; i < size; i++ )
            {
               FieldIDMapper fm = getFromAttributeProjection( attributeProjectionID[i] );
               //the compositeBounds might be redundant for AJDS
               ISIDMapper im = new ISIDMapper( getFromBounds( dim,
                     componentBoundsID[i] ), getFromBounds( dim,
                     compositeBoundsID[i] ) );
               DataSource temp;

               String tempName = getName( componentDSID[i] );
               if ( dsc.nameExists( tempName ) )
                  temp = dsc.get( dsc.indexOfName( tempName ) );
               else temp = getDS( componentDSID[i], dsc );

               ci[i] = new ComponentInfo( temp, fm, im );
            }

            if ( dsType.equals( "AJDS" ) )
            {
               ds = new AttributeJoinDataSource( name, ci );
               if ( verbose )
                  System.err.println( "-----------------a AJDS has been created "
                              + name );
            }

            if ( dsType.equals( "BDS" ) )
            {
               ds = new BlockedDataSource( name, ci );
               if ( verbose )
                  System.err
                        .println( "------------------a BDS has been created "
                              + name );
            }

         }

         if ( dsType.equals( "VRDS" ) )
         {
            int cid = 0;

            query = "select * from Component where CompositeDSID = " + dsid;

            if ( verbose )
               System.err.println( "query: " + query );

            s = c.createStatement();

            rs = s.executeQuery( query );

            while ( rs.next() )
            {
               cid = rs.getInt( Table.cCOMPONENTDSID );
            }

            s.close();

            DataSource cds = dsc.getDataSource( getName( cid ) );
 
            if ( cds != null )
               //ds = new VRDataSource(name, cds, res);
               ds = new VRDataSource( name, cds ); // PJR Removed res to
                                                   // acoommodate
            // Li's changes to VRDataSource
         }

         if ( dsType.equals( "AL" ) || dsType.equals( "FR" ) )
         {
            String alias = "";
            String originName = "";
            String dbName = "";
            URL url = null;

            query = "select * from DSMapping where DSID = " + dsid;

            if ( verbose )
               System.err.println( "query: " + query );

            s = c.createStatement();

            rs = s.executeQuery( query );

            while ( rs.next() )
            {
               originName = rs.getString( Table.dsName );
               alias = rs.getString( Table.dsAlias );
               dbName = rs.getString( Table.dsDBName );
            }

            s.close();

            url = getUrlByDBName( dbName );

            WorkSpace newWorkSpace = WorkSpaceManager.getWorkSpace( url );

            if ( newWorkSpace != null )
            {
               DataSource rDS = newWorkSpace.loadDS( originName );

               if ( dsType.equals( "AL" ) )
               {
                  ds = new AliasDataSource( originName, alias, dbName, rDS );
                  currentWorkSpace.addWorkSpace( alias, newWorkSpace );
               }
               if ( dsType.equals( "FR" ) )
               {
                  ds = new ForeignReferenceDataSource( originName, dbName, rDS );
                  currentWorkSpace.addWorkSpace( originName, newWorkSpace );
               }

            }
         }

      }

      return ds;

   }

   //----------------getName(int
   // dsid)---------------------------------------------
   private String getName( int dsid ) throws SQLException
   {
      String name = new String();

      String query = "select DSName from DataSource where DSID = " + dsid;

      if ( verbose )
         System.err.println( "query: " + query );

      s = c.createStatement();

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         name = rs.getString( "DSName" );
      }

      s.close();

      return name;
   }

   //---------getFromAttributeProjection(int apid)---------------------------------
   private FieldIDMapper getFromAttributeProjection( int apid )
         throws SQLException
   {
      int size = 0;

      String query = "select * from AttributeProjection where AttributeProjectionID = "
            + apid;

      s = c.createStatement();
      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         size++ ;
      }

      s.close();

      int[] compositeP = new int[size];
      int[] componentP = new int[size];
      int i = 0;

      query = "select * from AttributeProjection where AttributeProjectionID = "
            + apid;
      s = c.createStatement();
      rs = s.executeQuery( query );

      while ( rs.next() )
      {
         compositeP[i] = rs.getInt( Table.aCOMPOSITEPOSITION );
         componentP[i] = rs.getInt( Table.aCOMPONENTPOSITION );
         i++ ;
      }

      s.close();

      return new FieldIDMapper( componentP, compositeP );

   }

   //---------getFromComponent(long dsid, int[] componentDSID, 
   //                          int[] attributeProjectionID,
   //                          int[] compositeBoundsID, int[] componentBoundsID)
   //-------------------------------------------------------------------
   private void getFromComponent( long dsid, int[] componentDSID,
         int[] attributeProjectionID, int[] compositeBoundsID,
         int[] componentBoundsID ) throws SQLException
   {
      int i = 0;

      String query = " select * from Component where CompositeDSID = " + dsid;

      s = c.createStatement();
      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         componentDSID[i] = rs.getInt( Table.cCOMPONENTDSID );
         attributeProjectionID[i] = rs.getInt( Table.cATTRIBUTEPROJECTIONID );
         compositeBoundsID[i] = rs.getInt( Table.cCOMPOSITEBOUNDSID );
         componentBoundsID[i] = rs.getInt( Table.cCOMPONENTBOUNDSID );
         i++ ;
      }

      s.close();
   }

   //------------------getComponent(long dsid)----------------------------
   private int getComponent( long dsid ) throws SQLException
   {
      int rowCount = 0;

      String query = " select * from Component where CompositeDSID = " + dsid;

      s = c.createStatement();
      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         rowCount++ ;
      }

      return rowCount;
   }

   //-------------------getFromAxisOrdering(int dim, long dsid)------------
   /**
    * helper method query AxisOrdering table
    */
   private AxisOrdering getFromAxisOrdering( int dim, long dsid )
         throws SQLException
   {

      int[] orderingarray = new int[dim];

      for ( int i = 0; i < dim; i++ )
         orderingarray[i] = i;

      s = c.createStatement();

      String query = "select * from AxisOrdering where DSID = " + dsid;

      if ( verbose )
         System.err.println( "query: " + query );

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         orderingarray[rs.getInt( Table.aAXIS )] = rs
               .getInt( Table.aDESTINATION );

      }

      s.close();

      return new AxisOrdering( orderingarray );
   }

   //--------------getFromBounds(int dim, long boundsid)----------------
   /**
    * helper method, query Bounds table
    */
   private ISBounds getFromBounds( int dim, long boundsid ) throws SQLException
   {
      int[] corner1 = new int[dim];
      int[] corner2 = new int[dim];

      s = c.createStatement();

      String query = "select * from Bounds where BoundsID = " + boundsid;

      if ( verbose )
         System.err.println( "query: " + query );

      ResultSet rs = s.executeQuery( query );
      while ( rs.next() )
      {
         corner1[rs.getInt( Table.bAXIS )] = rs.getInt( Table.bLOWERBOUNDS );
         corner2[rs.getInt( Table.bAXIS )] = rs.getInt( Table.bUPPERBOUNDS );
      }

      s.close();
      return new ISBounds( new IntegerIndexSpaceID( corner1 ),
            new IntegerIndexSpaceID( corner2 ) );
   }

   //-----------------getRecordDescriptor(long recordid, stringname)---------------
   /**
    * helper method, query Record and Type table
    */
   private RecordDescriptor getRecordDescriptor( long recordid, String name )
         throws SQLException
   {
      ArrayList<String> v1 = new ArrayList<String>();
      ArrayList<Integer> v2 = new ArrayList<Integer>();
      int fp = 0;

      s = c.createStatement();

      String query = "select * from Record where RecordID = " + recordid;

      if ( verbose )
         System.err.println( "query " + query );

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         fp++ ;
         v1.add( rs.getInt( Table.rFIELDPOSITION ), 
        		 rs.getString( Table.rFIELDNAME ) );
         v2.add( rs.getInt( Table.rFIELDPOSITION ), 
        		 new Integer( rs.getInt( Table.rTYPEID ) ) );
      }

      s.close();

      AttributeDescriptor[] atts = new AttributeDescriptor[fp];

      for ( int i = 0; i < fp; i++ )
      {
         Integer tmp = v2.get( i );
         int base = 0;
         String typeName = new String();
         long rid = 0;

         s = c.createStatement();
         query = "select * from Type where TypeID = " + tmp;

         if ( verbose )
            System.err.println( "query " + query );

         rs = s.executeQuery( query );
         while ( rs.next() )
         {
            base = rs.getInt( Table.tBASETYPE );
            typeName = rs.getString( Table.tTYPENAME );
            rid = rs.getLong( Table.tRECORDID );
         }
         s.close();

         // if the type is not in type table, need to add it to the type table

         if ( t.get( typeName ) == null )
         {
            //base type is record
            if ( base == 11 )
            {
               t.insert( getRecordDescriptor( rid, typeName ) );
            }
            else
            {
               query = "select * from Range where TypeID = " + tmp;

               if ( verbose )
                  System.err.println( "query " + query );

               s = c.createStatement();
               rs = s.executeQuery( query );

               while ( rs.next() )
               {
                  double d1 = rs.getDouble( Table.raMINIMUM );
                  double d2 = rs.getDouble( Table.raMAXIMUM );
                  if ( rs.wasNull() )
                     t.insert( new UserType( typeName, base ) );
                  else t.insert( new RangedType( typeName, base, d1, d2 ) );

               }

               s.close();
            }
         }

         atts[i] = new AttributeDescriptor( t, typeName, v1.get( i ) );
      }

      if ( name == null )
         return new RecordDescriptor( t, atts );
      else return new RecordDescriptor( name, t, atts );
   }

   //-------------------getFromPhysicalDS(long dsid)-----------------------
   /**
    * helper method, query PhysicalDS table
    */
   private PDSDescriptor getFromPhysicalDS( long dsid ) throws SQLException
   {
      PDSDescriptor pds = new PDSDescriptor();
      String endianness = null;

      s = c.createStatement();

      String query = "select * from PhysicalDS where DSID = " + dsid;

      if ( verbose )
         System.err.println( "query " + query );

      ResultSet rs = s.executeQuery( query );

      while ( rs.next() )
      {
         pds.url = rs.getString( Table.pURL );
         pds.fileOffset = rs.getInt( Table.pFILEOFFSET );
         pds.fieldLayout = rs.getInt( Table.pFIELDLAYOUT );
         endianness = rs.getString( Table.pENDIANNESS );
         if ( endianness.equals( ByteOrder.BIG_ENDIAN.toString() ) )
            pds.endianness = ByteOrder.BIG_ENDIAN;
         if ( endianness.equals( ByteOrder.LITTLE_ENDIAN.toString() ) )
            pds.endianness = ByteOrder.LITTLE_ENDIAN;
      }

      s.close();

      return pds;
   }

   //---------------------show(String tableName)-------------------
   /**
    * For debugging, print out the table content
    */
   public void show( String tableName )
   {

      printTitle( convert( tableName ) );

      try
      {
         connection();
         //c = DriverManager.getConnection(dbUrl, user, password);

         s = c.createStatement();

         String query = "select * from " + tableName;

         ResultSet rs = s.executeQuery( query );

         while ( rs.next() )
         {
            int i = 1;
            try
            {
               while ( true )
                  System.out.print( rs.getString( i++ ) + "\t" );
            } catch ( Exception e )
            {
               System.out.println();
            }
         }

         System.out.println();
         s.close();
         c.close();
      } catch ( Exception e )
      {
         new MetaDataBaseError( SDBError.Recoverable,
               "SQL Error in showing table" );
      }

      finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

   }

   //-------------------showAll()---------------------------------------
   public void showAll()
   {
      show( "DataSource" );
      show( "PhysicalDS" );
      show( "AxisOrdering" );
      show( "Bounds" );
      show( "AttributeProjection" );
      show( "Component" );
      show( "Record" );
      show( "Type" );
      show( "Range" );
      show( "DBMapping" );
   }

   //-----------------convert(String name)------------------------------
   private int convert( String name )
   {
      int result = 0;
      String temp = name.toUpperCase();

      if ( temp.equals( "DATASOURCE" ) )
         result = Table.DATASOURCE;

      if ( temp.equals( "PHYSICALDS" ) )
         result = Table.PHYSICALDS;

      if ( temp.equals( "AXISORDERING" ) )
         result = Table.AXISORDERING;

      if ( temp.equals( "COMPONENT" ) )
         result = Table.COMPONENT;

      if ( temp.equals( "BOUNDS" ) )
         result = Table.BOUNDS;

      if ( temp.equals( "ATTRIBUTEPROJECTION" ) )
         result = Table.ATTRIBUTEPROJECTION;

      if ( temp.equals( "RECORD" ) )
         result = Table.RECORD;

      if ( temp.equals( "TYPE" ) )
         result = Table.TYPE;

      if ( temp.equals( "RANGE" ) )
         result = Table.RANGE;

      if ( temp.equals( "DBMAPPING" ) )
         result = Table.DBMAPPING;

      return result;
   }

   //-----------------printTitle(int i)----------------------------------
   private void printTitle( int i )
   {
      switch ( i )
      {
      case Table.DATASOURCE:
         System.out.println( "\nDataSource Table" );
         System.out
               .println( "DSID DSName ISDIM BoundsID DataSourceType RecordID Des" );
         break;

      case Table.PHYSICALDS:
         System.out.println( "\nPhysicalDS Table" );
         System.out.println( "DSID\tURL\tFileOffset\tFieldLayout\tEndianness" );
         break;

      case Table.AXISORDERING:
         System.out.println( "\nAxisOrdering Table" );
         System.out.println( "DSID\tAxis\tSource\tDestination" );
         break;

      case Table.COMPONENT:
         System.out.println( "\nComponent Table" );
         System.out
               .println( "CompositeDSID ComponentDSID AttributeProjectionID CompositeBoundsID ComponentBoundsID" );
         break;

      case Table.BOUNDS:
         System.out.println( "\nBounds Table" );
         System.out.println( "BoundsID  Axis LowerBounds UpperBounds" );
         break;

      case Table.ATTRIBUTEPROJECTION:
         System.out.println( "\nAttributeProjection Table" );
         System.out
               .println( "AttributeProjectionID CompositePosition ComponentPosition" );
         break;

      case Table.RECORD:
         System.out.println( "\nRecord Table" );
         System.out
               .println( "RecordID FieldPosition FieldName TypeID\tReplication" );
         break;

      case Table.TYPE:
         System.out.println( "\nType Table" );
         System.out.println( "TypeID\tTypeName BaseType RecordID" );
         break;

      case Table.RANGE:
         System.out.println( "\nRange Table" );
         System.out.println( "TypeID\tSequence Minimum Maximum\tName" );
         break;

      case Table.DBMAPPING:
         System.out.println( "\nDBMapping Table" );
         System.out.println( "DBID\tDBName\tUserName\tPassword\tDBType" );
         break;

      default:
         System.out.println( "Table Name not recognized" );
      }
   }

   //-------------------------isDSNameUsed(String name)-----------------------
   public boolean isDSNameUsed( String name )
   {

      long id = 0;

      try
      {
         connection();

         String query = "select DSID from DataSource where DSName = " + "\""
               + name + "\"";

         if ( verbose )
            System.err.println( "query: " + query );

         s = c.createStatement();

         ResultSet rs = s.executeQuery( query );

         while ( rs.next() )
         {
            id = rs.getLong( "DSID" );
         }

         s.close();

         c.close();
      } catch ( SQLException s )
      {
         new MetaDataBaseError( SDBError.Recoverable, "SQL error from setDB" );
      } finally
      {
         if ( c != null )
         {
            try
            {
               c.close();
            } catch ( SQLException e )
            {
               e.printStackTrace();
            }
         }
      }

      if ( id == 0 )
         return false;
      else return true;
   }

}//end of DBIF class 

