/**
*    Table.java
*    Copyright 2000 University of New Hampshire
*    Durham, NH 03824				
*/


package edu.unh.sdb.mdb;

public class Table{
 /** define table name*/
  public final static int
  DATASOURCE =1, PHYSICALDS =2, AXISORDERING =3, COMPONENT = 4,
  BOUNDS = 5, ATTRIBUTEPROJECTION =6, RECORD =7, TYPE = 8,
  RANGE = 9, DBMAPPING = 10, DSMAPPING = 11;

  /** define the table fields*/
  public final static int
  dDSID =1, dDSNAME =2, dESDIM =3, dBOUNDSID=4, dDATASOURCETYPE = 5,
  dRECORDID = 6, dDES = 7;

  public final static int
  pDSID =1, pURL =2, pFILEOFFSET=3, pFIELDLAYOUT=4, pENDIANNESS=5;

  public final static int
  aDSID =1, aAXIS =2, aSOURCE = 3, aDESTINATION =4;

  public final static int
  cCOMPOSITEDSID =1, cCOMPONENTDSID =2, cATTRIBUTEPROJECTIONID = 3,
  cCOMPOSITEBOUNDSID =4, cCOMPONENTBOUNDSID =5;

  public final static int
  bBOUNDSID =1, bAXIS =2, bLOWERBOUNDS =3, bUPPERBOUNDS =4;

  public final static int
  aATTRIBUTEPROJECTIONID =1, aCOMPOSITEPOSITION =2, aCOMPONENTPOSITION =3;

  public final static int
  rRECORDID =1, rFIELDPOSITION =2, rFIELDNAME =3, rTYPEID =4, rREPICATION =5;

  public final static int
  tTYPEID =1, tTYPENAME =2, tBASETYPE =3, tRECORDID =4;

  public final static int
  raTYPEID =1, raSEQUENCE =2, raMINIMUM =3, raMAXIMUM =4, raNAME =5;

  public final static int
  dmDBID = 1, dmDBName =2, dmUserName =3, dmPassword = 4, dmDBType = 5;

  public final static int
  dsDSID = 1, dsName =2, dsAlias =3, dsDBName =4;


 static String[] sql ={
  
  //create new DataSource table
  "CREATE TABLE IF NOT EXISTS DataSource " +
  "(DSID BIGINT PRIMARY KEY, " +
  "DSName VARCHAR(50) NOT NULL, " +
  "ISDIM INT, " +
  "BoundsID INT REFERENCES Bounds(BoundsID), " +
  "DataSourceType VARCHAR(50), " +
  "RecordID INT REFERENCES Record(RecordID), " +
  "Des Text)",

  // create File table
  "CREATE TABLE IF NOT EXISTS PhysicalDS " +
  "(DSID BIGINT REFERENCES DataSource(DSID), " +
  "URL VARCHAR(50), " +
  "FileOffset INT, " +
  "FieldLayout INT, " +
  "Endianness VARCHAR(50))",

  // create AxisOrdering table
  "CREATE TABLE AxisOrdering " +
  "(DSID BIGINT REFERENCES DataSource(DSID), " +
  "Axis INT, " +
  "Source INT, " +
  "Destination INT)",

  // create Component table
  "CREATE TABLE IF NOT EXISTS Component " +
  "(CompositeDSID BIGINT REFERENCES DataSource(DSID), " +
  "ComponentDSID BIGINT REFERENCES DataSource(DSID), " +
  "AttributeProjectionID BIGINT REFERENCES AttributeProjection(AttributeProjectionID),"+
  "CompositeBoundsID BIGINT REFERENCES Bounds(BoundsID), " +
  "ComponentBoundsID BIGINT REFERENCES Bounds(BoundsID)) ",

  // create Bounds table 
  "CREATE TABLE IF NOT EXISTS Bounds " +
  "(BoundsID INT, " +
  "Axis INT, " +
  "LowerBounds INT, " +
  "UpperBounds INT)",

  // create AttributeProjection table
  "CREATE TABLE IF NOT EXISTS AttributeProjection " +
  "( AttributeProjectionID BIGINT, " +
  "CompositePosition INT, " +
  "ComponentPosition INT)",

  // create Record table
  "CREATE TABLE IF NOT EXISTS Record " +
  "(RecordID BIGINT, " +
  "FieldPosition INT, " +
  "FieldName Text, " +
  "TypeID BIGINT REFERENCES Type(TypeID), " +
  "Replication INT)",

  // create Type table
  "CREATE TABLE IF NOT EXISTS Type " +
  "(TypeID BIGINT PRIMARY KEY, " +
  "TypeName VARCHAR(50), " +
  "BaseType BIGINT, " +
  "RecordID BIGINT REFERENCES Record(RecordID))",

  // create Range table
  "CREATE TABLE IF NOT EXISTS Range " +
  "(TypeID BIGINT REFERENCES Type(TypeID), " +
  "Sequence BIGINT, " +
  "Minimum VARCHAR(50), " +
  "Maximum VARCHAR(50), " +
  "Name VARCHAR(50)) ",

  // create DBMapping table
  "CREATE TABLE IF NOT EXISTS DBMapping " +
  "(DBID BIGINT, " +
  "DBName VARCHAR(50), " +
  "UserName VARCHAR(50), " +
  "Password VARCHAR(50), " +
  "DBType INT )",

  //creat DSMapping table
  "CREATE TABLE IF NOT EXISTS DSMapping " +
  "(DSID BIGINT REFERENCES DataSource(DSID), " +
  "Name VARCHAR(50), " +
  "Alias VARCHAR(50), " +
  "DBName VARCHAR(50)) " 


  };

 static String[] baseType = {

  "insert into Type values(1,\"boolean\", NULL, NULL)",
  "insert into Type values(2, \"byte\", NULL, NULL)",
  "insert into Type values(3, \"unsigned byte\", NULL, NULL)",
  "insert into Type values(4, \"char\", NULL, NULL)",
  "insert into Type values(5, \"short\", NULL, NULL)",
  "insert into Type values(6, \"unsigned short\", NULL, NULL)",
  "insert into Type values(7, \"int\", NULL, NULL)",
  "insert into Type values(8, \"float\", NULL, NULL)",
  "insert into Type values(9, \"double\", NULL, NULL)",
  "insert into Type values(10, \"long\", NULL, NULL)",
  "insert into Type values(11, \"record\", NULL, NULL)"
  };

  static String[] drop = {
  
  "drop table IF EXISTS DataSource",
  "drop table IF EXISTS PhysicalDS",
  "drop table IF EXISTS AxisOrdering",
  "drop table IF EXISTS Component",
  "drop table IF EXISTS Bounds",
  "drop table IF EXISTS AttributeProjection",
  "drop table IF EXISTS Record",
  "drop table IF EXISTS Type",
  "drop table IF EXISTS Range",
  "drop table IF EXISTS DBMapping" ,
  "drop table IF EXISTS DSMapping"

  };
}
