
//----------------------- CommandDefException class ----------------------------
//
//  Define exceptions raised in definitions of Commands in CommandProcessor 
//  This shows how the constructor for an Exception can specify some 
//    arguments that can get used later in the catch code.
//------------------- Edit history --------------------------------------
// 10/21/05 rdb: Added getAction method and serialVersionUID variable
//-----------------------------------------------------------------------

package edu.unh.sdb.util.CommandProcessor;

public class CommandDefException extends RuntimeException
{
  private static final long serialVersionUID = 0; // for serialization inheritance
  private String           command;    // the string passed as the command
  private String           parmSpec;   // the string passed as param specif
  private CommandAction    action;     // the CommandAction parameter

  //----------CommandDefException ( String, String, CommandAction ) -----------
  public CommandDefException( String message, 
                              String cmd, String parms, CommandAction act )
  {
     super( message );  
     this.command = cmd;
     this.parmSpec = parms;
     this.action = act;
  }
  //----------CommandDefException ( String) ----------------------------------
  public CommandDefException( String message )
  {
     super( message );  
     this.command = null;
     this.parmSpec = null; 
     this.action = null;
  }
  //----------CommandDefException ( ) ----------------------------------------
  public CommandDefException( )
  {
     super( "Calculator CommandDef exception" );  
     this.command = null;
     this.parmSpec = null; 
     this.action = null;
  }
  //----------command ( ) ----------------------------------------
  // return the command field of the command that caused the exception
  //
  public String command()
  {
     return this.command;
  }
  //----------parmSpec ( ) ----------------------------------------
  // return the parmSpec field of the command that caused the exception
  //
  public String parmSpec()
  {
     return this.parmSpec;
  }
  //----------getAction ( ) ----------------------------------------
  // return the Action object associated with the exception
  //
  public CommandAction getAction()
  {
     return this.action;
  }
}
