///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//          U N I V E R S I T Y    O F    N E W    H A M P S H I R E         //
//                                                                           //
//                       COMPUTER  SCIENCE  DEPARTMENT                       //
//                                                                           //
//                    Graphics And Visualization Research                    //
//                                                                           //
//          --------------------------------------------------------         //
//  File: GraniteRemoteIF.h                                                  //
//                                                                           //
//  The GraniteRemoteIF provides a nice wrapper for functionality provided   //
//  by the RemoteConnection class.                                           // 
//                                                                           //
//                                                                           //
//  Programmer:  Jon Frain                                                   //
//  Creation Date: 08/05/09                                                   //
//                                                                           //
//  History:                                                                 //
///////////////////////////////////////////////////////////////////////////////



#ifndef _GRANITE_REMOTE_IF_H_
#define _GRANITE_REMOTE_IF_H_

#include "RemoteConnection.h"



class GraniteRemoteIF
{
public:  // constructors
    GraniteRemoteIF(char* server, int port);
    virtual ~GraniteRemoteIF();

public:  // api
 
    //-----------------------------------------------------------------------
    // createDataSource
    //-----------------------------------------------------------------------
    int   createDataSource(char* dsName, char* dsPath); 

    //-----------------------------------------------------------------------
    // subblock
    //------------------------------------------------------------------------
    int   subblock(int dsID, int start1, int start2, int start3,
                   int end1, int end2,   int end3);

    //------------------------------------------------------------------------
    // getFloat
    //------------------------------------------------------------------------
    float getFloat(int dsID, int attrID, int loc1, int loc2, int loc3);

    //------------------------------------------------------------------------
    // getFloatsPointOrder
    //------------------------------------------------------------------------
    void  getFloatsPointOrder(int dsID, float* arr, int szInBytes);

    //------------------------------------------------------------------------
    // getFloatsAttrOrder
    //------------------------------------------------------------------------
    void  getFloatsAttrOrder(int dsID, float* arr, int szInBytes);

    //------------------------------------------------------------------------
    // getByteOrder
    //------------------------------------------------------------------------
    int getByteOrder(int dsID);

    //------------------------------------------------------------------------
    // setByteOrder
    //------------------------------------------------------------------------
    void setByteOrder(int dsID, char* order);
    
    //------------------------------------------------------------------------
    // isConnected
    //------------------------------------------------------------------------
    bool isConnected() {return remoteConn->isConnected();};

    //------------------------------------------------------------------------
    // getDimensions
    //------------------------------------------------------------------------
    int* getDimensions(int dsID, int* sz);

    //------------------------------------------------------------------------
    // getNumAttributes
    //------------------------------------------------------------------------
    int getNumAttributes(int dsID);

    //------------------------------------------------------------------------
    // getAttributeName
    //------------------------------------------------------------------------
    char** getAttributeNames(int dsID, int* sz);

    //------------------------------------------------------------------------
    // enable/disableRemoteByteSwapping
    //------------------------------------------------------------------------
    void disableRemoteByteSwapping();
    void enableRemoteByteSwapping();


private:  // methods



private:  // members
    RemoteConnection* remoteConn;


private: // disable default and copy constructor and operator= 
         // - disabling to force passing in of a remote connection
    GraniteRemoteIF();  
    GraniteRemoteIF(GraniteRemoteIF&);
    GraniteRemoteIF& operator=(GraniteRemoteIF&);   
};


#endif // _GRANITE_REMOTE_IF_H_
