package edu.unh.sdb.util.VisitGraniteFramework.GraniteServer;

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//          U N I V E R S I T Y    O F    N E W    H A M P S H I R E         //
//                                                                           //
//                       COMPUTER  SCIENCE  DEPARTMENT                       //
//                                                                           //
//                    Graphics And Visualization Research                    //
//                                                                           //
//          --------------------------------------------------------         //
//                                                                           //
//  Professor:   Dan Bergeron <rdb@cs.unh.edu>                               //
//  Programmer:  Andy Foulks <rafoulks@cs.unh.edu>                           //
//  File:        RemoteServer.java                                           //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

import java.util.*;
import java.io.*;
import java.net.*;

/**
 *  The RemoteServer is the middle man between the RemoteDataSource
 *  on the C++ side, and the DataSource on the java side.  This 
 *  class takes commands from a RemoteDataSource (using sockets
 *  as the communication method), and sends the
 *  response to the RemoteDataSource.
 */

public class RemoteServer 
{
/* ========================================================================= */

private static final int PORT_NUMBER = 50000;
private GraniteServer scriptEngine = new GraniteServer(this);
private DataOutputStream output = null; // handles binary data

/* ========================================================================= */
/**
 *  Uses the socket connection to the client to process and decode 
 *  the messages from the client, and send appropriate responses to the 
 *  client.  If the clients sends the message "quit", we sever the 
 *  connection and return to the calling function to wait for 
 *  other clients to connect.
 */

private void processInputFromClient(Socket clientSocket)
{
    System.out.println("entered processInputFromClient");

    try {
        BufferedReader   input  = null; // reads ascii data commands

        output = new DataOutputStream(clientSocket.getOutputStream());
        input  = new BufferedReader(
                     new InputStreamReader(clientSocket.getInputStream()));


        String line = input.readLine();
        System.out.println("  line = " + line);


        for( ; line != null; line=input.readLine()) {
            System.out.printf("RemoteServer: got command '%s'\n", line);

            if(line.startsWith("msg")) {
                String reply = "got your message '" + 
                               line.replaceFirst("msg","").trim() + "'";
                output.writeInt(0);  // send error code 'success'
                output.writeInt(reply.length());
                output.writeBytes(reply);
            }
            else {
                scriptEngine.processCommand(line);
            }
        }

        output.close();
        input.close();
        clientSocket.close();
    }
    catch(IOException e) {
        System.err.println("Fatal I/O Error");
        System.err.println("Error was: '" + e + "'");

        System.exit(-1);
    }
}

/* ========================================================================= */
/**
 *  Methods for sending output to the client.
 */

public void sendMessageToClient(int errorcode, int value)
{
    System.err.printf("sendMessageToClient: output='%s', errorcode='%d', " +
                      "intvalue='%d'\n",output!=null?"ok":"null", 
                      errorcode,value);
    try {
        output.writeInt(errorcode); 
        output.writeInt(4);
        output.writeInt(value);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, int[] value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(value.length*4);
        for(int i=0; i<value.length; i++)
            output.writeInt(value[i]);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, float value)
{
    System.err.printf("sendMessageToClient: output='%s', errorcode='%d', " +
                      "floatvalue='%f'\n",output!=null?"ok":"null", 
                      errorcode,value);
    try {
        output.writeInt(errorcode); 
        output.writeInt(4);
        output.writeFloat(value);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, float[] value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(value.length*4);
        for(int i=0; i<value.length; i++)
            output.writeFloat(value[i]);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, byte value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(1);
        output.writeByte(value);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, byte[] value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(value.length);
        output.write(value);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, double value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(8);
        output.writeDouble(value);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, double[] value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(value.length*8);
        for(int i=0; i<value.length; i++)
            output.writeDouble(value[i]);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, short value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(2);
        output.writeShort(value);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */

public void sendMessageToClient(int errorcode, short[] value)
{
    try {
        output.writeInt(errorcode); 
        output.writeInt(value.length*2);
        for(int i=0; i<value.length; i++)
            output.writeFloat(value[i]);
    }
    catch(IOException e) {
        System.err.printf("sendMessageToClient: Error '%s'", e);
    }
}

/* ========================================================================= */
/**
 *  Starts the sdb server.  Creates a "listening" socket and advertises
 *  the hostname and the port on the standard output.  Accepts connections
 *  from clients and processes textual commands coming from the clients.
 */

public void waitForClients()
{
    // server socket is a listening socket that listens for clients
    ServerSocket serverSocket = null;

    // client socket is the actual socket used for communication
    Socket       clientSocket = null;

    try {
        System.out.println("Starting sdb server...");

        String hostName     = InetAddress.getLocalHost().getHostName();
        String hostAddress  = InetAddress.getLocalHost().getHostAddress();

        serverSocket = new ServerSocket(PORT_NUMBER);

        System.out.printf("Server is listening on host '%s' (%s), port %d\n\n",
                           hostName, hostAddress, PORT_NUMBER);

        System.out.println("Waiting for a client to connect...\n");

        while(true) {
            clientSocket = serverSocket.accept();

            InetAddress addr = clientSocket.getInetAddress().getLocalHost();
            String clientName = addr.getHostName() +": "+ addr.getHostAddress();
            System.out.printf("[accepted connection from %s]\n", clientName);

            // put multi-threaded code here ?
            processInputFromClient(clientSocket);

            System.out.printf("[client '%s' has disconnected]\n", clientName);
        }

        //serverSocket.close();
    } 
    catch(IOException e) {
        System.err.printf("Fatal I/O error: '%s'\n", e);
        System.exit(-1);
    }
}

/* ========================================================================= */

public static void main(String[] args) 
{
    RemoteServer server = new RemoteServer();

    server.waitForClients();
}

/* ========================================================================= */

} // end class RemoteServer

/*
private String decodeCommand(String message, DataOutputStream output)
    throws IOException
{
    System.out.println("decodeCommand: message was '" + message + "'");

    if(message.equals("getdata"))
    {
        // acknowledge the command
        output.writeBytes("data");
        output.flush();

        byte numDims = 2;   // For now, always return 2D arrays of floats.
        byte type    = 0;   // 0 signifies float
        int  numRows = mData.length;
        int  numCols = mData[0].length;

        //               numDims   type   numElemsDim1   numElemsDim2
        int headerSize =    1    +  1   +      4       +      4;
        int replyLength = numRows * numCols * 4 + headerSize;

        // first, send the size of the response (total number of bytes)
        System.out.println("sending replyLength= " + replyLength);
        output.writeInt(replyLength);
        output.flush();

        // send the header 
        output.writeByte(numDims);
        output.writeByte(type);
        output.writeInt(numRows);
        output.writeInt(numCols);

        // send the data
        for(int row=0; row<numRows; row++)
            for(int col=0; col<numCols; col++)
                output.writeFloat(mData[row][col]);

        output.flush();
    }
    else if(message.equals("quit"))
    {
        output.writeBytes("text");
        output.flush();

        output.writeInt("goodbye".length());
        output.flush();

        output.writeBytes("goodbye");
        output.flush();
    }
    else
    {
        output.writeBytes("text");
        output.flush();

        String reply = "error: unknown command '" + message + "'";
        output.writeInt(reply.length());
        output.flush();

        output.writeBytes(reply);
        output.flush();
    }

    return message;
}

*/
